package com.jiaoyinbrother.monkeyking.manager

import android.app.Activity
import android.app.Application
import android.content.Context
import android.os.Bundle
import android.support.multidex.MultiDexApplication
import android.view.View
import android.view.ViewGroup

import com.jiaoyinbrother.library.refresh.ClassicsFooter
import com.jiaoyinbrother.library.refresh.WKCarHeader
import com.jiaoyinbrother.library.util.InitUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.llew.huawei.verifier.LoadedApkHuaWei
import com.networkbench.agent.impl.instrumentation.NBSAppInstrumentation
import com.orhanobut.hawk.Hawk
import com.scwang.smartrefresh.layout.SmartRefreshLayout
import com.tencent.smtt.sdk.QbSdk

class CarApp : MultiDexApplication(), Application.ActivityLifecycleCallbacks {


    /*当前显示的Activity*/


    override fun attachBaseContext(base: Context) {
        NBSAppInstrumentation.attachBaseContextBeginIns(base)
        super.attachBaseContext(base)
        NBSAppInstrumentation.attachBaseContextEndIns()
    }


    override fun onCreate() {
        NBSAppInstrumentation.applicationCreateBeginIns()
        super.onCreate()

        CarApp.instance = this

        this.registerActivityLifecycleCallbacks(this)

        //搜集本地tbs内核信息并上报服务器，服务器返回结果决定使用哪个内核。

        val cb = object : QbSdk.PreInitCallback {

            override fun onViewInitFinished(arg0: Boolean) {
                //x5內核初始化完成的回调，为true表示x5内核加载成功，否则表示x5内核加载失败，会自动切换到系统内核。
                LogUtil.e(" onViewInitFinished is $arg0")
            }

            override fun onCoreInitFinished() {

                LogUtil.e(" onViewInitFinished is onCoreInitFinished")
            }
        }
        //x5内核初始化接口
        QbSdk.initX5Environment(applicationContext, cb)


        AppInitService().init(this)
        val currentProcessName = InitUtil(this.applicationContext).currentProcessName
        LogUtil.e("currentProcessName =$currentProcessName")


        LoadedApkHuaWei.hookHuaWeiVerifier(this)
        loadMoreLayoutInit()

        NBSAppInstrumentation.applicationCreateEndIns()

        Hawk.init(this).build()



    }

    private fun loadMoreLayoutInit() {
        SmartRefreshLayout.setDefaultRefreshInitializer { context, layout ->

            layout.setEnableAutoLoadMore(true)
            layout.setEnableOverScrollDrag(false)
            layout.setEnableOverScrollBounce(true)
            layout.setEnableLoadMoreWhenContentNotFull(true)
            layout.setEnableScrollContentWhenRefreshed(true)
        }

        SmartRefreshLayout.setDefaultRefreshHeaderCreator { context, layout ->
            WKCarHeader(context)
        }

        SmartRefreshLayout.setDefaultRefreshFooterCreator { context, layout ->
            ClassicsFooter(context)
        }

    }


    override fun onActivityPaused(activity: Activity?) {
    }

    override fun onActivityResumed(activity: Activity?) {

    }

    override fun onActivityStarted(activity: Activity?) {
        /*获取当前显示的Activity*/
        mActivity = activity
    }

    override fun onActivityDestroyed(activity: Activity?) {
    }

    override fun onActivitySaveInstanceState(activity: Activity?, outState: Bundle?) {
    }

    override fun onActivityStopped(activity: Activity?) {
    }

    override fun onActivityCreated(activity: Activity?, savedInstanceState: Bundle?) {
    }

    companion object {
        /*当前对象的静态实例*/
        private var instance: CarApp? = null

        fun getInstance(): CarApp {
            return CarApp.instance!!
        }

        var mActivity: Activity? = null

        fun getActivity(): Activity {
            return mActivity!!
        }

    }





    /**
     * 显示View
     *
     * @param view 需要显示到Activity的视图
     */
    fun showView(view: View) {
        /*Activity不为空并且没有被释放掉*/
        if (mActivity != null && mActivity?.isFinishing == false) {
            /*获取Activity顶层视图,并添加自定义View*/
            (mActivity?.window?.decorView as ViewGroup).addView(view)
        }

    }

    /**
     * 隐藏View
     *
     * @param view 需要从Activity中移除的视图
     */
    fun hideView(view: View) {
        /*Activity不为空并且没有被释放掉*/
        if (mActivity != null && mActivity?.isFinishing == false) {
            /*获取Activity顶层视图*/
            val root = mActivity?.window?.decorView as ViewGroup
            /*如果Activity中存在View对象则删除*/
            if (root.indexOfChild(view) != -1) {
                /*从顶层视图中删除*/
                root.removeView(view)
            }
        }
    }


}
