package com.jiaoyinbrother.monkeyking.manager.activity

import android.content.Intent
import android.os.AsyncTask
import android.view.View
import android.widget.Button
import android.widget.LinearLayout
import android.widget.TextView
import com.camnter.easyrecyclerview.adapter.EasyRecyclerViewAdapter
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.google.gson.Gson
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.network.CarLib
import com.jiaoyinbrother.monkeyking.manager.util.NetUtil
import com.jiaoyinbrother.library.base.BaseActivity
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.presenter.AccountPresenterImpl
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.library.util.Util
import com.jiaoyinbrother.library.view.IGeneralModel
import kotlinx.android.synthetic.main.act_account_detail.*
import kotlinx.android.synthetic.main.layout_account_detail_header.*
import kotlinx.android.synthetic.main.layout_no_data.*
import kotlinx.android.synthetic.main.main_title.*

class AccountDetailActivity : BaseActivity(), View.OnClickListener {
    private var pageNo = 1

    private var recyclerAdapter: AccountDetailRecyclerAdapter? = null
    private var mData: ArrayList<RecordsBean>? = null

    override fun getLayoutId(): Int {

        return R.layout.act_account_detail

    }

    override fun initView() {

    }

    override fun initToolBar() {

    }

    override fun initListeners() {

        recharge.setOnClickListener(this)

        refresh_layout.setRefreshListener {
            pageNo = 1
            getAccountDetail()
        }
        refresh_layout.setMoreListener { getAccountDetail() }
    }

    override fun initData() {
        ivTitleName.text = "账户余额"
        ivTitleBtnRight_ll.visibility = View.INVISIBLE

        imageview_no_data.setImageResource(R.mipmap.img_no_account)
        textview_no_data.text = "您还没有账户明细哦"

//        初始化Adapter...
        mData = ArrayList()
        refresh_layout.setFooter()
        recyclerAdapter = AccountDetailRecyclerAdapter(accountDetailAdapterCallback)
        recyclerview_account_detail.adapter = recyclerAdapter

//        初始化数据
        pageNo = 1
    }


    override fun onResume() {
        super.onResume()
        getAccountDetail()
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.recharge -> handleRecharge()
        }
    }

    /**
     * 获取用户账户余额
     */
    private fun getAccountRecordsDetail() {
        AccountGetRecordsAsyncTask().execute()
    }

    private inner class AccountGetRecordsAsyncTask : AsyncTask<Void, Void, GetRecordsResult>() {

        override fun doInBackground(vararg params: Void): GetRecordsResult {

            val gre = GetRecordsRequest()
            var grr = GetRecordsResult()
            gre.uid = UserUtil(this@AccountDetailActivity).uid
            gre.page = pageNo
            gre.page_size = PAGE_SIZE
            var gson = Gson()
            try {
                grr = CarLib.getInstance(applicationContext).postRequest(gson.toJson(gre), "account/get_records",
                        GetRecordsResult::class.java) as GetRecordsResult
            } catch (e: Exception) {
                NetUtil.handlePostExceptionBaseResult(grr, e)
            }

            return grr
        }

        override fun onPostExecute(result: GetRecordsResult?) {
            super.onPostExecute(result)
            handleRunningFinish()

            if (result == null) return

            when (result.code) {
                CarEntity.SUCC -> {

                    if (result.records != null && result.records.size > 0) {

//                    第一页，清除数据，footer
                        if (pageNo == 1) {
                            mData?.clear()
//                        添加header使用数据
                            val header = RecordsBean()
                            recyclerAdapter?.setHeaderData(accountDetailsResult)
//                        header.amount = result.amount.toInt()
                            mData?.add(header)
                            deleteFooter()
                        }

                        mData?.addAll(result.records ?: ArrayList())

                        if (result.records.size < PAGE_SIZE) {
//                        最后的数据
                            addFooter()
                        } else {
                            // 还有数据
                        }

                        pageNo++

                    }else{
//                            个数为0， 如果是第一页，空页面；不是第一页，添加footer
                        if (pageNo > 1) {
                            addFooter()
                        }
                    }

                }
                else -> showToast(result.msg)
            }
            recyclerAdapter?.list = mData
            recyclerAdapter?.notifyDataSetChanged()

            LogUtil.e("recyclerAdapter?.itemCount =" + recyclerAdapter?.itemCount)
            if (recyclerAdapter?.itemCount!! <= 1) {
                no_data_view.visibility = View.VISIBLE
                refresh_layout.visibility = View.GONE
            } else {
                no_data_view.visibility = View.GONE
                refresh_layout.visibility = View.VISIBLE
            }
        }
    }

    private fun handleRunningFinish() {
        dismissLoadingDialog()

        refresh_layout.refreshIsEnd()
        refresh_layout.loadMoreIsEnd()

        isRunning = false

    }

    private var isRunning = false
    private var accountDetailsResult = AccountResult()

    //    footer处理函数
    private var isFooterAdded = false

    private fun deleteFooter() {
        recyclerAdapter?.deleteFooter()
        isFooterAdded = false
    }

    private fun addFooter() {

        if (isFooterAdded)
            return

        recyclerAdapter?.addFooter()
        val footer = RecordsBean()// bug 重复添加
        mData?.add(footer)
        isFooterAdded = true
    }


    companion object {
        private val PAGE_SIZE = 10
    }


    //充值
    private val accountDetailAdapterCallback = object : AccountDetailAdapterCallback {
        override fun recharge() {
            handleRecharge()
        }

    }

    /* 充值 */
    private fun handleRecharge() {
        val intent = Intent(this@AccountDetailActivity, RechargeActivity::class.java)
        startActivity(intent)
    }


    interface AccountDetailAdapterCallback {
        fun recharge()

    }


    internal class AccountDetailRecyclerAdapter(private var callback: AccountDetailAdapterCallback) : EasyRecyclerViewAdapter() {
        private var footer = false
        private var accountDetailsResult = AccountResult()

        override fun getRecycleViewItemType(position: Int): Int {
            return when (position) {
                0 -> 0
                itemCount - 1 -> {
                    if (footer) 2 else 1
                }
                else -> 1
            }
        }

        override fun onBindRecycleViewHolder(viewHolder: EasyRecyclerViewHolder, position: Int) {
            val recycleViewItemType = getRecycleViewItemType(position)
            when (recycleViewItemType) {
                0 -> onBindHeaderViewHolder(viewHolder, position)
                1 -> onBindItemViewHolder(position, viewHolder)
            }
        }

        private fun onBindHeaderViewHolder(viewHolder: EasyRecyclerViewHolder, position: Int) {


            val part3 = viewHolder.findViewById<LinearLayout>(R.id.part3)
            part3.visibility = View.VISIBLE
            val tv_balance = viewHolder.findViewById<TextView>(R.id.tv_balance)
            val tv_frozen = viewHolder.findViewById<TextView>(R.id.tv_frozen)
            val tv_available = viewHolder.findViewById<TextView>(R.id.tv_available)
            tv_balance.text = accountDetailsResult.total_amount.toInt().toString()
            tv_frozen.text = accountDetailsResult.frozen_amount.toInt().toString()
            tv_available.text = accountDetailsResult.available_amount.toInt().toString()


//        按键事件回调

            val recharge = viewHolder.findViewById<Button>(R.id.recharge)
            recharge.setOnClickListener { callback.recharge() }

        }

        private fun onBindItemViewHolder(position: Int, viewHolder: EasyRecyclerViewHolder) {
            val entity = list[position] as RecordsBean

            val type = viewHolder.findViewById<TextView>(R.id.type)
            val count = viewHolder.findViewById<TextView>(R.id.count)
            val time = viewHolder.findViewById<TextView>(R.id.time)

            if (null != entity) {
                if (null != entity.time) {
                    val strs = entity.time.split("[ ]".toRegex()).dropLastWhile({ it.isEmpty() }).toTypedArray()
                    time.text = strs[0]
                }

                count.text = Util.getMoneyString(entity.amount.toInt())
                // 类型（1：收入，2：支出，3：充值，4：提现，5：其他）
                when (entity.type) {
                    1 -> type.text = "收入"
                    2 -> type.text = "支出"
                    3 -> type.text = "充值"
                    4 -> type.text = "提现"
                    5 -> type.text = "冻结"
                    6 -> type.text = "解冻"
                    else -> type.text = "其他"
                }
        }
        }

        override fun getItemLayouts(): IntArray {
            return intArrayOf(R.layout.layout_account_detail_header, R.layout.adapter_wkcoinitem, R.layout.layout_footer_all_data)
        }

        fun setHeaderData(accountDetailsResult: AccountResult) {
            this.accountDetailsResult = accountDetailsResult
        }

        fun addFooter() {
            this.footer = true
        }

        fun deleteFooter() {
            this.footer = false
        }

    }

    //    先调用accountdetail接口，之后再调用get_record接口
    private fun getAccountDetail() {
        showLoadingDialog()
        val presenter = AccountPresenterImpl(this@AccountDetailActivity, AccountResult::class.java,accountModel)

        val request = UserDetailRequestBean()
        request.uid = UserUtil(this@AccountDetailActivity).uid
        presenter.getAccount(request)
    }

    private val accountModel = object : IGeneralModel {
        override fun getCodeBeanFail(errorCode: Int) {
            dismissLoadingDialog()
            handleRunningFinish()
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            dismissLoadingDialog()
            val result = bean as AccountResult
            when (result.code) {
                CarEntity.SUCC -> {
                    tv_balance.text = result.total_amount.toInt().toString()
                    tv_frozen.text = result.frozen_amount.toInt().toString()
                    tv_available.text = result.available_amount.toInt().toString()
                    accountDetailsResult = result
                    getAccountRecordsDetail()
                }
                else -> showToast(result.msg)
            }
        }
    }
}
