package com.jiaoyinbrother.monkeyking.manager.activity

import android.app.Activity
import android.content.Intent
import android.content.IntentFilter
import android.os.AsyncTask
import android.support.v4.content.ContextCompat
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.view.View.OnClickListener
import android.widget.Button
import android.widget.RelativeLayout
import com.google.gson.Gson
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.network.CarLib
import com.jiaoyinbrother.monkeyking.manager.util.DialogUtil
import com.jiaoyinbrother.monkeyking.manager.util.NetUtil
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.library.base.BaseActivity
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.presenter.CodePresenterImpl
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.library.view.IGeneralModel
import com.jiaoyinbrother.library.widget.LoadingDialog
import io.reactivex.Flowable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import kotlinx.android.synthetic.main.activity_binding_phone.*
import org.json.JSONException
import org.json.JSONObject
import java.util.concurrent.TimeUnit


class BindPhoneActivity : BaseActivity(), OnClickListener {
    override fun initView() {
        
    }

    override fun initData() {
        mLoading = LoadingDialog(this@BindPhoneActivity)
        voice_code_button.text = TextUtil.setForegroundColor(voice_code_button.text.toString(),
                ContextCompat.getColor(this, R.color.color_8), 11, 16)
    }

    override fun getLayoutId(): Int {
        extras
        return R.layout.activity_binding_phone
    }

    override fun initToolBar() {
        (findViewById<Button>(R.id.ivTitleName)).text = "微信登录"
        findViewById<RelativeLayout>(R.id.ivTitleBtnRight_ll).visibility = View.INVISIBLE
    }

    override fun initListeners() {
        vcode_button.setOnClickListener(this)
        voice_code_button.setOnClickListener(this)
        binding_button.setOnClickListener(this)
    }

    private var mLoading: LoadingDialog? = null
    private var mCarLib: CarLib? = null
    private var codeType = 0
    private var wxur: WeChatUserInfoResult? = null

    private var mDisposable: Disposable? = null

    override fun getExtras(): String? {
        val intent = intent
        try{
            wxur = intent.getSerializableExtra("wx_user_info") as WeChatUserInfoResult
        }catch (e:Exception){

        }
        return ""
    }


    override fun onDestroy() {
        try {

            if (mLoading != null)
                mLoading?.dismiss()
        } catch (e: Exception) {
        }

        super.onDestroy()
    }

    override fun onClick(v: View) {
        when (v?.id) {
            R.id.vcode_button -> {
                onGetVerificationCode(0)
            }
            R.id.voice_code_button -> {
                onGetVerificationCode(2)
            }
            R.id.binding_button -> {
                goToBindingLogin()
                StatisticsUtil.onEvent(this, ConstantsUtil.LOGIN_WX_SUBMIT, "")
            }
        }
    }

    private fun activationCode(t: Int) {
        if (t == 2)
            vcode_button.isEnabled = true
        else
            voice_code_button.isEnabled = true
    }

    /**
     * 获取验证码
     */
    private fun onGetVerificationCode(t: Int) {
        if (TextUtil.checkPhoneNum(this@BindPhoneActivity, phone_edittext.text.toString().trim { it <= ' ' })) {
            activationCode(t)
            return
        }

        codeType = t

        val request = CodeRequest()
        request.phone = phone_edittext.text.toString()
        request.tag = "LOGIN"
        request.type = codeType
        request.pic_flag = 1

        val presenter = CodePresenterImpl(this@BindPhoneActivity, CodeResult::class.java, codeModel)
        presenter.getCodeNum(request)

        StatisticsUtil.onEvent(this, ConstantsUtil.LOGIN_WX_GET_CODE, "")
    }


    private val codeModel = object : IGeneralModel {
        override fun getCodeBeanFail(errorCode: Int) {
            showErrorToast(errorCode)
            activationCode(codeType)

            handleSensordata(false)
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            val result = bean as CodeResult
            when (result.code) {
                CarEntity.SUCC -> {
                    if (TextUtils.isEmpty(result.error_tips)) {
                        // 没有错误信息
                        countDown(codeType)
                        showToast("获取验证码成功")
                        handleSensordata(true)
                    } else {
                        IOSAlertDialog(this@BindPhoneActivity).builder().setTitle("温馨提示").
                                setMsg(result.error_tips, true)
                                .setRightButton("确定") {

                                }.setLeftButton("联系客服") {
                            DialogUtil.onServiceDial(this@BindPhoneActivity)

                        }.show()
                        activationCode(codeType)
                    }


                }
                else -> {
                    activationCode(codeType)
                    showToast(result.msg)
                    handleSensordata(false)
                }
            }
        }
    }
    private fun countDown(type: Int) {
        voice_code_button.visibility = View.INVISIBLE
        if (type == VOICE) IOSAlertDialog(this@BindPhoneActivity).builder().setMsg("语音验证码将以来电的形式提示，请稍后注意接听", true).setLeftButton("我知道了", {}).show()
        mDisposable = Flowable.intervalRange(0, 31, 0, 1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .doOnNext { count ->
                    vcode_button.text = "${30 - count}s"
                    TextViewUtil.setButtonStatus(vcode_button, this@BindPhoneActivity, TextViewUtil.BUTTON_DISABLE)
                }
                .doOnComplete {
                    voice_code_button.visibility = View.VISIBLE
                    activationCode(type)
                    vcode_button.text = "获取验证码"
                    TextViewUtil.setButtonStatus(vcode_button, this@BindPhoneActivity, TextViewUtil.BUTTON_ENABLE_NORMAL)
                }
                .subscribe()
    }

    /**
     * 绑定登陆
     */
    private fun goToBindingLogin() {
        if (!NetUtil.isNetworkAvailable(this@BindPhoneActivity)) {
            OldToastUtil.showerror(this@BindPhoneActivity, OldToastUtil.TIMEOUT_EXCEPTION)
            return
        }
        if (TextUtil.checkPhoneNum(this@BindPhoneActivity, phone_edittext.text.toString().trim { it <= ' ' }))
            return
        if (TextUtils.isEmpty(vcode_edittext.text.toString())) {
            OldToastUtil.show(this@BindPhoneActivity, "验证码不能为空")
            return
        }
        QueryBindAsyncTask().execute()
    }

    /**
     * 查询绑定结果
     */
    internal inner class QueryBindAsyncTask : AsyncTask<Void, Void, GetThirdDetailResult>() {
        override fun doInBackground(vararg params: Void): GetThirdDetailResult {
            mCarLib = CarLib.getInstance(applicationContext)
            val gtde = GetThirdDetailRequest()
            var gtdr = GetThirdDetailResult()
            if (!TextUtils.isEmpty(phone_edittext.text.toString())) {
                gtde.uid = phone_edittext.text.toString()
            }
            if (!TextUtils.isEmpty(vcode_edittext.text.toString())) {
                gtde.vcode = vcode_edittext.text.toString()
            }
            try {
                gtdr = mCarLib?.postRequest(Gson().toJson(gtde), "user/thirdparty/get_detail",
                        GetThirdDetailResult::class.java) as GetThirdDetailResult
                Log.e("BindPhoneActivity", "get_detail : " + gtdr)
            } catch (e: Exception) {
                NetUtil.handlePostExceptionBaseResult(gtdr, e)
            }

            return gtdr
        }

        override fun onPostExecute(result: GetThirdDetailResult) {
            super.onPostExecute(result)
            if (result.errCode == -1) {
                if (result.code == CarEntity.SUCC) {
                    if (result.exist == 0) {
                        if (result.thirdparty != null && result.thirdparty.size > 0) {
                            for (thirdparty in result.thirdparty) {
                                if (thirdparty.openid == wxur?.openid) {
                                    OldToastUtil.show(this@BindPhoneActivity, "此手机号已绑定")
                                } else {
                                    BindFinishAsyncTask().execute()
                                }
                            }
                        } else {
                            BindFinishAsyncTask().execute()
                        }
                    } else {
                        BindFinishAsyncTask().execute()
                    }
                } else {
                    OldToastUtil.show(this@BindPhoneActivity, result.msg)
                }
            } else {
                OldToastUtil.showerror(this@BindPhoneActivity, result.errCode)
            }
        }
    }

    /**
     * 输入密码后调用绑定成功接口
     */
    internal inner class BindFinishAsyncTask : AsyncTask<Void, Void, BindThirdResult>() {
        override fun doInBackground(vararg params: Void): BindThirdResult {
            mCarLib = CarLib.getInstance(applicationContext)
            val bte = BindThirdEntitiy()
            var btr = BindThirdResult()
            bte.uid = phone_edittext!!.text.toString()
            bte.openid = wxur?.openid
            bte.unionid = wxur?.unionid
            if (!TextUtils.isEmpty(wxur?.nickname)) {
                bte.nickname = wxur?.nickname
            }
            if (!TextUtils.isEmpty(wxur?.sex)) {
                if (wxur?.sex == "2") {
                    bte.gender = "女"
                } else {
                    bte.gender = "男"
                }
            }
            if (!TextUtils.isEmpty(wxur?.province)) {
                bte.province = wxur?.province
            }
            if (!TextUtils.isEmpty(wxur?.city)) {
                bte.city = wxur?.city
            }
            if (!TextUtils.isEmpty(wxur?.country)) {
                bte.country = wxur?.country
            }
            if (!TextUtils.isEmpty(wxur?.headimgurl)) {
                bte.headimgurl = wxur?.headimgurl
            }
            bte.type = CarEntity.ThirdLoginType.WEIXIN.typeName
            val channel = ChannelUtil.getAppChannel(this@BindPhoneActivity)
            Log.e(TAG, "channel=" + channel)
            if (!TextUtils.isEmpty(channel)) {
                bte.channel = channel
            }
            try {
                btr = mCarLib?.postRequest(Gson().toJson(bte), "user/thirdparty/binding",
                        BindThirdResult::class.java) as BindThirdResult
                Log.e("BindPhoneActivity", "binding : " + btr)
            } catch (e: Exception) {
                NetUtil.handlePostExceptionBaseResult(btr, e)
            }

            return btr
        }

        override fun onPostExecute(result: BindThirdResult) {
            super.onPostExecute(result)
            if (result.errCode == -1) {
                if (result.code == CarEntity.SUCC) {
                    // 绑定成功 调用Login接口
                    val isNew = result.isIsnew
                    val intent = Intent()
                    intent.putExtra("mWxResult", wxur)
                    setResult(Activity.RESULT_OK, intent)
                    finish()
                    //                    new LoginAsyncTask(isNew).execute();
                } else {
                    OldToastUtil.show(this@BindPhoneActivity, result.msg)
                }
            } else {
                OldToastUtil.showerror(this@BindPhoneActivity, result.errCode)
            }
        }
    }

    private fun handleSensordata(success: Boolean) {
        val phone = phone_edittext.text.toString()
        val properties = JSONObject()
        try {
            properties.put("phone_number", TextUtil.formatPhoneNum(phone))
            properties.put("is_success", success)
            StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_GETCODE, properties)
        } catch (e: JSONException) {

        }

    }

    companion object {
        private const val TAG = "BindPhoneActivity"
        private const val VCODE = 0
        private const val VOICE = 2
        private const val ACTION = "android.provider.Telephony.SMS_RECEIVED"
    }

}
