package com.jiaoyinbrother.monkeyking.manager.activity

import android.app.Activity
import android.content.Intent
import android.text.TextUtils
import android.view.View
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.adapter.PayMethodAdapter
import com.jiaoyinbrother.monkeyking.manager.util.SharedPreferencesUtil
import com.jiaoyinbrother.monkeyking.manager.wxutil.WXUitls
import com.jiaoyinbrother.library.base.BaseActivity
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.presenter.PayMoneyPresenterImpl
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.PayUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.library.util.Util
import com.jiaoyinbrother.library.view.IGeneralModel
import com.jiaoyinbrother.library.widget.GeneralBottomButton.SubmitListener
import com.tencent.mm.opensdk.openapi.IWXAPI
import kotlinx.android.synthetic.main.activity_recharge.*
import kotlinx.android.synthetic.main.layout_recharge.*
import java.util.*

/**
 * 充值
 * by soda  2018/01/13
 * */
class RechargeActivity : BaseActivity(), EasyRecyclerViewHolder.OnItemClickListener {

    private var mWxApi: IWXAPI? = null

    private var mPayTn: String? = null
    private var mPayAmount = 0
    private var mPayMethod = -1
    private var mAdapter: PayMethodAdapter? = null

    override fun getLayoutId(): Int {
        return R.layout.activity_recharge
    }

    override fun initView() {
        mAdapter = PayMethodAdapter(this@RechargeActivity)
    }

    override fun initToolBar() {
        mainTitle.text = "充值"
    }

    override fun initListeners() {
        mAdapter?.setOnItemClickListener(this)
        goPay.normal()
        goPay.setListener(object : SubmitListener {
            override fun submit() {
                onRecharge()
            }
        })
    }

    override fun initData() {
        goPay.setText("立即支付")

        mWxApi = WXUitls.initWxApi(this)
        val json = UserUtil(this).userDetail
        val result = Util.jsonToEntity(json, UserDetailResult::class.java) as UserDetailResult
//        val payMethodList = result.account?.pay_method_avai
        var payUtil = PayUtil(this@RechargeActivity)
        val payMethodList = payUtil.getAvaiPayMethod(result.account?.pay_method_avai ?: ArrayList())
        mPayMethod = getPayMethod(payMethodList)
        mAdapter?.list = payMethodList
        payMethodRv.adapter = mAdapter
        mAdapter?.selectPayMethod(mPayMethod)
    }

    /**
     * 初始化默认支付渠道(UNIONPAY:0，WECHATPAY:6，ALIPAY:7，ACCOUNTPAY:8,YIBAOPAY:12)
     * */
    private fun getPayMethod(payMethodList: ArrayList<PayMethodAvaiBean>?): Int {
        // 后台提供勾选项
        for (payMethod: PayMethodAvaiBean in payMethodList ?: ArrayList()) {
            if (payMethod.check == 1) {// 选中
                return payMethod.id
            }
        }
        // 存在用户习惯， 并且支付方式包含用户习惯
        for (payMethod: PayMethodAvaiBean in payMethodList ?: ArrayList()) {
            if (SharedPreferencesUtil.getInstance().payments != -1 && payMethod.id == SharedPreferencesUtil.getInstance().payments) {// 选中
                return payMethod.id
            }
        }
        // 默认勾选第一项
        return payMethodList?.get(0)?.id ?: -1
    }

    private fun onRecharge() {
        if (TextUtils.isEmpty(edInputMoney.text.toString())) {
            showToast("请输入充值金额")
            return
        }
        if (edInputMoney.text.toString().toInt() == 0) {
            showToast("充值金额不能为0")
            return
        }
        if (mPayMethod == 6 && mWxApi?.isWXAppInstalled == false) {
            showToast("请先安装微信")
            return
        }
        showLoadingDialog()
        val presenter = PayMoneyPresenterImpl(this, RechargeResult::class.java, rechargeModel)

        val request = RechargeRequest()
        request.uid = UserUtil(this@RechargeActivity).uid
        mPayAmount = edInputMoney.text.toString().toInt()
        request.amount = mPayAmount * 100
        request.pay_method = mPayMethod
        request.type = "RECHARGE"
        presenter.goRecharge(request)
    }

    private val rechargeModel = object : IGeneralModel {
        override fun getCodeBeanFail(errorCode: Int) {
            showErrorToast(errorCode)
            dismissLoadingDialog()
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            dismissLoadingDialog()
            val result = bean as RechargeResult
            when (result.code) {
                CarEntity.SUCC -> {
                    mPayTn = result.tn
                    SharedPreferencesUtil.getInstance().savePayments(mPayMethod)
                    val payMethodBean = PayInfoBean(PayInfoBean.PAYMENT_TYPE_ZUCHE, result.url, result.tn, mPayAmount.toFloat(), result.pay_method, result.order_str)
                    val intent = Intent(this@RechargeActivity, PayMethodActivity::class.java)
                    intent.putExtra(ConstantsUtil.START_PAYMETHOD, payMethodBean)
                    intent.putExtra("disable_pay_channels", result.disable_pay_channels)
                    intent.putExtra("enable_pay_channels", result.enable_pay_channels)
                    startActivityForResult(intent, 101)
                }
                else -> showToast(result.msg)
            }
        }
    }

    override fun onItemClick(holder: View?, position: Int) {
        val method = mAdapter?.getItem<PayMethodAvaiBean>(position)
        mPayMethod = method?.id ?: 0
        mAdapter?.selectPayMethod(method?.id ?: -1)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 101 && resultCode == Activity.RESULT_OK) {
            val code = data?.extras?.getInt(PayMethodActivity.RESULT_CODE)
            val message = data?.extras?.getString(PayMethodActivity.RESULT_MSG)
            val intent = Intent(this@RechargeActivity, PayResultActivity::class.java)
            intent.putExtra(ConstantsUtil.PAY_RESULT_CODE, code)
            intent.putExtra(ConstantsUtil.PAY_RESULT_MSG, message)
            intent.putExtra(ConstantsUtil.PAY_RESULT_AMOUNT, mPayAmount)
            startActivity(intent)
            finish()
        }
    }
}
