package com.jiaoyinbrother.monkeyking.manager.activity

import android.content.Intent
import android.support.v4.content.ContextCompat
import android.text.TextUtils
import com.bumptech.glide.Glide
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.library.base.BaseActivity
import com.jiaoyinbrother.library.bean.DepositBean
import com.jiaoyinbrother.library.bean.OrderDetailRequest
import com.jiaoyinbrother.library.bean.OrderDetailResult
import com.jiaoyinbrother.library.presenter.OrderPresenterImpl
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.TextUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.library.util.Util
import com.jiaoyinbrother.library.view.IGeneralModel
import com.jiaoyinbrother.library.widget.GeneralBottomButton
import kotlinx.android.synthetic.main.activity_smart_key_deposit.*
import kotlinx.android.synthetic.main.layout_smart_key_status.*

class SmartKeyDepositActivity : BaseActivity() {

    private var mOrderInfo: OrderDetailResult? = null
    private var mOrderID: String? = null


    override fun getLayoutId(): Int {
        getExtras(intent)
        return R.layout.activity_smart_key_deposit
    }

    override fun initView() {
        mainTitle.text = "确认还车"
    }

    override fun initToolBar() {
    }

    override fun initListeners() {
        goPay.setListener(object : GeneralBottomButton.SubmitListener {
            override fun submit() {
//                handleSensorsData()
                val intent = Intent(this@SmartKeyDepositActivity, CashierDeskActivity::class.java)
                intent.putExtra("entrance", "确认还车")
                intent.putExtra(ConstantsUtil.BUNDLE_EXTRA_KEY, ConstantsUtil.PAY_PROJECT_ILLEGAL)
                intent.putExtra(ConstantsUtil.ORDER_RESULT, mOrderInfo)
                intent.putExtra("where are you from", "TakingPic")
                startActivity(intent)
            }
        })
    }

    override fun initData() {
        goPay.disable()
        goPay.setText("立即支付")
        setStatusInfo()
        orderDetail()

    }

    private fun setDepositInfo(result: OrderDetailResult?) {
        if (result == null)
            return
        try {
            Glide.with(this@SmartKeyDepositActivity).load(result.pic).into(carImg)

            val depositBean = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_ILLEGAL)
            val sp = depositBean.type_name + "¥" + depositBean.amount.toInt()
            val sIndex = sp.length - (depositBean.amount.toInt().toString().length + 1)
            depositAmount.text = TextUtil.setForegroundColor(sp, ContextCompat.getColor(this@SmartKeyDepositActivity, R.color.color_6), sIndex, sp.length)
            depositDesc.text = depositBean.type_description
            depositPrompt.text = depositBean.description
            if (depositBean.enabled == 1 && depositBean.status_code != DepositBean.DEPOSIT_PAYED) {
                // 可以支付
                goPay.normal()
            }
        } catch (e: Exception) {

        }
    }

    private fun setStatusInfo() {
        tvStatusOne.text = "支付违章押金"
        tvStatusTwo.text = "拍照反馈车况"
        ivStatusOne.setBackgroundResource(R.mipmap.h_1)
        ivStatusTwo.setBackgroundResource(R.mipmap.w_2)
        tvStatusOne.setTextColor(ContextCompat.getColor(this, R.color.color_6))
        tvStatusTwo.setTextColor(ContextCompat.getColor(this, R.color.color_3))
        statusLine.setBackgroundColor(ContextCompat.getColor(this, R.color.color_3))
    }


    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        getExtras(intent)
        orderDetail()
    }

    private fun getExtras(intent: Intent?) {
        mOrderInfo = intent?.getSerializableExtra(ConstantsUtil.ORDER_RESULT) as OrderDetailResult?
        mOrderID = mOrderInfo?.orderid
    }

    private fun orderDetail() {
        showLoadingDialog()
        val presenter = OrderPresenterImpl(this, OrderDetailResult::class.java,mOrderDetailModel)
//        presenter.addListener(mOrderDetailModel)
        val request = OrderDetailRequest()
        request.orderid = mOrderInfo?.orderid
        request.uid = UserUtil(this@SmartKeyDepositActivity).uid
        presenter.getOrderDetail(request)
    }

    private val mOrderDetailModel = object : IGeneralModel {
        override fun getCodeBeanFail(errorCode: Int) {
            dismissLoadingDialog()
        }

        override fun getCodeObjectSuccess(bean: Any) {
            dismissLoadingDialog()
            val result = bean as OrderDetailResult
            when (result.code) {
                CarEntity.SUCC -> {
                    if (getDepositPayed(result)) {
                        val intent = Intent(this@SmartKeyDepositActivity, SmartKeyFeedBackActivity::class.java)
                        intent.putExtra(ConstantsUtil.FLAG_SMART_KEY, SmartKeyFeedBackActivity.STATUS_RETURN)
                        intent.putExtra(ConstantsUtil.ORDER_RESULT, result)
                        startActivity(intent)
                        finish()
                    } else {
                        setDepositInfo(result)
                    }
                }
                else -> showToast(result.msg)
            }
        }
    }

    private fun getDepositPayed(result: OrderDetailResult?): Boolean {
        var deposit: DepositBean? = null
        result?.deposit?.filter { TextUtils.equals(it.type, ConstantsUtil.PAY_PROJECT_ILLEGAL) }
                ?.forEach { deposit = it }
        return deposit?.status_code == DepositBean.DEPOSIT_PAYED
    }


//    private fun handleSensorsData() {
//        val properties = JSONObject()
//        try {
//            properties.put("entrance", "确认还车")
//            properties.put("order_id", mOrderInfo?.orderid)
//            properties.put("payment_type", "支付违章押金")
//            properties.put("is_deposit", true)
//            properties.put("is_offline", false)
//            properties.put("order_amount", Util.getDepositAmount(ConstantsUtil.PAY_PROJECT_ILLEGAL, mOrderInfo))
//        } catch (e: JSONException) {
//            e.printStackTrace()
//        }
//        StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_PAYMENT, properties)
//    }
}