package com.jiaoyinbrother.monkeyking.manager.adapter

import android.content.Context
import android.net.Uri
import android.text.TextUtils
import android.view.View
import android.view.WindowManager
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.RelativeLayout
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.load.resource.bitmap.RoundedCorners
import com.bumptech.glide.request.RequestOptions
import com.camnter.easyrecyclerview.adapter.EasyRecyclerViewAdapter
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.library.bean.PicBean
import com.jiaoyinbrother.library.util.DensityUtil
import com.jiaoyinbrother.library.util.LogUtil
import java.util.ArrayList

class SelectImagesAdapter() : EasyRecyclerViewAdapter() {

    companion object {

        const val TYPE_NORMAL = 0
        const val TYPE_OPTIONAL = 1
    }

    private var viewWidth = 0
    private var viewHeight = 0
    private var mType = -1
    private var optionalList: ArrayList<PicBean>? = null
    private var mContext:Context? = null

    constructor(context: Context, type: Int) : this() {
        this.mType = type
        mContext = context
        optionalList = ArrayList()
        val wm = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        viewWidth = (wm.defaultDisplay.width - DensityUtil.dip2px(context, 40f)) / 2
        viewHeight = viewWidth / 3 * 2
    }

    override fun getRecycleViewItemType(position: Int): Int {
        val entity = list[position] as PicBean
        return if (TextUtils.isEmpty(entity.imgUri) && TextUtils.isEmpty(entity.imgUrl)) 0 else 1
    }

    override fun onBindRecycleViewHolder(viewHolder: EasyRecyclerViewHolder?, position: Int) {
        val recycleViewItemType = getRecycleViewItemType(position)
        if (recycleViewItemType == 0) {
            imageDefaultViewHolder(viewHolder, position)
        } else {
            showImageViewHolder(viewHolder, position)
        }
    }

    private fun imageDefaultViewHolder(viewHolder: EasyRecyclerViewHolder?, position: Int) {
        // 父布局宽高
        val mLayout = viewHolder?.findViewById<RelativeLayout>(R.id.relativeLayout)
        mLayout?.layoutParams = LinearLayout.LayoutParams(viewWidth, viewHeight)
        val defaultImg = viewHolder?.findViewById<ImageView>(R.id.item_image_default)
        val resource = if (mType == TYPE_NORMAL) {
            when (position) {
                0 -> R.mipmap.bg_pic_0
                1 -> R.mipmap.bg_pic_1
                2 -> R.mipmap.bg_pic_2
                else -> R.mipmap.bg_pic_3
            }
        } else {
            R.mipmap.bg_pic_default
        }
        defaultImg?.setImageResource(resource)
        if(mContext!=null && defaultImg!=null)
            Glide.with(mContext!!).load(resource).placeholder(R.mipmap.bg_pic_default)
                    .apply(RequestOptions.bitmapTransform(RoundedCorners(5))).into(defaultImg!!)
    }

    private fun showImageViewHolder(viewHolder: EasyRecyclerViewHolder?, position: Int) {
        // 父布局宽高
        val mainLayout = viewHolder?.findViewById<RelativeLayout>(R.id.relativeLayout)
        mainLayout?.layoutParams = LinearLayout.LayoutParams(viewWidth, viewHeight)
        // imageView
        val normalImg = viewHolder?.findViewById<ImageView>(R.id.item_image_normal)
        normalImg?.layoutParams = RelativeLayout.LayoutParams(viewWidth, viewHeight)
        LogUtil.e("showImageViewHolder ---------->>> start")
        val entity = list[position] as PicBean
        if (!TextUtils.isEmpty(entity.imgUrl)){

            if(mContext!=null && normalImg!=null)
                Glide.with(mContext!!).load(entity.imgUrl).placeholder(R.mipmap.bg_place_image).into(normalImg!!)
        } else{
            val path = "file://${entity.imgUri}"
            val uri = Uri.parse(path)

            //不使用缓存
            if(mContext!=null && normalImg!=null)
                Glide.with(mContext!!).load(uri).placeholder(R.mipmap.bg_place_image)
                        .skipMemoryCache(true).diskCacheStrategy(DiskCacheStrategy.NONE)
                        .into(normalImg!!)

        }

        LogUtil.e("showImageViewHolder ---------->>> end")
    }

    override fun getItemLayouts(): IntArray {
        return intArrayOf(R.layout.item_select_image_default, R.layout.item_select_image_normal)
    }

//    fun addItem() {
//        list.add(list.size, PicBean())
//        notifyDataSetChanged()
//    }

//    fun setOptionalList(data: ArrayList<PicBean>) {
//        list = data
//        optionalList?.addAll(data)
//        if (optionalList?.size == 0) {
//            optionalList?.add(PicBean())
//        } else if (optionalList?.size ?: 0 < 6 && optionalList?.get(optionalList?.size ?: 0
//                -1)?.imgUri != null) {
//            optionalList?.add(PicBean())
//        }
//    }

}