package com.jiaoyinbrother.monkeyking.manager.fragment.mainordermanager

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import android.widget.Toast
import com.alibaba.android.arouter.launcher.ARouter
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.MainCommentDialogResult
import com.jiaoyinbrother.library.bean.MainListItemBean
import com.jiaoyinbrother.library.bean.ManagerGetTaskRequest
import com.jiaoyinbrother.library.bean.ManagerGetTaskResult
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.commentlist.CommentListActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.list.OrderManageActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.list.fragment.OrdersFragment
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.workflowlist.WorkFlowListActivity
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class MainOrderManagerPresenter(context: Context, view: IMainOrderManagerContract.View)
    : BasePresenterImpl<IMainOrderManagerContract.View>(context, view), IMainOrderManagerContract.Presenter {
    private var mOrderList: ArrayList<MainListItemBean> = ArrayList<MainListItemBean>()
    private var mWorkList: ArrayList<MainListItemBean> = ArrayList<MainListItemBean>()

    private var mOrderItems = arrayOf("待用车","用车中","待结算","3天内评价")
    private var mWorkFlowItems = arrayOf("续租工单","取消订单工单","修改时间工单","提前还车工单","异常价格工单")


    private var notice: String? = null
    private var depositFailedNum:Int = 0
    private var isNotice = true
    private var isPayedFail = true


    override fun onOrderListItemClicked(position: Int) {
        LogUtil.e("onOrderListItemClicked, position =$position")

        when (mOrderList[position].type) {
            MainListItemBean.TYPE_ORDER_USING -> onOrderListUsing()
            MainListItemBean.TYPE_ORDER_NEED_CONFIRM -> onOrderListConfirmed()
            MainListItemBean.TYPE_ORDER_EVALUATE -> onComment()
            MainListItemBean.TYPE_ORDER_PENDING -> onOrderListPending()

        }

    }

    override fun onWorkListItemClicked(position: Int) {
        LogUtil.e("onWorkListItemClicked, position =$position")
        when (mWorkList[position].type) {
            MainListItemBean.TYPE_MODIFY_WORK_LIST -> onOrderModifyWorkFlow()
            MainListItemBean.TYPE_CANCEL_WORK_LIST -> onCancelFlow()
            MainListItemBean.TYPE_RELET_WORK_LIST -> onWorkFlowList()
            MainListItemBean.TYPE_ADVANCE_WORK_LIST -> onOrderAdvanceWorkFlow()
            MainListItemBean.TYPE_AUDITING_WORK_LIST -> onOrderAuditingWorkFlow()
        }
    }


    override fun requestComment() {
        val userUtil = UserUtil(context)
        if (userUtil.userId.toString().isNotEmpty()) {
            val request = ManagerGetTaskRequest()
            request.manager_id = userUtil.userId.toString()
            val mRetrofitService = RetrofitHelper.getInstance(context).service
            val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
            val observable = mRetrofitService.getMainCommentDialogContent(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
            observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<MainCommentDialogResult>(context, this) {
                override fun onSuccess(response: MainCommentDialogResult?) {
                    notice = if (!TextUtils.isEmpty(response?.notice)) response?.notice else ""
                    setDialog(depositFailedNum)
                }

                override fun onAPIError(apiException: ApiException?) {
                    setDialog(depositFailedNum)
                }

                override fun onError(e: Throwable) {
                    setDialog(depositFailedNum)
                }
            })
        }
    }

    override fun requestGetManagerTask() {
        val userUtil = UserUtil(context)
        val request = ManagerGetTaskRequest()
        request.manager_id = userUtil.userId.toString()
        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = mRetrofitService.managerGetTasks(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<ManagerGetTaskResult>(context, this) {
            override fun onSuccess(response: ManagerGetTaskResult?) {
                if (response != null) {
                    view.setManagerTaskNum(response)
                    mOrderList = getMainOrderList(response)
                    mWorkList = getMainWorkList(response)
                    view.setOrderTaskList(mOrderList)
                    view.setWorkFlowTaskList(mWorkList)
                    depositFailedNum = response.deposit_payed_fail_num
                    requestComment()
                }
            }

            override fun onAPIError(apiException: ApiException?) {
                depositFailedNum = 0
                requestComment()
            }

            override fun onError(e: Throwable) {
                depositFailedNum = 0
                requestComment()
            }
        })
    }

    private fun setDialog(num: Int) {
        if (num > 0) {
            setPayedFailDialogInfo(num)
        } else {
            setCommentDialogInfo()
        }
    }

    private fun setPayedFailDialogInfo(num: Int) {
        if (isPayedFail){
            isPayedFail = false
            IOSAlertDialog(context)
                    .builder()
                    .setCancelable(false)
                    .setTitle("结算异常提醒")
                    .setMsg("您有${num}个订单存在结算异常情况，请尽快与客户沟通处理！结算异常订单可前往“订单-＞待结算“中查看！", true)
                    .setRightButton("我知道了") {
                        setCommentDialogInfo() }
                    .show()
        }
    }


    private fun setCommentDialogInfo() {
        if (!TextUtils.isEmpty(notice) && isNotice) {
            isNotice = false
            view.setCommentDialog(notice ?: "")
        }
    }

    private fun getMainWorkList(response: ManagerGetTaskResult): ArrayList<MainListItemBean> {
        val list = ArrayList<MainListItemBean>()
        addOneItemToList(list, MainListItemBean.TYPE_RELET_WORK_LIST, "续租工单", response.order_relet_apply_auditing)
        addOneItemToList(list, MainListItemBean.TYPE_CANCEL_WORK_LIST, "取消订单工单", response.order_cancel_workcard_num)
        addOneItemToList(list, MainListItemBean.TYPE_MODIFY_WORK_LIST, "修改时间工单", response.order_time_apply_auditing)
        addOneItemToList(list, MainListItemBean.TYPE_ADVANCE_WORK_LIST, "提前还车工单", response.order_advance_apply_auditing)
        addOneItemToList(list, MainListItemBean.TYPE_AUDITING_WORK_LIST, "异常价格工单", response.abnormal_price_auditing)

        LogUtil.e("getMainWorkList list =$list")

        return list
    }

    private fun getMainOrderList(response: ManagerGetTaskResult): ArrayList<MainListItemBean> {
        val orderList = ArrayList<MainListItemBean>()

        addOneItemToList(orderList, MainListItemBean.TYPE_ORDER_PENDING, "待用车", response.order_illegal_payed)
        addOneItemToList(orderList, MainListItemBean.TYPE_ORDER_USING, "用车中", response.order_executing)

        addOneItemToList(orderList, MainListItemBean.TYPE_ORDER_NEED_CONFIRM, "待结算", response.order_bill_applied)
        addOneItemToList(orderList, MainListItemBean.TYPE_ORDER_EVALUATE, "待处理差评", response.mark_num_within_three_day)

        LogUtil.e("orderList =$orderList")
        return orderList
    }

    private fun addOneItemToList(orderList: ArrayList<MainListItemBean>, type: Int, title: String, number: Int) {
        val mainListItemBean1 = MainListItemBean()
        mainListItemBean1.type = type
        mainListItemBean1.title = title
        mainListItemBean1.number = number
        orderList.add(mainListItemBean1)
    }

    private fun onOrderListUsing() {
        val intent = Intent(context, OrderManageActivity::class.java)
        intent.putExtra(TimeOrderUtil.MANAGAGER_IS_TIME, false)
        intent.putExtra(ConstantsUtil.FRAGMENT_FLAG, OrdersFragment.ORDER_STATUS_ONGOING)
        context?.startActivity(intent)
    }


    private fun onComment() {
        val intent = Intent(context, CommentListActivity::class.java)
        intent.putExtra(CommentListActivity.COMMENT_TITLE_VISIBLE, 0)
        context?.startActivity(intent)
    }


    private fun onOrderListConfirmed() {
        val intent = Intent(context, OrderManageActivity::class.java)
        intent.putExtra(TimeOrderUtil.MANAGAGER_IS_TIME, false)
        intent.putExtra(ConstantsUtil.FRAGMENT_FLAG, OrdersFragment.ORDER_STATUS_SETTLE)
        context?.startActivity(intent)
    }

    private fun onOrderListPending() {
        val intent = Intent(context, OrderManageActivity::class.java)
        intent.putExtra(TimeOrderUtil.MANAGAGER_IS_TIME, false)
        intent.putExtra(ConstantsUtil.FRAGMENT_FLAG, OrdersFragment.ORDER_STATUS_HANDOVER)
        context?.startActivity(intent)
    }

    private fun onWorkFlowList() {
        WorkFlowListActivity.newInstance(context, 0, WorkFlowListActivity.PAGE_TYPE_RELET)
    }

    private fun onCancelFlow() {
        ARouter.getInstance().build(ARouterConstantUtil.ACTIVITY_ORDER_CANCEL_FLOW_LIST)
                .withInt(ARouterConstantUtil.PARAM_ORDER_CANCEL_INIT_TAB_NUM, 0)
                .navigation()
    }


    private fun onOrderModifyWorkFlow() {
        WorkFlowListActivity.newInstance(context, 0, WorkFlowListActivity.PAGE_TYPE_MODIFY)
    }


    private fun onOrderAdvanceWorkFlow() {
        //复用WorkFlowListActivity
        WorkFlowListActivity.newInstance(context, 0, WorkFlowListActivity.PAGE_TYPE_ADVANCE)
    }

    private fun onOrderAuditingWorkFlow() {
        //复用WorkFlowListActivity
        WorkFlowListActivity.newInstance(context, 0, WorkFlowListActivity.PAGE_TYPE_AUDITING)
    }

    private fun initItems(items: Array<String>): ArrayList<MainListItemBean> {
        val list = ArrayList<MainListItemBean>()
        for (item in items){
            val entity = MainListItemBean()
            entity.number = 0
            entity.title = item
            list.add(entity)
        }
        return list
    }



    fun initData(){
        // 初始化
        mOrderList = initItems(mOrderItems)
        view.setOrderTaskList(mOrderList)
        mWorkList = initItems(mWorkFlowItems)
        view.setWorkFlowTaskList(mWorkList)
    }



}

