package com.jiaoyinbrother.monkeyking.manager.fragment.ordercancelworkflowlist

import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.library.widget.OrderListSortView
import com.jiaoyinbrother.monkeyking.manager.fragment.workflowlist.WorkflowListPresenter
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit


class OrderCancelWorkFlowListPresenter(context: Context, view: IOrderCancelWorkFlowListContract.View)
    : BasePresenterImpl<IOrderCancelWorkFlowListContract.View>(context, view), IOrderCancelWorkFlowListContract.Presenter {
    private var mSortDirection = OrderListSortView.DESC
    private var mSortKey = WorkflowListPresenter.WORK_LIST_SORT_CREATED_STAMP

    private var mWorkListApplyStatus = OrderCancelWorkListRequest.STATUS_PENDING
    private var mPageNo = 1

    private var mWorkFlowList = ArrayList<OrderCancelWorkListRowBean>()

    override fun getIntentData(arguments: Bundle?) {
        if (arguments != null) {
            LogUtil.e("getIntentData, arguments =$arguments")
            mWorkListApplyStatus = arguments.getInt(WorkflowListPresenter.WORK_LIST_APPLY_STATUS_KEY, OrderCancelWorkListRequest.STATUS_PENDING)
            LogUtil.e("getIntentData, mWorkListApplyStatus= $mWorkListApplyStatus")
        }
    }

    override fun onItemClicked(orderReletID: Int) {

    }

    override fun onChargeClicked(orderReletID: Int) {
        LogUtil.e("WorkflowListPresenter WorkflowListPresenter, orderReletID=$orderReletID")
        IOSAlertDialog(context).builder().setTitle("操作提示")
                .setMsg("请确认收取违约金？", true)
                .setLeftButton("取消") {  }.setRightButton("确认") {
                    apiOrderCancelOperation(getOrderCancelOperationRequest(orderReletID, OrderCancelOperationRequest.ORDER_CANCEL_OPERATION_STATUS_CHARGE))
                }.show()

    }


    override fun onFreeClicked(orderReletID: Int) {
        IOSAlertDialog(context).builder().setTitle("操作提示")
                .setMsg("请确认不收取违约金？", true)
                .setLeftButton("取消") {  }.setRightButton("确认") {
                    apiOrderCancelOperation(getOrderCancelOperationRequest(orderReletID, OrderCancelOperationRequest.ORDER_CANCEL_OPERATION_STATUS_FREE))
                }.show()
    }

    override fun onRefresh() {
        view?.setEnableLoadMore(true)
        mPageNo = 1
        apiOrderCancelWorkList(getOrderCancelWorkListRequest())
    }

    private fun getOrderCancelWorkListRequest(): OrderCancelWorkListRequest {
        val request = OrderCancelWorkListRequest()

        request.manager_id = UserUtil(context).userId.toString()
        request.workcard_status = mWorkListApplyStatus

        request.page = mPageNo
        request.page_size = ConstantsUtil.PAGE_SIZE

        //默认，不传递sidx/sord
        if(mSortKey != WorkflowListPresenter.WORK_LIST_SORT_CREATED_STAMP){
            request.sidx = mSortKey
            request.sord = mSortDirection
        }

        return request
    }

    override fun onLoadMore() {
        mPageNo++
        apiOrderCancelWorkList(getOrderCancelWorkListRequest())
    }

    private fun apiOrderCancelWorkList(request: OrderCancelWorkListRequest) {
        val mRetrofitService = RetrofitHelper.getInstance(context).service

        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        val observable = mRetrofitService.orderCancelWorkList(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderCancelWorkListResult>(context, this) {
            override fun onSuccess(response: OrderCancelWorkListResult?) {

                view.dismissLoadingDialog()
                when(mPageNo){
                    1 -> handleListAPIRefresh(response)
                    else -> handleListAPILoadMore(response)
                }
            }

            override fun onError(e: Throwable) {
                super.onError(e)
                handleListAPIError()
            }

            override fun onAPIError(apiException: ApiException?) {
                super.onAPIError(apiException)
                handleListAPIError()
            }
        })
    }

    override fun onSort(type: Int?, b: Boolean) {
        val sortFactor = type ?: 0
        mSortDirection = if (b) OrderListSortView.ASC else OrderListSortView.DESC
        mSortKey = when (sortFactor) {
            OrderListSortView.TAKE_CAR -> WorkflowListPresenter.WORK_LIST_SORT_OLD_START_TIME
            OrderListSortView.RETURN_CAR -> WorkflowListPresenter.WORK_LIST_SORT_OLD_END_TIME
            else -> WorkflowListPresenter.WORK_LIST_SORT_CREATED_STAMP
        }
        //用户点击了排序；没有切tab,刷新界面；
        onRefresh()
    }




    private fun handleListAPILoadMore(response: OrderCancelWorkListResult?) {
        val dataList = response?.rows
        if(dataList!= null && dataList.isNotEmpty()){
            mWorkFlowList.addAll(dataList)

            view?.showListData(mWorkFlowList)
            if(dataList.size<ConstantsUtil.PAGE_SIZE){
                //加载更多，数据小于PAGE_SIZE，一页
                view?.finishLoadMoreWithNoMoreData()
            }else{
                //加载更多，一整页
                view?.finishLoadMore()
            }
        }else{
            //加载更多，没有数据
            view?.finishLoadMoreWithNoMoreData()
        }
    }

    private fun handleListAPIRefresh(response: OrderCancelWorkListResult?) {
        val dataList = response?.rows
        if(dataList!= null && dataList.isNotEmpty()){
            //刷新成功，有数据
            mWorkFlowList.clear()
            mWorkFlowList.addAll(dataList)
            view?.showListData(mWorkFlowList)

            if(dataList.size<ConstantsUtil.PAGE_SIZE){
                //刷新成功，数据小于PAGE_SIZE，一页
                view?.finishRefreshWithNoMoreData()
            }else{
                //刷新成功，一整页
                view?.finishRefresh()
            }

        }else{
            //刷新没有数据，必须显示no data view
            view?.finishRefresh()
            view?.showNoDataView()
        }

    }

    private fun handleListAPIError() {
        when(mPageNo){
            1 -> {
                //刷新
                view?.finishRefresh()
                if(mWorkFlowList.isEmpty()){
                    //原来无数据，no_net
                    view?.showNoNetView()
                }
                //原来有数据，不处理
            }
            else -> {
                //加载更多
                view?.finishLoadMore()
            }
        }

    }



    private fun apiOrderCancelOperation(request: OrderCancelOperationRequest) {
        val mRetrofitService = RetrofitHelper.getInstance(context).service

        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        val observable = mRetrofitService.orderCancelOperation(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderCancelOperationResult>(context, this) {
            override fun onSuccess(response: OrderCancelOperationResult?) {

                view.dismissLoadingDialog()

                if(TextUtils.isEmpty(response?.result_msg)){
                    when(request.operation_status){
                        OrderCancelOperationRequest.ORDER_CANCEL_OPERATION_STATUS_FREE -> view.showToast("该工单不收取违约金已完成！")
                        OrderCancelOperationRequest.ORDER_CANCEL_OPERATION_STATUS_CHARGE -> view.showToast("该工单收取违约金已完成！")
                    }
                }else{
                    view.showToast(response?.result_msg)
                }

                //刷新当前界面
                onRefresh()
            }

        })
    }


    private fun getOrderCancelOperationRequest(orderReletID: Int, orderCancelOperationStatus: Int): OrderCancelOperationRequest {
        val request = OrderCancelOperationRequest()
        request.manager_id = UserUtil(context).userId.toString()
        request.workcard_id = orderReletID
        request.operation_status = orderCancelOperationStatus

        return request

    }


}
