package com.jiaoyinbrother.monkeyking.manager.fragment.workflowlist

import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.library.widget.OrderListSortView
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.workflowlist.WorkFlowListActivity
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.*
import java.util.concurrent.TimeUnit
import kotlin.collections.ArrayList

class WorkflowListPresenter(context: Context, view: IWorkflowListContract.View)
    : BasePresenterImpl<IWorkflowListContract.View>(context, view), IWorkflowListContract.Presenter {


    private var mSortDirection = OrderListSortView.DESC
    private var mSortKey = WORK_LIST_SORT_CREATED_STAMP

    private var mWorkListApplyStatus = OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_AUDITING
    private var mOrderActionType = WorkFlowListActivity.PAGE_TYPE_RELET
    private var mPageNo = 1

    private var mWorkFlowList = ArrayList<OrderReletApplyListRowsBean>()

    private var mSpiderList = ArrayList<SpiderListItemBean>()

    override fun getOrderType(): Int {
        return mOrderActionType
    }

    override fun onApproveWorkflow(orderReletID: Int, amount: Int) {
        when (mOrderActionType) {
            WorkFlowListActivity.PAGE_TYPE_RELET -> onReletApprove(orderReletID, amount)
            WorkFlowListActivity.PAGE_TYPE_MODIFY -> onModifyApprove(orderReletID, amount)
            WorkFlowListActivity.PAGE_TYPE_ADVANCE -> onAdvanceApprove(orderReletID, amount)
        }
        LogUtil.e("WorkflowListPresenter WorkflowListPresenter, orderReletID=$orderReletID")

    }

    private fun onAdvanceApprove(orderReletID: Int, amount: Int) {

        IOSAlertDialog(context).builder().setTitle("操作提示")
                .setMsg("用户提前还车会给用户退款，请确认是否同意提前还车？", true)
                .setLeftButton("取消") { }.setRightButton("确定") {
                    apiOrderReletApplyHandle(getOderReletApplyHandleRequest(orderReletID, OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_AUDITED))
                }.show()
    }

    private fun onModifyApprove(orderReletID: Int, amount: Int) {
        LogUtil.e("onReletApprove, amount =$amount")

        if (amount > 0) {
            IOSAlertDialog(context).builder().setTitle("操作提示")
                    .setMsg("同意用户修改申请后，会给用户退款，请确认是否同意修改？", true)
                    .setLeftButton("取消") { }.setRightButton("同意修改") {
                        apiOrderReletApplyHandle(getOderReletApplyHandleRequest(orderReletID, OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_AUDITED))
                    }.show()
        } else {
            apiOrderReletApplyHandle(getOderReletApplyHandleRequest(orderReletID, OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_AUDITED))
        }


    }

    private fun onReletApprove(orderReletID: Int, amount: Int) {
        apiOrderReletApplyHandle(getOderReletApplyHandleRequest(orderReletID, OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_AUDITED))
    }

    override fun onDenyWorkflow(orderReletID: Int, amount: Int) {

        LogUtil.e("onDenyWorkflow, amount =$amount")
        when (mOrderActionType) {
            WorkFlowListActivity.PAGE_TYPE_RELET -> onReletDeny(orderReletID)
            WorkFlowListActivity.PAGE_TYPE_MODIFY -> onModifyDeny(orderReletID, amount)
            WorkFlowListActivity.PAGE_TYPE_ADVANCE -> onAdvanceDeny(orderReletID, amount)
        }


    }

    private fun onAdvanceDeny(orderReletID: Int, amount: Int) {

        IOSAlertDialog(context).builder().setTitle("操作提示")
                .setMsg("拒绝用户提前还车后，会给用户退还费用，请确认是否拒绝提前还车？", true)
                .setLeftButton("取消") { }.setRightButton("确定") {
                    apiOrderReletApplyHandle(getOderReletApplyHandleRequest(orderReletID, OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_REJECT))
                }.show()
    }

    private fun onModifyDeny(orderReletID: Int, amount: Int) {
        LogUtil.e("onModifyDeny, amount =$amount")
        if (amount > 0) {
            IOSAlertDialog(context).builder().setTitle("操作提示")
                    .setMsg("拒绝用户修改申请后，会给用户退款，请确认是否拒绝修改？", true)
                    .setLeftButton("取消") { }.setRightButton("拒绝修改") {
                        apiOrderReletApplyHandle(getOderReletApplyHandleRequest(orderReletID, OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_REJECT))
                    }.show()
        } else {
            apiOrderReletApplyHandle(getOderReletApplyHandleRequest(orderReletID, OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_REJECT))
        }

    }

    private fun onReletDeny(orderReletID: Int) {
        LogUtil.e("onReletDeny")
        IOSAlertDialog(context).builder().setTitle("操作提示")
                .setMsg("续租拒绝后会产生用户退款，请确认是否拒绝？", true)
                .setLeftButton("取消") { }.setRightButton("拒绝续租") {
                    apiOrderReletApplyHandle(getOderReletApplyHandleRequest(orderReletID, OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_REJECT))
                }.show()
    }

    override fun onItemClicked(orderReletID: Int) {
        //不跳订单详情
    }

    override fun onRefresh() {
        view?.setEnableLoadMore(true)
        mPageNo = 1
        apiWorkFlow(getOrderReletApplyListRequest())
    }

    override fun onLoadMore() {
        mPageNo++
        apiWorkFlow(getOrderReletApplyListRequest())
    }

    //设置sort参数
    override fun onSort(type: Int?, b: Boolean) {
        val sortFactor = type ?: 0
        mSortDirection = if (b) OrderListSortView.ASC else OrderListSortView.DESC
        mSortKey = when (sortFactor) {
            OrderListSortView.TAKE_CAR -> WORK_LIST_SORT_OLD_START_TIME
            OrderListSortView.RETURN_CAR -> WORK_LIST_SORT_OLD_END_TIME
            else -> WORK_LIST_SORT_CREATED_STAMP
        }
        //用户点击了排序；没有切tab,刷新界面；
        onRefresh()
    }

    private fun getOrderReletApplyListRequest(): OrderReletApplyListRequest {
        val orderReletApplyListRequest = OrderReletApplyListRequest()

        orderReletApplyListRequest.manager_id = UserUtil(context).userId.toString()

//        orderReletApplyListRequest.type = if(mOrderActionType == WorkFlowListActivity.PAGE_TYPE_MODIFY)
//            "TIME" else "RELET"

        var orderType = ""
        when (mOrderActionType) {
            WorkFlowListActivity.PAGE_TYPE_MODIFY -> {
                orderType = "TIME"
                orderReletApplyListRequest.type = orderType
                orderReletApplyListRequest.apply_status = mWorkListApplyStatus
            }
            WorkFlowListActivity.PAGE_TYPE_RELET -> {
                orderType = "RELET"
                orderReletApplyListRequest.type = orderType
                orderReletApplyListRequest.apply_status = mWorkListApplyStatus
            }
            WorkFlowListActivity.PAGE_TYPE_ADVANCE -> {
                orderType = "ADVANCE"
                orderReletApplyListRequest.type = orderType
                orderReletApplyListRequest.apply_status = mWorkListApplyStatus
            }
            WorkFlowListActivity.PAGE_TYPE_AUDITING -> {
                orderReletApplyListRequest.status = mWorkListApplyStatus
            }
        }

        orderReletApplyListRequest.page = mPageNo
        orderReletApplyListRequest.page_size = ConstantsUtil.PAGE_SIZE

        //默认，不传递sidx/sord
        if (mSortKey != WORK_LIST_SORT_CREATED_STAMP) {
            orderReletApplyListRequest.sidx = mSortKey
            orderReletApplyListRequest.sord = mSortDirection
        }

        return orderReletApplyListRequest
    }

    override fun apiSpiderDeal(site_car_type_id: Int, status: String, id: Int) {

        val request = SpiderDealRequest()
        request.site_car_type_id = site_car_type_id
        request.status = status
        request.adjust_price_result = "CONFIRM"
        request.id = id
        request.handle_user_name = if (TextUtils.isEmpty(UserUtil(context).userName)) {
            UserUtil(context).nikeName ?: ""
        } else {
            UserUtil(context).userName ?: ""
        }

        val mRetrofitService = RetrofitHelper.getInstance(context).service

        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        val observable = mRetrofitService.spiderDeal(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult?) {

                view.dismissLoadingDialog()

                onRefresh()

            }
        })

    }

    private fun apiWorkFlow(request: OrderReletApplyListRequest) {
        val mRetrofitService = RetrofitHelper.getInstance(context).service

        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        if (mOrderActionType == WorkFlowListActivity.PAGE_TYPE_AUDITING) {

            mRetrofitService.orderAuditingList(hashMap)
                    .debounce(1000, TimeUnit.MILLISECONDS)
                    .subscribeOn(Schedulers.io())
                    .doOnSubscribe {
                        view.showLoadingDialog()
                    }
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(object : BaseObserver<SpiderListResult>(context, this) {
                        override fun onSuccess(response: SpiderListResult?) {

                            view.dismissLoadingDialog()

                            when (mPageNo) {
                                1 -> handleSpiderListAPIRefresh(response)
                                else -> handleSpiderListAPILoadMore(response)
                            }
                        }

                        override fun onError(e: Throwable) {
                            super.onError(e)
                            handleListAPIError()
                        }

                        override fun onAPIError(apiException: ApiException?) {
                            super.onAPIError(apiException)
                            handleListAPIError()
                        }
                    })
        } else {

            mRetrofitService.orderReletAppyList(hashMap)
                    .debounce(1000, TimeUnit.MILLISECONDS)
                    .subscribeOn(Schedulers.io()).doOnSubscribe {
                        view.showLoadingDialog()
                    }
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(object : BaseObserver<OrderReletApplyListResult>(context, this) {
                        override fun onSuccess(response: OrderReletApplyListResult?) {

                            view.dismissLoadingDialog()

                            when (mPageNo) {
                                1 -> handleListAPIRefresh(response)
                                else -> handleListAPILoadMore(response)
                            }
                        }

                        override fun onError(e: Throwable) {
                            super.onError(e)
                            handleListAPIError()
                        }

                        override fun onAPIError(apiException: ApiException?) {
                            super.onAPIError(apiException)
                            handleListAPIError()
                        }
                    })

        }
    }

    private fun handleListAPILoadMore(response: OrderReletApplyListResult?) {
        val dataList = response?.rows
        if (dataList != null && dataList.isNotEmpty()) {
            mWorkFlowList.addAll(dataList)

            view?.showListData(mWorkFlowList)

            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //加载更多，数据小于PAGE_SIZE，一页
                view?.finishLoadMoreWithNoMoreData()
            } else {
                //加载更多，一整页
                view?.finishLoadMore()
            }
        } else {
            //加载更多，没有数据
            view?.finishLoadMoreWithNoMoreData()
        }
    }

    private fun handleListAPIRefresh(response: OrderReletApplyListResult?) {
        val dataList = response?.rows
        if (dataList != null && dataList.isNotEmpty()) {
            //刷新成功，有数据
            mWorkFlowList.clear()
            mWorkFlowList.addAll(dataList)

            view?.showListData(mWorkFlowList)

            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //刷新成功，数据小于PAGE_SIZE，一页
                view?.finishRefreshWithNoMoreData()
            } else {
                //刷新成功，一整页
                view?.finishRefresh()
            }

        } else {
            //刷新没有数据，必须显示no data view
            view?.finishRefresh()
            view?.showNoDataView()
        }

    }

    private fun handleSpiderListAPILoadMore(response: SpiderListResult?) {
        val dataList = response?.list
        if (dataList != null && dataList.isNotEmpty()) {
            mSpiderList.addAll(dataList)

            view?.showSpiderListData(mSpiderList)

            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //加载更多，数据小于PAGE_SIZE，一页
                view?.finishLoadMoreWithNoMoreData()
            } else {
                //加载更多，一整页
                view?.finishLoadMore()
            }
        } else {
            //加载更多，没有数据
            view?.finishLoadMoreWithNoMoreData()
        }
    }

    private fun handleSpiderListAPIRefresh(response: SpiderListResult?) {
        val dataList = response?.list
        if (dataList != null && dataList.isNotEmpty()) {
            //刷新成功，有数据
            mSpiderList.clear()
            mSpiderList.addAll(dataList)

            view?.showSpiderListData(mSpiderList)

            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //刷新成功，数据小于PAGE_SIZE，一页
                view?.finishRefreshWithNoMoreData()
            } else {
                //刷新成功，一整页
                view?.finishRefresh()
            }

        } else {
            //刷新没有数据，必须显示no data view
            view?.finishRefresh()
            view?.showNoDataView()
        }

    }

    private fun handleListAPIError() {
        when (mPageNo) {
            1 -> {
                //刷新
                view?.finishRefresh()
                if (mWorkFlowList.isEmpty() || mSpiderList.isEmpty()) {
                    //原来无数据，no_net
                    view?.showNoNetView()
                }
                //原来有数据，不处理
            }
            else -> {
                //加载更多
                view?.finishLoadMore()
            }
        }

    }

    private fun getOderReletApplyHandleRequest(orderReletID: Int, type: String): OrderReletApplyHandleRequest {
        val orderReletApplyHandleRequest = OrderReletApplyHandleRequest()
        orderReletApplyHandleRequest.manager_id = UserUtil(context).userId.toString()
        orderReletApplyHandleRequest.relet_id = orderReletID
        orderReletApplyHandleRequest.type = type
        orderReletApplyHandleRequest.source = "PARTY"

        return orderReletApplyHandleRequest
    }

    private fun apiOrderReletApplyHandle(request: OrderReletApplyHandleRequest) {
        val mRetrofitService = RetrofitHelper.getInstance(context).service

        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        val observable = mRetrofitService.orderRenewalApplyHandle(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult?) {

                view.dismissLoadingDialog()
                when (request.type) {
                    OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_AUDITED -> {
                        when (mOrderActionType) {
                            WorkFlowListActivity.PAGE_TYPE_RELET -> view.showToast("同意续租操作完成！")
                            WorkFlowListActivity.PAGE_TYPE_MODIFY -> view.showToast("同意修改已完成！")
                            WorkFlowListActivity.PAGE_TYPE_ADVANCE -> view.showToast("已同意提前还车！")
                        }
                    }
                    OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_REJECT -> {
                        when (mOrderActionType) {
                            WorkFlowListActivity.PAGE_TYPE_RELET -> view.showToast("拒绝续租操作完成！")
                            WorkFlowListActivity.PAGE_TYPE_MODIFY -> view.showToast("拒绝修改已完成！")
                            WorkFlowListActivity.PAGE_TYPE_ADVANCE -> view.showToast("已拒绝提前还车！")
                        }
                    }

                }
                //刷新当前界面
                onRefresh()
            }

        })
    }


    override fun getIntentData(arguments: Bundle?) {

        if (arguments != null) {
            LogUtil.e("getIntentData, arguments =$arguments")
            mWorkListApplyStatus = arguments.getString(WORK_LIST_APPLY_STATUS_KEY, OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_AUDITING)
            mOrderActionType = arguments.getInt(WorkFlowListActivity.PAGE_TYPE_KEY)
            LogUtil.e("getIntentData, mWorkListApplyStatus= $mWorkListApplyStatus")
            LogUtil.e("getIntentData, mOrderActionType= $mOrderActionType")
        }
    }

    companion object {
        //排序key
        const val WORK_LIST_SORT_CREATED_STAMP = "CREATED_STAMP"
        const val WORK_LIST_SORT_OLD_START_TIME = "OLD_START_TIME"
        const val WORK_LIST_SORT_OLD_END_TIME = "OLD_END_TIME"

        //worklist type key
        const val WORK_LIST_APPLY_STATUS_KEY = "WORK_LIST_APPLY_STATUS_KEY"


        const val WORK_ORDER_STATUS_FOR_PRICE = 0x99

    }
}
