package com.jiaoyinbrother.monkeyking.manager.mvpactivity.billrecord

import android.content.Context
import com.alibaba.android.arouter.launcher.ARouter

import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.BillRecordRequest
import com.jiaoyinbrother.library.bean.BillRecordResult
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class BillDataPresenter(context: Context, view: IBillDataContract.view) : BasePresenterImpl<IBillDataContract.view>(context, view), IBillDataContract.presenter {


    override fun requestData(request: BillRecordRequest) {

        val service = RetrofitHelper.getInstance(context).service

        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        val observable = service.billPhotoDetail(params).debounce(1000, TimeUnit.MILLISECONDS)

        observable.subscribeOn(Schedulers.io()).doOnSubscribe {

            view.showLoadingDialog()

        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BillRecordResult>(context,this) {
            override fun onSuccess(response: BillRecordResult?) {
                view.dismissLoadingDialog()
                // 备注
                view.setComment(response?.comment)
                // 租车单
                view.setPrintCarRentList(response?.print_car_rent_list)
                // 验车单
                view.setPrintCarCheckList(response?.print_car_check_list)
                // 客户信息
                view.setPrintCustomerInfo(response?.print_customer_info)
                // 结算单
                view.setPrintSettleList(response?.print_settle_list)
                // pos小票
                view.setPrintPosTicket(response?.print_pos_ticket)
                // 其它
                view.setPrintOther(response?.print_other)
            }

        })
    }


    override fun handleException(apiExcpetion: ApiException) {
        view.dismissLoadingDialog()
        when (apiExcpetion.code) {
            ApiException.SERVER_EXCEPTION, ApiException.OTHER_EXCEPTION, ApiException.API_ERROR_EXCEPTION -> view.onFailed()
            ApiException.TOKEN_EXPIRED_EXCEPTION -> {
                view.showToast(apiExcpetion.message)
                ARouter.getInstance().build("/user/loginactivity").navigation()
            }
            else -> view.onFailed()
        }
    }
}
