package com.jiaoyinbrother.monkeyking.manager.mvpactivity.billrecord

import android.content.Intent
import android.os.Bundle
import android.support.v4.view.ViewPager
import android.text.TextUtils
import android.view.View
import android.widget.LinearLayout
import com.jiaoyinbrother.monkeyking.manager.Constants
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.adapter.TabAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.billrecord.billRecordUpdate.BillNoteFragment
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.billrecord.billRecordUpdate.BillPhotoFragment
import com.jiaoyinbrother.monkeyking.manager.util.ImageSelectorUtils
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.library.bean.BillPhotoBean
import com.jiaoyinbrother.library.bean.BillRecordRequest
import com.jiaoyinbrother.library.listeners.BillRecordListener
import com.jiaoyinbrother.library.util.ConstantsUtil
import kotlinx.android.synthetic.main.activity_ticker_photo.*
import java.util.*

class BillRecordActivity : MvpBaseActivity<BillDataPresenter>(), IBillDataContract.view {
    override fun initView() {

    }


    // constant
    companion object {

        // 票据类型 （ 租车单、 验车单、 客户信息、 结算单、 pos小票、 其它）
        const val PRINT_CAR_RENT_LIST = "PRINT_CAR_RENT_LIST"     // 租车单
        const val PRINT_CAR_CHECK_LIST = "PRINT_CAR_CHECK_LIST"   // 验车单
        const val PRINT_SETTLE_LIST = "PRINT_SETTLE_LIST"         // 结算单
        const val PRINT_POS_TICKET = "PRINT_POS_TICKET"           // POS小票
        const val PRINT_CUSTOMER_INFO = "PRINT_CUSTOMER_INFO"     // 客户信息
        const val PRINT_OTHER = "PRINT_OTHER"                     // 其他
        // ViewPager page name
        const val PAGE_RENT = 0
        const val PAGE_CHECK = 1
        const val PAGE_SETTLE = 2
        const val PAGE_POS = 3
        const val PAGE_INFO = 4
        const val PAGE_OTHER = 5
        const val PAGE_NOTE = 6
    }

    // variable
    private var mOrderID: String? = null
    private var isEditor = false
    private var mCurrentPosition = PAGE_RENT

    // 租车单
    private var mRentFragment: BillPhotoFragment? = null
    // 验车单
    private var mCheckFragment: BillPhotoFragment? = null
    // 结算单
    private var mSettleFragment: BillPhotoFragment? = null
    // POS小票
    private var mPOSFragment: BillPhotoFragment? = null
    // 客户信息
    private var mInfoFragment: BillPhotoFragment? = null
    // 其他
    private var mOtherFragment: BillPhotoFragment? = null
    // 备注
    private var mNoteFragment: BillNoteFragment? = null

    override fun getLayoutId(): Int {
        return R.layout.activity_ticker_photo
    }

    override fun initPresenter(): BillDataPresenter {
        return BillDataPresenter(this, this)
    }

    override fun initToolBar() {
        mainTitle.text = "票据记录"
        subTitle.text = "编辑"
    }

    override fun initData() {
        mOrderID = intent?.getStringExtra(ConstantsUtil.ORDER_ID)
        // ViewPager
        val mAdapter = TabAdapter(supportFragmentManager, this@BillRecordActivity)
        viewPager.offscreenPageLimit = 7
        viewPager.adapter = getAdapter(mAdapter)
        // Custom TabLayout
        tabLayout.setupWithViewPager(viewPager)
        for (i in 0 until tabLayout.tabCount) {
            val tab = tabLayout.getTabAt(i)
            tab?.customView = mAdapter.getTabView(i)
        }
        presenter.requestData(getDataRequest())
    }

    private fun getDataRequest(): BillRecordRequest {
        val request = BillRecordRequest()
        request.order_id = mOrderID
        if (!TextUtils.isEmpty("")) request.file_type = ""
        return request
    }

    private fun getAdapter(adapter: TabAdapter): TabAdapter {
        // 租车单
        mRentFragment = BillPhotoFragment()
        mRentFragment?.arguments = setBundle(PRINT_CAR_RENT_LIST)
        adapter.addFragment(mRentFragment)
        // 验车单
        mCheckFragment = BillPhotoFragment()
        mCheckFragment?.arguments = setBundle(PRINT_CAR_CHECK_LIST)
        adapter.addFragment(mCheckFragment)
        // 结算单
        mSettleFragment = BillPhotoFragment()
        mSettleFragment?.arguments = setBundle(PRINT_SETTLE_LIST)
        adapter.addFragment(mSettleFragment)
        // POS小票
        mPOSFragment = BillPhotoFragment()
        mPOSFragment?.arguments = setBundle(PRINT_POS_TICKET)
        adapter.addFragment(mPOSFragment)
        // 客户信息
        mInfoFragment = BillPhotoFragment()
        mInfoFragment?.arguments = setBundle(PRINT_CUSTOMER_INFO)
        adapter.addFragment(mInfoFragment)
        // 其他
        mOtherFragment = BillPhotoFragment()
        mOtherFragment?.arguments = setBundle(PRINT_OTHER)
        adapter.addFragment(mOtherFragment)
        // 备注
        mNoteFragment = BillNoteFragment()
        mNoteFragment?.arguments = setBundle("")
        adapter.addFragment(mNoteFragment)
        return adapter
    }

    private fun setBundle(type: String): Bundle {
        val bundle = Bundle()
        bundle.putString(ConstantsUtil.ORDER_ID, mOrderID)
        bundle.putString("BILL_TYPE", type)
        return bundle
    }

    @Suppress("DEPRECATION")
    override fun initListeners() {
        mRentFragment?.setListener(myFragmentListener)
        mCheckFragment?.setListener(myFragmentListener)
        mSettleFragment?.setListener(myFragmentListener)
        mPOSFragment?.setListener(myFragmentListener)
        mInfoFragment?.setListener(myFragmentListener)
        mOtherFragment?.setListener(myFragmentListener)
        mNoteFragment?.setListener(myFragmentListener)
        // 编辑
        subTitle.setOnClickListener({
            editStatus(isEditor)
        })
        // fragment viewPager
        viewPager.setOnPageChangeListener(object : ViewPager.OnPageChangeListener {
            override fun onPageScrollStateChanged(state: Int) {}

            override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {}

            override fun onPageSelected(position: Int) {
                mCurrentPosition = position
                when (position) {
                    PAGE_RENT -> mRentFragment?.setDisplayEditing()
                    PAGE_CHECK -> mCheckFragment?.setDisplayEditing()
                    PAGE_SETTLE -> mSettleFragment?.setDisplayEditing()
                    PAGE_POS -> mPOSFragment?.setDisplayEditing()
                    PAGE_INFO -> mInfoFragment?.setDisplayEditing()
                    PAGE_OTHER -> mOtherFragment?.setDisplayEditing()
                    PAGE_NOTE -> mNoteFragment?.setDisplayEditing()
                }
            }
        })
    }

    private fun editStatus(isEditing: Boolean) {
        if (isEditing) {
            subTitle.text = "编辑"
            switchEditor(false)
        } else {
            subTitle.text = "取消"
            switchEditor(true)
        }
        isTabClickable(isEditing)
        isEditor = !isEditing
    }

    private val myFragmentListener = object : BillRecordListener {

        // 提交成功后编辑状态重置
        override fun editCompleted(isCompleted: Boolean) {
            editStatus(isCompleted)
        }

        // 编辑按钮是否显示
        override fun displayEditing(isDisplay: Boolean) {
            subTitle.visibility = if (isDisplay) View.VISIBLE else View.GONE
        }
    }

    /**
     * 切换 编辑/取消 状态
     * @param isEditing
     * */
    private fun switchEditor(isEditing: Boolean) {
        when (mCurrentPosition) {
            PAGE_RENT -> mRentFragment?.setIsEditor(isEditing)
            PAGE_CHECK -> mCheckFragment?.setIsEditor(isEditing)
            PAGE_SETTLE -> mSettleFragment?.setIsEditor(isEditing)
            PAGE_POS -> mPOSFragment?.setIsEditor(isEditing)
            PAGE_INFO -> mInfoFragment?.setIsEditor(isEditing)
            PAGE_OTHER -> mOtherFragment?.setIsEditor(isEditing)
            PAGE_NOTE -> mNoteFragment?.setIsEditor(isEditing)
        }
    }

    /**
     * 禁用&启用 TabLayout点击事件
     * @param isClickable
     * */
    private fun isTabClickable(isClickable: Boolean) {
        val tabStrip = tabLayout.getChildAt(0) as LinearLayout
        for (i in 0 until tabStrip.childCount) {
            val tabView = tabStrip.getChildAt(i)
            if (tabView != null) tabView.isClickable = isClickable
        }
    }

    /**
     * 获取data 失败提示拦截
     * */
    private fun failurePrompt() {
        IOSAlertDialog(this@BillRecordActivity).builder().setCancelable(false).setMsg("票据记录获取失败，\n请稍后重试！", true)
                .setRightButton("重试") { presenter?.requestData(getDataRequest()) }.setLeftButton("取消") { finish() }.show()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (data != null) {
            if (requestCode == Constants.REQUEST_PIC) {
                val images = data.getStringArrayListExtra(ImageSelectorUtils.SELECT_RESULT)
                val type = data.getStringExtra("TYPE")
                if (images != null && images.size > 0) {
                    when (type) {
                        PRINT_CAR_RENT_LIST -> mRentFragment?.setImages(images)
                        PRINT_CAR_CHECK_LIST -> mCheckFragment?.setImages(images)
                        PRINT_SETTLE_LIST -> mSettleFragment?.setImages(images)
                        PRINT_POS_TICKET -> mPOSFragment?.setImages(images)
                        PRINT_CUSTOMER_INFO -> mInfoFragment?.setImages(images)
                        PRINT_OTHER -> mOtherFragment?.setImages(images)
                    }
                }
            }
        }
    }

    override fun setPrintCarRentList(list: ArrayList<BillPhotoBean>?) {
        mRentFragment?.setData(list)
        mRentFragment?.setDisplayEditing()
    }

    override fun setPrintCarCheckList(list: ArrayList<BillPhotoBean>?) {
        mCheckFragment?.setData(list)
    }

    override fun setPrintSettleList(list: ArrayList<BillPhotoBean>?) {
        mSettleFragment?.setData(list)
    }

    override fun setPrintPosTicket(list: ArrayList<BillPhotoBean>?) {
        mPOSFragment?.setData(list)
    }

    override fun setPrintCustomerInfo(list: ArrayList<BillPhotoBean>?) {
        mInfoFragment?.setData(list)
    }

    override fun setPrintOther(list: ArrayList<BillPhotoBean>?) {
        mOtherFragment?.setData(list)
    }

    override fun setComment(comment: String?) {
        mNoteFragment?.setData(comment)
    }

    override fun onFailed() {
        failurePrompt()
    }
}
