package com.jiaoyinbrother.monkeyking.manager.mvpactivity.choosesite

import android.content.Context
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.ChooseSiteBean
import com.jiaoyinbrother.library.bean.ChooseSiteResult
import com.jiaoyinbrother.library.bean.ManagerGetTaskRequest
import com.jiaoyinbrother.library.bean.ManagerGetTaskResult
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.UserUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class ChooseSitePresenter(context: Context?, view: IChooseSiteContract.IChooseSiteView?)
    : BasePresenterImpl<IChooseSiteContract.IChooseSiteView>(context, view), IChooseSiteContract.IChooseSitePresenter {

    private var mDefaultCheck: Int = 0
    private var mTotalPage: Int = 1

    private var list = ArrayList<ChooseSiteBean>()

    override fun setDefaultPosition(position: Int) {
        mDefaultCheck = position
    }

    override fun getChooseSiteList(page: Int) {
        if (page > mTotalPage){

            view.onLoadMoreEnd()

            view.onRefreshEnd()
            return
        }


        val userUtil = UserUtil(context)
        val request = ManagerGetTaskRequest()
        request.manager_id = userUtil.userId.toString()

        request.page_size = ConstantsUtil.PAGE_SIZE

        request.page = page

        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = mRetrofitService.getCommentSiteList(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {

        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<ChooseSiteResult>(context, this) {
            override fun onSuccess(response: ChooseSiteResult?) {
                mTotalPage = response?.total ?: 1

                if (response?.rows != null) {

                    if (page == 1) {

                        list = response.rows!!

                        val firstBean = ChooseSiteBean()
                        firstBean.site_name = "全部网点"
                        list.add(0, firstBean)

                    } else {

                        list.addAll(response.rows!!)

                    }

                    if (mDefaultCheck < list.size) {

                        list[mDefaultCheck].isCheck = true

                    }

                    view.setList(list)
                }

                view.onLoadMoreEnd()

                view.onRefreshEnd()

            }

            override fun onAPIError(apiException: ApiException?) {
                super.onAPIError(apiException)
                LogUtil.releaseLog(apiException.toString())
                view.onLoadMoreEnd()

                view.onRefreshEnd()
            }

            override fun onError(e: Throwable) {
                super.onError(e)
                LogUtil.releaseLog(e.toString())
                view.onLoadMoreEnd()

                view.onRefreshEnd()
            }
        })
    }
}