package com.jiaoyinbrother.monkeyking.manager.mvpactivity.codepay

import android.content.Intent
import android.graphics.Bitmap
import android.text.TextUtils
import android.view.View
import android.widget.LinearLayout
import android.widget.RadioButton
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.request.RequestOptions
import com.jiaoyinbrother.library.bean.PayMethodAvaiBean
import com.jiaoyinbrother.library.util.QRCodeUtil
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import kotlinx.android.synthetic.main.activity_qrcode_pay.*
import kotlinx.android.synthetic.main.main_title.*

class QRCodePayActivity : MvpBaseActivity<QRCodePayPresenter>(), IQRCodePayContract.View {

    private var bgWidth = 0
    private var qrCodeWidth = 0


    override fun initPresenter(): QRCodePayPresenter {
        return QRCodePayPresenter(this, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_qrcode_pay
    }

    override fun initToolBar() {
        ivTitleName.text = "收款码"
    }


    override fun initView() {
        val screenWidth = windowManager.defaultDisplay.width
        bgWidth = screenWidth * 83 / 100
        qrCodeWidth = screenWidth * 11 / 20
        qrcode_bg1.layoutParams = LinearLayout.LayoutParams(bgWidth, bgWidth)
        qrcode_bg2.layoutParams = LinearLayout.LayoutParams(bgWidth, bgWidth / 5)
        img_qrcode.layoutParams = LinearLayout.LayoutParams(qrCodeWidth, qrCodeWidth)
        setRadioBtnDrawable(btn_0, R.drawable.selector_bankcard)
        setRadioBtnDrawable(btn_1, R.drawable.selector_wx)
        setRadioBtnDrawable(btn_2, R.drawable.selector_alipay)
        setRadioBtnDrawable(btn_3, R.drawable.selector_credit)
    }

    override fun initData() {
        presenter.getData(intent)
    }

    override fun initListeners() {
        select_rg.setOnCheckedChangeListener { _, checkedId ->
            when (checkedId) {
                R.id.btn_0 ->
                    presenter.doDepositQRCode(PayMethodAvaiBean.YIBAOPAY)
                R.id.btn_1 ->
                    presenter.doDepositQRCode(PayMethodAvaiBean.WECHATPAY)
                R.id.btn_2 ->
                    presenter.doDepositQRCode(PayMethodAvaiBean.ALIPAY)
                R.id.btn_3 ->
                    presenter.doDepositQRCode(PayMethodAvaiBean.ALIASSURE)
            }
        }
    }

    private fun setRadioBtnDrawable(btn: RadioButton, imgId: Int) {
        val drawable = resources.getDrawable(imgId)
        drawable.setBounds(0, 0, drawable.intrinsicWidth * 3 / 4, drawable.intrinsicHeight * 3 / 4)
        btn.setCompoundDrawables(null, drawable, null, null)
    }

    override fun setDepositType(type: String) {
        deposit_tv?.text = type
    }

    override fun setDepositAmount(amount: String) {
        amount_tv?.text = amount
    }

    override fun setDepositQrCode(url: String?) {
        if (!TextUtils.isEmpty(url)) {
            val qrCodeBitmap = QRCodeUtil.createQRCodeBitmap(url, qrCodeWidth, qrCodeWidth)
            img_qrcode?.let {
                Glide.with(this).load(qrCodeBitmap).apply(RequestOptions()
                        .placeholder(R.mipmap.empty_bg_1_1)
                        .error(R.mipmap.empty_bg_1_1)).into(img_qrcode)
            }
        }
    }

    override fun setDepositQrCode(url: Bitmap?) {
        if (url != null) {
            img_qrcode?.let {
                Glide.with(this)
                        .load(url)
                        .apply(RequestOptions().placeholder(R.mipmap.empty_bg_1_1).error(R.mipmap.empty_bg_1_1))
                        .into(img_qrcode)
            }
        }
    }

    override fun setMethodVisible(type: Int) {
        when (type) {
            QRCodePayPresenter.METHOD_YIBAO -> btn_0?.visibility = View.VISIBLE
            QRCodePayPresenter.METHOD_WECHAT -> btn_1?.visibility = View.VISIBLE
            QRCodePayPresenter.METHOD_ALPAY -> btn_2?.visibility = View.VISIBLE
            QRCodePayPresenter.METHOD_MIN_PROGRAM -> btn_3?.visibility = View.VISIBLE
        }
    }

    override fun setDescription(desc: String) {
        pre_tv?.text = desc
    }

    override fun setBankCardChecked() {
        btn_0?.isChecked = true
    }

    override fun setWeChatChecked() {
        btn_1?.isChecked = true
    }

    override fun setAliChecked() {
        btn_2?.isChecked = true
    }

    override fun setGuaranteeChecked() {
        btn_3?.isChecked = true
    }

    override fun setResult(name: String) {
        val intent = Intent()
        intent.putExtra(QRCodePayPresenter.DEPOSIT_STATUS_NAME, name)
        setResult(0, intent)
        finish()
    }


    override fun onDestroy() {
        presenter.onDestroy()
        super.onDestroy()
    }
}