package com.jiaoyinbrother.monkeyking.manager.mvpactivity.codepay

import android.content.Context
import android.content.Intent
import android.graphics.BitmapFactory
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.listeners.DownloadFileListener
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.DownloadFileManager
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.*
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import org.json.JSONException
import org.json.JSONObject
import java.io.File
import java.util.concurrent.TimeUnit

class QRCodePayPresenter(context: Context, view: IQRCodePayContract.View)
    : BasePresenterImpl<IQRCodePayContract.View>(context, view), IQRCodePayContract.Presenter {

    companion object {
        const val ORDER_ID = "order_id"
        const val AMOUNT = "amount"
        const val PAY_TYPE = "pay_type"
        const val PAY_AVAI_METHOD = "pay_method_avai"

        //车辆押金
        const val DEPOSIT_CAR = 1

        //违章押金
        const val DEPOSIT_ILLEGAL = 2

        const val METHOD_YIBAO = 0
        const val METHOD_WECHAT = 1
        const val METHOD_ALPAY = 2
        const val METHOD_MIN_PROGRAM = 3

        const val DEPOSIT_STATUS_NAME = "deposit_status_name"
    }


    private var orderId: String? = null
    private var amount: Int = 0
    private var payType: Int? = null
    private var payMethod: Int = -1
    private var aliAssureDescription = ""
    private var bankCardDescription = ""
    private var mLoopQuery: Disposable? = null

    private var mCountDown = 60

    override fun getData(intent: Intent) {
        orderId = intent.getStringExtra(ORDER_ID)
        amount = intent.getIntExtra(AMOUNT, 0)
        payType = intent.getIntExtra(PAY_TYPE, 0)
        val avaiMethods = intent.getSerializableExtra(PAY_AVAI_METHOD) as? ArrayList<PayMethodAvaiBean>
        // 支付类型
        view.setDepositType(if (payType == DEPOSIT_ILLEGAL) "请支付违章押金" else "请支付车辆押金")

        view.setDepositAmount(" ¥${amount}")

        if (avaiMethods != null) {
            val methodId = ArrayList<Int>()

            for (method in avaiMethods) {
                when (method.id) {
                    PayMethodAvaiBean.YIBAOPAY -> {
                        view.setMethodVisible(METHOD_YIBAO)
                        bankCardDescription = method.description ?: ""
                    }
                    PayMethodAvaiBean.WECHATPAY -> view.setMethodVisible(METHOD_WECHAT)
                    PayMethodAvaiBean.ALIPAY -> view.setMethodVisible(METHOD_ALPAY)
                    PayMethodAvaiBean.ALIASSURE -> {
                        view.setMethodVisible(METHOD_MIN_PROGRAM)
                        aliAssureDescription = method.description ?: ""
                    }
                }

                // 仅记录支持扫码Id用于判断控件显示默认勾选
                if (method.id == PayMethodAvaiBean.YIBAOPAY
                        || method.id == PayMethodAvaiBean.WECHATPAY
                        || method.id == PayMethodAvaiBean.ALIPAY
                        || method.id == PayMethodAvaiBean.ALIASSURE) {
                    methodId.add(method.id)
                }
            }

            when {
                methodId.contains(PayMethodAvaiBean.YIBAOPAY) -> {
                    view.setBankCardChecked()
                    doDepositQRCode(PayMethodAvaiBean.YIBAOPAY)
                }
                methodId.contains(PayMethodAvaiBean.WECHATPAY) -> {
                    view.setWeChatChecked()
                    doDepositQRCode(PayMethodAvaiBean.WECHATPAY)
                }
                methodId.contains(PayMethodAvaiBean.ALIPAY) -> {
                    view.setAliChecked()
                    doDepositQRCode(PayMethodAvaiBean.ALIPAY)
                }
                methodId.contains(PayMethodAvaiBean.ALIASSURE) -> {
                    view.setGuaranteeChecked()
                    doDepositQRCode(PayMethodAvaiBean.ALIASSURE)
                }
            }
        }

        // 开启轮训检查支付结果， 如果推出页面记得关闭轮训
        doLoopQuery()
    }


    override fun doDepositQRCode(type: Int) {
        when (type) {
            PayMethodAvaiBean.YIBAOPAY -> {
                apiOrderPay(PayMethodAvaiBean.YIBAOPAY)
            }
            PayMethodAvaiBean.WECHATPAY -> {
                apiOrderPay(PayMethodAvaiBean.WECHATPAY)
            }
            PayMethodAvaiBean.ALIPAY -> {
                apiOrderPay(PayMethodAvaiBean.ALIPAY)
            }
            PayMethodAvaiBean.ALIASSURE -> {
                apiGuarantee()
            }
        }

        if (type == PayMethodAvaiBean.YIBAOPAY && !TextUtils.isEmpty(bankCardDescription)) {
            view.setDescription("（$bankCardDescription）")
        } else if (type == PayMethodAvaiBean.ALIASSURE && !TextUtils.isEmpty(aliAssureDescription)) {
            view.setDescription("（$aliAssureDescription）")
        } else {
            view.setDescription("")
        }
    }


    private fun apiOrderPay(method: Int) {
        val request = OrderPayRequest()
        request.orderid = orderId
        request.amount = amount
        request.pay_method = method
        request.pay_type = payType ?: -1
        when (method) {
            // 微信和支付宝入参 trade_type = NATIVE
            PayMethodAvaiBean.WECHATPAY,
            PayMethodAvaiBean.ALIPAY -> {
                request.trade_type = "NATIVE"
            }
            // 银行卡入参 runtype = APP_SCAN
            PayMethodAvaiBean.YIBAOPAY -> {
                request.runtype = "APP_SCAN"
            }
        }
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.payOrder(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderPayResult>(context, this) {
            override fun onSuccess(response: OrderPayResult) {
                view?.dismissLoadingDialog()
                payMethod = response.pay_method

                when (payMethod) {
                    PayMethodAvaiBean.WECHATPAY -> {
                        view.setDepositQrCode(response.code_url)
                    }
                    PayMethodAvaiBean.ALIPAY,
                    PayMethodAvaiBean.YIBAOPAY -> {
                        view.setDepositQrCode(response.url)
                    }
                }
            }

            override fun onAPIError(apiException: ApiException?) {
                view?.dismissLoadingDialog()
                super.onAPIError(apiException)
            }

            override fun onError(e: Throwable) {
                view.dismissLoadingDialog()
                super.onError(e)
            }
        })
    }


    private fun apiGuarantee() {
        val request = OrderQRcodeRequest()
        request.order_id = orderId
        request.pay_type = payType.toString()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.apiGuarantee(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderQRcodeResult>(context, this) {
            override fun onSuccess(response: OrderQRcodeResult) {
                var name = "guarantee_${System.currentTimeMillis()}.jpg"
                DownloadFileManager
                        .getInstance(context)
                        .setDownloadUrl(response.qr_code_url)
                        .setDownloadPath(context.getExternalFilesDir(null)?.path)
                        .setTitle("")
                        .setFileName(name)
                        .setListener(object : DownloadFileListener {
                            override fun onSuccess(path: String?) {
                                val file = File(path, name)
                                if (file.exists()) {
                                    var bitmap = BitmapFactory.decodeFile(file.toString())
                                    bitmap = BitmapUtil.cropGuaranteeBitmap(bitmap)
                                    view.setDepositQrCode(bitmap)
                                    view.dismissLoadingDialog()
                                } else {
                                    view.dismissLoadingDialog()
                                }
                            }

                            override fun onFailed(throwable: Throwable?) {
                                LogUtil.e("Lyy download onFailed!")
                                view.dismissLoadingDialog()
                            }

                            override fun onProgress(total: Int, current: Int) {}

                            override fun onPrepare() {}
                        })
                        .download()
            }

            override fun onAPIError(apiException: ApiException?) {
                view?.dismissLoadingDialog()
                super.onAPIError(apiException)
            }

            override fun onError(e: Throwable) {
                view.dismissLoadingDialog()
                super.onError(e)
            }
        })
    }


    /**
     * 轮询支付结果
     *
     */
    private fun doLoopQuery() {

        mLoopQuery = Observable.interval(0, 5, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe { _ ->
                    mCountDown--
                    if (mCountDown == 0) {
                        mLoopQuery?.dispose()
                    }
                    apiOrderInfo()
                }
    }

    private fun apiOrderInfo() {
        val request = OrderDetailRequest()
        request.uid = UserUtil(context).uid
        request.orderid = orderId
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderDetailResult>(context, this) {
            override fun onSuccess(response: OrderDetailResult) {
                val deposits = response.deposit ?: return

                val type = when (payType) {
                    // 车辆押金
                    DEPOSIT_CAR -> ConstantsUtil.PAY_PROJECT_VEHICLE
                    // 违章押金
                    DEPOSIT_ILLEGAL -> ConstantsUtil.PAY_PROJECT_ILLEGAL
                    // 其他情况无
                    else -> ""
                }
                // 获取到对应的押金对象
                val deposit = getDepositInfo(deposits, type) ?: return

                getResultStatus(deposit)

            }

            override fun onAPIError(apiException: ApiException?) {
                LogUtil.e("loop query qrcode result apiException failed")
            }

            override fun onError(e: Throwable) {
                LogUtil.e("loop query qrcode result onError failed")
            }
        })
    }


    /**
     * 获取支付的押金对象
     *
     * @param deposits
     * @param type
     * @return
     */
    private fun getDepositInfo(deposits: ArrayList<DepositBean>, type: String): DepositBean? {
        for (deposit in deposits) {
            if (TextUtils.equals(deposit.type, type)) {
                return deposit
            }
        }
        return null
    }


    private fun getResultStatus(deposit: DepositBean) {
        if (deposit.status_code == DepositBean.DEPOSIT_PAYED) {
            mLoopQuery?.dispose()
            //已支付埋点
            try {
                val properties = JSONObject()
                properties.put("entrance", "")
                properties.put("order_id", orderId)
                if (payType == DEPOSIT_CAR) {
                    properties.put("payment_type", "支付车辆押金")
                } else {
                    properties.put("payment_type", "支付违章押金")
                }
                properties.put("is_deposit", true)
                properties.put("is_offline", false)
                properties.put("payment_platform", getMethodName(payMethod))
                properties.put("order_amount", amount)
                properties.put("is_success", true)
                StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_PAYMENT_RESULT, properties)
            } catch (e: JSONException) {
                e.printStackTrace()
            }
            view.setResult(deposit.status_name ?: "")

        }
    }


    private fun getMethodName(method: Int): String {
        return when (method) {
            PayMethodAvaiBean.WECHATPAY -> "微信支付"
            PayMethodAvaiBean.ALIPAY -> "支付宝支付"
            PayMethodAvaiBean.ALIASSURE -> "支付宝扫一扫"
            else -> "银行卡支付"
        }
    }


    override fun onDestroy() {
        if (mLoopQuery?.isDisposed == false) {
            mLoopQuery?.dispose()
        }
    }


}