package com.jiaoyinbrother.monkeyking.manager.mvpactivity.commentlist

import android.arch.lifecycle.Observer
import android.content.Context
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.monkeyking.mvpactivity.commentlist.ICommentListContract
import com.jiaoyinbrother.monkeyking.mvpactivity.commentlist.ICommentListContract.ICommentListView
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.choosesite.ChooseSiteActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.choosesite.ChooseSiteActivity.Companion.SITE_CHECK_BEAN_INFO
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.choosesite.ChooseSiteActivity.Companion.SITE_CHECK_POSITION
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.commentlist.CommentListActivity.Companion.COMMENT_TITLE_VISIBLE
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.commentreply.CommentReplyActivity.Companion.COMMENT_REPLY_KEY_BAD
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.*
import java.util.concurrent.TimeUnit

class CommentListPresenter(context: Context, view: ICommentListView) :
        BasePresenterImpl<ICommentListView>(context, view), ICommentListContract.ICommentListPresenter {

    override fun setSiteId(id: String?) {
        mSiteID = id
    }

    override fun setGroupId(id: String?) {
        mGroupID = id
    }


    companion object {
        const val COMMENT_LIST_TYPE_GOOD = 2
        const val COMMENT_LIST_TYPE_NEUTRAL = 1
        const val COMMENT_LIST_TYPE_BAD = 0
        const val COMMENT_LIST_TYPE_IMAGE = 3
        const val COMMENT_LIST_TYPE_ALL = 4
        const val COMMENT_LIST_DATA = "COMMENT_LIST_DATA"
        const val COMMENT_LIST_REFRESH = "COMMENT_LIST_REFRESH"
        const val COMMENT_LIST_DATA_SITE_ID = "COMMENT_LIST_DATA_SITE_ID"
        const val COMMENT_LIST_DATA_GROUP_ID = "COMMENT_LIST_DATA_GROUP_ID"
    }

    private var mSiteID: String? = null
    private var mGroupID: String? = null
    private var page = 1
    private var mType: String? = null
    private var mHasReply: Int? = null
    private var mCommentTitles: ArrayList<CommentTypeCheckBean>? = null
    private var comments = ArrayList<MarksBean>()


    override fun initLiveBus(activity: CommentListActivity) {

        mType = "BAD"

        LiveDataBus.get().with(COMMENT_LIST_REFRESH).observe(activity, Observer {

            LogUtil.e("getCommentList -> 1")
            getCommentList(true)
        })

        LiveDataBus.get().with(SITE_CHECK_POSITION, Int::class.java).observe(activity, Observer {
            view.setDefaultCheck(it ?: 0)
        })


        LiveDataBus.get().with(SITE_CHECK_BEAN_INFO, ChooseSiteBean::class.java).observe(activity, Observer {
            if (it != null) {

                mSiteID = it.site_id
                view.setDefaultCheckBean(it)
                LogUtil.e("getCommentList -> 2")
                getCommentList(true)
            }
        })
    }


    override fun getCommentListTyple(type: String?) {

        LogUtil.e("type==$type")
        LogUtil.e("getCommentList -> 3")

        mType = type
        getCommentList(true)
    }

    override fun getCommentListHasReply(has: Int?) {

        mHasReply = has
        LogUtil.e("getCommentList -> 4")
        getCommentList(true)
    }


    override fun getCommentList(isRefresh: Boolean) {


        view.showLoadingDialog()

        if (isRefresh) {
            page = 1
        }

        val request = CommentListRequest()
        request.group_id = mGroupID
        request.site_id = mSiteID
        request.manager_id = UserUtil(context).userId.toString()
        request.type = mType
        request.page_size = 20
        request.page = page
        if (mHasReply != null && mHasReply!! > -1) {
            request.has_reply = mHasReply
        }
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getStoreCommentList(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.setRefreshEnd()
        }.observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<CommentListResult>(context, this) {
                    override fun onSuccess(response: CommentListResult) {

                        val mTitleTypes = ArrayList<CommentTypeCheckBean>()
                        val mGood = response.good
                        val mNeutral = response.neutral
                        val mBad = response.bad
                        val mImage = response.image
                        val mAll = mGood + mNeutral + mBad
                        for (index in 0..4) {
                            val bean = CommentTypeCheckBean()
                            when (index) {
                                COMMENT_LIST_TYPE_ALL -> {

                                    bean.name = "全部 $mAll"
//                                bean.isCheck = true
                                }
                                COMMENT_LIST_TYPE_IMAGE -> {
                                    bean.name = "有图 $mImage"
                                    bean.type = "IMAGE"
                                }
                                COMMENT_LIST_TYPE_GOOD -> {
                                    bean.name = "好评 $mGood"
                                    bean.type = "GOOD"
                                }
                                COMMENT_LIST_TYPE_NEUTRAL -> {
                                    bean.name = "中评 $mNeutral"
                                    bean.type = "NEUTRAL"
                                }
                                COMMENT_LIST_TYPE_BAD -> {
                                    bean.name = "差评 $mBad"
                                    bean.type = "BAD"
                                }
                            }

                            mTitleTypes.add(bean)
                        }


                        val mTitleReply = ArrayList<CommentTypeCheckBean>()

                        for (index in 0..3) {
                            val bean = CommentTypeCheckBean()
                            when (index) {
                                CommentTypeCheckBean.TYPE_ALL -> {
                                    bean.name = "全部"
                                    bean.type = "-1"
                                }
                                CommentTypeCheckBean.TYPE_NO_REPLY -> {
                                    bean.name = "未回复"
                                    bean.type = "0"
                                }
                                CommentTypeCheckBean.TYPE_REPLIED -> {
                                    bean.name = "已回复"
                                    bean.type = "1"
                                }
                                CommentTypeCheckBean.TYPE_NONE -> {
                                    bean.name = "无需回复"
                                    bean.type = "2"
                                }
                            }
                            mTitleReply.add(bean)
                        }

                        mCommentTitles = mTitleTypes
                        view.setTitleData(mCommentTitles!!, mTitleReply)

                        if (isRefresh) {
                            comments.clear()
                            comments.add(MarksBean())
                        }
                        if (response.marks != null && response.marks!!.size > 0) {

                            comments.addAll(response.marks!!)
                            if (response.marks!!.size < 30) {
//                            comments = CommentListAdapter1.addFooter(comments)
                                view.setCommentNoMore()
//                            view.setCouponFooter(true)
                            }
                            page++
                        } else {
                            if (page > 1) {
//                            comments = CommentListAdapter1.addFooter(comments)
                                view.setCommentNoMore()
//                            view.setCouponFooter(true)
                            }
                        }
//                    if (page == 1)
//                        comments = CommentListAdapter1.addHeader(comments)
                        view.setCommentData(comments)
                        view.setNoDataView(comments.size == 1, ConstantsUtil.NO_DATA)

                        view.dismissLoadingDialog()

                    }

                    override fun onError(e: Throwable) {
                        super.onError(e)
                        view.setNoDataView(comments.size == 0, ConstantsUtil.NO_NET)
                    }

                })
    }


    private fun getCheck(type: String?): Boolean {

        return when (type) {
            "IMAGE" -> true
            "GOOD" -> true
            "NEUTRAL" -> true
            "BAD" -> true
            else -> true
        }
    }
}