package com.jiaoyinbrother.monkeyking.manager.mvpactivity.console.create

import android.arch.lifecycle.Observer
import android.content.Context
import android.content.Intent
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.request.RequestOptions
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.bean.CarTypeSearchRowsBean
import com.jiaoyinbrother.library.widget.GeneralBottomButton
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpdialog.cartypesearch.CarTypeSearchDialog
import com.jiaoyinbrother.monkeyking.manager.mvpdialog.cartypesearch.CarTypeSearchDialog.Companion.CAR_TYPE_SEARCH_CHECK
import com.jiaoyinbrother.monkeyking.manager.view.ClearEditText
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import kotlinx.android.synthetic.main.activity_car_create.*

class CarCreateActivity : MvpBaseActivity<CarCreatePresenter>(), ICarCreateContract.View, View.OnClickListener {

    override fun initPresenter(): CarCreatePresenter {
        return CarCreatePresenter(this, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_car_create
    }

    override fun initView() {
    }

    override fun initToolBar() {
        mainTitle.text = "添加车辆"
    }

    override fun initData() {
        doSubmit.disable()

        LiveDataBus.get().with(CAR_TYPE_SEARCH_CHECK, CarTypeSearchRowsBean::class.java)
                .observe(this, Observer {
                    if (it != null) {
                        presenter.doCarType(it)
                    }
                })
    }

    override fun initListeners() {
        btn_upload.setOnClickListener(this)
        btn_reload.setOnClickListener(this)
        iv_front_license.setOnClickListener(this)
        ll_plate_number.setOnClickListener(this)
        ll_register_time.setOnClickListener(this)
        ll_engine_no.setOnClickListener(this)
        ll_chassis_number.setOnClickListener(this)
        ll_mileage.setOnClickListener(this)
        ll_maintenance_mileage.setOnClickListener(this)
        cl_car_type.setOnClickListener(this)
        ll_car_color.setOnClickListener(this)
        iv_back_license.setOnClickListener(this)
        delete_back_license.setOnClickListener(this)
        et_plate_number.addTextChangedListener(MyTextWatcher(et_plate_number))
        et_engine_no.addTextChangedListener(MyTextWatcher(et_engine_no))
        et_vin.addTextChangedListener(MyTextWatcher(et_vin))
        et_mileage.addTextChangedListener(MyTextWatcher(et_mileage))
        et_maintenance_mileage.addTextChangedListener(MyTextWatcher(et_maintenance_mileage))
        doSubmit.setListener(object : GeneralBottomButton.SubmitListener {
            override fun submit() {
                presenter.doCreateCar()
            }
        })
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.btn_upload -> presenter.doFrontLicense()

            R.id.btn_reload -> presenter.doReloadFrontLicense()

            R.id.iv_front_license -> presenter.doFrontLicense()

            R.id.ll_plate_number -> viewHasFocus(et_plate_number)

            R.id.ll_register_time -> presenter.doRegisterDate(tv_register_time.text.toString())

            R.id.ll_engine_no -> viewHasFocus(et_engine_no)

            R.id.ll_chassis_number -> viewHasFocus(et_vin)

            R.id.ll_mileage -> viewHasFocus(et_mileage)

            R.id.ll_maintenance_mileage -> viewHasFocus(et_maintenance_mileage)

            R.id.cl_car_type -> {
               val mCarTypeSearchDialog = CarTypeSearchDialog()
                mCarTypeSearchDialog?.show(supportFragmentManager, null)
            }

            R.id.ll_car_color -> presenter.doCarColor()

            R.id.iv_back_license -> presenter.doBackLicense()

            R.id.delete_back_license -> presenter.doDeleteBackLicense()
        }
    }

    inner class MyTextWatcher(et: ClearEditText) : TextWatcher {

        var editText = et

        override fun afterTextChanged(s: Editable?) {
            when (editText) {
                et_plate_number -> presenter.doWatcherNumber(s.toString())
                et_engine_no -> presenter.doWatcherEngineNo(s.toString())
                et_vin -> presenter.doWatcherVin(s.toString())
                et_mileage -> presenter.doWatcherMileage(s.toString())
                et_maintenance_mileage -> presenter.doWatcherMaintenanceMileage(s.toString())
            }
        }

        override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
        override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        presenter.onActivityResult(requestCode, resultCode, data)
    }

    override fun onBackPressed() {
        if (presenter.isEdited()) {
            IOSAlertDialog(this)
                    .builder()
                    .setTitle("温馨提示")
                    .setMsg("确认退出添加车辆页面，确认后不保存之前填写信息", true)
                    .setRightButton("确认") { super.onBackPressed() }
                    .setLeftButton("取消") { }
                    .show()
        } else {
            super.onBackPressed()
        }
    }

    override fun setFrontLicense(uri: String) {
        Glide.with(this).load(uri).apply(RequestOptions()
                .placeholder(R.mipmap.bg_driving_license)
                .error(R.mipmap.bg_driving_license))
                .skipMemoryCache(true)
                .diskCacheStrategy(DiskCacheStrategy.NONE)
                .into(iv_front_license)
    }

    override fun setFrontLicenseShow() {
        iv_front_license.visibility = View.VISIBLE
        btn_reload.visibility = View.VISIBLE
        ll_upload.visibility = View.GONE
    }

    override fun setPlateNumber(s: String) {
        et_plate_number.setText(s)
    }

    override fun setRegisterDate(date: String) {
        tv_register_time.text = date
    }

    override fun setEngineNo(s: String) {
        et_engine_no.setText(s)
    }

    override fun setVIN(s: String) {
        et_vin.setText(s)
    }

    override fun setCarTypeName(s: String) {
        tv_car_type.text = s
    }

    override fun setMileage(s: String) {
        et_mileage.setText(s)
    }

    override fun setMaintenanceMileage(s: String) {
        et_maintenance_mileage.setText(s)
    }

    override fun setCarColor(s: String) {
        tv_car_color.text = s
    }

    override fun setBackLicense(uri: String) {
        Glide.with(this).load(uri).apply(RequestOptions()
                .placeholder(R.mipmap.icon_back_license)
                .error(R.mipmap.icon_back_license))
                .skipMemoryCache(true)
                .diskCacheStrategy(DiskCacheStrategy.NONE)
                .into(iv_back_license)
    }

    override fun setBackLicenseDelete(b: Boolean) {
        delete_back_license.visibility = if (b) View.VISIBLE else View.GONE
    }

    override fun setActiveSubmit(b: Boolean) {
        if (b) doSubmit.normal() else doSubmit.disable()
    }

    override fun setPageFinish() {
        finish()
    }

    private fun viewHasFocus(et: ClearEditText) {
        et.requestFocus()
        et.isFocusable = true
        et.isFocusableInTouchMode = true
    }

    companion object {
        fun newInstance(context: Context) {
            val intent = Intent(context, CarCreateActivity::class.java)
            context.startActivity(intent)
        }
    }
}