package com.jiaoyinbrother.monkeyking.manager.mvpactivity.console.create

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.support.v4.app.FragmentActivity
import android.text.TextUtils
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.impl.UploadListener
import com.jiaoyinbrother.library.retrofit.*
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.library.widget.BottomDialog
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.Constants
import com.jiaoyinbrother.monkeyking.manager.activity.ShowBigImageActivity
import com.jiaoyinbrother.monkeyking.manager.util.ImageSelectorUtils
import com.jiaoyinbrother.monkeyking.manager.view.camera.DrivingCamera
import com.jiaoyinbrother.monkeyking.manager.view.date.SelectBirthdayDialog
import com.jiaoyinbrother.monkeyking.manager.view.date.SelectColorDialog
import com.tbruyelle.rxpermissions2.RxPermissions
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.io.File
import java.text.SimpleDateFormat
import java.util.*
import java.util.concurrent.TimeUnit
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

class CarCreatePresenter(context: Context, view: ICarCreateContract.View)
    : BasePresenterImpl<ICarCreateContract.View>(context, view), ICarCreateContract.Presenter {

    private var mLicenseUrl: String? = null
    private var mNumber: String? = null
    private var mLicenseUrl2: String? = null
    private var mRegisterDate: String? = null
    private var mEngineNo: String? = null
    private var mVin: String? = null
    private var mMileage: String? = null
    private var mMaintenanceMileage: String? = null
    private var mCarTypeID: String? = null
    private var mColor: String? = null
    private var mOcrResult: OCRPageBean? = null
    private var mColorList: ArrayList<ColorBean>? = null
    private var mPicType: Int = FRONT
    private var mFrontUri: String? = null
    private var mBackUri: String? = null
    private var compressedImage: File? = null
    private var mUploadUtil: AliYunPictureUtil? = null

    companion object {
        const val FRONT = 0
        const val BACK = 1
        private val LICENSE_BACK = BitmapUtil.SYSTEM_BITMAP_PATH + "license_back.jpg"
    }

    override fun doFrontLicense() {
        if (TextUtils.isEmpty(mFrontUri)) {
            doNoneLicense(FRONT)
        } else {
            ShowBigImageActivity.newInstanceUri(context, mFrontUri ?: "", "图片浏览")
        }
    }

    fun doReloadFrontLicense() {
        doNoneLicense(FRONT)
    }

    override fun doWatcherNumber(s: String) {
        setNumber(s)
    }

    override fun doRegisterDate(date: String) {
        val mDate = if (TextUtils.isEmpty(date)) SimpleDateFormat("yyyy-MM-dd").format(Date()) else date
        SelectBirthdayDialog.Builder(context)
                .setTitle("注册日期")
                .setDefaultDate(mDate)
                .setOnResultListener { resultTime ->
                    setRegisterDate(resultTime ?: "")
                    view.setRegisterDate(mRegisterDate ?: "")
                }
                .show()
    }

    override fun doWatcherEngineNo(s: String) {
        setEngineNo(s)
    }

    override fun doWatcherVin(s: String) {
        setVIN(s)
    }

    override fun doCarType(b: CarTypeSearchRowsBean) {
        mCarTypeID = b.car_type_id
        view.setCarTypeName("${b.brand_name}\n${b.series_name}\n${b.name}")
        activeSubmit()
    }

    override fun doWatcherMileage(s: String) {
        mMileage = s
    }

    override fun doWatcherMaintenanceMileage(s: String) {
        mMaintenanceMileage = s
    }

    override fun doCarColor() {
        if (mColorList != null) {
            showColorDialog()
        } else {
            apiSystemColor()
        }
    }

    override fun doBackLicense() {
        if (TextUtils.isEmpty(mBackUri)) {
            doNoneLicense(BACK)
        } else {
            ShowBigImageActivity.newInstanceUri(context, mBackUri ?: "", "图片浏览")
        }
    }

    override fun doDeleteBackLicense() {
        mBackUri = ""
        view.setBackLicense(mBackUri ?: "")
        view.setBackLicenseDelete(false)
    }

    override fun isEdited(): Boolean {
        return isEdit()
    }

    @SuppressLint("CheckResult")
    private fun doNoneLicense(type: Int) {
        mPicType = type
        RxPermissions(context as FragmentActivity).request(Manifest.permission.READ_EXTERNAL_STORAGE,
                Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.CAMERA).subscribe { b ->
            if (b) {// 获取权限成功
                BottomDialog(context).builder().addItems(arrayOf("拍摄", "从相册选择")).setCancel("取消") {}.setItemClick { _, i ->
                    if (i == 0) { // 自定义相机
                        if (mPicType == FRONT) {
                            DrivingCamera.create(context as FragmentActivity).openCamera()
                        } else {
                            handleTakePicture(context)
                        }
                    } else {// 相册
                        ImageSelectorUtils.openPhoto(context as FragmentActivity, Constants.REQUEST_PIC, true, 1, null, "")
                    }
                }.show()
            } else {
                view.showToast("权限被拒绝")
            }
        }
    }

    /**
     * 拍照
     * */
    private fun handleTakePicture(context: Context) {
        PhotoUtils.takePicture(context as FragmentActivity, File(LICENSE_BACK), CarEntity.REQUEST_CAMERA)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            Constants.REQUEST_PIC -> {
                if (data != null) {
                    //images返回的是照片路径
                    val images = data.getStringArrayListExtra(ImageSelectorUtils.SELECT_RESULT)
                    if (images != null && images.size > 0) {
                        if (images[0] == null)
                            return
                        handlePhoto(File(images[0]))
                    }
                }
            }
            CarEntity.REQUEST_CAMERA -> {
                if (resultCode == Activity.RESULT_OK) {
                    handlePhoto(File(LICENSE_BACK))
                }
            }
            DrivingCamera.REQUEST_CODE -> {
                if (resultCode == DrivingCamera.RESULT_CODE) {
                    val path = DrivingCamera.getImagePath(data)
                    handlePhoto(File(path))
                }
            }
        }
    }

    private fun handlePhoto(actualImage: File?) {
        if (actualImage == null)
            return
        try {
            compressedImage = PhotoUtils.compressFile(context, actualImage)
            if (compressedImage != null) {
                if (mPicType == FRONT) {
                    apiOCR(compressedImage)
                    handleFrontLicense(compressedImage?.absolutePath ?: "")
                } else {
                    handleBackLicense(compressedImage?.absolutePath ?: "")
                }
            }
        } catch (e: Exception) {
            LogUtil.e("handlePhoto err ---->>> $e")
        }
    }

    private fun handleFrontLicense(uri: String) {
        mFrontUri = uri
        view.setFrontLicense(uri)
        view.setFrontLicenseShow()
        activeSubmit()
    }

    private fun handleBackLicense(uri: String) {
        mBackUri = uri
        view.setBackLicense(uri)
        view.setBackLicenseDelete(true)
    }

    override fun apiSystemColor() {
        val request = SystemColorRequest()
        request.type = "COLOUR"
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.systemColor(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<SystemColorResult>(context, this) {
            override fun onSuccess(response: SystemColorResult) {
                view.dismissLoadingDialog()
                mColorList = response.rows
                showColorDialog()
            }
        })
    }

    private fun showColorDialog() {
        SelectColorDialog
                .Builder(context)
                .setColorData(mColorList)
                .setOnResultListener {
                    view.setCarColor(it.description ?: "")
                    mColor = it.status_id
                }.show()
    }

    override fun doCreateCar() {
        view.showLoadingDialog()
        uploadAliCloud()
    }

    private fun apiCreateCar() {
        val request = CreateCarRequest()
        request.uid = UserUtil(context).uid
        request.vehicle_license_url = mLicenseUrl
        request.number = mNumber
        request.birth_date = mRegisterDate
        request.engine_no = mEngineNo
        request.vin = mVin
        request.car_type_id = mCarTypeID
        request.mileage = mMileage
        request.next_mileage = mMaintenanceMileage
        request.color = mColor
        if (!TextUtils.isEmpty(mLicenseUrl2)) {
            request.vehicle_license_url2 = mLicenseUrl2
        }
        request.ocr_result = mOcrResult
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.createCar(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<CreateCarResult>(context, this) {
            override fun onSuccess(response: CreateCarResult) {
                view.dismissLoadingDialog()
                LiveDataBus.get().with("CarCreate").postValue(true)
                view.setPageFinish()
            }
        })
    }

    private fun activeSubmit() {
        if (TextUtils.isEmpty(mFrontUri)
                || TextUtils.isEmpty(mNumber)
                || TextUtils.isEmpty(mRegisterDate)
                || TextUtils.isEmpty(mEngineNo)
                || TextUtils.isEmpty(mVin)
                || TextUtils.isEmpty(mCarTypeID)) {
            LogUtil.e("activeSubmit  ----- >>>> false")
            view.setActiveSubmit(false)
        } else {
            LogUtil.e("activeSubmit  ----- >>>> true")
            view.setActiveSubmit(true)
        }
    }

    private fun uploadAliCloud() {
        val postImages = ArrayList<String>()
        postImages.add(mFrontUri ?: "")
        if (!TextUtils.isEmpty(mBackUri)) {
            postImages.add(mBackUri!!)
        }
        if (mUploadUtil == null)
            mUploadUtil = AliYunPictureUtil.getInstance(context)

        mUploadUtil?.setDatas(AliYunPictureUtil.BUCKET_TYPE_PRIVATE, postImages, AliYunPictureUtil.PRIVATE_CAR_CRED, object : UploadListener {

            override fun onUploadComplete(net: String?, local: String?) {}

            override fun onUploadComplete(success: HashMap<Int, String>?, failure: HashMap<Int, String>?) {
                view.dismissLoadingDialog()
                if (success?.size != 0) {
                    LogUtil.e("onUploadComplete success ：$success")
                    if (!TextUtils.isEmpty(success?.get(1))) {
                        mLicenseUrl = success?.get(1)
                    }
                    if (!TextUtils.isEmpty(success?.get(2))) {
                        mLicenseUrl2 = success?.get(2)
                    }
                    apiCreateCar()
                } else {
                    view.showToast("上传失败")
                }
            }
        })
    }

    private fun apiOCR(file: File?) {

        val service = OCRRetrofitHelper.getInstance(context).service
        val requestBody = RequestBody.create(MediaType.parse("multipart/form-data"), file)
        val body = MultipartBody.Part.createFormData("image_file", file?.name, requestBody)

        val observable = service.upload(body).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : OCRObserver<OCRResult>(context, this) {
            override fun onSuccess(response: OCRResult) {
                view.dismissLoadingDialog()
                mOcrResult = response.data?.page_a
                setNumber(mOcrResult?.plate_number ?: "")
                setRegisterDate(mOcrResult?.register_date ?: "")
                setEngineNo(mOcrResult?.engine_number ?: "")
                setVIN(mOcrResult?.VIN ?: "")

                if (mOcrResult == null
                        || (TextUtils.isEmpty(mNumber) && TextUtils.isEmpty(mRegisterDate) && TextUtils.isEmpty(mEngineNo) && TextUtils.isEmpty(mVin))) {
                    view.showToast("未识别到行驶证主页，请认真拍摄，重新上传!")
                    return
                }

                view.setPlateNumber(mNumber ?: "")
                view.setRegisterDate(mRegisterDate ?: "")
                view.setEngineNo(mEngineNo ?: "")
                view.setVIN(mVin ?: "")

                if (!TextUtils.isEmpty(mNumber)
                        && !TextUtils.isEmpty(mRegisterDate)
                        && !TextUtils.isEmpty(mEngineNo)
                        && !TextUtils.isEmpty(mVin)) {
                    view.showToast("上传成功，请复核一遍!")
                } else {
                    toastString()
                }
            }

            override fun onAPIError(apiException: ApiException?) {
                view.dismissLoadingDialog()
                view.showToast("未识别到行驶证主页，请认真拍摄，重新上传!")
            }

            override fun onError(e: Throwable) {
                view.dismissLoadingDialog()
                view.showToast("未识别到行驶证主页，请认真拍摄，重新上传!")
            }
        })
    }


    private fun toastString() {
        val success = StringBuilder()
        val failed = StringBuilder()

        if (TextUtils.isEmpty(mNumber)) {
            failed.append("车牌号")
        } else {
            success.append("车牌号")
        }

        if (TextUtils.isEmpty(mRegisterDate)) {
            if (!TextUtils.isEmpty(failed.toString()))
                failed.append("、")
            failed.append("注册日期")
        } else {
            if (!TextUtils.isEmpty(success.toString()))
                success.append("、")
            success.append("注册日期")
        }
        if (TextUtils.isEmpty(mEngineNo)) {
            if (!TextUtils.isEmpty(failed.toString()))
                failed.append("、")
            failed.append("发动机号")
        } else {
            if (!TextUtils.isEmpty(success.toString()))
                success.append("、")
            success.append("发动机号")
        }
        if (TextUtils.isEmpty(mVin)) {
            if (!TextUtils.isEmpty(failed.toString()))
                failed.append("、")
            failed.append("车架号")
        } else {
            if (!TextUtils.isEmpty(success.toString()))
                success.append("、")
            success.append("车架号")
        }
        view.showToast("只识别到$success，未识别到$failed，可重新上传!")
    }

    private fun setNumber(s: String) {
        mNumber = s
        activeSubmit()
    }

    private fun setRegisterDate(s: String) {
        mRegisterDate = s
        activeSubmit()
    }

    private fun setEngineNo(s: String) {
        mEngineNo = s
        activeSubmit()
    }

    private fun setVIN(s: String) {
        mVin = s
        activeSubmit()
    }


    private fun isEdit(): Boolean {
        return !(TextUtils.isEmpty(mFrontUri)
                && TextUtils.isEmpty(mNumber)
                && TextUtils.isEmpty(mRegisterDate)
                && TextUtils.isEmpty(mEngineNo)
                && TextUtils.isEmpty(mVin)
                && TextUtils.isEmpty(mCarTypeID)
                && TextUtils.isEmpty(mMileage)
                && TextUtils.isEmpty(mMaintenanceMileage)
                && TextUtils.isEmpty(mColor)
                && TextUtils.isEmpty(mBackUri))
    }
}