package com.jiaoyinbrother.monkeyking.manager.mvpactivity.console.manager

import android.arch.lifecycle.Observer
import android.content.Context
import android.content.Intent
import android.support.design.widget.TabLayout
import android.view.View
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.bean.SimpleCarBean
import com.jiaoyinbrother.library.widget.GeneralBottomButton
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.activity.CarDetailForManageActivity
import com.jiaoyinbrother.monkeyking.manager.activity.SearchCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.console.create.CarCreateActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.console.manager.CarManagerPresenter.Companion.NOT_RENT
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.console.manager.CarManagerPresenter.Companion.TO_RENT
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import kotlinx.android.synthetic.main.activity_car_manager.*

class CarManagerActivity : MvpBaseActivity<CarManagerPresenter>(), ICarManagerContract.View, View.OnClickListener {


    private var mAdapter: CarManagerAdapter? = null

    override fun initPresenter(): CarManagerPresenter {
        return CarManagerPresenter(this, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_car_manager
    }

    override fun initView() {
        mAdapter = CarManagerAdapter(this)
        carManagerRv?.adapter = mAdapter
        carManagerRv?.isFocusable = false
    }

    override fun initToolBar() {
        mainTitle.text = "车辆管理"
        subImg1.setImageResource(R.mipmap.ic_order_search);
    }

    override fun initData() {
        presenter.apiCars()

        LiveDataBus.get().with("CarCreate", Boolean::class.java).observe(this, Observer {
            if (it == true) {
                IOSAlertDialog(this)
                        .builder()
                        .setTitle("添加成功").setMsg("请在后台设置好车型价格和所属网点后方可进行相关审核！", true)
                        .setRightButton("我知道了") {}
                        .show()
            }
        })
    }

    override fun initListeners() {
        subImg1.setOnClickListener(this)
        btn_to_rent.setOnClickListener(this)
        btn_not_rent.setOnClickListener(this)
        mAdapter?.setOnItemClickListener { _, position ->
            val entity = mAdapter?.getItem<SimpleCarBean>(position)
            val intent = Intent(this@CarManagerActivity, CarDetailForManageActivity::class.java)
            intent.putExtra("car_detail", entity)
            startActivity(intent)
        }
        refresh_layout?.setOnRefreshListener {
            presenter?.doRefresh()
        }
        refresh_layout?.setOnLoadMoreListener {
            presenter?.doLoadMore()
        }
        doAddCar.setListener(object : GeneralBottomButton.SubmitListener {
            override fun submit() {
                CarCreateActivity.newInstance(this@CarManagerActivity)
            }
        })
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.subImage1 ->
                startActivity(Intent(this, SearchCarActivity::class.java))
            R.id.btn_to_rent -> {
                presenter.doGetCars(TO_RENT)
                refresh_layout.autoRefresh()
            }
            R.id.btn_not_rent -> {
                presenter.doGetCars(NOT_RENT)
                refresh_layout.autoRefresh()
            }
        }
    }

    override fun finishRefresh() {
        refresh_layout?.finishRefresh()
    }

    override fun finishLoadMore() {
        refresh_layout?.finishLoadMore()
    }

    override fun finishRefreshWithNoMoreData() {
        refresh_layout?.finishRefreshWithNoMoreData()
    }

    override fun finishLoadMoreWithNoMoreData() {
        refresh_layout?.finishLoadMoreWithNoMoreData()
    }

    override fun setEnableLoadMore(b: Boolean) {
        refresh_layout?.setEnableLoadMore(b)
    }

    override fun showNoDataView() {
        setEnableLoadMore(false)
        loading_layout?.showEmpty()
    }

    override fun showListData(data: ArrayList<SimpleCarBean>) {
        loading_layout?.showContent()
        mAdapter?.list = data
        mAdapter?.notifyDataSetChanged()
    }

    companion object {

        fun newInstance(context: Context) {
            val intent = Intent(context, CarManagerActivity::class.java)
            context.startActivity(intent)
        }
    }
}