package com.jiaoyinbrother.monkeyking.manager.mvpactivity.console.manager

import android.content.Context
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.GetCarsRequest
import com.jiaoyinbrother.library.bean.GetCarsResult
import com.jiaoyinbrother.library.bean.SimpleCarBean
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.UserUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class CarManagerPresenter(context: Context, view: ICarManagerContract.View)
    : BasePresenterImpl<ICarManagerContract.View>(context, view), ICarManagerContract.Presenter {

    private var page: Int = 1
    private var mType = TO_RENT
    private var mCarData = ArrayList<SimpleCarBean>()


    override fun doRefresh() {
        view?.setEnableLoadMore(true)
        page = 1
        apiCars()
    }

    override fun doLoadMore() {
        apiCars()
    }

    override fun doGetCars(type: Int) {
        mType = type
//        doRefresh()
    }

    override fun apiCars() {
        val request = GetCarsRequest()
        request.uid = UserUtil(context).uid
        request.status = mType
        request.page = page
        request.page_size = ConstantsUtil.PAGE_SIZE
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getCars(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<GetCarsResult>(context, this) {
            override fun onSuccess(response: GetCarsResult) {
                view.dismissLoadingDialog()

                when (page) {
                    1 -> setAPIRefresh(response)
                    else -> setAPILoadMore(response)
                }
            }
        })
    }

    private fun setAPIRefresh(response: GetCarsResult) {
        val dataList = response.cars
        if (dataList != null && dataList.isNotEmpty()) {
            //刷新成功，有数据
            mCarData.clear()
            mCarData.addAll(dataList)
            view?.showListData(mCarData)
            page++
            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //刷新成功，数据小于PAGE_SIZE，一页
                view?.finishRefreshWithNoMoreData()
            } else {
                //刷新成功，一整页
                view?.finishRefresh()
            }
        } else {
            //刷新没有数据，必须显示no data view
            view?.finishRefresh()
            view?.showNoDataView()
        }
    }

    private fun setAPILoadMore(response: GetCarsResult) {
        val dataList = response.cars
        if (dataList != null && dataList.isNotEmpty()) {
            mCarData.addAll(dataList)
            view?.showListData(mCarData)
            page++
            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //加载更多，数据小于PAGE_SIZE，一页
                view?.finishLoadMoreWithNoMoreData()
            } else {
                //加载更多，一整页
                view?.finishLoadMore()
            }
        } else {
            //加载更多，没有数据
            view?.finishLoadMoreWithNoMoreData()
        }
    }


    companion object {
        const val TO_RENT = 0
        const val NOT_RENT = 1
    }
}