package com.jiaoyinbrother.monkeyking.manager.mvpactivity.console.price

import android.content.Context
import android.support.v7.app.AlertDialog
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.Gravity
import android.view.View
import android.widget.FrameLayout
import android.widget.TextView
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.library.util.Util
import com.jiaoyinbrother.monkeyking.manager.R
import com.leochuan.CenterSnapHelper
import com.leochuan.ScaleLayoutManager
import com.leochuan.ScrollHelper
import com.leochuan.ViewPagerLayoutManager
import com.scwang.smartrefresh.layout.util.SmartUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class CarPriceManagerPresenter(context: Context, view: ICarPriceManagerContract.View) : BasePresenterImpl<ICarPriceManagerContract.View>(context, view), ICarPriceManagerContract.Presenter {
    private var site_id: String? = null
    private var page: Int = 1
    private var mCarData = mutableListOf<PriceCarBean>()
    private val mSitesData = mutableListOf<ManagerSiteBean>()
    override fun doRefresh() {
        view?.setEnableLoadMore(true)
        page = 1
        apiCars()
    }

    override fun doLoadMore() {
        apiCars()
    }

    override fun showSiteDialog() {
        val view = View.inflate(context, R.layout.dialog_select_site, null)
        val dialog = AlertDialog.Builder(context, R.style.wk_pub_dialog_style).setView(view).create()
        dialog.show()
        // 调整dialog背景大小
        val window = dialog.window
        window?.setGravity(Gravity.BOTTOM)
        val display = window?.windowManager?.defaultDisplay
        val layoutParams = view.layoutParams
        layoutParams.height = ((display?.height?.times(0.4)) as? Int)
                ?: FrameLayout.LayoutParams.WRAP_CONTENT
        layoutParams.width = display?.width
                ?: FrameLayout.LayoutParams.MATCH_PARENT
        view.layoutParams = layoutParams
        val rvSites = view.findViewById<RecyclerView>(R.id.rv_site)
        val cancelTv = view.findViewById<TextView>(R.id.tv_cancel)
        val submitTv = view.findViewById<TextView>(R.id.tv_submit)

        val builder = ScaleLayoutManager.Builder(context, 0)
        builder.setMaxAlpha(1f)
        builder.setMinAlpha(0.7f)
        builder.setOrientation(ViewPagerLayoutManager.VERTICAL)
        val viewPagerLayoutManager = ScaleLayoutManager(builder)
        rvSites.layoutManager = viewPagerLayoutManager
        val adapter = SiteAdapter(mSitesData)
        rvSites.adapter = adapter
        val centerSnapHelper = CenterSnapHelper()
        centerSnapHelper.attachToRecyclerView(rvSites)
        var indexOfFirst = mSitesData.indexOfFirst { TextUtils.equals(site_id, it.id) }
        if (indexOfFirst<0){
            indexOfFirst = 0
        }
        rvSites.scrollToPosition(indexOfFirst)
//        ScrollHelper.smoothScrollToTargetView(rvSites, viewPagerLayoutManager.findViewByPosition(indexOfFirst))
        cancelTv.setOnClickListener {
            dialog.dismiss()
        }
        submitTv.setOnClickListener {
            dialog.dismiss()
            doSetSite(mSitesData[viewPagerLayoutManager.currentPosition])
        }
    }

    override fun apiCars() {
        val request = GetPriceCarsRequest()
        request.is_app = 1
        request.page = page
        request.site_id = site_id
        request.page_size = ConstantsUtil.PAGE_SIZE
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getPriceCars(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<GetPriceCarsResult>(context, this) {
            override fun onSuccess(response: GetPriceCarsResult) {
                view.dismissLoadingDialog()

                when (page) {
                    1 -> setAPIRefresh(response)
                    else -> setAPILoadMore(response)
                }
            }

        })
    }

    override fun apiSites() {
        val request = GetSitesRequest()
        request.manager_id = UserUtil(context).userId.toString()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getManagerSites(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<GetSitesResult>(context, this) {
            override fun onSuccess(response: GetSitesResult) {
                view.dismissLoadingDialog()
                mSitesData.clear()
                mSitesData.addAll(response.sites ?: emptyList())
                if (mSitesData.isEmpty()) {
                    view?.showAllNoDataView()
                } else {
                    doSetSite(mSitesData[0])
                }

            }

        })
    }

    override fun doSetSite(managerSiteBean: ManagerSiteBean) {
        site_id = managerSiteBean.id
        view.updateSite(managerSiteBean.name)
        doRefresh()
    }

    private fun setAPIRefresh(response: GetPriceCarsResult) {
        val dataList = response.rows
        if (dataList != null && dataList.isNotEmpty()) {
            //刷新成功，有数据
            mCarData.clear()
            mCarData.addAll(dataList)
            view?.showListData(mCarData)
            page++
            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //刷新成功，数据小于PAGE_SIZE，一页
                view?.finishRefreshWithNoMoreData()
            } else {
                //刷新成功，一整页
                view?.finishRefresh()
            }
        } else {
            //刷新没有数据，必须显示no data view
            view?.finishRefresh()
            view?.showNoDataView()
        }
    }

    private fun setAPILoadMore(response: GetPriceCarsResult) {
        val dataList = response.rows
        if (dataList != null && dataList.isNotEmpty()) {
            mCarData.addAll(dataList)
            view?.showListData(mCarData)
            page++
            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //加载更多，数据小于PAGE_SIZE，一页
                view?.finishLoadMoreWithNoMoreData()
            } else {
                //加载更多，一整页
                view?.finishLoadMore()
            }
        } else {
            //加载更多，没有数据
            view?.finishLoadMoreWithNoMoreData()
        }
    }

}