package com.jiaoyinbrother.monkeyking.manager.mvpactivity.cost

import android.arch.lifecycle.Observer
import android.support.v7.widget.GridLayoutManager
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.View
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.bean.BillBean
import com.jiaoyinbrother.library.bean.BillContentBean
import com.jiaoyinbrother.library.bean.BillHistoriesBean
import com.jiaoyinbrother.library.bean.OrderDetailResult
import com.jiaoyinbrother.library.util.DateTimeUtil
import com.jiaoyinbrother.library.util.MoneyUtil
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.adapter.CostAdapter
import com.jiaoyinbrother.monkeyking.manager.adapter.CostCalendarAdapter
import com.jiaoyinbrother.monkeyking.manager.adapter.OrderRecordAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.util.H5Utils
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import kotlinx.android.synthetic.main.activity_cost_detail.*

class CostDetailActivity : MvpBaseActivity<CostPresenter>(),CostContract.View{


    private var mRentAdapter: CostAdapter? = null
    private var mServiceAdapter: CostAdapter? = null
    private var mDiscountAdapter: CostAdapter? = null
    private var mRecordAdapter: OrderRecordAdapter? = null
    private var mCalendarAdapter: CostCalendarAdapter? = null


    override fun initPresenter(): CostPresenter {
        return CostPresenter(this,this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_cost_detail
    }
    override fun initView() {
        mRentAdapter = CostAdapter()
        rent_rv?.adapter = mRentAdapter
        mServiceAdapter = CostAdapter()
        service_rv?.adapter = mServiceAdapter
        mDiscountAdapter = CostAdapter()
        discount_rv?.adapter = mDiscountAdapter

        calendar_rv.layoutManager = GridLayoutManager(this,7, RecyclerView.VERTICAL,false)
        mCalendarAdapter = CostCalendarAdapter()
        calendar_rv.adapter = mCalendarAdapter
        mRecordAdapter = OrderRecordAdapter()
        record_rv?.adapter = mRecordAdapter
    }

    override fun initToolBar() {
        mainTitle?.text = "费用明细"
    }

    override fun initListeners() {
        cost_rule_tv?.setOnClickListener{v->
            H5Utils.onCostRule(this)
        }
        calendar_btn_tv?.setOnClickListener{
            if(calendar_rv?.visibility == View.VISIBLE){
                calendar_rv?.visibility = View.GONE
                calendar_btn_tv?.text = "查看价格日历"
            }else{
                calendar_rv?.visibility = View.VISIBLE
                calendar_btn_tv?.text = "收起价格日历"
            }
        }
        mRentAdapter?.setOnUrlListener(object : CostAdapter.OnUrlListener {
            override fun onUrl(billContentBean: BillContentBean?) {
                startH5(billContentBean)
            }
        })
        mServiceAdapter?.setOnUrlListener(object : CostAdapter.OnUrlListener {
            override fun onUrl(billContentBean: BillContentBean?) {
                startH5(billContentBean)
            }
        })
        mDiscountAdapter?.setOnUrlListener(object : CostAdapter.OnUrlListener {
            override fun onUrl(billContentBean: BillContentBean?) {
                startH5(billContentBean)
            }
        })

        discount_question_iv.setOnClickListener {

            IOSAlertDialog(this).builder()
                    .setTitle("优惠减免")
                    .setMsg("“会员优惠、其他支付（抵用金）、悟空币、打包价减免”四种费用均由悟空全额承担；“优惠券、优惠活动”的悟空承担金额以页面备注为准",false)
                    .setRightButton("知道了"){}
                    .show()
        }
    }


    override fun initData() {
        observerOrderBill()
    }
    private fun observerOrderBill() {
        LiveDataBus.get().with(CostPresenter.ORDER_DETAIL, OrderDetailResult::class.java).observeSticky(this, Observer {
            val billBean = it?.bill
            val prices = it?.prices
            val real_start_time = billBean?.real_start_time
            val real_end_time = billBean?.real_end_time
            val bill_show = billBean?.bill_show

            start_time_tv?.text = DateTimeUtil.getDate(real_start_time) + " " + DateTimeUtil.getTime(real_start_time)
            end_time_tv?.text = DateTimeUtil.getDate(real_end_time) + " " + DateTimeUtil.getTime(real_end_time)
            duration_tv?.text =
                    DateTimeUtil.getTimePeriodString(real_start_time, real_end_time, DateTimeUtil.TYPE_DAY_HOUR)


            mRentAdapter?.list = bill_show?.rent_show
            mRentAdapter?.notifyDataSetChanged()

            service_ll.visibility = if (bill_show?.service_show?.size == 0) View.GONE else View.VISIBLE
            mServiceAdapter?.list = bill_show?.service_show
            mServiceAdapter?.notifyDataSetChanged()


            discount_ll.visibility = if (bill_show?.discount_show?.size == 0) View.GONE else View.VISIBLE
            mDiscountAdapter?.list = bill_show?.discount_show
            mDiscountAdapter?.notifyDataSetChanged()

            total_amount_tv?.text = MoneyUtil().getMoneyText(bill_show?.total_amount ?: "")
            prepaid_amount_ll?.visibility = if(TextUtils.isEmpty(bill_show?.prepaid_amount)) View.GONE else View.VISIBLE
            prepaid_amount_tv?.text = MoneyUtil().getMoneyText(bill_show?.prepaid_amount ?: "")

            mCalendarAdapter?.list = DateTimeUtil.getCostCalendarPrices(prices)
            mCalendarAdapter?.notifyDataSetChanged()

            presenter?.requestBillHistories(it?.orderid)
        })
    }

    override fun onBillHistories(history_list: ArrayList<BillHistoriesBean>?) {
        record_ll?.visibility = if(history_list?.size?:0 > 0) View.VISIBLE else View.GONE
        mRecordAdapter?.list = history_list
        mRecordAdapter?.notifyDataSetChanged()
    }

    private fun startH5(billContentBean: BillContentBean?) {
        H5Utils.onCostExplain(this, billContentBean?.title ?: "", billContentBean?.url ?: "")
    }
}
