package com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.car

import android.content.Context
import android.os.Bundle
import android.support.v4.app.FragmentManager
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.OperatingCarStockResult
import com.jiaoyinbrother.library.bean.OrderStatsBean
import com.jiaoyinbrother.library.bean.OrderStatsRequest
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.dialog.DataCenterPickerDialog
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.text.SimpleDateFormat
import java.util.*
import java.util.concurrent.TimeUnit

class CarDataAnalysisPresenter(context: Context, view: ICarDataAnalysisContract.View) :
    BasePresenterImpl<ICarDataAnalysisContract.View>(context, view),
    ICarDataAnalysisContract.Presenter, DataCenterPickerDialog.DialogListener {


    private val upSimpleDateFormat = SimpleDateFormat("yyyy-MM-dd")
    private val showSimpleDateFormat = SimpleDateFormat("yyyy年MM月dd日")

    private val dialog by lazy {
        DataCenterPickerDialog()
    }

    override var site_id: String? = null
    override var data_type: String? = OrderStatsRequest.DAY

    private var day_start_time: Long = 0L
    private var week_start_time: Long = 0L
    private var week_end_time: Long = 0L
    private var month_start_time: Long = 0L

    private var jsonStock: String? = null
    private var jsonSales: String? = null


    override fun initData() {
        val instance = Calendar.getInstance()
        val isMonday = instance.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY
        instance.add(Calendar.DATE, -1)
        val currentTimeMillis = instance.timeInMillis
        instance.timeInMillis = currentTimeMillis
        if (isMonday){
            instance.add(Calendar.DATE, -1 * 7);
        }
        instance.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY)
        day_start_time = currentTimeMillis
        week_end_time = currentTimeMillis
        month_start_time = currentTimeMillis
        week_start_time = instance.timeInMillis
        data_type = OrderStatsRequest.DAY
        showTime()
    }

    private fun showTime() {
        view?.setDate(
            when (data_type) {
                OrderStatsRequest.DAY -> {
                    showSimpleDateFormat.format(Date(day_start_time))
                }
                OrderStatsRequest.WEEK -> {
                    "${showSimpleDateFormat.format(Date(week_start_time))}-${showSimpleDateFormat.format(
                        Date(week_end_time)
                    )}"
                }
                OrderStatsRequest.MONTH -> {
                    SimpleDateFormat("yyyy年MM月").format(Date(month_start_time))
                }
                else -> {
                    ""
                }
            }
        )
    }

    override fun setDataType(type: String) {
        data_type = type
        showTime()
        apiCarSales()
    }

    override fun datetimeDialog(fragmentManager: FragmentManager) {
        val (startTime, endTime) = when (data_type) {
            OrderStatsRequest.DAY -> {
                Pair(day_start_time, day_start_time)
            }
            OrderStatsRequest.WEEK -> {
                Pair(week_start_time, week_end_time)
            }
            OrderStatsRequest.MONTH -> {
                Pair(month_start_time, month_start_time)
            }
            else -> {
                Pair(day_start_time, day_start_time)
            }
        }
        dialog.arguments = Bundle().apply {
            putString(DataCenterPickerDialog.TYPE, data_type)
            putLong(DataCenterPickerDialog.START_TIME, startTime)
            putLong(DataCenterPickerDialog.END_TIME, endTime)
            putInt(DataCenterPickerDialog.DATE_TAG, DataCenterPickerDialog.NO_TODAY)
        }
        dialog.show(fragmentManager, this)
    }

    override fun onLoadECarStockCharts() {
        view.setCarStockData(jsonStock)
    }

    override fun onLoadECarSalesCharts() {
        view.setCarSalesData(jsonSales)
    }

    override fun refreshData() {
        if (TextUtils.isEmpty(site_id))
            return
        apiCarStock()
        apiCarSales()
    }

    private fun apiCarStock() {
        val request = OrderStatsRequest()
        request.site_id = site_id
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.operatingCarStock(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : BaseObserver<OperatingCarStockResult>(context, this) {
                override fun onSuccess(response: OperatingCarStockResult) {
                    /**
                     * 车型销量榜。webView 执行
                     * "setCarStockData([{name: '在租', value: 100}, {name: '整备车辆', value: 80}, {name: '暂不可租', value: 40}, {name: '维修、保养、其他', value: 20}], '全部车辆 99356 辆')"
                     * 即可
                     */
                    jsonStock = setCarStockAnalysis(response.rows, response.all_sum)
                    onLoadECarStockCharts()
                }
            })
    }

    private fun apiCarSales() {
        val request = OrderStatsRequest()
        request.site_id = site_id
        request.type = data_type
        when (data_type) {
            OrderStatsRequest.DAY -> {
                request.start_time = upSimpleDateFormat.format(Date(day_start_time))
                request.end_time = upSimpleDateFormat.format(Date(day_start_time))
            }
            OrderStatsRequest.WEEK -> {
                request.start_time = upSimpleDateFormat.format(Date(week_start_time))
                request.end_time = upSimpleDateFormat.format(Date(week_end_time))
            }
            OrderStatsRequest.MONTH -> {
                val instance = Calendar.getInstance()
                instance.timeInMillis = month_start_time
                instance.set(Calendar.DAY_OF_MONTH, 1)
                request.start_time = upSimpleDateFormat.format(instance.time)
            }
        }
        request.is_more = OrderStatsRequest.NO_MORE
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.operatingCarSales(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : BaseObserver<OperatingCarStockResult>(context, this) {
                override fun onSuccess(response: OperatingCarStockResult) {
                    view.dismissLoadingDialog()
                    /**
                     * 车型销量榜。webView 执行
                     * "setCarRentTop([5, 1, 10, 1000, 500], ['car1', 'car2', 'car3', 'car4', 'car5'])"
                     * 即可
                     */
                    response.rows?.reverse()
                    jsonSales = setCarSalesAnalysis(response.rows)
                    onLoadECarSalesCharts()
                }

                override fun onError(e: Throwable) {
                    view.dismissLoadingDialog()
                }

                override fun onAPIError(apiException: ApiException?) {
                    view.dismissLoadingDialog()
                }
            })
    }

    override fun onDismiss() {}

    override fun onSubmit(type: String, startTime: Long, endTime: Long) {
        when (type) {
            OrderStatsRequest.DAY -> {
                day_start_time = startTime
            }
            OrderStatsRequest.WEEK -> {
                week_start_time = startTime
                week_end_time = endTime
            }
            OrderStatsRequest.MONTH -> {
                month_start_time = startTime
            }
            else -> {
            }
        }
        setDataType(type)
    }


    private fun setCarStockAnalysis(data: ArrayList<OrderStatsBean>?, total: Int? = 0): String {
        if (data == null || data.isEmpty()) {
            // 空数据
            return "[{name:'',value:0}], '全部车辆 0辆'"
        }
        val buffer = StringBuffer()
        buffer.append("[")
        data.forEach {
            buffer.append("{name:'${it.type_name}',value:${it.sum ?: 0}},")
        }
        buffer.append("], '全部车辆 ${total}辆'").toString()
        return buffer.toString()
    }

    private fun setCarSalesAnalysis(data: ArrayList<OrderStatsBean>?): String {
        if (data == null || data.isEmpty()) {
            // 空数据
//            [5, 1, 10, 1000, 500], ['car1', 'car2', 'car3', 'car4', 'car5']
            return "[], []"
        }
        val key = StringBuffer()
        key.append("[")
        val value = StringBuffer()
        value.append("[")
        data.forEach {
            key.append("'${it.car_type_name}',")
            value.append("${it.sum},")
        }

        key.append("]")
        value.append("]")
        return "$value, $key"
    }

}