package com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.comment

import android.content.Context
import android.os.Bundle
import android.support.v4.app.FragmentManager
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.dialog.DataCenterPickerDialog
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.text.SimpleDateFormat
import java.util.*
import java.util.concurrent.TimeUnit

class CommentDataAnalysisPresenter(context: Context, view: ICommentDataAnalysisContract.View) :
    BasePresenterImpl<ICommentDataAnalysisContract.View>(context, view),
    ICommentDataAnalysisContract.Presenter, DataCenterPickerDialog.DialogListener {
    private val dialog by lazy {
        DataCenterPickerDialog()
    }
    override var data_type: String = OrderStatsRequest.DAY
    override var site_id: String? = null
    override var day_start_time: Long = 0L
    override var week_start_time: Long = 0L
    override var week_end_time: Long = 0L
    override var month_start_time: Long = 0L
    override val list = mutableListOf<OrderStatsBean>()
    private val upSimpleDateFormat = SimpleDateFormat("yyyy-MM-dd")
    private val showSimpleDateFormat = SimpleDateFormat("yyyy年MM月dd日")
    override fun refresh() {
        if (TextUtils.isEmpty(site_id)) return
        getData()
        getRating()
    }

    private fun getRating() {
        val request = SiteMarkRequest()
        request.site_id = site_id
        request.page = 1
        request.page_size = 1
        request.plat_form = "WKOON"
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getSiteMark(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : BaseObserver<SiteMarkResult>(context, this) {
                override fun onSuccess(response: SiteMarkResult) {
                    val split = response.rate_desc?.split(" ")
                    if (split?.size ?: 0 > 1) {
                        response.rate_desc = split?.get(1) ?: response.rate_desc
                    }
                    view.showRating(response)
                }
            })
    }

    override fun setDataType(type: String) {
        data_type = type
        showTime()
        getData()
    }

    override fun getData() {
        val request = OrderStatsRequest()
        request.site_id = site_id
        request.type = data_type
        request.source = "MARK"
        when (data_type) {
            OrderStatsRequest.DAY -> {
                request.start_time = upSimpleDateFormat.format(Date(day_start_time))
                request.end_time = upSimpleDateFormat.format(Date(day_start_time))
            }
            OrderStatsRequest.WEEK -> {
                request.start_time = upSimpleDateFormat.format(Date(week_start_time))
                request.end_time = upSimpleDateFormat.format(Date(week_end_time))
            }
            OrderStatsRequest.MONTH -> {
                val instance = Calendar.getInstance()
                instance.timeInMillis = month_start_time
                instance.set(Calendar.DAY_OF_MONTH, 1)
                request.start_time = upSimpleDateFormat.format(instance.time)
            }
        }
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getDataStats(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : BaseObserver<OrderStatsResult>(context, this) {
                override fun onSuccess(response: OrderStatsResult) {
                    view?.dismissLoadingDialog()
                    list.clear()
                    list.addAll(response.rows ?: emptyList())
                    view?.updateView()

                }

                override fun onAPIError(apiException: ApiException?) {
                    super.onAPIError(apiException)
                    view?.dismissLoadingDialog()
                }
            })
    }

    override fun initData() {
        val instance = Calendar.getInstance()
        val isMonday = instance.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY
        instance.add(Calendar.DATE, -1)
        val currentTimeMillis = instance.timeInMillis
        instance.timeInMillis = currentTimeMillis
        if (isMonday){
            instance.add(Calendar.DATE, -1 * 7);
        }
        instance.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY)
        day_start_time = currentTimeMillis
        week_end_time = currentTimeMillis
        month_start_time = currentTimeMillis
        week_start_time = instance.timeInMillis
        data_type = OrderStatsRequest.DAY
        showTime()
    }

    override fun datetimeDialog(fragmentManager: FragmentManager) {
        val (startTime, endTime) = when (data_type) {
            OrderStatsRequest.DAY -> {
                Pair(day_start_time, day_start_time)
            }
            OrderStatsRequest.WEEK -> {
                Pair(week_start_time, week_end_time)
            }
            OrderStatsRequest.MONTH -> {
                Pair(month_start_time, month_start_time)
            }
            else -> {
                Pair(day_start_time, day_start_time)
            }
        }
        dialog.arguments = Bundle().apply {
            putString(DataCenterPickerDialog.TYPE, data_type)
            putLong(DataCenterPickerDialog.START_TIME, startTime)
            putLong(DataCenterPickerDialog.END_TIME, endTime)
            putInt(DataCenterPickerDialog.DATE_TAG, DataCenterPickerDialog.NO_TODAY)
        }
        dialog.show(fragmentManager, this)
    }

    private fun showTime() {
        view?.showTime(
            when (data_type) {
                OrderStatsRequest.DAY -> {
                    showSimpleDateFormat.format(Date(day_start_time))
                }
                OrderStatsRequest.WEEK -> {
                    "${showSimpleDateFormat.format(Date(week_start_time))}-${showSimpleDateFormat.format(
                        Date(week_end_time)
                    )}"
                }
                OrderStatsRequest.MONTH -> {
                    SimpleDateFormat("yyyy年MM月").format(Date(month_start_time))
                }
                else -> {
                    ""
                }
            }
        )
    }

    override fun onDismiss() {}

    override fun onSubmit(type: String, startTime: Long, endTime: Long) {
        when (type) {
            OrderStatsRequest.DAY -> {
                day_start_time = startTime
            }
            OrderStatsRequest.WEEK -> {
                week_start_time = startTime
                week_end_time = endTime
            }
            OrderStatsRequest.MONTH -> {
                month_start_time = startTime
            }
            else -> {
            }
        }
        setDataType(type)
    }

}