package com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.deal

import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.library.bean.OrderStatsBean
import com.jiaoyinbrother.library.bean.OrderStatsRequest
import com.jiaoyinbrother.monkeyking.manager.R

class DealDataAnalysisAdapter(
    private val list: MutableList<OrderStatsBean>
) : RecyclerView.Adapter<EasyRecyclerViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): EasyRecyclerViewHolder {
        return EasyRecyclerViewHolder(
            LayoutInflater.from(parent.context)
                .inflate(R.layout.item_data_analysis_deal, parent, false)
        )
    }

    override fun getItemCount(): Int {
        return list.size
    }

    override fun onBindViewHolder(holder: EasyRecyclerViewHolder, position: Int) {
        holder.findViewById<View>(R.id.v_line)?.visibility =
            if (position == itemCount - 1 || (position == itemCount - 2 && position % 2 == 0)) {
                View.GONE
            } else {
                View.VISIBLE
            }
        val orderStatsBean = list[position]
        holder.itemView.setOnClickListener {
            if (type == OrderStatsRequest.DAY) {
                dataAnalysisItemClickListener?.onItemClick(orderStatsBean, position)
            }
        }
        holder.findViewById<View>(R.id.iv_question)?.setOnClickListener {
            dataAnalysisItemClickListener?.onQuestionClick(orderStatsBean, position)
        }
        holder.findViewById<TextView>(R.id.tv_name)?.text = orderStatsBean.type_name
        val (sum, compare_sum) = getSumString(
            orderStatsBean.type,
            orderStatsBean.sum,
            orderStatsBean.compare_sum
        )
        holder.findViewById<TextView>(R.id.tv_sum)?.text = sum
        holder.findViewById<TextView>(R.id.tv_compare_sum_head)?.text = when (type) {
            OrderStatsRequest.WEEK -> "较上周："
            OrderStatsRequest.MONTH -> "较上月："
            else -> "较前一日："
        }

        holder.findViewById<ImageView>(R.id.iv_chart)?.visibility =
            if (type == OrderStatsRequest.DAY) {
                View.VISIBLE
            } else {
                View.GONE
            }

        val tvSum = holder.findViewById<TextView>(R.id.tv_compare_sum)
        tvSum?.text = compare_sum

        val tvStats = holder.findViewById<View>(R.id.tv_stats)
        val ivStats = holder.findViewById<ImageView>(R.id.iv_stats)
        if (TextUtils.equals("equal", orderStatsBean.status)) {
            tvStats?.visibility = View.VISIBLE
            ivStats.visibility = View.GONE
            tvSum?.visibility = View.GONE
        } else {
            tvStats?.visibility = View.GONE
            ivStats.visibility = View.VISIBLE
            tvSum?.visibility = View.VISIBLE
            if (isReverse(orderStatsBean.type)) {
                ivStats?.setImageResource(
                    if (TextUtils.equals("down", orderStatsBean.status))
                        R.mipmap.icon_data_analysis_up
                    else
                        R.mipmap.icon_data_analysis_down
                )
                ivStats?.rotation = 180F
            } else {
                ivStats?.setImageResource(
                    if (TextUtils.equals("down", orderStatsBean.status))
                        R.mipmap.icon_data_analysis_down
                    else
                        R.mipmap.icon_data_analysis_up
                )
                ivStats?.rotation = 0F
            }
        }

    }

    private fun getSumString(
        type: String?,
        sum: String?,
        compare_sum: String?
    ): Pair<String?, String?> {
        return when (type) {
            //(payed_amount-付款订单金额 payed_orders-付款订单数 canceled_rate-付款后取消率 canceled_orders-付款后取消单数 accepted_amount-结单金额 accepted_orders-结单数 mark_rate-评价率 good_mark_rate-好评率 bad_mark_rate-差评率)
            "payed_amount", "accepted_amount" -> {//付款订单金额//结单金额
                Pair("¥$sum", "¥$compare_sum")
            }
            "payed_orders", "canceled_orders", "accepted_orders" -> {//付款订单数//付款后取消单数//结单数
                Pair("${sum}单", "${compare_sum}单")
            }
            "canceled_rate", "mark_rate", "good_mark_rate", "bad_mark_rate" -> {//付款后取消率//评价率//好评率//差评率
                Pair("$sum%", "$compare_sum%")
            }
            else -> {
                Pair("$sum", "$compare_sum")
            }
        }
    }

    private fun isReverse(type: String?): Boolean {
        return TextUtils.equals(type, "canceled_rate")
                || TextUtils.equals(type, "canceled_orders")
                || TextUtils.equals(type, "bad_mark_rate")
    }

    private var type: String = OrderStatsRequest.DAY

    fun setDataType(type: String) {
        this.type = type
    }

    var dataAnalysisItemClickListener: DataAnalysisItemClickListener? = null

    interface DataAnalysisItemClickListener {
        fun onQuestionClick(data: OrderStatsBean, position: Int)

        fun onItemClick(data: OrderStatsBean, position: Int)
    }
}