package com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.deal

import android.content.Context
import android.os.Bundle
import android.support.v4.app.FragmentManager
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.OrderStatsBean
import com.jiaoyinbrother.library.bean.OrderStatsRequest
import com.jiaoyinbrother.library.bean.OrderStatsResult
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.dialog.DataCenterPickerDialog
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.text.SimpleDateFormat
import java.util.*
import java.util.concurrent.TimeUnit

class DealDataAnalysisPresenter(context: Context, view: IDealDataAnalysisContract.View) :
        BasePresenterImpl<IDealDataAnalysisContract.View>(context, view), IDealDataAnalysisContract.Presenter, DataCenterPickerDialog.DialogListener {
    private val dialog by lazy {
        DataCenterPickerDialog()
    }
    override var data_type: String = OrderStatsRequest.DAY
    override var site_id: String? = null
    override var day_start_time: Long = 0L
    override var week_start_time: Long = 0L
    override var week_end_time: Long = 0L
    override var month_start_time: Long = 0L
    override val list = mutableListOf<OrderStatsBean>()
    private val upSimpleDateFormat = SimpleDateFormat("yyyy-MM-dd")
    private val showSimpleDateFormat = SimpleDateFormat("yyyy年MM月dd日")
    override fun refresh() {
        if (TextUtils.isEmpty(site_id)) return
        val request = OrderStatsRequest()
        request.site_id = site_id
        request.type = data_type
        request.source = "TRANSACTION"
        when (data_type) {
            OrderStatsRequest.DAY -> {
                request.start_time = upSimpleDateFormat.format(Date(day_start_time))
                request.end_time = upSimpleDateFormat.format(Date(day_start_time))
            }
            OrderStatsRequest.WEEK -> {
                request.start_time = upSimpleDateFormat.format(Date(week_start_time))
                request.end_time = upSimpleDateFormat.format(Date(week_end_time))
            }
            OrderStatsRequest.MONTH -> {
                val instance = Calendar.getInstance()
                instance.timeInMillis = month_start_time
                instance.set(Calendar.DAY_OF_MONTH,1)
                request.start_time = upSimpleDateFormat.format(instance.time)
            }
        }
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getDataStats(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderStatsResult>(context, this) {
            override fun onSuccess(response: OrderStatsResult) {
                view?.dismissLoadingDialog()
                list.clear()
                list.addAll(response.rows ?: emptyList())
                view?.updateView()

            }

            override fun onAPIError(apiException: ApiException?) {
                super.onAPIError(apiException)
                view?.dismissLoadingDialog()
            }
        })
    }

    override fun setDataType(type: String) {
        data_type = type
        showTime()
        refresh()
    }

    override fun initData() {
        val currentTimeMillis = System.currentTimeMillis()
        val instance = Calendar.getInstance()
        instance.timeInMillis = currentTimeMillis
        instance.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY)
        day_start_time = currentTimeMillis
        week_start_time = instance.timeInMillis
        week_end_time = currentTimeMillis
        month_start_time = currentTimeMillis
        data_type = OrderStatsRequest.DAY
        showTime()
    }

    override fun datetimeDialog(fragmentManager: FragmentManager) {
        val (startTime, endTime) = when (data_type) {
            OrderStatsRequest.DAY -> {
                Pair(day_start_time, day_start_time)
            }
            OrderStatsRequest.WEEK -> {
                Pair(week_start_time, week_end_time)
            }
            OrderStatsRequest.MONTH -> {
                Pair(month_start_time, month_start_time)
            }
            else -> {
                Pair(day_start_time, day_start_time)
            }
        }
        dialog.arguments = Bundle().apply {
            putString(DataCenterPickerDialog.TYPE, data_type)
            putLong(DataCenterPickerDialog.START_TIME, startTime)
            putLong(DataCenterPickerDialog.END_TIME, endTime)
            putInt(DataCenterPickerDialog.DATE_TAG, DataCenterPickerDialog.HAS_TODAY)
        }
        dialog.show(fragmentManager, this)
    }

    private fun showTime() {
        view?.showTime(when (data_type) {
            OrderStatsRequest.DAY -> {
                showSimpleDateFormat.format(Date(day_start_time))
            }
            OrderStatsRequest.WEEK -> {
                "${showSimpleDateFormat.format(Date(week_start_time))}-${showSimpleDateFormat.format(Date(week_end_time))}"
            }
            OrderStatsRequest.MONTH -> {
                SimpleDateFormat("yyyy年MM月").format(Date(month_start_time))
            }
            else -> {
                ""
            }
        })
    }

    override fun onDismiss() {}

    override fun onSubmit(type: String, startTime: Long, endTime: Long) {
        when (type) {
            OrderStatsRequest.DAY -> {
                day_start_time = startTime
            }
            OrderStatsRequest.WEEK -> {
                week_start_time = startTime
                week_end_time = endTime
            }
            OrderStatsRequest.MONTH -> {
                month_start_time = startTime
            }
            else -> {
            }
        }
        setDataType(type)
    }

}