package com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.detail

import android.support.constraint.ConstraintLayout
import android.support.v4.content.ContextCompat
import android.support.v7.widget.RecyclerView
import android.util.SparseArray
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.jiaoyinbrother.library.bean.OrderStatsBean
import com.jiaoyinbrother.monkeyking.manager.R
import com.tendcloud.tenddata.Order

class AnalysisTabAdapter : RecyclerView.Adapter<AnalysisTabAdapter.ViewHolder>() {


    private var selectType: String? = null
    private var onItemClickListener: OnItemClickListener? = null

    inner class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        private val views: SparseArray<View> by lazy {
            SparseArray<View>()
        }

        fun <T : View?> findViewById(viewId: Int): T {
            var view = views.get(viewId)
            if (view == null) {
                view = itemView.findViewById(viewId)
                views.put(viewId, view)
            }
            return view as T
        }
    }

    var list: ArrayList<OrderStatsBean> = ArrayList()
        set(value) {
            field.clear()
            field.addAll(value)
            notifyDataSetChanged()
        }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        return ViewHolder(
            LayoutInflater.from(parent.context).inflate(R.layout.item_analysis_tab, parent, false)
        )
    }

    override fun getItemCount(): Int {
        return list.size
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.let {
            val rootView = it.findViewById<ConstraintLayout>(R.id.item_analysis_tab_root)
            val mTitle = it.findViewById<TextView>(R.id.item_analysis_tab_title)
            val mContent = it.findViewById<TextView>(R.id.item_analysis_tab_content)
            val mEmpty = it.findViewById<View>(R.id.empty_view)

            val entity = list[position] as? OrderStatsBean
            entity?.let { bean ->
                if (selectType == bean.type) {
                    rootView.setBackgroundResource(R.drawable.tab_item_checked_bg)
                    mTitle.setTextColor(
                        ContextCompat.getColor(it.itemView.context, R.color.color_6)
                    )
                    mContent.setTextColor(
                        ContextCompat.getColor(it.itemView.context, R.color.color_6)
                    )
                } else {
                    rootView.setBackgroundResource(R.drawable.tab_item_uncheck_bg)
                    mTitle.setTextColor(
                        ContextCompat.getColor(it.itemView.context, R.color.color_333333)
                    )
                    mContent.setTextColor(
                        ContextCompat.getColor(it.itemView.context, R.color.color_333333)
                    )
                }

                mTitle.text = bean.type_name
                mContent.text = getContentInfo(bean)
            }

            mEmpty.visibility = if (position == 0) View.VISIBLE else View.GONE

            rootView.setOnClickListener {
                onItemClickListener?.onItemClick(entity, position)
            }
        }
    }

    private fun getContentInfo(bean: OrderStatsBean): String {
        return when (bean.type) {
            // 付款订单数 | 付款后取消单数 | 结单数
            OrderStatsBean.PAYED_ORDERS,
            OrderStatsBean.CANCELED_ORDERS,
            OrderStatsBean.ACCEPTED_ORDERS -> "${bean.sum}单"
            // 结单金额 | 付款订单金额
            OrderStatsBean.ACCEPTED_AMOUNT,
            OrderStatsBean.PAYED_AMOUNT -> "￥${bean.sum}"
            // 付款后取消率
            OrderStatsBean.MARK_RATE,
            OrderStatsBean.GOOD_MARK_RATE,
            OrderStatsBean.BAD_MARK_RATE,
            OrderStatsBean.CANCELED_RATE -> "${bean.sum}%"
            else -> "￥${bean.sum}"

        }
    }

    fun onSelectItem(type: String?) {
        selectType = type
    }


    fun setOnItemClickListener(listener: OnItemClickListener) {
        this.onItemClickListener = listener
    }

    /**
     * the click listeners callback
     * 点击事件回调
     */
    interface OnItemClickListener {
        /**
         * on item click call back
         *
         */
        fun onItemClick(entity: OrderStatsBean?, position: Int)
    }

}