package com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.detail

import android.content.Context
import android.content.Intent
import com.google.gson.Gson
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.DateTimeUtil
import com.jiaoyinbrother.library.util.LogUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import org.json.JSONArray
import java.util.concurrent.TimeUnit

class DetailPresenter(context: Context, view: IDetailContract.View) :
    BasePresenterImpl<IDetailContract.View>(context, view), IDetailContract.Presenter {
    companion object {
        const val ANALYSIS_ORDER = 0
        const val ANALYSIS_MARK = 1
    }

    // 区分 入口是交易 or 评价
    private var entrance: Int = ANALYSIS_ORDER

    // 通用参数
    private var site_id: String? = null

    // 初始化 type 勾选
    private var mTypeData: OrderStatsBean? = null

    /*交易分析图表Json格式数据*/
    private var orderJson: String? = null

    /*评价分析图表Json格式数据*/
    private var marksJson: String? = null

    override fun initData(intent: Intent) {
        entrance = intent.getIntExtra(DetailAnalysisActivity.ANALYSIS_ITEMS, ANALYSIS_ORDER)
        site_id = intent.getStringExtra(DetailAnalysisActivity.SITE_ID)
        mTypeData =
            intent.getSerializableExtra(DetailAnalysisActivity.ANALYSIS_CLASS) as? OrderStatsBean

        view.setToolbarInfo(if (entrance == ANALYSIS_ORDER) "交易分析" else "评价分析")

        setDayType(OrderStatsRequest.DAY_SEVEN)
    }

    override fun setDayType(day: Int) {
        when (entrance) {
            ANALYSIS_ORDER -> apiOrderAnalysis(day)
            ANALYSIS_MARK -> apiMarkAnalysis(day)
            else -> {
                LogUtil.e("entrance error ")
            }
        }
    }

    override fun setTabItem(entity: OrderStatsBean) {
        mTypeData = entity
        onRefreshChart()
    }

    override fun onRefreshChart() {
        when (entrance) {
            ANALYSIS_ORDER -> view.setAnalysisData(setChartData(orderJson, mTypeData))
            ANALYSIS_MARK -> view.setAnalysisData(setChartData(marksJson, mTypeData))
            else -> {
                LogUtil.e("entrance error ")
            }
        }

    }


    /**
     * 通用线形图表处理
     * 通过StringBuffer进行参数封装
     *
     * @param data 原数据Json格式
     * @param type 当前tab对应的OrderStatsBean对象
     *
     * @return 图表所需要数据格式
     */
    private fun setChartData(data: String?, type: OrderStatsBean?): String {
        val buffer = StringBuffer()
        buffer.append("[")
        if (data == null || data.isEmpty()) {
            // 空数据
            return buffer.append("], '${'$'}{value}'").toString()
        }
        val array = JSONArray(data)
        for (i in 0 until array.length()) {
            val obj = array.getJSONObject(i)
            val date = obj.optString("date_str")
            val value = obj.optString(type?.type)
            buffer.append(
                "{name:'${DateTimeUtil.getMDForCN(date)}<br/>${type?.type_name} ${formatTipValue(
                    type?.type, value
                )}',value:['$date', ${value}]},"
            )
        }
        // '${'$'}{value}' 需要根据type类型进行拼接
        buffer.append("]").append(formatValue(type?.type))
        return buffer.toString()
    }

    private fun formatValue(type: String?): String {
        return when (type) {
            // ￥
            OrderStatsBean.PAYED_AMOUNT,
            OrderStatsBean.ACCEPTED_AMOUNT -> ", '￥${'$'}{value}'"
            // %
            OrderStatsBean.MARK_RATE,
            OrderStatsBean.BAD_MARK_RATE,
            OrderStatsBean.CANCELED_RATE,
            OrderStatsBean.GOOD_MARK_RATE -> ", '${'$'}{value}%'"
            // 单
            OrderStatsBean.PAYED_ORDERS,
            OrderStatsBean.ACCEPTED_ORDERS,
            OrderStatsBean.CANCELED_ORDERS -> ", '${'$'}{value}单'"
            else -> ", '${'$'}{value}单'"
        }
    }

    private fun formatTipValue(type: String?, value: String): String =
        when (type) {
            // ￥
            OrderStatsBean.PAYED_AMOUNT,
            OrderStatsBean.ACCEPTED_AMOUNT -> "￥${value}"
            // %
            OrderStatsBean.MARK_RATE,
            OrderStatsBean.BAD_MARK_RATE,
            OrderStatsBean.CANCELED_RATE,
            OrderStatsBean.GOOD_MARK_RATE -> "${value}%"
            // 单
            OrderStatsBean.PAYED_ORDERS,
            OrderStatsBean.ACCEPTED_ORDERS,
            OrderStatsBean.CANCELED_ORDERS -> "${value}单"
            else -> "${value}单"
        }

    /**
     *  交易分析接口
     *
     * 处理入口为交易tab事件
     */
    private fun apiOrderAnalysis(day: Int) {
        val request = OrderStatsRequest()
        request.site_id = site_id
        request.day = day
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.orderAnalysis(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : BaseObserver<OrderAnalysisResult>(context, this) {
                override fun onSuccess(response: OrderAnalysisResult) {
                    view.setModifyData(response.modify_time_str ?: "")
                    // tab data
                    if (response.rows != null && response.rows?.size ?: 0 > 0) {
                        view.setTabData(response.rows, mTypeData)
                    }
                    // line chart data
                    /**
                     * 折线图，js字符串："setLine([{name: '8月1日<br/>结单金额 100',value:['2020-01-02', 10]}], '${value}')"
                     *
                     * @param data 是一个数组，数组结构：name 是 tooltip 显示的内容，value 是 x，y 轴的数据，如
                     * [{
                     *      name: '8月1日<br/>结单金额 100',
                     *      value:['2020-01-02', 10]
                     * }, {
                     *      name: '8月1日<br/>结单金额 100',
                     *      value:['2020-01-02', 10]
                     * }, {
                     *      name: '8月1日<br/>结单金额 100',
                     *      value:['2020-01-02', 10]
                     * }]"
                     *
                     * @param yAxisFormat 是 y 轴格式，${value} 和单位的组合，如 '¥${value}' 或 '${value}元'
                     */
                    if (response.curves != null && response.curves?.size ?: 0 > 0) {
                        orderJson = Gson().toJson(response.curves)
                        view.setAnalysisData(setChartData(orderJson, mTypeData))
                    }
                    view.dismissLoadingDialog()
                }

                override fun onError(e: Throwable) {
                    view.dismissLoadingDialog()
                }

                override fun onAPIError(apiException: ApiException?) {
                    view.dismissLoadingDialog()
                }
            })
    }

    /**
     * 评价分析接口
     *
     * 处理入口为评价tab事件
     */
    private fun apiMarkAnalysis(day: Int) {
        val request = OrderStatsRequest()
        request.site_id = site_id
        request.day = day
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.markAnalysis(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : BaseObserver<OrderAnalysisResult>(context, this) {
                override fun onSuccess(response: OrderAnalysisResult) {
                    view.dismissLoadingDialog()
                    view.setModifyData(response.modify_time_str ?: "")
                    // tab data
                    if (response.rows != null && response.rows?.size ?: 0 > 0) {
                        view.setTabData(response.rows, mTypeData)
                    }
                    // line chart data
                    if (response.curves != null && response.curves?.size ?: 0 > 0) {
                        marksJson = Gson().toJson(response.curves)
                        view.setAnalysisData(setChartData(marksJson, mTypeData))
                    }
                }

                override fun onError(e: Throwable) {
                    view.dismissLoadingDialog()
                }

                override fun onAPIError(apiException: ApiException?) {
                    view.dismissLoadingDialog()
                }
            })
    }
}