package com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.dialog

import android.os.Bundle
import android.support.v4.app.DialogFragment
import android.support.v4.app.FragmentManager
import android.view.Gravity
import android.view.View
import android.view.WindowManager
import com.jiaoyinbrother.library.bean.OrderStatsRequest
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpdialog.MVPDialogFragment
import com.zyyoona7.wheel.WheelView
import kotlinx.android.synthetic.main.dialog_data_center_picker.*
import java.lang.Exception

class DataCenterPickerDialog : MVPDialogFragment<DataCenterPickerPresenter>(),
    IDataCenterPickerContract.View {

    var listener: DialogListener? = null

    fun show(manager: FragmentManager?, listener: DialogListener) {
        try {
            this.listener = listener
            show(manager, "DataCenterPickerDialog")
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    override fun initListeners() {
        dialog.setOnDismissListener {
            listener?.onDismiss()
        }
        wv_year?.onItemSelectedListener = WheelView.OnItemSelectedListener<String> { _, data, _ ->
            presenter.updateDay(data)
        }
    }

    override fun initViews(view: View?, savedInstanceState: Bundle?) {
        tv_cancel?.setOnClickListener {
            dismiss()
        }
        tv_submit?.setOnClickListener {
            presenter.callBack(listener, wv_year.selectedItemPosition, wv_day.selectedItemPosition)
            dismiss()
        }
    }

    override fun initData() {
        presenter.updateData(
            arguments?.getString(TYPE) ?: OrderStatsRequest.DAY,
            arguments?.getLong(START_TIME) ?: System.currentTimeMillis(),
            arguments?.getLong(END_TIME) ?: System.currentTimeMillis(),
            arguments?.getInt(DATE_TAG) ?: HAS_TODAY
        )
    }

    override fun getLayoutId(): Int {
        return R.layout.dialog_data_center_picker
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setStyle(DialogFragment.STYLE_NO_FRAME, R.style.Dialog_Transparent)
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        //初始化样式
        dialog?.window?.setLayout(
            WindowManager.LayoutParams.MATCH_PARENT,
            WindowManager.LayoutParams.WRAP_CONTENT
        )
        dialog?.window?.setGravity(Gravity.BOTTOM)
        dialog?.window?.setWindowAnimations(R.style.dialog_anim_down2up)
    }

    override fun initToolbar(savedInstanceState: Bundle?) {

    }

    override fun initPresenter(): DataCenterPickerPresenter {
        return DataCenterPickerPresenter(requireContext(), this)
    }

    override fun showDay(
        yearList: List<String>,
        dayList: List<String>,
        yearPosition: Int,
        dayPosition: Int
    ) {
        wv_year?.visibility = View.VISIBLE
        wv_year?.data = yearList
        wv_year.selectedItemPosition = if (yearPosition < 0) 0 else yearPosition

        wv_day?.visibility = View.VISIBLE
        wv_day?.data = dayList
        wv_day.selectedItemPosition = if (dayPosition < 0) 0 else dayPosition

    }

    override fun updateDay(list: List<String>, position: Int) {
        wv_day?.visibility = View.VISIBLE
        wv_day?.data = list
        wv_day.selectedItemPosition = if (position < 0) 0 else position
    }

    override fun showWeek(list: List<String>, position: Int) {
        wv_year?.visibility = View.VISIBLE
        wv_year?.data = list
        wv_year.selectedItemPosition = if (position < 0) 0 else position
        wv_day?.visibility = View.GONE
    }

    override fun showMonth(list: List<String>, position: Int) {
        wv_year?.visibility = View.VISIBLE
        wv_year?.data = list
        wv_year.selectedItemPosition = if (position < 0) 0 else position
        wv_day?.visibility = View.GONE
    }

    interface DialogListener {
        fun onDismiss()
        fun onSubmit(type: String, startTime: Long, endTime: Long)
    }

    companion object {
        const val TYPE = "TYPE"
        const val START_TIME = "START_TIME"
        const val END_TIME = "END_TIME"
        const val DATE_TAG = "DATE_TAG"
        const val NO_TODAY = 0
        const val HAS_TODAY = 1
    }
}