package com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.dialog

import android.content.Context
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.OrderStatsRequest
import java.lang.Exception
import java.text.SimpleDateFormat
import java.util.*


class DataCenterPickerPresenter(context: Context, view: IDataCenterPickerContract.View) :
    BasePresenterImpl<IDataCenterPickerContract.View>(context, view),
    IDataCenterPickerContract.Presenter {
    private var type: String = OrderStatsRequest.DAY
    private var tag: Int = DataCenterPickerDialog.HAS_TODAY

    private val year_of_day = mutableListOf<String>()//日期选择的年份
    private val day_of_day = mutableMapOf<String, MutableList<String>>()
    private val date_of_week = mutableListOf<String>()//周选择的列表
    private val month_of_month = mutableListOf<String>()//月份选择的月份列表

    override fun updateData(type: String, startTime: Long, endTime: Long, tag: Int) {
        this.type = type
        this.tag = tag
        when (type) {
            OrderStatsRequest.DAY -> {
                if (day_of_day.isEmpty()) {
                    getDayData()
                }
                val instance = getCalendarForTag(tag)
                instance.timeInMillis = startTime
                val element = "${instance.get(Calendar.YEAR)}年${instance.get(Calendar.MONTH) + 1}月"
                val dayList = day_of_day[element] ?: emptyList<String>()
                view.showDay(
                    year_of_day,
                    dayList,
                    year_of_day.indexOf(element),
                    dayList.indexOf("${instance.get(Calendar.DAY_OF_MONTH)}日")
                )
            }
            OrderStatsRequest.WEEK -> {
                if (date_of_week.size == 0) {
                    getWeekData()
                }
                val start = getCalendarForTag(tag)
                start.timeInMillis = startTime
                val end = getCalendarForTag(tag)
                end.timeInMillis = endTime
                view.showWeek(
                    date_of_week,
                    date_of_week.indexOf("${start[Calendar.YEAR]}年${start[Calendar.MONTH] + 1}月${start[Calendar.DAY_OF_MONTH]}日-${end[Calendar.YEAR]}年${end[Calendar.MONTH] + 1}月${end[Calendar.DAY_OF_MONTH]}日")
                )

            }
            OrderStatsRequest.MONTH -> {
                if (month_of_month.size == 0) {
                    getMonthData()
                }
                val instance = getCalendarForTag(tag)
                instance.timeInMillis = startTime
                val element = "${instance.get(Calendar.YEAR)}年${instance.get(Calendar.MONTH) + 1}月"
                view.showMonth(month_of_month, month_of_month.indexOf(element))
            }
            else -> {
            }
        }
    }

    override fun updateDay(data: String?) {
        if (TextUtils.equals(OrderStatsRequest.DAY, type)) {
            view.updateDay(day_of_day[data] ?: emptyList<String>(), 0)
        }
    }

    override fun callBack(
        listener: DataCenterPickerDialog.DialogListener?,
        yearSelectPosition: Int,
        daySelectPosition: Int
    ) {
        val simpleDateFormat = SimpleDateFormat("yyyy年MM月dd日", Locale.CHINA)
        val (startTime, endTime) = try {
            when (type) {
                OrderStatsRequest.DAY -> {
                    val year = year_of_day[yearSelectPosition]
                    val time = simpleDateFormat
                        .parse("$year${day_of_day[year]?.get(daySelectPosition)}")?.time ?: -1
                    Pair(time, time)
                }
                OrderStatsRequest.WEEK -> {
                    val split = date_of_week[yearSelectPosition].split("-")
                    if (split.size == 2) {
                        Pair(
                            simpleDateFormat.parse(split[0])?.time ?: -1,
                            simpleDateFormat.parse(split[1])?.time ?: -1
                        )
                    } else {
                        Pair(-1L, -1L)
                    }
                }
                OrderStatsRequest.MONTH -> {
                    val time = SimpleDateFormat("yyyy年MM月", Locale.CHINA)
                        .parse(month_of_month[yearSelectPosition])?.time ?: -1
                    Pair(time, time)
                }
                else -> {
                    Pair(-1L, -1L)
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
            Pair(-1L, -1L)
        }
        listener?.onSubmit(type, startTime, endTime)
    }

    private fun getDayData() {
        year_of_day.clear()
        day_of_day.clear()
        val instance = getCalendarForTag(tag)
        var year = "${instance.get(Calendar.YEAR)}年${instance.get(Calendar.MONTH) + 1}月"
        year_of_day.add(0, year)
        day_of_day[year] = mutableListOf<String>()
        for (i in 0 until 30) {
            val currYear = "${instance.get(Calendar.YEAR)}年${instance.get(Calendar.MONTH) + 1}月"
            if (TextUtils.equals(year, currYear)) {
                val mutableList = day_of_day[year]
                mutableList?.add(0, "${instance.get(Calendar.DAY_OF_MONTH)}日")
            } else {
                year = currYear
                year_of_day.add(0, year)
                day_of_day[year] = mutableListOf<String>().apply {
                    add(0, "${instance.get(Calendar.DAY_OF_MONTH)}日")
                }
            }
            instance.add(Calendar.DAY_OF_MONTH, -1)
        }
    }

    private fun getWeekData() {
        date_of_week.clear()
        val endCalendar = getCalendarForTag(tag)
        endCalendar.firstDayOfWeek = Calendar.MONDAY

        val startCalendar = Calendar.getInstance(Locale.CHINA)
        startCalendar.timeInMillis = endCalendar.timeInMillis
        startCalendar.set(Calendar.DAY_OF_MONTH, 1)
        startCalendar.add(Calendar.MONTH, -2)
        while (startCalendar[Calendar.DAY_OF_WEEK] == Calendar.MONDAY) {
            startCalendar.add(Calendar.DAY_OF_MONTH, 1)
        }
        var weekEnd =
            "${endCalendar[Calendar.YEAR]}年${endCalendar[Calendar.MONTH] + 1}月${endCalendar[Calendar.DAY_OF_MONTH]}日"
        endCalendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY)
        var weekStart =
            "${endCalendar[Calendar.YEAR]}年${endCalendar[Calendar.MONTH] + 1}月${endCalendar[Calendar.DAY_OF_MONTH]}日"
        date_of_week.add(0, "${weekStart}-${weekEnd}")
        do {
            endCalendar.add(Calendar.DAY_OF_YEAR, -1)
            weekEnd =
                "${endCalendar[Calendar.YEAR]}年${endCalendar[Calendar.MONTH] + 1}月${endCalendar[Calendar.DAY_OF_MONTH]}日"
            endCalendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY)
            weekStart =
                "${endCalendar[Calendar.YEAR]}年${endCalendar[Calendar.MONTH] + 1}月${endCalendar[Calendar.DAY_OF_MONTH]}日"
            date_of_week.add(0, "${weekStart}-${weekEnd}")
        } while (endCalendar > startCalendar)
    }

    private fun getMonthData() {
        month_of_month.clear()
        val instance = getCalendarForTag(tag)
        for (i in 0 until 3) {
            month_of_month.add(0, "${instance[Calendar.YEAR]}年${instance[Calendar.MONTH] + 1}月")
            instance.add(Calendar.MONTH, -1)
        }
    }



    private fun getCalendarForTag(tag:Int) :Calendar{
        val instance = Calendar.getInstance(Locale.CHINA)
        instance.timeInMillis = System.currentTimeMillis()
        if (tag == DataCenterPickerDialog.NO_TODAY){
            instance.add(Calendar.DATE, -1)
        }
        return instance
    }
}
