package com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.sales

import android.content.Context
import android.content.Intent
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.OperatingCarStockResult
import com.jiaoyinbrother.library.bean.OrderStatsBean
import com.jiaoyinbrother.library.bean.OrderStatsRequest
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.dataanalysis.detail.DetailAnalysisActivity
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit
import kotlin.collections.ArrayList

class SalesPresenter(context: Context, view: ISalesContract.View) :
    BasePresenterImpl<ISalesContract.View>(context, view), ISalesContract.Presenter {

    override var site_id: String? = null
    override var type: String? = null
    override var day: Int? = OrderStatsRequest.DAY_SEVEN
    override var data: ArrayList<OrderStatsBean> = ArrayList()


    override fun initData(intent: Intent) {
        site_id = intent.getStringExtra(DetailAnalysisActivity.SITE_ID) ?: return
//        type = intent.getStringExtra(DetailAnalysisActivity.ANALYSIS_CLASS)
        apiCarSales()
    }

    override fun setDayType(day: Int) {
        this.day = day
        apiCarSales()
    }

    private fun apiCarSales() {
        val request = OrderStatsRequest()
        request.site_id = site_id
        request.day = day
        request.is_more = OrderStatsRequest.IS_MORE
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.operatingCarSales(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : BaseObserver<OperatingCarStockResult>(context, this) {
                override fun onSuccess(response: OperatingCarStockResult) {
                    view.dismissLoadingDialog()
                    view.setSalesRefreshDate(response.modify_time_str ?: "")
                    view.setCarSalesData(setData(response.rows))
                }

                override fun onError(e: Throwable) {
                    view.dismissLoadingDialog()
                    view.setCarSalesData(setData(null))
                }

                override fun onAPIError(apiException: ApiException?) {
                    view.dismissLoadingDialog()
                    view.setCarSalesData(setData(null))
                }
            })
    }


    private fun setData(list: ArrayList<OrderStatsBean>?): ArrayList<OrderStatsBean> {
        data.clear()
        // 标题
        data.add(OrderStatsBean())
        // 数据
        if (list == null || list.size == 0) {
            return data
        }
        data.addAll(list)
        return data
    }
}