package com.jiaoyinbrother.monkeyking.manager.mvpactivity.dropinthecar

import android.content.Context
import android.content.Intent
import android.support.v4.app.FragmentActivity
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.base.OrderBaseBean

import com.jiaoyinbrother.library.bean.*

import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.adapter.OrderListAdapter.Companion.ORDER_LIST_TO_DROP_IN
import com.jiaoyinbrother.monkeyking.manager.util.DialogUtil
import com.jiaoyinbrother.monkeyking.manager.util.PhoneConvertUtil
import com.jiaoyinbrother.monkeyking.manager.util.WkAlertDialogFactory
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit
import kotlin.random.Random

class DropInTheCarPresenter(context: Context, view: IDropInTheCarContract.view) :
        BasePresenterImpl<IDropInTheCarContract.view>(context, view), IDropInTheCarContract.presenter {


    companion object {
        //订单ID
        const val ORDER_ID = "ORDER_ID"

        // 上门操作类型 1：出发送车，2：确认送达，3：出发取车，4：确认取车
        const val ONSITE_OPERATION = "ONSITE_OPERATION"

        //intent 是默认intent   button是页面底部按钮    refresh订单详情刷新
        const val FLAG_INTENT = "FLAG_INTENT"
        const val FLAG_BUTTON = "FLAG_BUTTON"
        const val FLAG_REFRESH = "FLAG_REFRESH"

//        const val ORDER_REFRESH = "ORDER_REFRESH"

    }

    private var onSiteOperation: String? = null
    private var orderId: String? = null
    private var ordersBean: OrderBaseBean? = null
    private var isOnSiteService = 0


    fun getIntent(intent: Intent) {

        ordersBean = intent.getSerializableExtra(ORDER_LIST_TO_DROP_IN) as OrderBaseBean?
        LogUtil.e("ordersBean====" + ordersBean.toString())
        //类型
        onSiteOperation = ordersBean?.onsite_operation.toString()
        //订单id
        orderId = ordersBean?.orderid

        view.setCarModel(ordersBean?.type ?: "")
        view.setNumberPlate(ordersBean?.number ?: "")
        view.setPhoneInfo(ordersBean?.renter_name ?: "", ordersBean?.renterid ?: "")
        isOnSiteService = ordersBean?.is_onsite_service ?: 0
//        区分送车还是接送
        when (isOnSiteService) {
            OrdersBean.SEND_TO_CAR -> {
                setSendCarInfo()
            }
            OrdersBean.PICK_UP_PERSON -> {
                setPickupPersonInfo()
            }
        }
    }

    private fun setPickupPersonInfo() {
        view.setAddressTitle("接人地址")
        whenPickType(onSiteOperation ?: "", FLAG_INTENT)
        when (onSiteOperation) {
            "1", "2" -> {
                if (ordersBean?.location != null) {
                    view.setDeliveryAddress(ordersBean?.address ?: "")
                    view.setLocation(ordersBean?.location!!, ordersBean?.address ?: "")
                    view.setPickUpTime(ordersBean?.start_time ?: "")
                    view.setPickUpTheCar("预计取车时间")
                }
            }
            else -> {
                if (ordersBean?.location != null)
                    view.setLocation(ordersBean?.location!!, ordersBean?.address ?: "")
            }
        }
    }

    private fun whenPickType(onSiteOperation: String, flag: String) {
        when (onSiteOperation) {
            "1" -> {
                when (flag) {
                    FLAG_INTENT -> {
                        view.setTitleName("出发接人")
                        view.setButtonName("出发接人")
                        view.setDialogMsg("现在开始出发接人吗？", "出发", "再等等", orderId ?: "", onSiteOperation)
                    }
                    FLAG_BUTTON ->
                        view.setDialogMsg("现在开始出发接人吗？", "出发", "再等等", orderId ?: "", onSiteOperation)
                    FLAG_REFRESH -> {
                        view.setTitleName("出发接人")
                        view.setButtonName("出发接人")
                    }
                    else -> {
                        view.setTitleName("出发接人")
                        view.setButtonName("出发接人")
                    }
                }
            }
            "2" -> {
                when (flag) {
                    FLAG_INTENT -> {
                        view.setTitleName("接人中")
                        view.setButtonName("完成接人")
                    }
                    FLAG_BUTTON ->
                        view.setDialogMsg("确认已完成接人？", "确认", "取消", orderId ?: "", onSiteOperation)
                    FLAG_REFRESH -> {
                        view.setTitleName("接人中")
                        view.setButtonName("完成接人")
                    }
                    else -> {
                        view.setViewFinish()
                        LiveDataBus.get().with(CarEntity.BROADCAST_JPUSH_ORDER_STATUS_CHANGED).postValue(FLAG_REFRESH)
                    }
                }
            }
            else -> {
                view.setViewFinish()
                LiveDataBus.get().with(CarEntity.BROADCAST_JPUSH_ORDER_STATUS_CHANGED).postValue(FLAG_REFRESH)
            }
        }
    }

    private fun setSendCarInfo() {
        view.setAddressTitle("送车地址")
        //根据类型判断
        whenType(onSiteOperation ?: "", FLAG_INTENT)
        when (onSiteOperation) {
            "1", "2" -> {
                if (ordersBean?.location != null) {
                    view.setDeliveryAddress(ordersBean?.address ?: "")
                    view.setLocation(ordersBean?.location!!, ordersBean?.address ?: "")
                    view.setPickUpTime(ordersBean?.start_time ?: "")
                    view.setPickUpTheCar("预计取车时间")
                }
            }
            "3", "4" -> {
                if (ordersBean?.return_location != null) {
                    view.setDeliveryAddress(ordersBean?.return_address ?: "")
                    view.setLocation(ordersBean?.return_location!!, ordersBean?.return_address
                            ?: "")
                    view.setPickUpTime(ordersBean?.end_time ?: "")
                    view.setPickUpTheCar("预计还车时间")
                }
            }
            else -> {
                if (ordersBean?.location != null)
                    view.setLocation(ordersBean?.location!!, ordersBean?.address ?: "")
            }
        }
    }

    private fun whenType(onSiteOperation: String, flag: String) {//  //根据类型判断
        when (onSiteOperation) {
            "1" -> {
                when {
                    FLAG_INTENT == flag -> {
                        view.setTitleName("出发送车")
                        view.setButtonName("出发送车")
                        view.setDialogMsg("现在开始出发送车吗？", "出发", "再等等", orderId ?: "", onSiteOperation
                                ?: "")


                    }
                    FLAG_BUTTON == flag -> view.setDialogMsg("现在开始出发送车吗？", "出发", "再等等", orderId
                            ?: "", onSiteOperation
                            ?: "")
                    FLAG_REFRESH == flag -> {
                        view.setTitleName("出发送车")
                        view.setButtonName("出发送车")
                    }
                    else -> {
                        view.setTitleName("出发送车")
                        view.setButtonName("出发送车")
                    }
                }
            }

            "2" -> {
                when {
                    FLAG_INTENT == flag -> {
                        view.setTitleName("送车中")
                        view.setButtonName("确认送达")
                    }
                    FLAG_BUTTON == flag -> view.setDialogMsg("确认车辆已送达？", "确认", "取消", orderId
                            ?: "", onSiteOperation ?: "")
                    FLAG_REFRESH == flag -> {
                        view.setTitleName("送车中")
                        view.setButtonName("确认送达")
                    }
                    else -> {
                        view.setViewFinish()
                        LiveDataBus.get().with(CarEntity.BROADCAST_JPUSH_ORDER_STATUS_CHANGED).postValue(FLAG_REFRESH)
                    }
                }
            }


            "3" -> {
                when {
                    FLAG_INTENT == flag -> {
                        view.setTitleName("出发取车")
                        view.setButtonName("出发取车")
                        view.setDialogMsg("现在开始出发取车吗？", "出发", "再等等", orderId ?: "", onSiteOperation
                                ?: "")

                    }
                    FLAG_BUTTON == flag -> view.setDialogMsg("现在开始出发取车吗？", "出发", "再等等", orderId
                            ?: "", onSiteOperation
                            ?: "")
                    FLAG_REFRESH == flag -> {
                        view.setTitleName("出发取车")
                        view.setButtonName("出发取车")
                    }
                    else -> {
                        view.setTitleName("出发取车")
                        view.setButtonName("出发取车")
                    }
                }
            }

            "4" -> {
                when {
                    FLAG_INTENT == flag -> {
                        view.setTitleName("取车中")
                        view.setButtonName("确认取车")
                    }
                    FLAG_BUTTON == flag -> view.setDialogMsg("确认已取车？", "确认", "取消", orderId
                            ?: "", onSiteOperation ?: "")
                    FLAG_REFRESH == flag -> {
                        view.setTitleName("取车中")
                        view.setButtonName("确认取车")
                    }
                    else -> {
                        view.setViewFinish()
                        LiveDataBus.get().with(CarEntity.BROADCAST_JPUSH_ORDER_STATUS_CHANGED).postValue(FLAG_REFRESH)
                    }
                }

            }
            else -> {
                view.setViewFinish()
                LiveDataBus.get().with(CarEntity.BROADCAST_JPUSH_ORDER_STATUS_CHANGED).postValue(FLAG_REFRESH)
            }
        }
    }


    override fun onsiteOperation(orderId: String, onsiteOperation: String) {
        val request = SiteOperationRequest()
        request.order_id = orderId
        request.onsite_operation = onsiteOperation.toInt()
        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = mRetrofitService.onsiteOperation(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult?) {
                requestOrderInfo(orderId)
            }

            override fun onError(e: Throwable) {
                super.onError(e)
                view.dismissLoadingDialog()
            }
        })
    }


    override fun confirmedDelivery() {
        if (isOnSiteService == OrdersBean.PICK_UP_PERSON) {
            whenPickType(onSiteOperation ?: "", FLAG_BUTTON)
        } else {
            //根据类型判断
            whenType(onSiteOperation ?: "", FLAG_BUTTON)
        }
    }


    override fun onRenterPhone() {
        if (context is FragmentActivity) {
            DialogUtil.onSiteDial(context as FragmentActivity, ordersBean?.renterid)
        }
    }


    /**
     * 获取订单详情
     * */
    private fun requestOrderInfo(orderId: String) {
        val request = OrderDetailRequest()
        request.uid = UserUtil(context).uid
        request.orderid = orderId
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderDetailResult>(context, this) {
            override fun onSuccess(response: OrderDetailResult) {
                view.dismissLoadingDialog()
//                siteId= response.site?.siteid
                //赋值 刷新订单
                onSiteOperation = response.onsite_operation.toString()

                if (isOnSiteService == OrdersBean.PICK_UP_PERSON) {
                    whenPickType(onSiteOperation ?: "", FLAG_REFRESH)
                } else {
                    //根据类型判断
                    whenType(onSiteOperation ?: "", FLAG_REFRESH)
                }

            }

            override fun onError(e: Throwable) {
                super.onError(e)
                view.dismissLoadingDialog()
            }
        })
    }
}