package com.jiaoyinbrother.monkeyking.manager.mvpactivity.login

import android.arch.lifecycle.Observer
import android.content.Intent
import android.support.v4.content.ContextCompat
import android.text.Editable
import android.text.InputType
import android.text.TextUtils
import android.text.TextWatcher
import android.view.View
import com.alibaba.android.arouter.facade.annotation.Route
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.bean.WeChatBean
import com.jiaoyinbrother.library.bean.WeChatUserInfoResult
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.TextUtil
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.activity.BindPhoneActivity
import com.jiaoyinbrother.monkeyking.manager.activity.IdentificationActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity

import com.jiaoyinbrother.monkeyking.manager.util.DialogUtil
import com.jiaoyinbrother.monkeyking.manager.view.GraphicCodeDialog
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import kotlinx.android.synthetic.main.activity_login.*


@Route(path = "/user/loginactivity")
class LoginActivity : MvpBaseActivity<LoginPresenter>(), LoginContract.View, View.OnClickListener, TextWatcher {

    private var exitFirstTime: Long = 0
    private var picCodeDialog: GraphicCodeDialog? = null

    override fun onDestroy() {
        presenter.unSmsReceiver()

        super.onDestroy()
    }

    override fun initPresenter(): LoginPresenter {
        return LoginPresenter(this, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_login
    }

    override fun initView() {
        onSmsCode.disable()
        onLogin.disable()
    }

    override fun initToolBar() {
        mainTitle.text = "登录"
    }

    override fun initData() {
        presenter.getData(intent)
        presenter.init()
        picCodeDialog = GraphicCodeDialog(this).builder()
        // 语音验证码字体变色
        onVoiceCode.text = TextUtil.setForegroundColor(onVoiceCode.text.toString(),
                ContextCompat.getColor(this@LoginActivity, R.color.color_8), 11, 16)

        LiveDataBus.get().with(ConstantsUtil.WX_AUTH_RESULT).observe(this, Observer {
            presenter.doWeChatAuth(it as WeChatBean)
        })
    }

    override fun initListeners() {
        rb_password.setOnClickListener(this)
        rb_code.setOnClickListener(this)

        etPhone.addTextChangedListener(this)
        etPassword.addTextChangedListener(this)
        etCode.addTextChangedListener(this)

        ll_password_checked.setOnClickListener(this)


        onSmsCode.setOnClickListener(this)
        onVoiceCode.setOnClickListener(this)

        onLogin.setOnClickListener(this)
        onWeChat.setOnClickListener(this)

        picCodeDialog?.setListener(object : GraphicCodeDialog.GraphicCodeListener {
            override fun onCancel() {
                presenter.resetCode()
                picCodeDialog?.dismiss()
            }

            override fun onImgCode() {
                presenter.doPicCode()
            }

            override fun onSure(code: String) {
                picCodeDialog?.dismiss()
                presenter.getPicCode(code)
            }
        })
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.onSmsCode -> presenter.doSmsCode()
            R.id.onVoiceCode -> presenter.doVoiceCode()
            R.id.onLogin -> presenter.doLogin()
            R.id.onWeChat -> presenter.doWeChat()
            R.id.rb_password -> presenter.doPasswordChecked()
            R.id.rb_code -> presenter.doCodeChecked()
            R.id.ll_password_checked -> {
                cbPassword.isChecked = !cbPassword.isChecked

                etPassword.inputType = if (cbPassword.isChecked) InputType.TYPE_TEXT_VARIATION_VISIBLE_PASSWORD else InputType.TYPE_CLASS_TEXT or InputType.TYPE_TEXT_VARIATION_PASSWORD
                val index = etPassword.text?.length ?: 0
                LogUtil.e("etPassword length = $index")
                etPassword.setSelection(index)
            }
        }
    }

    /**
     * 不显示返回
     * @return 默认true显示，false不显示。
     */
    override fun isShowBacking(): Boolean {
        return false
    }



    override fun onBackPressed() {
        //不运行返回。
        val secondTime = System.currentTimeMillis()
        if (secondTime - exitFirstTime > 2000) {
            exitFirstTime = secondTime
            showToast("再按一次退出租车帮·悟空")
        } else {
            LiveDataBus.get().with(ConstantsUtil.LIVEDATA_EXIT_APPLICATION).setValue(true)
            finish()
        }
    }


    override fun afterTextChanged(s: Editable?) {
        presenter.doTextChangeWatcher(s.toString())
    }

    override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
    }

    override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {

    }

    override fun setLoginEnable(b: Boolean) {
        if (b) onLogin.normal() else onLogin.disable()
    }

    override fun setThirdPartyLogin(b: Boolean) {
        thirdLoginLayout.visibility = if (b) View.VISIBLE else View.GONE
    }

    override fun setPasswordShow() {
        llPassword.visibility = View.VISIBLE
        llCode.visibility = View.GONE
    }

    override fun setCodeShow() {
        llPassword.visibility = View.GONE
        llCode.visibility = View.VISIBLE
    }

    override fun getPhone(): String {
        return etPhone.text.toString()
    }

    override fun getCode(): String {
        return etCode.text.toString()
    }

    override fun getPassword(): String {
        return etPassword.text.toString()
    }

    override fun setCodeSmsEnable(b: Boolean) {
        onSmsCode.isEnabled = b
    }

    override fun setCodeVoiceEnable(b: Boolean) {
        onVoiceCode.isEnabled = b
    }

    override fun setCodeFocus() {
        etCode.isFocusable = true
        etCode.isFocusableInTouchMode = true
        etCode.requestFocus()
    }

    override fun setCountDown(s: String) {
        onSmsCode.text = s
    }

    override fun setPicCode(pic: String, msg: String) {
        if (!TextUtils.isEmpty(pic)) {
            picCodeDialog?.setImgCode(pic)
        }
        picCodeDialog?.setErrMsg(msg)
        if (picCodeDialog?.isShowing() == false)
            picCodeDialog?.show()
    }

    override fun setCodeInfo(code: String) {
        etCode.setText(code)
    }

    override fun setErrorTips(tips: String) {
        IOSAlertDialog(this@LoginActivity)
                .builder()
                .setTitle("温馨提示")
                .setMsg(tips, true).setRightButton("确定") {
                }.setLeftButton("联系客服") {
                    DialogUtil.onServiceDial(this@LoginActivity)
                }
                .show()
    }

    override fun setIMParty(flag: String, b: Boolean) {
        if (flag == CarEntity.EXTRA_FLAG_CARDETAIL_NOLOGIN && b) {
            IdentificationActivity.startIdentificationActivity(this, IdentificationActivity.ACTIVITY_TYPE_LOGIN, "")
        } else {
            val intent = Intent()
            intent.putExtra(CarEntity.EXTRA_BUNDLE_KEY, "setResult")
            setResult(RESULT_OK, intent)
        }
        finish()
    }

    override fun setUserBind(result: WeChatUserInfoResult) {
        val intent = Intent(this@LoginActivity, BindPhoneActivity::class.java)
        intent.putExtra("wx_user_info", result)
        intent.putExtra("third_type", CarEntity.ThirdLoginType.WEIXIN.typeName)
        startActivityForResult(intent, 10)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 10) {
            presenter.doBindLogin(data)
        }
    }

}