package com.jiaoyinbrother.monkeyking.manager.mvpactivity.login

import android.content.Context
import android.content.Intent
import com.jiaoyinbrother.monkeyking.manager.util.EMUtil
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit
import android.text.TextUtils
import cn.jpush.android.api.JPushInterface
import com.baidu.mobstat.StatService
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.monkeyking.manager.jpush.MKPushUtil
import com.jiaoyinbrother.monkeyking.manager.push.PushUtil

import com.jiaoyinbrother.monkeyking.manager.util.MKUtil
import com.jiaoyinbrother.monkeyking.manager.util.NetUtil
import com.sensorsdata.analytics.android.sdk.SensorsDataAPI
import io.reactivex.Flowable
import io.reactivex.disposables.Disposable
import org.json.JSONException
import org.json.JSONObject
import java.lang.IllegalStateException


class LoginPresenter(context: Context, view: LoginContract.View) : BasePresenterImpl<LoginContract.View>(context, view), LoginContract.Presenter {

    private var fromFlag: String? = null
    private var entrance: String? = null

    private var currentType = PASSWORD
    private var mPicCode = ""
    private var codeType = SMS
    private var mCountDown: Disposable? = null
    private var mWxResult: WeChatUserInfoResult? = null
    private var mIsNew = false

    override fun getData(intent: Intent) {
        entrance = intent.getStringExtra("entrance") ?: ""
        fromFlag = intent.getStringExtra(CarEntity.EXTRA_BUNDLE_KEY) ?: ""
    }

    fun init() {
        view.setThirdPartyLogin(WeChatUtil(context).isWxInstalled())
    }

    override fun doPasswordChecked() {
        currentType = PASSWORD
        view.setPasswordShow()
        view.setLoginEnable(getLoginPassword())
    }

    override fun doCodeChecked() {
        currentType = CODE
        view.setCodeShow()
        view.setLoginEnable(getLoginCode())
    }

    override fun doTextChangeWatcher(s: String) {
        LogUtil.e("currentType = $currentType")
        view.setCodeSmsEnable(view.getPhone().length == 11)
        if (currentType == CODE) {
            view.setLoginEnable(getLoginCode())
        } else {
            view.setLoginEnable(getLoginPassword())
        }
    }

    override fun doSmsReceiver() {

    }

    override fun unSmsReceiver() {
        mCountDown?.dispose()

    }

    override fun doSmsCode() {
        view.setCodeSmsEnable(false)
        StatisticsUtil.onEvent(context, ConstantsUtil.LOGIN_GET_CODE, ConstantsUtil.LOGIN_GET_CODE)
        sendCode(SMS)
    }

    override fun doVoiceCode() {
        view.setCodeVoiceEnable(false)
        StatisticsUtil.onEvent(context, ConstantsUtil.LOGIN_VOICE_CODE, ConstantsUtil.LOGIN_VOICE_CODE)
        sendCode(VOICE)
    }

    override fun getPicCode(code: String) {
        mPicCode = code
        doPicCode()
    }

    override fun doPicCode() {
        sendCode(codeType)
    }

    fun resetCode() {
        activeCode(codeType)
    }

    override fun doLogin() {
        if (!NetUtil.isNetworkAvailable(context)) {
            view.showToast("网络不给力，请重试!")
            return
        }
        if (TextUtil.checkPhoneNum(context, view.getPhone()))
            return

        if (currentType == CODE && TextUtil.checkCode(context, view.getCode()))
            return

        if (currentType == PASSWORD && TextUtil.checkPassword(context, view.getPassword())) {
            return
        }
        StatisticsUtil.onEvent(context, ConstantsUtil.LOGIN_SUBMIT, "")
        mWxResult = null
        apiLogin(null)
    }


    override fun doWeChat() {
        StatisticsUtil.onEvent(context, ConstantsUtil.LOGIN_WX_LOGIN, "")
        view.showLoadingDialog()
        WeChatUtil(context).weChatLogin()
    }

    private fun sendCode(type: Int) {
        view.setCodeFocus()
        codeType = type
        if (TextUtil.checkPhoneNum(context, view.getPhone())) {
            activeCode(type)
            return
        }
        apiCode(type)
    }

    private fun apiCode(type: Int) {
        val request = CodeRequest()
        request.phone = view.getPhone()
        request.tag = "LOGIN"
        request.type = type
        request.pic_flag = 1
        if (!TextUtils.isEmpty(mPicCode)) {
            request.pic_vcode = mPicCode
        }
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.sendCode(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<CodeResult>(context, this) {

                    override fun onAPIError(apiException: ApiException?) {
                        view.dismissLoadingDialog()
                        activeCode(type)
                        handleSensorsData(false)
                    }

                    override fun onSuccess(response: CodeResult) {
                        view.dismissLoadingDialog()
                        mPicCode = ""
                        if (TextUtils.isEmpty(response.error_tips)) {
                            if (!TextUtils.isEmpty(response.pic) || !TextUtils.isEmpty(response.pic_message)) {
                                view.setPicCode(response.pic ?: "", response.pic_message ?: "")
                            } else {
                                view.showToast("获取验证码成功")
                                doCountDown(type)
                            }
                        } else {
                            activeCode(type)
                            view.setErrorTips(response.error_tips)
                            handleSensorsData(false)
                        }
                    }
                })
    }

    private fun activeCode(type: Int) {
        if (type == SMS)
            view.setCodeSmsEnable(true)
        else
            view.setCodeVoiceEnable(true)
    }

    private fun doCountDown(type: Int) {
        handleSensorsData(true)
        // 倒计时
        mCountDown = Flowable.intervalRange(0, 61, 0, 1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .doOnNext { count -> view.setCountDown("${60 - count}s") }
                .doOnComplete {
                    mCountDown?.dispose()
                    view.setCountDown("重新获取")
                    activeCode(type)
                }
                .subscribe()
    }


    private fun handleSensorsData(success: Boolean) {
        try {
            val properties = JSONObject()
            properties.put("phone_number", TextUtil.formatPhoneNum(view.getPhone()))
            properties.put("is_success", success)
            StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_GETCODE, properties)
        } catch (e: JSONException) {

        }
    }


    private fun apiLogin(result: WeChatUserInfoResult?) {
        val request = LoginRequest()
        request.encrypt = "RSA"
        request.channel = ChannelUtil.getAppChannel(context)
        LogUtil.e("Login channel --->>> ${ChannelUtil.getAppChannel(context)}")
        if (result == null) {
            request.uid = view.getPhone()
            if (currentType == PASSWORD) {
                request.password = RSAEncryptor.encrypt(view.getPassword(), null)
            } else {
                request.vcode = view.getCode()
            }
        } else {
            request.type = CarEntity.ThirdLoginType.WEIXIN.typeName
            request.uid = result.unionid
            request.unionid = result.unionid
            request.openid = result.openid
            if (!TextUtils.isEmpty(result.nickname))
                request.nickname = result.nickname
            if (!TextUtils.isEmpty(result.province))
                request.province = result.province
            if (!TextUtils.isEmpty(result.country))
                request.country = result.country
            if (!TextUtils.isEmpty(result.city))
                request.city = result.city
            if (!TextUtils.isEmpty(result.headimgurl))
                request.headimgurl = result.headimgurl
        }
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.login(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<LoginResult>(context, this) {

                    override fun onAPIError(apiException: ApiException?) {
                        super.onAPIError(apiException)
                        view.dismissLoadingDialog()
                        handleLoginSensordata(false, null)
                    }

                    override fun onSuccess(response: LoginResult) {
                        view.dismissLoadingDialog()
                        handleLoginSensordata(true, response)
                        // 统计使用
                        StatService.setUserId(context, response.id.toString())// 百度统计上传UID
                        SensorsDataAPI.sharedInstance().login(response.id.toString())

                        val userUtil = UserUtil(context)

                        // 登录TOKEN
                        userUtil.login(response.token)

                        userUtil.uid = response.uid
                        userUtil.userId = response.id

                        val json = Util.entityToJson(response)
                        userUtil.loginResultPref = json
                        MKUtil.saveUserInfo(response.name, response.uid, response.id_no)
                        if (AppCacheUtil(context).jPushStatus) {
                            JPushInterface.resumePush(context)
                            MKPushUtil.updateJPush(context)
                        }

                        LiveDataBus.get().with(ConstantsUtil.LIVEDATA_LOGIN).postValue(true);

                        if (!TextUtils.isEmpty(response.name)) {
                            userUtil.userName = response.name
                        } else {
                            userUtil.userName = "${response.uid.substring(0, 3)}****${response.uid.substring(7)}"
                        }
                        userUtil.userRole = response.role
                        userUtil.userRoleName = response.rolename
                        mIsNew = response.isIsnew

                        PushUtil.apiPush(context,this@LoginPresenter,false)
                        //环信
                        apiIMParty()


                    }
                })
    }

    private fun handleLoginSensordata(success: Boolean, result: LoginResult?) {

        val properties = JSONObject()
        properties.put("entrance", entrance)
        properties.put("phone_number", TextUtil.formatPhoneNum(result?.uid))
//        如果isIsNEW没有数据，则先传递0
        properties.put("is_first_time", 0)
        properties.put("is_first_time", result?.isIsnew)
        if (mWxResult == null) {
            properties.put("login_method", "手机号")
        } else {
            properties.put("login_method", "微信")
        }
        properties.put("is_success", success)
        StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_LOGIN, properties)
    }

    private fun apiIMParty() {
        val userUtil = UserUtil(context)
        val request = ImPartyRequest()
        request.user_id = userUtil?.userId?.toString()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getIMParty(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<ImPartyResult>(context, this) {
                    override fun onSuccess(response: ImPartyResult) {
                        view.dismissLoadingDialog()
                        if (response?.rows?.size ?: 0 > 0) {
                            val imPartyBean = response?.rows?.get(0)
                            userUtil.im_username = imPartyBean?.im_username ?: ""
                            userUtil.im_password = imPartyBean?.im_password ?: ""
                            userUtil.im_headimg_url = imPartyBean?.im_headimg_url ?: ""
                            userUtil.im_nickname = imPartyBean?.im_nickname ?: ""
                            EMUtil.instance.loginEM(userUtil.im_username, userUtil.im_password)
                        } else {
                            userUtil.im_username = ""
                            userUtil.im_password = ""
                            userUtil.im_headimg_url = ""
                            userUtil.im_nickname = ""
                        }

                        view.showToast("登录成功")
                        view.setIMParty(fromFlag ?: "", mIsNew)
                    }
                })
    }

    override fun doWeChatAuth(it: WeChatBean) {
        if (it.isSuccess) {
            mWxResult = it.weChatUserInfo
            if (it.weChatUserInfo != null) {
                apiCheckWx(it.weChatUserInfo!!)
            }
        } else {
            view.dismissLoadingDialog()
            view.showToast(FAIL)
        }
    }

    /**
     * 第三方验证
     */
    private fun apiCheckWx(result: WeChatUserInfoResult) {
        val request = WeChatCheckRequest()
        request.openid = result.openid
        request.unionid = result.unionid
        request.type = "ZUCHE_WEIXIN"
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.checkParty(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<WeChatCheckResult>(context, this) {
            override fun onSuccess(response: WeChatCheckResult) {
                view.dismissLoadingDialog()
                if (response.exist > 0) {
                    view.setUserBind(result)
                } else {
                    apiLogin(result)
                }
            }
        })
    }


    override fun doBindLogin(data: Intent?) {
        if (data != null) {
            try {
                mWxResult = data.getSerializableExtra("mWxResult") as WeChatUserInfoResult?
            } catch (e: Exception) {

            }
            apiLogin(mWxResult)
        } else {
            //拉起用户微信信息成功，但是不bindphone----此时，mWxResult清空
            mWxResult = null
        }
    }

    private fun getLoginPassword(): Boolean {
        return view.getPhone().length == 11 && !TextUtils.isEmpty(view.getPassword()) && view.getPassword().length > 7
    }


    private fun getLoginCode(): Boolean {
        return view.getPhone().length == 11 && view.getCode().length == 4
    }




    companion object {
        private const val PASSWORD = 0
        private const val CODE = 1

        private const val SMS = 0
        private const val VOICE = 2

        private const val FAIL = "授权失败"
        private const val SUCC = "授权成功"
    }
}