package com.jiaoyinbrother.monkeyking.manager.mvpactivity.message

import android.arch.lifecycle.Observer
import android.content.Intent
import android.os.Bundle
import android.support.v4.app.FragmentActivity
import android.text.TextUtils
import android.view.View
import android.widget.RadioButton
import com.hyphenate.chat.EMClient
import com.hyphenate.chat.EMConversation
import com.hyphenate.chat.EMMessage
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.activity.MyMessageActivity
import com.jiaoyinbrother.monkeyking.manager.adapter.ConversationAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseFragment
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.systemnotification.SystNotificationActivity
import com.jiaoyinbrother.monkeyking.manager.util.EMUtil
import com.jiaoyinbrother.monkeyking.manager.util.H5Utils
import com.jiaoyinbrother.library.bean.ImInfoBean
import com.jiaoyinbrother.library.bean.UnreadMessagesBean
import com.jiaoyinbrother.library.impl.NotifyMsgClickListener
import com.jiaoyinbrother.library.util.ARouterConstantUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.monkeyking.manager.CarApp
import com.jiaoyinbrother.monkeyking.manager.util.NotifyUtil
import kotlinx.android.synthetic.main.fragment_message.*
import java.util.ArrayList

class MainMessageFragment : MvpBaseFragment<MessagePresenter>(), MessageContract.View {

    private var mConversationAdapter: ConversationAdapter? = null
    private var mConversationsMap: Map<String, EMConversation>? = null
    private var mConversationIdlist: List<String>? = null
    private var fromId = ""

    companion object {
        const val REFRESH_MSG = "REFRESH_MSG"
    }

    override fun initPresenter(): MessagePresenter {
        return MessagePresenter(context!!, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.fragment_message
    }

    override fun initViews(view: View?, savedInstanceState: Bundle?) {

    }

    override fun initToolbar(savedInstanceState: Bundle?) {
    }

    override fun initListeners() {
        mConversationAdapter?.setOnItemClickListener { v, position ->
            LogUtil.e("infoBean -- " + mImInfoList)
            val infoBean = mImInfoList?.get(position)
            EMUtil.instance.startEMChat(activity as FragmentActivity, infoBean)
        }
//        EMUtil.instance.addMessageListener(mIMListener)
        order_msg_ll?.setOnClickListener { v ->
            //订单消息
            val intent = Intent(context, MyMessageActivity::class.java)
            intent.putExtra(MyMessageActivity.MSG_TYPE, MyMessageActivity.MSG_ORDER)
            startActivity(intent)
        }
        system_msg_ll?.setOnClickListener { v ->
            //系统公告
            startActivity(Intent(context, SystNotificationActivity::class.java))
        }
        wk_msg_ll?.setOnClickListener { v ->
            //悟空小美
            H5Utils.contactService(context)
        }
    }

    override fun initData() {

        mConversationAdapter = ConversationAdapter(context!!)
        conversation_rv?.adapter = mConversationAdapter
        LiveDataBus.get().with(REFRESH_MSG).observe(this, Observer {
            refreshMsg()
        })


        LiveDataBus.get().with("MSG_LISTENER").observe(this, Observer {
            val messages = it as List<EMMessage>
            activity?.runOnUiThread {
                NotifyUtil.msgNotify(CarApp.getActivity(), messages)
                setConversations()
            }
        })

        LiveDataBus.get().with("toChat").observeSticky(this, Observer {
            val id = it as String
            fromId = id
            setConversations()
        })


        LiveDataBus.get().with("PUSH_BULLETIN_ID",String::class.java).observeSticky(this, Observer {
            presenter.setPushBulletin(TextUtils.isEmpty(it))
            presenter.requestMessageStatus(it)
        })
    }

    override fun updataFragment() {

    }

    override fun onResume() {
        super.onResume()
        refreshMsg()
    }

    private fun refreshMsg() {
        if (UserUtil(activity as FragmentActivity).isLogin()) {
            presenter?.requestMsgUnread()
        }
        setConversations()
    }

    private fun setConversations() {
        EMUtil.instance.loadAllConversations()
        mConversationsMap = EMClient.getInstance()?.chatManager()?.allConversations
        mConversationIdlist = mConversationsMap?.keys?.toList()
        presenter?.requestImInfos(mConversationIdlist)
        handleNoData()
        if (isHasImMsgUnread()) {
            activity?.findViewById<RadioButton>(R.id.radioButtonMsg)?.setCompoundDrawablesRelativeWithIntrinsicBounds(null, context?.resources?.getDrawable(R.drawable.tab_item_main_bottom_msg_dot), null, null)
        }
    }

    private var mImInfoList: List<ImInfoBean>? = null

    override fun onImInfos(ims: List<ImInfoBean>?) {
        mImInfoList = ims

        mImInfoList = EMUtil.instance.sortImInfo(ims, mConversationsMap)


        mConversationAdapter?.list = mImInfoList
        mConversationAdapter?.mConversationsMap = mConversationsMap
        mConversationAdapter?.notifyDataSetChanged()


        if (!TextUtils.isEmpty(fromId)) {
            toEMChat(fromId)
            fromId = ""
        }
        handleNoData()
    }

    override fun onMsgUnread(unread_messages: ArrayList<UnreadMessagesBean>?) {
        updateMessageStatus(unread_messages)
    }


    private fun handleNoData() {
        if (mConversationAdapter?.itemCount ?: 0 == 0) {
            no_data_view?.visibility = View.VISIBLE
            conversation_rv?.visibility = View.GONE
        } else {
            no_data_view?.visibility = View.GONE
            conversation_rv?.visibility = View.VISIBLE
        }
    }

    private fun updateMessageStatus(unreadMessageList: ArrayList<UnreadMessagesBean>?) {
        order_flag_tv?.visibility = View.GONE
        notification_flag_tv?.visibility = View.GONE
        activities_flag_iv?.visibility = View.GONE
        var isHasNativeMsg = false
        if (unreadMessageList?.size ?: 0 > 0) {
            for (msg in unreadMessageList!!) {
                when (msg?.type) {
                    MyMessageActivity.MSG_ORDER -> {
                        order_flag_tv?.visibility =
                                if (msg?.count > 0) View.VISIBLE else View.GONE
                        order_flag_tv?.text = if(msg?.count > 99) "99+" else msg?.count?.toString()
                        if (msg?.count > 0) isHasNativeMsg = true
                    }
                    MyMessageActivity.BULLETIN -> {
                        notification_flag_tv?.visibility =
                                if (msg?.count > 0) View.VISIBLE else View.GONE
                        notification_flag_tv?.text = if(msg?.count > 99) "99+" else msg?.count?.toString()
                        if (msg?.count > 0) isHasNativeMsg = true
                    }
                }
            }
        }
        var isHasImMsg = isHasImMsgUnread()
        if (isHasNativeMsg || isHasImMsg) {
            activity?.findViewById<RadioButton>(R.id.radioButtonMsg)?.setCompoundDrawablesRelativeWithIntrinsicBounds(null, context?.resources?.getDrawable(R.drawable.tab_item_main_bottom_msg_dot), null, null)
        } else {
            activity?.findViewById<RadioButton>(R.id.radioButtonMsg)?.setCompoundDrawablesRelativeWithIntrinsicBounds(null, context?.resources?.getDrawable(R.drawable.tab_item_main_bottom_msg), null, null)
        }
    }

    private fun isHasImMsgUnread(): Boolean {
        var isHasImMsg = false
        EMUtil.instance.loadAllConversations()
        var onversactionsMap = EMClient.getInstance().chatManager().allConversations
        val conversationList = onversactionsMap?.values?.toList()
        if (conversationList?.size ?: 0 > 0) {
            for (conversation in conversationList!!) {
                if (conversation?.unreadMsgCount ?: 0 > 0) {
                    isHasImMsg = true
                    break
                }
            }
        }
        return isHasImMsg
    }

    private fun toEMChat(id: String) {
        if (mImInfoList == null) {
            LogUtil.e("NotifyUtil toEMChat ------>>>> mImInfoList == null")
            return
        }
        for (entity in mImInfoList!!) {
            if (entity.im_username.equals(id))
                if (mActivity.isAlive){
                    EMUtil.instance.startEMChat(mActivity as FragmentActivity, entity)
                }
        }
    }

}