package com.jiaoyinbrother.monkeyking.manager.mvpactivity.message

import android.content.Context
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.monkeyking.manager.util.DialogUtil
import com.jiaoyinbrother.monkeyking.manager.util.H5Utils
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class MessagePresenter(context: Context, view: MessageContract.View) :
    BasePresenterImpl<MessageContract.View>(context, view), MessageContract.Presenter {

    private var mImInfoList: List<ImInfoBean>? = null
    override fun requestImInfos(imUserList: List<String>?) {

        if (!UserUtil(context).isLogin())
            return
        if (imUserList?.size ?: 0 == 0) {
//            return
            view?.onImInfos(mImInfoList)
        }
        val request = ImInfoRequest()
        request.im_usernames = imUserList
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getIMInfo(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
            object : BaseObserver<ImInfoResult>(context, this) {
                override fun onSuccess(response: ImInfoResult) {
                    view?.dismissLoadingDialog()
                    view?.onImInfos(response.ims)

                }

            })
    }

    private var isDialogShowed = false

    override fun requestMsgUnread() {

        LogUtil.e("Lyy push requestMsgUnread start")

        val request = MsgUnreadRequest()
        request.user_id = UserUtil(context).userId?.toString()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getMsgUnread(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
            object : BaseObserver<MsgUnreadResult>(context, this) {
                override fun onSuccess(response: MsgUnreadResult) {
                    view?.dismissLoadingDialog()
                    view?.onMsgUnread(response?.unread_messages)
                    LogUtil.e("Lyy push pop_up_window start")
                    if (response?.pop_up_window != null && !isDialogShowed && isPushBulletin) {
                        LogUtil.e("Lyy push pop_up_window show")
                        IOSAlertDialog(context)
                            .builder()
                            .setTitle(response?.pop_up_window?.title)
                            .setMsg(response?.pop_up_window?.content, true)
                            .setLeftButton("我知道了") {
                                isDialogShowed = true
                            }
                            .setRightButton("立即查看") {
                                if (!TextUtils.isEmpty(response?.pop_up_window?.url)) {
                                    isDialogShowed = true
                                    requestMessageStatus(response.pop_up_window?.bulletin_id ?: "")
                                    H5Utils.onCommon(context, "", response?.pop_up_window?.url)
                                }
                            }
                            .show()
                    }
                }

            })
    }


    //修改消息状态
    fun requestMessageStatus(id: String?) {
        val request = MessageStatusRequest()
        request.user_id = UserUtil(context).userId.toString()
        request.bulletin_id = id
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.messageStatusRead(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
            object : BaseObserver<BaseResult>(context, this) {
                override fun onSuccess(response: BaseResult) {
                }

                override fun onAPIError(apiException: ApiException?) {
                }
            })
    }

    private var isPushBulletin = true

    fun setPushBulletin(b: Boolean) {
        LogUtil.e("Lyy push setPushBulletin boolean == $b")
        isPushBulletin = b
    }
}