package com.jiaoyinbrother.monkeyking.manager.mvpactivity.monkeymain

import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Handler
import android.os.Message
import android.support.v4.app.FragmentActivity
import android.text.TextUtils

import com.jiaoyinbrother.monkeyking.manager.util.CheckVersionUtil
import com.jiaoyinbrother.monkeyking.manager.util.SharedPreferencesUtil
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.monkeyking.manager.push.PushUtil
import com.jiaoyinbrother.monkeyking.manager.util.DialogUtil

import java.util.concurrent.TimeUnit

import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers

/**
 * Created by zhangbo on 2018/6/26.
 */

class MonkeyMainPresenter(context: Context, view: IMonkeyMainView.View) : BasePresenterImpl<IMonkeyMainView.View>(context, view), IMonkeyMainView.Presenter {

    private var alreadyCheckVersion: Boolean = false
    private var isCheckingVersion: Boolean = false


    override fun doPushInit() {
        if (UserUtil(context).isLogin()){
            PushUtil.apiPush(context,this,false)
        }
    }

    override fun requestAdBanner() {
        val request = MainFragmentRequest()
        request.city_id = 110000
        request.publish_channel = "APP"

        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getAdBanner(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe { }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<CityConfigBean>(context, this) {
                    override fun onSuccess(response: CityConfigBean?) {
                        if (response != null && response.share != null) {
                            if (!TextUtils.isEmpty(response.share.url)) {
                                SharedPreferencesUtil.getInstance().saveShareUrl(response.share.url)
                            }
                        }
                    }
                })
    }


    fun checkVersion() {
        if (!alreadyCheckVersion && !isCheckingVersion) {
            isCheckingVersion = true
            requestVersion()
        }
    }


    override fun requestVersion() {
        val request = CheckVersionUtil(context).getCheckVersionEntity()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.checkVersion(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe { }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<CheckVersionResult>(context, this) {
                    override fun onSuccess(response: CheckVersionResult) {
                        isCheckingVersion = false
                        if (CheckVersionUtil(context).checkVersion(response)) {
                            if (response.android != null) {
                                view.setVersionUpdateDialog(response.android)
                            }
                        }
                    }

                    override fun onAPIError(apiException: ApiException?) {
                        isCheckingVersion = false
                        super.onAPIError(apiException)
                    }

                    override fun onError(e: Throwable) {
                        isCheckingVersion = false
                        super.onError(e)
                    }
                })
    }

    private var mProgressReceiver: ProgressReceiver? = null

    fun registerCheckVersionReceiver() {
        mProgressReceiver = ProgressReceiver()
        val filter = IntentFilter()
        filter.addAction(ConstantsUtil.ACTION_DOWNLOAD_PROGRESS)
        filter.addAction(ConstantsUtil.ACTION_DOWNLOAD_COMPOLETED)
        context.registerReceiver(mProgressReceiver, filter)
    }


    private inner class ProgressReceiver : BroadcastReceiver() {

        override fun onReceive(context: Context, intent: Intent) {
            val action = intent.action
            when (action) {
                ConstantsUtil.ACTION_DOWNLOAD_PROGRESS -> {
                    val total = intent.getLongExtra(ConstantsUtil.EXTRA_TOTAL, 0)
                    val current = intent.getLongExtra(ConstantsUtil.EXTRA_CURRENT, 0)
                    mHandler.sendMessage(mHandler.obtainMessage(MSG_UPDATE_PROGRESS, total.toInt(), current.toInt()))
                }
                ConstantsUtil.ACTION_DOWNLOAD_COMPOLETED -> {
                    val result = intent.getIntExtra(ConstantsUtil.EXTRA_RESULT, -1)
                    CheckVersionUtil(context).handleDownloadComplete(intent)
                    mHandler.sendMessage(mHandler.obtainMessage(MSG_DOWNLOAD_COMPLETED, result))
                }
            }
        }
    }

    private val mHandler = @SuppressLint("HandlerLeak")
    object : Handler() {

        override fun handleMessage(msg: Message) {
            when (msg.what) {
                ConstantsUtil.DESTROY -> removeMessages(MSG_UPDATE_PROGRESS)
                MSG_UPDATE_PROGRESS -> {
                    //升级进度条开启，关闭loading
                    view.dismissLoadingDialog()
                    val total = msg.arg1 / 1000
                    val current = msg.arg2 / 1000
                    var progress = 0
                    if (total > 0) {
                        if (total > current)
                            progress = (current * 100 / total)
                    }
                    view.setUpdateProgress(progress)
                }
                MSG_DOWNLOAD_COMPLETED -> {
                    view.dismissLoadingDialog()
                    view.showToast("下载成功")
                    view.setDownloadCompleted()
                }
                else -> {
                }
            }
        }
    }


    fun onDestroy() {
        context.unregisterReceiver(mProgressReceiver)
    }

    companion object {
        private const val MSG_UPDATE_PROGRESS = 0x30
        private const val MSG_DOWNLOAD_COMPLETED = 0x31
    }
}
