package com.jiaoyinbrother.monkeyking.manager.mvpactivity.notrent

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.DateTimeUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.widget.BottomDialog
import com.monkey.datetimelibrary.entity.NoRentDateEntity
import com.monkey.datetimelibrary.entity.NoRentMonthEntity
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.*
import java.util.concurrent.TimeUnit
import kotlin.collections.ArrayList

class NotRentPresenter(context: Context, view: INotRentContract.View)
    : BasePresenterImpl<INotRentContract.View>(context, view), INotRentContract.Presenter {

    private var mCarID: String? = null
    private var reason = arrayOf("可租", "维修保养", "车辆出险", "车辆借调", "线下租出", "其他")

    override fun getData(intent: Intent) {
        mCarID = intent.getStringExtra(ConstantsUtil.CAR_ID)
        requestNoRent()
    }

    override fun requestNoRent() {
        val request = NotRentRequest()
        request.carid = mCarID
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getNoRent(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<NotRentResult>(context, this) {
            override fun onSuccess(response: NotRentResult) {
                view.dismissLoadingDialog()
                if (response.unavailables != null && response.unavailables?.size ?: 0 > 0) {
                    view.setNoRentDate(response.unavailables ?: ArrayList())
                }
            }
        })
    }

    override fun showReason(sd: Date, ed: Date?) {
        BottomDialog(context)
                .builder()
                .addItems(reason)
                .setCancel("取消") {
                    view.setNoRentCancel()
                }
                .setItemClick { _, index ->
                    view.setActiveSave(true)
                    view.setNoRentReason(index, sd, ed)
                }
                .show()
    }

    fun updateNoRent(monthList: ArrayList<NoRentMonthEntity>) {

        requestUpdateNoRent(monthList)
    }

    override fun requestUpdateNoRent(monthList: ArrayList<NoRentMonthEntity>) {
        val request = NotRentUpdateRequest()
        request.carid = mCarID
        request.unavailables = getUnavailableList(monthList)
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.updataNoRent(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult) {
                view.dismissLoadingDialog()
                view.showToast("保存成功")
                view.setNoRentSuccess()
                LogUtil.e("requestUpdateNoRent : $response")
            }
        })
    }

    private fun getUnavailableList(monthList: ArrayList<NoRentMonthEntity>): ArrayList<UnavailableBean> {
        val unavailableList = ArrayList<UnavailableBean>()
        var unavailableBean: UnavailableBean? = null
        var code = 0

        for (i in monthList.indices) {
            val monthEntity = monthList[i]
            val dateList = monthEntity.list ?: ArrayList()
            for (j in dateList.indices) {
                val dateEntity = dateList[j]

                if (code != dateEntity.code && dateEntity.type == NoRentDateEntity.DATE_TYPE_NORMAL) {
                    code = dateEntity.code
                    if (code != UnavailableBean.FREE && code != UnavailableBean.ONLINE) {
                        unavailableBean = UnavailableBean()

                        unavailableList.add(unavailableBean)
                    } else {
                        continue
                    }
                }

                if (dateEntity.code == UnavailableBean.FREE || code == UnavailableBean.ONLINE) {
                    continue
                }

//                if (dateEntity.code == UnavailableBean.ONLINE) {
//                    unavailableBean?.orderid = dateEntity.orderId
//                }
                LogUtil.e("setCode1111 =====>>>>> ${dateEntity.code}")
                if (dateEntity.code != UnavailableBean.ONLINE) {

                    unavailableBean?.code = dateEntity.code
                    LogUtil.e("setCode =====>>>>> ${dateEntity.code}")
                }

                if (TextUtils.isEmpty(unavailableBean?.from_time)) {
                    unavailableBean?.from_time = DateTimeUtil.dateToString(monthEntity.year, monthEntity.month, dateEntity.date, 0)
                }

                unavailableBean?.to_time = DateTimeUtil.dateToString(monthEntity.year, monthEntity.month, dateEntity.date, 1)

            }
            LogUtil.e("unavailableList --->>> 111 : $unavailableList")
        }
        LogUtil.e("unavailableList ---->$unavailableList")
        return unavailableList
    }


}