package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.delivercar.fragments.contract

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.activity.FaDDWebViewActivity
import com.jiaoyinbrother.monkeyking.manager.activity.GeneralWebViewActivity
import com.jiaoyinbrother.monkeyking.manager.view.EditCarNumDialog
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.monkeyking.manager.view.TakeAlsoCarTimeLine
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class DeliverCarContractPresenter(context: Context, view: IDeliverCarContractModel.View)
    : BasePresenterImpl<IDeliverCarContractModel.View>(context, view), IDeliverCarContractModel.Presenter {

    private var mOrderInfo: OrderDetailResult? = null
    private var mDeleteType: Int = ContractPreRequest.DELETE
    private var mUserIDNum: String? = null
    private var mContractState: String? = null
    private var mCarNum: String? = null
    private var mOrderExecuteRequest: OrderExecuteRequest? = null
    private var isSignSuccess = false

    override fun getOrderInfo(result: OrderDetailResult) {
        mOrderInfo = result
        mCarNum = result.number
        initTimeLine(result)
    }

    override fun initPreContract() {
        mDeleteType = ContractPreRequest.DELETE
        apiUserDetail()
    }

    override fun getOrderExecuteRequest(request: OrderExecuteRequest) {
        this.mOrderExecuteRequest = request
    }

    override fun doContractSuccess() {
        isSignSuccess = true
        mDeleteType = ContractPreRequest.RETAIN
        apiPreContract()
    }

    override fun onBackPressedSupport() {
        if (mOrderInfo != null && mOrderInfo?.channel == "schoolapp") {
            view?.setSchoolBack()
        } else {
            view.setNormalBack()
        }
    }

    override fun doContractSignBack() {
        mDeleteType = ContractPreRequest.RETAIN
        apiPreContract()
    }

    override fun apiUserDetail() {
        if (TextUtils.isEmpty(mOrderInfo?.renterid))
            return
        val request = UserDetailRequestBean()
        request.uid = mOrderInfo?.renterid
        request.public_classification = "N"
        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = mRetrofitService.userGetDetail(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<UserDetailResult>(context, this) {
            override fun onSuccess(response: UserDetailResult) {
                val member = mOrderInfo?.member
                // renter id card
                mUserIDNum = if (TextUtils.isEmpty(member?.id_card_no))
                    response.id_no ?: ""
                else
                    member?.id_card_no ?: ""
                apiPreContract()
            }

            override fun onAPIError(apiException: ApiException?) {
                super.onAPIError(apiException)
                view?.dismissLoadingDialog()
                view.setErrorPager(true)
            }

            override fun onError(e: Throwable) {
                view?.dismissLoadingDialog()
                view.setErrorPager(true)
            }
        })
    }

    override fun apiPreContract() {
        val request = ContractPreRequest()
        request.order_id = mOrderInfo?.orderid
        request.user_id = UserUtil(context).userId.toString()
        request.id_card_no = mUserIDNum
        request.contract_type = ContractPreRequest.RENT_CONTRACT
        request.delete_contract = mDeleteType
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.preContract(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<ContractPreResult>(context, this) {
            override fun onSuccess(response: ContractPreResult) {
                view?.dismissLoadingDialog()
                view.setErrorPager(false)
                mContractState = response.contract_sign_status
                // 交车信息至合同页面下一步时mDeleteType == 1，从法大大页面返回时mDeleteType == 0，mDeleteType后台是否删除合同：0否1是
                view.setContractState(mContractState ?: "", mDeleteType)
            }

            override fun onAPIError(apiException: ApiException?) {
                super.onAPIError(apiException)
                view?.dismissLoadingDialog()
                view.setErrorPager(true)
            }

            override fun onError(e: Throwable) {
                view?.dismissLoadingDialog()
                view.setErrorPager(true)
            }
        })
    }

    override fun apiSignContract(mSignType : String?) {
        val request = ContractSignRequest()
        request.orderid = mOrderInfo?.orderid
        if (!TextUtils.isEmpty(mCarNum)) {
            request.car_number = mCarNum
        }

        if (!TextUtils.isEmpty(mSignType)){

            request.sign_type = mSignType
        }else {

            request.sign_type = "SMS"
        }

        request.user_id = UserUtil(context).userId.toString()
        request.contract_type = ContractPreRequest.RENT_CONTRACT
        request.fuel = mOrderExecuteRequest?.fuel
        request.miles = mOrderExecuteRequest?.miles.toString()
        request.fuel_url = mOrderExecuteRequest?.fuel_url
        request.front_url = mOrderExecuteRequest?.front_url
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.signContract(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<ContractSignResult>(context, this) {
            override fun onSuccess(response: ContractSignResult) {
                view?.dismissLoadingDialog()
                if (!TextUtils.isEmpty(response.sign_url)) {

                    val webViewConfigEntity = WebViewConfigEntity()
                    webViewConfigEntity.title = "签署文件"
                    webViewConfigEntity.url = response.sign_url
                    webViewConfigEntity.isNeedProgressBar = true
                    webViewConfigEntity.isNeedShareButton = false
                    webViewConfigEntity.imageUrl = ""
                    webViewConfigEntity.description = ""

                    if (mSignType.equals("FACE")){

                        val intent = Intent(context, FaDDWebViewActivity::class.java)
                        intent.putExtra(CarEntity.FEATURE_ITEM, webViewConfigEntity)
                        context.startActivity(intent)
                    }else{
                        val intent = Intent(context, GeneralWebViewActivity::class.java)
                        intent.putExtra(CarEntity.FEATURE_ITEM, webViewConfigEntity)
                        context.startActivity(intent)
                    }
                }
            }
        })
    }

    override fun doContractElectronic(mSignType : String?) {
        IOSAlertDialog(context)
                .builder()
                .setCancelable(false)
                .setTitle("信息核对提醒")
                .setMsg("请您核对车牌号：${mOrderInfo?.number}，和当前车辆车牌号是否一致，以免产生风险", true)
                .setRightButton("一致") {
                    apiSignContract(mSignType)
                }.setLeftButton("不一致") {
                    diffCarNum(mSignType,mOrderInfo?.number ?: "")
                }.show()
    }

    override fun doContractSubmit() {
        when (mContractState) {
            ContractPreResult.SUCCESS -> {
                // 签署成功 直接确认交车
                if (mOrderExecuteRequest != null) {
                    mOrderExecuteRequest?.real_car_number = mCarNum
                    apiOrderExecute(mOrderExecuteRequest!!)
                }
            }
            ContractPreResult.SIGN_FAILED,
            ContractPreResult.CERTIFICATION_FAILED -> {
                // 签署失败 & 认证失败 跳转最后一个页面
                if (mOrderInfo != null && mOrderInfo?.channel == "schoolapp") {
                    view.setSchoolContractPaper()
                } else {
                    view.setNormalContractPaper()
                }
            }
            else -> {
                // 未签署 跳转最后一个页面
                if (mOrderInfo != null && mOrderInfo?.channel == "schoolapp") {
                    view.setSchoolContractPaper()
                } else {
                    view.setNormalContractPaper()
                }
            }
        }
    }

    /**
     * 设置引导标题
     * @param result
     */
    private fun initTimeLine(result: OrderDetailResult) {
        if (result.school_roll != null) {
            // 0 否  1 是
            if (result.e_contract_status == 0) {
                view.setTimeLine(TakeAlsoCarTimeLine.DELIVER_SCHOOL, TakeAlsoCarTimeLine.INDEX_FOUR)
            } else {
                view.setTimeLine(TakeAlsoCarTimeLine.DELIVER_CONTRACT_SCHOOL, TakeAlsoCarTimeLine.INDEX_FOUR)
            }
        } else {
            if (result.e_contract_status == 0) {
                view.setTimeLine(TakeAlsoCarTimeLine.DELIVER_NORMAL, TakeAlsoCarTimeLine.INDEX_THREE)
            } else {
                view.setTimeLine(TakeAlsoCarTimeLine.DELIVER_CONTRACT_NORMAL, TakeAlsoCarTimeLine.INDEX_THREE)
            }
        }
    }

    /**
     * 车牌号不一致弹窗
     * @param num 车牌号
     */
    private fun diffCarNum(mSignType : String?,num: String) {
        EditCarNumDialog(context)
                .builder()
                .setOldCarNum(num)
                .setFinish {
                    mCarNum = it
                    apiSignContract(mSignType)
                }
                .show()
    }

    /**
     * 直接确认交车
     * @param request
     */
    private fun apiOrderExecute(request: OrderExecuteRequest) {
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.orderExecute(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult?) {
                view?.dismissLoadingDialog()
                view?.showToast("提交成功")
                view?.setActivityFinish()
            }
        })
    }
}
