package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.delivercar.fragments.first

import android.content.Context
import android.content.Intent
import android.support.v4.app.Fragment
import android.text.TextUtils
import android.view.View
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.util.MKUtil
import com.jiaoyinbrother.monkeyking.manager.util.RxTimer
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.codepay.QRCodePayActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.codepay.QRCodePayPresenter
import com.jiaoyinbrother.monkeyking.manager.view.TakeAlsoCarTimeLine
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.io.Serializable
import java.util.*
import java.util.concurrent.TimeUnit


class DeliverCarFirstPresenter(context: Context, view: IDeliverCarFirstContract.View)
    : BasePresenterImpl<IDeliverCarFirstContract.View>(context, view), IDeliverCarFirstContract.Presenter {


    private var mOrderResult: OrderDetailResult? = null
    private var orderID: String? = null
    private var carDepositAmount: Float? = null
    private var mScanPayMethods: ArrayList<PayMethodAvaiBean>? = null
    private var rxTimer: RxTimer? = null
    private var isTimeStarted = false

    override fun getOrderInfo(result: OrderDetailResult) {
        LogUtil.e("DeliverCarFirstPresenter,getOrderInfo, result = $result")

        mOrderResult = result

        initTimeLine(result)

        initCarNum(result)

        initDeposit(result)

        view.LoadDataDone()
    }

    override fun isStudent(): Boolean {
        LogUtil.e("isStudent = " + (mOrderResult != null && mOrderResult?.channel == "schoolapp"))
        return mOrderResult != null && mOrderResult?.channel == "schoolapp"
    }

    private fun initTimeLine(result: OrderDetailResult) {
        if (result.school_roll != null) {
//            0 否  1 是
            if (result.e_contract_status == 0) {
                view.setTimeLine(TakeAlsoCarTimeLine.DELIVER_SCHOOL, TakeAlsoCarTimeLine.INDEX_ONE)
            } else {
                view.setTimeLine(TakeAlsoCarTimeLine.DELIVER_CONTRACT_SCHOOL, TakeAlsoCarTimeLine.INDEX_ONE)
            }
        } else {
            if (result.e_contract_status == 0) {
                view.setTimeLine(TakeAlsoCarTimeLine.DELIVER_NORMAL, TakeAlsoCarTimeLine.INDEX_ZERO)
            } else {
                view.setTimeLine(TakeAlsoCarTimeLine.DELIVER_CONTRACT_NORMAL, TakeAlsoCarTimeLine.INDEX_ZERO)
            }
        }
    }

    private fun initCarNum(result: OrderDetailResult) {
        orderID = result.orderid
        // 订单号
        view.setOrderNum("订单号：${result.getOrderShowID()}")
        // 订单渠道
        if (TextUtils.isEmpty(result.channel_name)) {
            view.setOrderChannel("")
        } else {
            view.setOrderChannel("(${result.channel_name})")
        }
        // 车牌号
        view.setCarNum(result.number ?: "")
    }


    private fun initDeposit(result: OrderDetailResult) {
        if (isStudent()) {
            refreshStudentUI(result)
        } else {
            refreshNormalUI(result)
        }
    }


    private fun refreshStudentUI(result: OrderDetailResult) {
        LogUtil.e("refreshStudentOnlyUI, credit_auth_code = " + mOrderResult?.credit_auth_code)
        val carDeposit = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_VEHICLE) ?: return
        setVehiclePay(carDeposit)

        val illDeposit = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_ILLEGAL) ?: return
        setIllegalPay(illDeposit)

        when (mOrderResult?.credit_auth_code) {
            OrderDetailResult.STUDENT_AUTH_UNKNOWN -> {
                // 显示刷新状态，开始定时刷新
                handleStudentAuthUnknown(carDeposit, illDeposit)
            }
            OrderDetailResult.STUDENT_AUTH_SUCCESS -> {
                handleStudentAuthSuccess(carDeposit, illDeposit)
            }
            OrderDetailResult.STUDENT_AUTH_FAIL -> {
                handleStudentAuthFail(carDeposit, illDeposit)
            }
        }
    }

    private fun setVehiclePay(deposit: DepositBean) {
        carDepositAmount = deposit.amount
        // 车辆押金金额
        view.setDepositCostForCar(deposit.type_name
                ?: "", MoneyUtil().getMoneyText(deposit.amount.toString()))
        // 车辆押金描述
        view.setDepositDescForCar(deposit.type_description ?: "")
    }

    private fun setIllegalPay(deposit: DepositBean) {
        LogUtil.e("illegalDeposit =$deposit")
        val payAmount = deposit.amount
        // 违章押金金额
        view.setDepositCostForIllegal(deposit.type_name
                ?: "", MoneyUtil().getMoneyText(payAmount.toString()))
        // 违章押金描述
        view.setDepositDescForIllegal(deposit.type_description ?: "")
    }

    //当未获取到芝麻信用授权结果时，
    // 1. loding提示：显示
    //    2. 取消订单：不显示
    //    3. 车辆押金置灰
    //    4. 违章押金操作置灰
    private fun handleStudentAuthUnknown(carDeposit: DepositBean, illDeposit: DepositBean) {
        LogUtil.e("handleStudentAuthUnknown")
        // 显示刷新状态，开始定时刷新
        view.showStudentAuthStatusView(true)
        view.showStudentCancelOrderCheckBox(true)

        setStudentButtonState(carDeposit, illDeposit)

        view.enableNext(false)
    }

    //    授权成功，车辆押金、违章押金状态为“支付宝预授权已冻结”。
    //    授权失败，
    //    1. loding提示： 不显示
    //    2. 取消订单，不显示
    //    3. 车辆押金无
    //    4. 违章押金无
    //    5. 取消timer
    private fun handleStudentAuthSuccess(carDeposit: DepositBean, illDeposit: DepositBean) {
        LogUtil.e("handleStudentAuthSuccess")
        view.showStudentAuthStatusView(false)
        view.showStudentCancelOrderCheckBox(false)

        setStudentButtonState(carDeposit, illDeposit)

        cancelTimer()
    }

    //    授权失败，
    //    1. loding提示： 不显示
    //    2. 取消订单，显示 --- 已经支付,不显示;未支付,显示
    //    3. 车辆押金点亮可操作
    //    4. 违章押金操作置灰
    //    5. next 已经支付,enable, 未支付, disable,
    private fun handleStudentAuthFail(carDeposit: DepositBean, illDeposit: DepositBean) {
        LogUtil.e("handleStudentAuthFail")
        view.showStudentAuthStatusView(false)

        setStudentButtonState(carDeposit, illDeposit)
//        //车辆
//        val deposit = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_VEHICLE) ?: return
//
//        when (deposit.status_code) {
//            DepositBean.DEPOSIT_PAYED,
//            DepositBean.DEPOSIT_REFUNDED -> {
//                // 已支付 | 退款
//                view.setDepositForCarOffline(View.GONE, false)
//                view.setDepositForCarScan(View.GONE, false)
//                view.setDepositForCarFreed(View.GONE, false)
//                view.showStudentCancelOrderCheckBox(false)
//                view.enableNext(true)
//            }
//
//            DepositBean.DEPOSIT_WAIT_PAYED -> {
//                // 未支付
//                view.setDepositForCarOffline(View.VISIBLE, true)
//                view.setDepositForCarScan(View.VISIBLE, true)
//                view.setDepositForCarFreed(View.VISIBLE, true)
//                view.showStudentCancelOrderCheckBox(true)
//                view.enableNext(false)
//            }
//        }
//
//        val illegalDeposit = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_ILLEGAL)
//        // 违章押金支付状态
//        when (illegalDeposit?.status_code) {
//            DepositBean.DEPOSIT_PAYED,
//            DepositBean.DEPOSIT_REFUNDED -> {
//                // 已支付 | 退款
//                view.setDepositForIllegalOffline(View.GONE, false)
//                view.setDepositForIllegalScan(View.GONE, false)
//                view.setDepositIllegalFreed(View.GONE, false)
//                view.setDepositForIllegalState(View.VISIBLE, illegalDeposit.status_name ?: "")
//            }
//            DepositBean.DEPOSIT_WAIT_PAYED -> {
//                // 未支付
//                //违章
//                view.setDepositForIllegalOffline(View.VISIBLE, false)
//                view.setDepositForIllegalScan(View.VISIBLE, false)
//                view.setDepositIllegalFreed(View.VISIBLE, false)
//                view.setDepositForIllegalState(View.VISIBLE, "")
//            }
//        }
        cancelTimer()
    }


    private fun setStudentButtonState(carDeposit: DepositBean, illDeposit: DepositBean) {

        // 1.3.2与产品最后沟通，校园订单 三个按钮判断放开，全展示，点击后是否可支付由后台拦截，前台不做处理
        // 车辆押金支付状态
        when (carDeposit.status_code) {
            DepositBean.DEPOSIT_PAYED,
            DepositBean.DEPOSIT_REFUNDED -> {
                // 已支付 | 退款
                view.setDepositForCarOffline(View.GONE, false)
                view.setDepositForCarScan(View.GONE, false)
                view.setDepositForCarFreed(View.GONE, false)
                view?.setDepositStateForCar(View.VISIBLE, carDeposit.status_name ?: "")
                view.showStudentCancelOrderCheckBox(false)
                view.enableNext(true)
            }
            DepositBean.DEPOSIT_WAIT_PAYED -> {
                // 未支付
                view.setDepositForCarOffline(View.VISIBLE, true)
                view.setDepositForCarScan(View.VISIBLE, true)
                view.setDepositForCarFreed(View.VISIBLE, true)
                view?.setDepositStateForCar(View.VISIBLE, "")
                view.showStudentCancelOrderCheckBox(true)
                view.enableNext(false)
                mScanPayMethods = carDeposit.pay_method_avai
            }
        }
        // 违章押金支付状态
        when (illDeposit.status_code) {
            DepositBean.DEPOSIT_PAYED,
            DepositBean.DEPOSIT_REFUNDED -> {
                // 已支付 | 退款
                view.setDepositForIllegalOffline(View.GONE, false)
                view.setDepositForIllegalScan(View.GONE, false)
                view.setDepositIllegalFreed(View.GONE, false)
                view.setDepositForIllegalState(View.VISIBLE, illDeposit.status_name ?: "")
            }
            DepositBean.DEPOSIT_WAIT_PAYED -> {
                // 未支付
                view.setDepositForIllegalOffline(View.VISIBLE, false)
                view.setDepositForIllegalScan(View.VISIBLE, false)
                view.setDepositIllegalFreed(View.VISIBLE, false)
                view.setDepositForIllegalState(View.VISIBLE, "")
            }
        }
    }

    private fun refreshNormalUI(result: OrderDetailResult) {
        view.setDepositForCarFreed(View.GONE, false)
        view.setDepositIllegalFreed(View.GONE, false)

        setNormalVehiclePay(result)
        setNormalIllegalPay(result)
    }

    private fun setNormalVehiclePay(result: OrderDetailResult) {
        val deposit = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_VEHICLE) ?: return
        setVehiclePay(deposit)

        // 车辆押金支付状态
        when (deposit.status_code) {
            DepositBean.DEPOSIT_PAYED,
            DepositBean.DEPOSIT_REFUNDED -> {
                // 已支付 | 退款
                view.setDepositForCarOffline(View.GONE, false)
                view.setDepositForCarScan(View.GONE, false)
                view?.setDepositStateForCar(View.VISIBLE, deposit.status_name ?: "")
                view?.enableNext(true)
            }

            DepositBean.DEPOSIT_WAIT_PAYED -> {
                // 未支付
                view.setDepositForCarOffline(if (showOffline(deposit)) View.VISIBLE else View.GONE, deposit.enabled == 1)

                view.setDepositForCarScan(if (showScan(deposit)) View.VISIBLE else View.GONE, deposit.enabled == 1)

                view.setDepositStateForCar(View.GONE, "")

                mScanPayMethods = deposit.pay_method_avai

                view.enableNext(false)
            }
        }
    }

    private fun setNormalIllegalPay(result: OrderDetailResult) {
        // 违章押金
        val deposit = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_ILLEGAL) ?: return
        setIllegalPay(deposit)

        // 违章押金支付状态
        when (deposit.status_code) {
            DepositBean.DEPOSIT_PAYED,
            DepositBean.DEPOSIT_REFUNDED -> {
                // 已支付 | 退款
                view.setDepositForIllegalOffline(View.GONE, false)
                view.setDepositForIllegalScan(View.GONE, false)
                view.setDepositForIllegalState(View.VISIBLE, deposit.status_name ?: "")
            }

            DepositBean.DEPOSIT_WAIT_PAYED -> {
                // 未支付
                view.setDepositForIllegalOffline(if (showOffline(deposit)) View.VISIBLE else View.GONE, deposit.enabled == 1)

                view.setDepositForIllegalScan(if (showScan(deposit)) View.VISIBLE else View.GONE, deposit.enabled == 1)

                view.setDepositForIllegalState(View.GONE, "")
                mScanPayMethods = deposit.pay_method_avai
            }
        }
    }

    private fun showOffline(deposit: DepositBean): Boolean {
        return deposit.offline_button?.show == 1
    }

    private fun showScan(deposit: DepositBean): Boolean {
        return deposit.scan_button?.show == 1
    }

    override fun startTimer() {
        LogUtil.e("DeliverCarFirstPresenter startTimer, isTimeStarted = $isTimeStarted")
        if (!isStudent() || mOrderResult == null)
            return
//        credit_auth_code --- 是否免押判断
//信用免押code（校园APP必返，0：未申请、1：免押通过、2：免押失败
//        0， 显示loading
//        2.  失败
//        1， 通过
        view.showRefreshButton(true)
        if (mOrderResult!!.credit_auth_code == 0) {
            if (isTimeStarted)
                return
            LogUtil.e("DeliverCarFirstPresenter rxTimer?.interval *********************** started")
            isTimeStarted = true
            rxTimer = RxTimer()
            rxTimer?.interval(10000) {
                LogUtil.e("DeliverCarFirstPresenter rxTimer?.interval ***********************8")
                if (isTimeStarted) {
                    LogUtil.e("DeliverCarFirstPresenter rxTimer?.interval ************** isTimeStarted = $isTimeStarted")
                    requestAPIOrderInfo()
                }
            }
        } else {
            //获取状态，停止timer
            cancelTimer()
        }
    }

    override fun cancelTimer() {
        isTimeStarted = false
        LogUtil.e("DeliverCarFirstPresenter cancelTimer")
        rxTimer?.cancel()
    }

    private fun requestAPIOrderInfo() {
        val request = OrderDetailRequest()
        request.uid = UserUtil(context).uid
        request.orderid = mOrderResult?.orderid
        if (TextUtils.isEmpty(mOrderResult?.orderid))
            return
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderDetailResult>(context, this) {
            override fun onSuccess(response: OrderDetailResult) {
                view.dismissLoadingDialog()
                getOrderInfo(response)
            }
        })
    }

    override fun onOfflinePay() {
        IOSAlertDialog(context).builder().setMsg("您确认线下收取押金吗？线下收取后不可更改", true)
                .setRightButton("取消") { }.setLeftButton("确定") {
                    StatisticsUtil.paymentSensorData("订单详情", orderID, "支付车辆押金",
                            true, true, carDepositAmount?.toInt() ?: 0)
                    requestOrderPay()
                }.show()
    }

    override fun onScanPay(fragment: Fragment) {
        StatisticsUtil.paymentSensorData("订单详情", orderID, "支付车辆押金",
                true, false, carDepositAmount?.toInt() ?: 0)
        if (MKUtil.isQRCodePay(mScanPayMethods)) {
            val intent = Intent(context, QRCodePayActivity::class.java)
            intent.putExtra(QRCodePayPresenter.ORDER_ID, orderID)
            intent.putExtra(QRCodePayPresenter.AMOUNT, carDepositAmount?.toInt())
            intent.putExtra(QRCodePayPresenter.PAY_TYPE, QRCodePayPresenter.DEPOSIT_CAR)
            intent.putExtra(QRCodePayPresenter.PAY_AVAI_METHOD, mScanPayMethods as Serializable)
            fragment.startActivityForResult(intent, 999)
        } else {
            view.showToast("暂无收款码可用")
        }
    }

    override fun requestOrderPay() {
        val request = OrderPayRequest()
        request.orderid = orderID
        request.pay_method = 10// 到店支付
        request.pay_type = OrderPayRequest.PAY_TYPE_VEHICLE_DEPOSIT// 车辆押金
        request.amount = (carDepositAmount!! * 100).toInt()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.payOrder(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderPayResult>(context, this) {
            override fun onSuccess(response: OrderPayResult) {
                view.dismissLoadingDialog()
                requestAPIOrderInfo()
            }
        })
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (data == null) return
        if (requestCode == 999) {
            LogUtil.e("onActivityResult 999")
            requestAPIOrderInfo()
        }
    }

    override fun onNext() {
        if (mOrderResult != null && mOrderResult?.channel == "schoolapp") {
            if (view.getStudentCancelOrderCheckStatus()) {
                IOSAlertDialog(context).builder().setMsg("您是要取消用户的订单吗？", true)
                        .setRightButton("取消订单") { updateUserAPIRequest() }.setLeftButton("返回") { }.show()
            } else {
                view?.toSchoolNextPage()
            }
        } else {
            view.toNormalNextPage()
        }
    }

    /**
     * 更新个人信息(身份证URL和驾驶证url)
     */
    private fun updateUserAPIRequest() {
        val request = DeleteOrderRequest()
        request.uid = UserUtil(context).uid
        if (null != mOrderResult?.orderid)
            request.orderid = mOrderResult?.orderid
        request.type = "CANCEL"
        request.reason = "授权失败"
        request.nopenalty = "TRUE"
        request.credit_noauth = true
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.deleteOrder(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult?) {
                view?.dismissLoadingDialog()
                if (response != null) {
                    view?.showToast("订单取消成功")
                    view.finishActivity()
                    LiveDataBus.get().with(CarEntity.BROADCAST_JPUSH_ORDER_STATUS_CHANGED).postValue("refreshOrderDetail")
                }
            }
        })
    }

    override fun onStudentCancelOrderCheckChanged() {
        LogUtil.e("onStudentCancelOrderCheckChanged")
        //学生免押
        if (view.getStudentCancelOrderCheckStatus()) {
            view.enableNext(true)
        } else {
            view.enableNext(false)
        }
    }

    override fun onCarDepositFreeButtonClick() {
        IOSAlertDialog(context).builder().setMsg("请确定要给商户免押", true)
                .setRightButton("确认免押") {
                    orderFreeDepositSchool()
                }.setLeftButton("取消") { }.show()
    }

    private fun orderFreeDepositSchool() {
        val request = FreeDepositSchoolRequest()
        request.order_id = orderID
        request.owner_id = UserUtil(context).userId.toString()
        request.comments = "商户免押"
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.orderFreeDepositSchool(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult) {
                view.dismissLoadingDialog()
                view.showToast("提交成功")
                requestAPIOrderInfo()
            }
        })
    }

    override fun onRefreshButtonClicked() {
        requestAPIOrderInfo()
    }
}
