package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.delivercar.fragments.fourth

import android.content.Context
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.BaseResult
import com.jiaoyinbrother.library.bean.OrderDetailResult
import com.jiaoyinbrother.library.bean.OrderExecuteRequest
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.view.EditCarNumDialog
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class DeliverCarFourthPresenter(context: Context, view: IDeliverCarFourthContract.View)
    : BasePresenterImpl<IDeliverCarFourthContract.View>(context, view), IDeliverCarFourthContract.Presenter {
    private var orderDetailResult: OrderDetailResult? = null


    override fun getData() {

        val contract = ArrayList<String>()
        contract.add("1、已签署验车单，一式两份")
        contract.add("2、声明禁行区域、限行/限牌规则")
        contract.add("3、向客服简述安全须知和车辆使用方法")
        contract.add("4、确保用户了解合同详情，并由用户本人签署")
        view.setTipsForContract("验车单和电子合同", contract)

        val deposit = ArrayList<String>()
        deposit.add("1、悟空平台订单，只允许线上收取客户的相关费用，禁止任何线下收取押金、保险费等行为")
        deposit.add("2、第三方订单：只能通过pos机进行押金预授权。禁止任何微信支付宝转账、现金交易、银行卡刷消费进行押金收取")
        view.setTipsForDeposit("押金支付方式", deposit)

        val insurance = ArrayList<String>()
        insurance.add("1、告知客户出险和理赔流程")
        insurance.add("2、禁止线下收取客户任何费用，如有车辆升级，新增保险等请联系平台进行收费报备")
        insurance.add("3、如收车时产生费用，请在现场与客户确认。并向平台进行报备，严格禁止线下收取客户费用")
        insurance.add("4、如产生费用争议，专员端订单页详情点击暂缓结算，避免车辆押金解冻")
        view.setTipsForInsurance("保险及费用争议", insurance)

        val userCar = ArrayList<String>()
        userCar.add("1、请确保专员电话畅通，及时配合用户解决用车中遇到的问题。并实时关注车辆GPS状况，如发现异常请及时向运营经理报备")
        view.setTipsForUseCar("关注用车情况", userCar)

        val comment = ArrayList<String>()
        comment.add("1、请在还车后，提示客户如对本次用车比较满意，给一个5分好评。商家好评越多，车辆展示越靠前")
        view.setTipsForComment("提醒用户好评", comment)


    }

    override fun onBackPressed() {
        if (orderDetailResult != null && orderDetailResult?.channel == "schoolapp") {
            view.onSchoolBack(orderDetailResult?.e_contract_status == 0)
        } else {
            view.onNormalBack(orderDetailResult?.e_contract_status == 0)
        }

    }

    override fun onNext(orderExecuteRequest: OrderExecuteRequest) {
        IOSAlertDialog(context)
                .builder()
                .setCancelable(false)
                .setTitle("信息核对提醒")
                .setMsg("请您核对车牌号：${orderDetailResult?.number}，和当前车辆车牌号是否一致，以免产生风险", true)
                .setRightButton("一致") {
                    orderExecuteRequest.real_car_number = orderDetailResult?.number
                    postUserUpdateRequest(orderExecuteRequest)
                }.setLeftButton("不一致") {
                    diffCarNum(orderExecuteRequest,orderDetailResult?.number ?: "")
                }.show()
    }

    /**
     * 车牌号不一致弹窗
     * @param num 车牌号
     */
    private fun diffCarNum(orderExecuteRequest: OrderExecuteRequest,num: String) {
        EditCarNumDialog(context)
                .builder()
                .setOldCarNum(num)
                .setFinish {
                    orderExecuteRequest.real_car_number = it
                    postUserUpdateRequest(orderExecuteRequest)
                }
                .show()
    }

    override fun setOrderInfo(result: OrderDetailResult) {
        orderDetailResult = result

        view.setOwnerConfirmCheckBoxText("我已确认以上信息准确无误，如有风险我来承担")

    }

    private var checkBoxStatus = false;

    override fun onCheckBoxChanged() {
        checkBoxStatus = !checkBoxStatus
        LogUtil.e("onCheckBoxChanged")
        LogUtil.e("info2CheckBoxStatus=$checkBoxStatus")
        view.setCheckBoxStatus(checkBoxStatus)
        if (checkBoxStatus) {
            view.enableNext(true)
        } else {
            view.enableNext(false)
        }
    }


    /**
     * 更新个人信息(身份证URL和驾驶证url)
     */
    private fun postUserUpdateRequest(request: OrderExecuteRequest) {

        val service = RetrofitHelper.getInstance(context).service

        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        val observable = service.orderExecute(params).debounce(1000, TimeUnit.MILLISECONDS)

        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult?) {
                view?.dismissLoadingDialog()
                if (response != null) {
                    view?.showToast("提交成功")
                    view?.finishActivity()
                }

            }
        })

    }

}
