package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.delivercar.fragments.main

import android.content.Context
import android.os.Bundle
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.BaseResult
import com.jiaoyinbrother.library.bean.ContractRegisterRequest
import com.jiaoyinbrother.library.bean.OrderDetailRequest
import com.jiaoyinbrother.library.bean.OrderDetailResult
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.UserUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class DeliverMainPresenter(context: Context, view: IDeliverCarMainContract.View) :
        BasePresenterImpl<IDeliverCarMainContract.View>(context, view), IDeliverCarMainContract.Presenter {
    override fun isStudent(): Boolean {
        return mIsStudent
    }

    private var orderDetailResult: OrderDetailResult? = null
    private var mIsStudent = false

    override fun argumentsData(b: Bundle) {
        orderDetailResult = b.getSerializable(ConstantsUtil.ORDER_RESULT) as OrderDetailResult?
        LogUtil.e("argumentsData, orderDetailResult = " + orderDetailResult.toString())
        apiRegisterContract(orderDetailResult?.orderid ?: "")
    }

    override fun requestOrderInfo() {
        //是否学生，初始化，不同界面
        if (orderDetailResult != null && orderDetailResult?.channel == "schoolapp") {
            LogUtil.e("schoolapp")
            view.initFragmentPagesWithSchool()
        } else {
            LogUtil.e("not schoolapp")
            view.initFragmentPagesWithoutSchool()
        }
        //直接调用setOrderInfoWithoutSchool， fragment为null
        requestAPIOrderInfo()
    }

    private fun requestAPIOrderInfo() {
        val request = OrderDetailRequest()
        request.uid = UserUtil(context).uid
        request.orderid = orderDetailResult?.orderid
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderDetailResult>(context, this) {
            override fun onSuccess(response: OrderDetailResult) {
                view.dismissLoadingDialog()

                if (response.channel == "schoolapp") {
                    mIsStudent = true
//                    view.initFragmentPagesWithSchool()
                    view.setOrderInfoWithSchool(response)
                } else {
                    mIsStudent = false
//                    view.initFragmentPagesWithoutSchool()
                    view.setOrderInfoWithoutSchool(response)
                }
            }
        })
    }

    override fun apiRegisterContract(orderId: String) {
        val request = ContractRegisterRequest()
        request.order_id = orderId
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.registerContract(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult) {
                LogUtil.e("法大大api注册成功")
            }

            override fun onAPIError(apiException: ApiException?) {
                LogUtil.e("法大大api注册失败")
            }

            override fun onError(e: Throwable) {
                LogUtil.e("法大大http请求失败")
            }
        })
    }

}
