package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.delivercar.fragments.schoolinfo

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentActivity
import android.text.TextUtils
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.Constants
import com.jiaoyinbrother.monkeyking.manager.activity.ShowBigImageActivity
import com.jiaoyinbrother.monkeyking.manager.util.ImageSelectorUtils
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.impl.UploadListener
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.library.widget.BottomDialog
import com.jiaoyinbrother.monkeyking.manager.view.TakeAlsoCarTimeLine
import com.tbruyelle.rxpermissions2.RxPermissions
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import java.io.File
import java.util.concurrent.TimeUnit

class DeliverCarSchoolInfoPresenter(context: Context, view: IDeliverSchoolInfoContract.View)
    : BasePresenterImpl<IDeliverSchoolInfoContract.View>(context, view), IDeliverSchoolInfoContract.Presenter{
    override fun onRefreshButtonClicked() {
        requestAPIOrderInfo()
    }

    //0：不通过，1：通过， 3：永久拒绝
    private val USER_SCHOOL_DENY = 0
    private val USER_SCHOOL_APPROVE = 1
    private val USER_SCHOOL_DENY_FOREVER = 3

    //图片路径
    private val ID_FILE_PATH = BitmapUtil.SYSTEM_BITMAP_PATH + "id_card.jpg"
    private val ONE_CARD_FILE_PATH = BitmapUtil.SYSTEM_BITMAP_PATH + "one_card.jpg"

    private var orderDetailResult:OrderDetailResult? = null

    private var mUploadUtil: AliYunPictureUtil? = null

    private var disposable: Disposable? = null
    private var compressedImage: File? = null

    private var mIDCardUrl: String = ""
    private var mOneCardUrl: String = ""

    private var mOriginIDCardUrl = ""
    private var mOriginOneCardUrl = ""

    private var mIDCardFilePath = ""
    private var mOneCardFilePath = ""

    private var isOneCardUploadToAliyun = false
    private var isIDCardUploadToAliyun = false

    private var isIDCard = true

    override fun onRenterIDPhoto(fragment: Fragment) {
//       1. 判断用户状态；某个状态，直接next，
//                if 有IDPhoto，显示大图；else, 上传IDPhoto
//        2. else,
//                if 有IDPhoto，显示大图；else, 上传IDPhoto

//        1. 几个condition: 学籍状态；IDPhoto; OneCardPhoto

        LogUtil.e("SchoolInfoPresenter,onRenterIDPhoto")
        //初始有图片，则不许编辑
        if (TextUtils.isEmpty(mOriginIDCardUrl)) {
            isIDCard = true

            //如果没有url,也没有file,不显示看大图
            if(TextUtils.isEmpty(mIDCardFilePath) && TextUtils.isEmpty(mIDCardUrl)){
                showMenuWithoutBigImage(fragment)
            }else{
                showMenuWithBigImage(fragment)
            }
        } else {
            ShowBigImageActivity.newInstanceUri(context, mOriginIDCardUrl!!, false)
        }
    }

    override fun onRenterOneCarPhoto(fragment: Fragment) {
        LogUtil.e("SchoolInfoPresenter,onRenterOneCarPhoto")

        if (TextUtils.isEmpty(mOriginOneCardUrl)) {
            isIDCard = false

            //如果没有url,也没有file,不显示看大图
            if(TextUtils.isEmpty(mOriginOneCardUrl) && TextUtils.isEmpty(mOneCardFilePath)){
                showMenuWithoutBigImage(fragment)
            }else{
                showMenuWithBigImage(fragment)
            }
        } else {
            ShowBigImageActivity.newInstanceUri(context, mOriginOneCardUrl!!, false)
        }
    }


    override fun onNext() {
        LogUtil.e("SchoolInfoPresenter,onNext")

        LogUtil.e("mOriginOneCardUrl = $mOriginOneCardUrl")
        LogUtil.e("mOriginIDCardUrl = $mOriginIDCardUrl")

        if(isOneCardUploadToAliyun || isIDCardUploadToAliyun){
            var apiOneCarUrl = mOriginOneCardUrl
            var apiIDCardUrl = mOriginIDCardUrl
            if(isOneCardUploadToAliyun)
                apiOneCarUrl = mOneCardUrl

            if(isIDCardUploadToAliyun)
                apiIDCardUrl = mIDCardUrl

            updateStudentPhoto(apiIDCardUrl, apiOneCarUrl)

        }else{
            //初始有照片，不需要更新user
            view.toNextPage()
        }

    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        LogUtil.e("onActivityResult requestCode = $requestCode")
        if (data != null)
            LogUtil.e("onActivityResult data = $data")
        when (requestCode) {
            Constants.REQUEST_PIC -> {
                LogUtil.e("REQUEST_PIC=")
                if (data != null) {
                    //images返回的是照片路径
                    val images = data.getStringArrayListExtra(ImageSelectorUtils.SELECT_RESULT)
                    LogUtil.e("iamges=$images")
                    if (images != null && images.size > 0) {
                        LogUtil.e("images->$images")
                        if (images[0] == null)
                            return

                        handlePhoto(File(images[0]))
                    }
                }
            }
            CarEntity.REQUEST_CAMERA -> {
                if (resultCode == Activity.RESULT_OK) {
                    handlePhoto(File(getImageFilePath()))
                }
            }
        }
    }

    private fun updateStudentPhoto(idUrl:String, oneCarUrl:String){
        LogUtil.e("SchoolInfoPresenter,updateStudentPhoto")
        if(orderDetailResult == null || orderDetailResult?.school_roll?.user_id == null)
            return

        val request = StudentPhotoUpdateRequest()
        request.user_id = orderDetailResult?.school_roll?.user_id
        request.channel = "schoolapp"
        request.one_card_url = oneCarUrl
        request.card_url = idUrl

        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = mRetrofitService.userSchoolUpdate(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context,this) {
            override fun onSuccess(response: BaseResult) {
                view.dismissLoadingDialog()
                view.toNextPage()
            }
        })
    }

    override fun onSchoolApproval(context: Context) {
        IOSAlertDialog(context).builder().setMsg("请确认学籍认证通过？", true)
                .setRightButton("认证通过") {
                    updateUserSchoolApproveStatus(USER_SCHOOL_APPROVE)
                }.setLeftButton("取消") {}.show()
    }

    override fun onSchoolDeny(context: Context) {
        IOSAlertDialog(context).builder().setMsg("请确认永久拒绝此用户学籍认证？拒绝后会同时取消订单。", true)
                .setRightButton("永久拒绝") {
                    updateUserSchoolApproveStatus(USER_SCHOOL_DENY_FOREVER)
                }.setLeftButton("取消") {}.show()

    }

    override fun onSchoolReapply(context: Context) {
        IOSAlertDialog(context).builder().setTitle("重写填写学籍信息？").setMsg("请用户在悟空租车校园版APP的身份认证里重新编辑重新认证。", true)
                .setRightButton("认证失败") {
                    updateUserSchoolApproveStatus(USER_SCHOOL_DENY)

                }.setLeftButton("取消") {}.show()

    }

    override fun getOrderInfo(result: OrderDetailResult) {
        LogUtil.e("getOrderInfo")
        refreshOrderDetail(result)
    }

    private fun refreshOrderDetail(result: OrderDetailResult){

        LogUtil.e("refreshOrderDetail")
        orderDetailResult = result

        initTimeLine(result)

        initCarNum(result)

        initRenterSchoolInfo(result)

        initRenterSchoolPhoto(result)


    }

    private fun initTimeLine(result: OrderDetailResult) {
        if (result.e_contract_status == 0) {
            view.setTimeLine(TakeAlsoCarTimeLine.DELIVER_SCHOOL, TakeAlsoCarTimeLine.INDEX_ZERO)
        } else {
            view.setTimeLine(TakeAlsoCarTimeLine.DELIVER_CONTRACT_SCHOOL, TakeAlsoCarTimeLine.INDEX_ZERO)
        }
    }

    private fun initRenterSchoolPhoto(result: OrderDetailResult) {

        val schoolRoll = result.school_roll
        if( schoolRoll!= null) {
            if(!TextUtils.isEmpty(schoolRoll.card_url)) {
                mOriginIDCardUrl = schoolRoll.card_url!!
                setIDCardUrl(schoolRoll.card_url)
            }
            if(!TextUtils.isEmpty(schoolRoll.one_card_url)) {
                mOriginOneCardUrl = schoolRoll.one_card_url!!
                setOneCardUrl(schoolRoll.one_card_url)
            }
        }
        LogUtil.e("After IDCarUrl")
//        school auth status，必须放在photo初始化之后
        initRenterSchoolAuthStatus(result)
    }

    private fun initRenterSchoolAuthStatus(result: OrderDetailResult) {
        //学籍相关信息
        val schoolRoll = result.school_roll
        LogUtil.e("initRenterSchoolAuthStatus, schoolRoll =" + schoolRoll.toString())
        if(schoolRoll?.status != null && !TextUtils.isEmpty(schoolRoll.status)) {

            when (schoolRoll.status) {
                SchoolRollBean.NOAUTH, SchoolRollBean.BLACKLIST -> {
                    handleAuthFailed()
                    view.enableNext(false)
                }
                SchoolRollBean.AUTHING-> {
                    handleAuthing()
                    view.enableNext(false)
                }
                SchoolRollBean.AUTHED ->{
                    handleAuthed(result)
                    checkNextCondition()
                }

            }

        }

    }

    private fun handleAuthed(result: OrderDetailResult) {

        view.enableRenterSchoolApproveButton(enable = false, visibility = false)
        view.enableRenterSchoolDenyButton(enable = false, visibility = false)
        view.enableRenterSchoolDenyForeverButton(enable = false, visibility = false)

        view.showRenterSchoolPhotoLayout(true)
        if(!TextUtils.isEmpty(result.school_roll?.card_url))
            view.setUserIDPhotoUrl(result.school_roll?.card_url!!)

        if(!TextUtils.isEmpty(result.school_roll?.one_card_url))
            view.setUserOneCarPhotoUrl(result.school_roll?.one_card_url!!)
    }

    private fun handleAuthing() {

        view.enableRenterSchoolApproveButton(enable = true, visibility = true)
        view.enableRenterSchoolDenyButton(enable = true, visibility = true)
        view.enableRenterSchoolDenyForeverButton(enable = true, visibility = true)

        view.showRenterSchoolPhotoLayout(false)
    }

    private fun handleAuthFailed() {

        view.enableRenterSchoolApproveButton(enable = false, visibility = true)
        view.enableRenterSchoolDenyButton(enable = false, visibility = true)
        view.enableRenterSchoolDenyForeverButton(enable = false, visibility = true)

        view.showRenterSchoolPhotoLayout(false)
    }

    private fun initRenterSchoolInfo(result: OrderDetailResult) {
        LogUtil.e("initRenterSchoolInfo")
        //学籍相关信息
        val schoolRoll = result.school_roll
        if(schoolRoll?.status != null){

            LogUtil.e("initRenterSchoolInfo, schoolRoll = $schoolRoll")
            if(!TextUtils.isEmpty(schoolRoll.school))
                view.setUserSchool(schoolRoll.school!!)
            if(!TextUtils.isEmpty(schoolRoll.specialty))
                view.setUserSpecialty(schoolRoll.specialty!!)
            if(!TextUtils.isEmpty(schoolRoll.education_name))
                view.setUserEducation(schoolRoll.education_name!!)
            if(!TextUtils.isEmpty(schoolRoll.grade_name))
                view.setUserEducationYear(schoolRoll.grade_name!!)
            if(!TextUtils.isEmpty(schoolRoll.status_desc))
                view.setUserSchoolStatus(schoolRoll.status_desc!!)

        }

    }


    private fun initCarNum(result: OrderDetailResult) {
        // 订单号
        view.setOrderNum("订单号：${result.getOrderShowID()}")
        // 订单渠道
        if (TextUtils.isEmpty(result.channel_name)) {
            view.setOrderChannel("")
        } else {
            view.setOrderChannel("(${result.channel_name})")
        }
        // 车牌号
        view.setCarNum(result.number ?: "")
    }

    private fun updateUserSchoolApproveStatus(userSchoolAuthStatus:Int){
        if(orderDetailResult == null)
            return

        val request = UserSchoolApproveRequest()
        request.user_id = orderDetailResult?.school_roll?.user_id
        request.channel = "schoolapp"
        request.auth = userSchoolAuthStatus
        request.order_id = orderDetailResult?.orderid

        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = mRetrofitService.userSchoolApprove(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context,this) {
            override fun onSuccess(response: BaseResult) {
                view.dismissLoadingDialog()


                if(userSchoolAuthStatus == USER_SCHOOL_DENY_FOREVER) {
                    LogUtil.e("LiveDataBus  BROADCAST_JPUSH_ORDER_STATUS_CHANGED ")
                    LiveDataBus.get().with(CarEntity.BROADCAST_JPUSH_ORDER_STATUS_CHANGED).postValue("refreshOrderDetail")
                    view.finishActivity()
                }else{
                    requestAPIOrderInfo()
                }
            }
        })

    }


    private fun requestAPIOrderInfo() {
        val request = OrderDetailRequest()
        request.uid = UserUtil(context).uid
        if(TextUtils.isEmpty(orderDetailResult?.orderid))
            return

        request.orderid = orderDetailResult?.orderid
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderDetailResult>(context,this) {
            override fun onSuccess(response: OrderDetailResult) {
                view.dismissLoadingDialog()

                refreshOrderDetail(response)
            }
        })
    }


    private fun showMenuWithBigImage(fragment: Fragment) {
        BottomDialog(context).builder().addItems(arrayOf("查看大图", "拍照", "相册")).setCancel("取消") {}.setItemClick { _, i ->
            when (i) {
                // 相机(单选)
                0 -> {
                    // 查看大图
                    handleBigImage(context as FragmentActivity)
                }
                1 -> {
                    // 跳转到调用系统相机
                    handleTakePicture(context, fragment)
                }
                else -> {
                    // 跳转到调用系统图库
                    ImageSelectorUtils.openPhoto(context as FragmentActivity, fragment, Constants.REQUEST_PIC, true, 1, null, "")
                }
            }
        }.show()
    }


    private fun showMenuWithoutBigImage(fragment: Fragment) {
        BottomDialog(context).builder().addItems(arrayOf("拍照", "相册")).setCancel("取消", {}).setItemClick { _, i ->
            when (i) {
                // 相机(单选)
                0 -> {
                    // 跳转到调用系统相机
                    handleTakePicture(context,fragment)

                }
                1 -> {
                    // 跳转到调用系统图库
                    ImageSelectorUtils.openPhoto(context as FragmentActivity,fragment, Constants.REQUEST_PIC, true, 1, null, "")
                }
            }
        }.show()
    }

    private fun handleTakePicture(context: Context, fragment: Fragment) {
        disposable = RxPermissions(context as FragmentActivity).request(Manifest.permission.CAMERA,
                Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE).subscribe({ granted ->
            if (granted) { // Always true pre-M
                PhotoUtils.takePicture(context as Activity, fragment, File(getImageFilePath()), CarEntity.REQUEST_CAMERA)
            } else {
                //permission denied
            }
        }, { throwable ->
            LogUtil.e("RxJava throwable = $throwable")
        })
    }


    private fun getImageFilePath(): String {
        val filePath: String
        if (isIDCard)
            filePath = ID_FILE_PATH
        else
            filePath = ONE_CARD_FILE_PATH
        LogUtil.e("getImageFilePath =$filePath")
        return filePath
    }

    /**
     * 查看大图功能
     */
    private fun handleBigImage(activity: FragmentActivity) {

        if(isIDCard){
            if (!TextUtils.isEmpty(mIDCardUrl))
                ShowBigImageActivity.newInstanceUri(context, mIDCardUrl!!, false)
            else{
                ShowBigImageActivity.newInstanceUri(context, mIDCardFilePath, false)
            }
        }else{
            if (!TextUtils.isEmpty(mOneCardUrl))
                ShowBigImageActivity.newInstanceUri(context, mOneCardUrl!!, false)
            else{
                ShowBigImageActivity.newInstanceUri(context, mOneCardFilePath, false)
            }
        }

    }


    private fun handlePhoto(actualImage: File?) {
        LogUtil.e("handlePhoto")
        if (actualImage == null)
            return

        try {
            view?.showLoadingDialog()

            compressedImage = PhotoUtils.compressFile(context, actualImage)

            if (compressedImage != null) {
                LogUtil.e("compressedImage!=null")
                if (isIDCard) {
                    view.setUserIDPhotoUrl(compressedImage!!.absolutePath)
                    mIDCardFilePath = compressedImage!!.absolutePath
                    uploadeImageAliyunOnePic(mIDCardFilePath)
                } else {
                    view.setUserOneCarPhotoUrl(compressedImage!!.absolutePath)
                    mOneCardFilePath = compressedImage!!.absolutePath
                    uploadeImageAliyunOnePic(mOneCardFilePath)
                }
            }
        } catch (e: Exception) {

        }
    }

    private var postImages = ArrayList<String>()

    private fun uploadeImageAliyunOnePic(filePath: String) {
        if (TextUtils.isEmpty(filePath)) {
            view?.dismissLoadingDialog()
            return
        }
        //清除list,否则会有多张图片
        postImages.clear()
        postImages.add(filePath)

        mUploadUtil = AliYunPictureUtil.getInstance(context)
        mUploadUtil?.setDatas(AliYunPictureUtil.BUCKET_TYPE_PUBLIC,postImages, AliYunPictureUtil.VALIDATE_CAR, uploadListenerOnePic)
    }

    private val uploadListenerOnePic = object : UploadListener {
        override fun onUploadComplete(success: java.util.HashMap<Int, String>, failure: java.util.HashMap<Int, String>) {
            view.dismissLoadingDialog()

            LogUtil.e("onUploadComplete， success.size=" + success.size)
            LogUtil.e("onUploadComplete， success = $success")
            if (success.size == 1) {
                if(isIDCard){
                    isIDCardUploadToAliyun  = true
                    setIDCardUrl(success[1].toString())
                }else{
                    setOneCardUrl(success[1].toString())
                    isOneCardUploadToAliyun = true
                }

                checkNextCondition()

            } else {
                view.showToast("上传失败")
            }
        }

        override fun onUploadComplete(net: String?, local: String?) {}
    }

    private fun checkNextCondition() {
        //已经检查过 用户状态
        LogUtil.e("DeliverCarSchoolInfoPresenter checkNextCondition ")
        view?.enableNext(false)

        //初始有图片
        if(!TextUtils.isEmpty(mOriginIDCardUrl) ||!TextUtils.isEmpty(mOriginOneCardUrl)){
            view.enableNext(true)
            LogUtil.e("DeliverCarSchoolInfoPresenter checkNextCondition mOriginID!= null")
        }
        else{
            //上传过图片
            if(isIDCardUploadToAliyun||isOneCardUploadToAliyun) {
                view.enableNext(true)
                LogUtil.e("DeliverCarSchoolInfoPresenter checkNextCondition isIDCardUploadToAliyun!= null")
            }
            else {
                LogUtil.e("DeliverCarSchoolInfoPresenter checkNextCondition all == null")
                view.enableNext(false)
            }
        }

    }

    private fun setIDCardUrl(url: String?) {
        LogUtil.e("setIDCardUrl+url=$url")


        if (!TextUtils.isEmpty(url)) {
            mIDCardUrl = url!!
            view?.setUserIDPhotoUrl(url!!)
        }
    }

    private fun setOneCardUrl(url: String?) {


        if ( !TextUtils.isEmpty(mOneCardUrl)) {
            LogUtil.e("setOneCardUrl,mOneCardUrl=$mOneCardUrl")

        } else {
            LogUtil.e("setOneCardUrl,mOneCardUrl == null")
        }

        if (!TextUtils.isEmpty(url)) {
            view?.setUserOneCarPhotoUrl(url!!)
            mOneCardUrl = url!!
        }

    }

}
