package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.detail


import android.app.Activity
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Bundle
import android.support.v4.app.FragmentActivity
import android.support.v7.app.AppCompatActivity
import android.text.TextUtils
import android.view.View
import com.alibaba.android.arouter.launcher.ARouter
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.Constants
import com.jiaoyinbrother.monkeyking.manager.Constants.RELET_CODE
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.activity.ChangeCarActivity
import com.jiaoyinbrother.monkeyking.manager.adapter.OrderListAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.billrecord.BillRecordActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.checkcar.CheckCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.dropinthecar.DropInTheCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.delivercar.DeliverCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.refundcar.RefundTheCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.storeinfo.StoreInfoActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.storeinfo.StoreInfoPresenter
import com.jiaoyinbrother.monkeyking.manager.mvpdialog.renthistorydialog.RentHistoryDialog
import com.jiaoyinbrother.monkeyking.manager.util.DialogUtil
import com.jiaoyinbrother.monkeyking.manager.util.H5Utils
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.cost.CostDetailActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.cost.CostPresenter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.operatingdata.OrderOperatingDataActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.ordercontract.OrderContractActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.payforother.PayForOtherActivity
import com.jiaoyinbrother.monkeyking.manager.selfcheck.SelfCheckActivity
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import org.json.JSONObject
import java.util.concurrent.TimeUnit
import kotlin.collections.ArrayList

class OrderDetailPresenter(context: Context, view: IOrderDetailContract.View) :
        BasePresenterImpl<IOrderDetailContract.View>(context, view), IOrderDetailContract.Presenter {

    private var isDepositPayedFail = true

    /**
     * 自助验车图片
     * @param
     */
    fun onSelfCheck() {
        SelfCheckActivity.newInstance(context, orderStatus, orderID ?: "")
    }

    override fun onSuspendOrderConfirm() {

        val request = OrderBaseRequest()
        request.order_id = orderID
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.setupAutoApply(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<BaseResult>(context, this) {
                    override fun onSuccess(response: BaseResult) {
                        view.dismissLoadingDialog()
//                        view.showToast("取消成功")
                        requestOrderInfo()
                    }
                })
    }

    override fun cancelRelet() {

        val request = OrderBaseRequest()
        request.order_id = orderID
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.cancelRelet(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<BaseResult>(context, this) {
                    override fun onSuccess(response: BaseResult) {
                        view.dismissLoadingDialog()
                        view.showToast("取消成功")
                        requestOrderInfo()
                    }
                })
    }

    override fun goRelet() {

        checkReletRequest()
    }


    /**
     * 检查
     */
    fun checkReletRequest() {

        val request = CheckReletRequest()
        request.order_id = orderID
        request.type = "RELET" // RELET or ADVANCE
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.checkRelet(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<CheckReletResult>(context, this) {
                    override fun onSuccess(result: CheckReletResult) {

                        //跳转续租
                        ARouter.getInstance().build(ARouterConstantUtil.ACTIVITY_ORDER_RELET_URL)
                                .withSerializable(
                                        ARouterConstantUtil.ACTIVITY_PARAM_ORDER_DETAIL,
                                        orderInfo
                                )
                                .withSerializable(
                                        ARouterConstantUtil.ACTIVITY_PARAM_RELET_DEADLINE,
                                        result.last_time
                                )
                                .navigation(context as Activity, RELET_CODE)
                    }
                })
    }

    /**
     * 跳转到查看合同页面
     */
    override fun onGoContract() {
        OrderContractActivity.newInstance(context, orderStatus, orderID ?: "")
    }

    companion object {
        private const val TAG = "OrderDetailPresenter"
        const val ORDER_MODEL_CHECK_RELET_INFO = "ORDER_MODEL_CHECK_RELET_INFO"
        const val ORDER_MODEL_CHECK_RELET_LIST = "ORDER_MODEL_CHECK_RELET_LIST"
    }

    private var orderID: String? = null
    private var orderStatus: Int = 0
    private var orderInfo: OrderDetailResult? = null

    /**
     * 初始化页面传递params
     * @param i
     * */
    override fun getIntentData(i: Intent) {
        orderID = i.getStringExtra(ConstantsUtil.ORDER_ID) ?: ""
        LogUtil.e("$TAG --->>> $orderID")
    }

    override fun onRentNumberClicked(activity: AppCompatActivity) {
        LogUtil.e("onRentNumberClicked")

        val rentHistoryDialog = RentHistoryDialog()

        val bundle = Bundle()
        bundle.putString("RENTER_USER_ID", orderInfo?.renter_user_id ?: "")
        rentHistoryDialog.arguments = bundle
        rentHistoryDialog.show(activity.supportFragmentManager, null)

    }

    /**
     * 获取订单详情
     * */
    override fun requestOrderInfo() {
        val request = OrderDetailRequest()
        request.uid = UserUtil(context).uid
        request.orderid = orderID
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
            view.setRefreshEnd()
        }.observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<OrderDetailResult>(context, this) {
                    override fun onSuccess(response: OrderDetailResult) {
                        view.showPage()
                        view.dismissLoadingDialog()

                        orderInfo = response

                        orderID = response.orderid

                        orderStatus = response.status

                        // order
                        setOrderInfo(response)
                        // car
                        setCarInfo(response)
                        // time & site
                        setTimeAndSiteInfo(response)
                        // renter
                        setRenterInfo(response)
                        // deposit
                        setDepositInfo(response)
                        // cost & discount
                        setCostDiscountAndServiceInfo(response)

                        view.setFlightAndTrainsNote(!TextUtils.isEmpty(response.remarks), response.remarks
                                ?: "")

                        if (response.status >= OrdersBean.ORDER_STATUS_5) {
                            view.setStateColor(R.color.color_5)
                        } else {
                            view.setStateColor(R.color.color_3)
                        }

                        // 是否结算异常单（1：是，0：不是）
                        if (response.deposit_payed_fail ?: 0 == 1 && isDepositPayedFail) {
                            isDepositPayedFail = false
                            view.setDepositPayedFailDialog()
                        }

                        // By state set order bottom button
                        view.setBottomButton(response)

                        //安飞士
                        val listPlatForm = response.plat_form_tags
                        if (response.is_avis == true && listPlatForm != null) {
                            view.setPlatFormContent(listPlatForm)
                        }

                        //学籍信息
                        updateSchoolInfo(response)

                        //查看合同
                        if (response.is_e_contract == 1) {
                            view.setShowContract(View.VISIBLE)
                        } else {
                            view.setShowContract(View.GONE)
                        }

                        //是否出险
                        if (response.show_accident_button == 1) {
                            view.setShowCallPoliceLayout(View.VISIBLE, response.accident_apply_url)
                        } else
                            view.setShowCallPoliceLayout(View.GONE, "")

                    }
                })
    }

    /**
     * order info
     * */
    private fun setOrderInfo(result: OrderDetailResult) {
        // order number
        view.setOrderNum("订单号：${result.getOrderShowID()}")
        // order channel
        view.setOrderChannel("(${result.channel_name ?: ""})")
        // order state

        view.setOrderState(result.status_name ?: "", isShowSuspendOrderConfirmStatus(result))
        // order time line
        view.setOrderTimeInfo(result)

        setRelet(result.relet_enable)

    }

    private fun isShowSuspendOrderConfirmStatus(result: OrderDetailResult): Boolean {
        var ret = false

        //用车中；non_auto_apply ==1
        if (OrdersBean.ORDER_STATUS_5 == result.status && (result.non_auto_apply != null && result.non_auto_apply == 1)) {
            ret = true
        }

        LogUtil.e("isShowSuspendOrderConfirm, ret =$ret")

        return ret
    }

    private fun setRelet(status: Int) {

        when (status) {
            0 -> {
                view.setReletCancelButton(View.VISIBLE)
            }
            1 -> {
                view.setReletButton(View.VISIBLE)
            }
            else -> {
                view.setReletButton(View.GONE)
                view.setReletCancelButton(View.GONE)
            }
        }
    }

    /**
     * car info
     * */
    private fun setCarInfo(result: OrderDetailResult) {
        // car picture
        view.setCarPic(result.pic ?: "")
        // car type
        view.setCarType(result.type ?: "")
        // car info
        view.setCarInfo(Util.getCarCfgInfo(result))
        // car number
        view.setCarNum(result.number ?: "")
        // car remote
//        view.setCarRemote(result.remote > 0)
    }

    /**
     * time & site info
     * */
    private fun setTimeAndSiteInfo(result: OrderDetailResult) {
        // order take expect time
        if (!TextUtils.isEmpty(result.start_time)) {
            view.setTakeExpectTime("预计取车时间：${result.start_time}")
        } else {
            view.setTakeExpectTime("预计取车时间：------")
        }
        // actual take time
        if (!TextUtils.isEmpty(result.real_start_time)) {
            view.setTakeActualTime("实际取车时间：${result.real_start_time}")
        } else {
            view.setTakeActualTime("实际取车时间：------")
        }
        val site = result.site
        // take site name
        view.setTakeSiteName("${site?.city} - ${site?.name}")
        // take site address
        view.setTakeAddress(result.address ?: "")

        // expect return time
        if (!TextUtils.isEmpty(result.end_time)) {
            view.setReturnExpectTime("预计还车时间：${result.end_time}")
        } else {
            view.setReturnExpectTime("预计还车时间：------")
        }
        // actual return time
        if (!TextUtils.isEmpty(result.real_end_time)) {
            view.setReturnActualTime("实际还车时间：${result.real_end_time}")
        } else {
            view.setReturnActualTime("实际还车时间：------")
        }
        val returnSite = result.return_site
        // return site name
        view.setReturnSiteName("${returnSite?.city} - ${returnSite?.name}")
        // return site address
        view.setReturnAddress(result.return_address ?: "")


        when (result.is_onsite_service) {
            OrdersBean.SEND_TO_CAR -> {
                view.setReturnCarToDoor(true, "上门取送")
                view.setTakeCarFromDoor(true, "上门取送")
            }
            OrdersBean.PICK_UP_PERSON -> {
                view.setReturnCarToDoor(true, "接送至门店")
                view.setTakeCarFromDoor(true, "接送至门店")
            }
            else -> {
                if (result.remote == 1) {
                    view.setReturnCarToDoor(true, "自助取还")
                    view.setTakeCarFromDoor(true, "自助取还")
                } else {
                    view.setReturnCarToDoor(false, "")
                    view.setTakeCarFromDoor(false, "")
                }
            }
        }
    }

    /**
     * renter info
     * */
    private fun setRenterInfo(result: OrderDetailResult) {
        // renter name
        if (!TextUtils.isEmpty(result.renter_name)) {
            view.setRenterName(result.renter_name ?: "")
        } else {
            view.setRenterName("------")
        }
        // renter phone
        if (!TextUtils.isEmpty(result.renterid)) {
            view.setRenterPhone(result.renterid ?: "")
        } else {
            view.setRenterPhone("------")
        }
        view.setRenterAuthDisplay(true)
        if (!TextUtils.isEmpty(result.renterid)) {
            requestUserDetail()
        }
        var rentTimes = ""
        if (result.rent_times > 0) {
            rentTimes = "租车<font color=\"#CB2A1E\">" + result.rent_times + "</font>次"
        }

        view.setRentTimes(rentTimes)

        val member = result.member
        if (member != null) {
            view.setTakeCarPersonCard(true)
            view.setMemberName(member.name ?: "")

            view.setMemberPhone(member.member_phone ?: "")

            view.setMemberID(member.id_card_no ?: "")

            view.setMemberAuth(member.auth_status ?: "", member.auth_status_name ?: "")

            view.setMemberRate(member.rate_name ?: "")

        } else {
            view.setTakeCarPersonCard(false)
        }
    }

    /**
     * deposit info
     * */
    private fun setDepositInfo(result: OrderDetailResult) {
        // 车辆押金
        val depositForCar = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_VEHICLE)
        view.setDepositForCar(
                depositForCar.type_name ?: "",
                MoneyUtil().getMoneyText(depositForCar.amount.toString()),
                depositForCar.status_name ?: "",
                depositForCar.type_description ?: ""
        )
        // 违章押金
        val depositForIllegal = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_ILLEGAL)
        view.setDepositForIllegal(
                depositForIllegal.type_name ?: "",
                MoneyUtil().getMoneyText(depositForIllegal.amount.toString()),
                depositForIllegal.status_name ?: "",
                depositForIllegal.type_description ?: ""
        )
    }

    /**
     * cost & discount info
     * */
    private fun setCostDiscountAndServiceInfo(result: OrderDetailResult) {
        // 费用
        view.setOrderCost(result.status <= 5, "¥${result.bill?.total_amount}")

        view.setDepositPayedTip(!TextUtils.isEmpty(result.pay_type), "(${result.pay_type})")

        // 活动优惠
        view.setOrderDiscount(result.activity != null, result.activity?.name ?: "")
        // 保障服务
        view.setOrderService(result.optional_service_names ?: ArrayList())
        //店铺服务
        view.setStoreOptionalService(result.service_tags ?: ArrayList())

    }

    /**
     * 网点详情
     * */
    override fun onSiteInfo(isTake: Boolean) {

        val siteID = if (isTake) orderInfo?.site?.siteid else orderInfo?.return_site?.siteid
        LiveDataBus.get().with(StoreInfoPresenter.STORE_INFO_SITE_ID).postValue(siteID)
        val intent = Intent(context, StoreInfoActivity::class.java)
        intent.putExtra("SiteDetailFromType", StoreInfoActivity.TYPE_FROM_CAR_DETAIL)
        intent.putExtra("siteDetailId", siteID)
        context.startActivity(intent)
    }

    /**
     * 拨打客户电话
     * */
    fun onRenterPhone() {
        if (context is FragmentActivity) {
            DialogUtil.onSiteDial(context as FragmentActivity, orderInfo?.renterid)
        }
    }

    /**
     * 退款说明
     * */
    override fun onRefundTips() {
        H5Utils.onBackMoney(context)
    }

    /**
     * 保障范围
     * */
    override fun onServiceTips() {
        H5Utils.onInsurance(context)
    }

    /**
     * 费用详情
     * */
    override fun onCostInfo() {
        StatisticsUtil.onEvent(
                context,
                ConstantsUtil.ORDER_DETAIL_BILL,
                ConstantsUtil.ORDER_DETAIL_BILL
        )

        LiveDataBus.get().with(CostPresenter.ORDER_DETAIL).postValue(orderInfo)
        val intent = Intent(context, CostDetailActivity::class.java)
        context.startActivity(intent)
    }

    /**
     * 验车单
     * */
    override fun onCheckCarInfo() {
        if (orderInfo?.status ?: 0 >= OrdersBean.ORDER_STATUS_5) {
            StatisticsUtil.onEvent(
                    context,
                    ConstantsUtil.ORDER_DETAIL_VALIDATE_CAR,
                    ConstantsUtil.ORDER_DETAIL_VALIDATE_CAR
            )
            LiveDataBus.get().with("CheckCar").postValue(orderInfo)
            val intent = Intent(context, CheckCarActivity::class.java)
            context.startActivity(intent)
        }
    }

    /**
     * 票据中心
     * */
    override fun onRecordsInfo() {
        if (orderInfo?.status ?: 0 >= OrdersBean.ORDER_STATUS_5) {
            val intent = Intent(context, BillRecordActivity::class.java)
            intent.putExtra(ConstantsUtil.ORDER_ID, orderID)
            context.startActivity(intent)
        }
    }

    //办理取车
    override fun onConfirmTakeCar() {
        LogUtil.e("onConfirmTakeCar presenter")
        val intent = Intent(context, DeliverCarActivity::class.java)
        intent.putExtra(ConstantsUtil.ORDER_ID, orderInfo?.orderid)
//        //因为需要先判断是否是学生，所有传递orderdetail
//        if (orderInfo != null)
//            intent.putExtra(ConstantsUtil.ORDER_RESULT, orderInfo)
        (context as Activity).startActivityForResult(intent, Constants.CONFIRM_SALOON_CAR_CODE)

    }

    //办理还车
    override fun onConfirmReturnCar() {
        val intent = Intent(context, RefundTheCarActivity::class.java)
        intent.putExtra(ConstantsUtil.ORDER_ID, orderInfo?.orderid)
        (context as Activity).startActivityForResult(intent, Constants.CLOSE_ACCT_CODE)

    }

    //确认结算
    override fun onSettlementForOwner() {
        val intent = Intent(context, PayForOtherActivity::class.java)
        intent.putExtra("Flag", "代客支付")
        intent.putExtra("order_detail", orderInfo)
        (context as Activity).startActivityForResult(intent, Constants.CLOSE_ACCT_CODE)
    }

    //换车
    override fun onChangeCar() {
        val intent = Intent(context, ChangeCarActivity::class.java)
        intent.putExtra(ConstantsUtil.ORDER_RESULT, orderInfo)
        (context as Activity).startActivityForResult(intent, Constants.CHANGE_CAR_CODE)

    }

    //出发取车
    override fun onTakeCarBackFromDoor() {
        goDropInTheCarActivity()
    }


    //出发送车
    override fun onSendCarToDoor() {
        goDropInTheCarActivity()
    }

    //确认出发取车
    override fun onConfirmTakeCarBackFromDoor() {
        goDropInTheCarActivity()
    }

    //确认出发送车
    override fun onConfirmSendCarToDoor() {
        goDropInTheCarActivity()
    }


    override fun requestUserDetail() {
        val request = UserDetailRequestBean()
        request.uid = orderInfo?.renterid
        request.public_classification = "N"
        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable =
                mRetrofitService.userGetDetail(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<UserDetailResult>(context, this) {
                    override fun onSuccess(response: UserDetailResult) {
                        view.dismissLoadingDialog()
                        // renter id card
                        if (!TextUtils.isEmpty(response.id_no)) {
                            view.setRenterIDCard(response.id_no)
                        } else {
                            view.setRenterIDCard("------")
                        }
                        view.setRenterAuthDisplay(false)
                        // renter auth state
                        view.setRenterAuthState(response.auth)
                        // renter level
                        view.setRenterLevel(response.rolename)
                    }
                })
    }

    private fun updateSchoolInfo(orderDetail: OrderDetailResult) {
        LogUtil.e("updateSchoolInfo")
        val schoolRoll = orderDetail.school_roll
        if (schoolRoll?.status != null) {

            view.showUserSchool(true)
            LogUtil.e("schoolRoll = $schoolRoll")
            if (!TextUtils.isEmpty(schoolRoll.school))
                view.setUserSchool(schoolRoll.school!!)
            if (!TextUtils.isEmpty(schoolRoll.specialty))
                view.setUserSpecialty(schoolRoll.specialty!!)
            if (!TextUtils.isEmpty(schoolRoll.education_name))
                view.setUserEducation(schoolRoll.education_name!!)
            if (!TextUtils.isEmpty(schoolRoll.grade_name))
                view.setUserEducationYear(schoolRoll.grade_name!!)

            view.setUserEducationAuth(schoolRoll.status, schoolRoll.status_desc)

        } else {
            view.showUserSchool(false)
            LogUtil.e("schoolRoll == null")
        }
    }

    /**
     * 订单信息推送
     * */
    override fun getOrderInfoReceiver() {
        val filter = IntentFilter()
        filter.addAction(CarEntity.BROADCAST_JPUSH_ORDER_STATUS_CHANGED)
        context.registerReceiver(receiver, filter)
    }

    /**
     * 订单信息广播接收
     * */
    private val receiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            val mAction = intent?.action
            if (mAction == CarEntity.BROADCAST_JPUSH_ORDER_STATUS_CHANGED && intent.hasExtra(
                            CarEntity.JPUSH_ORDER_KEY
                    )
            ) {
                val pushJson = intent.extras?.getString(CarEntity.JPUSH_ORDER_KEY)
                if (!TextUtils.isEmpty(pushJson)) {
                    try {
                        val obj = JSONObject(pushJson)
                        val mOrderID = obj.optString("orderid")
                        if (TextUtils.equals(orderID, mOrderID)) {
                            requestOrderInfo()
                        }
                    } catch (e: Exception) {
                        LogUtil.e("OrderDetailPresenter error")
                    }
                }
            }
        }
    }

    private fun goDropInTheCarActivity() {

        val intent = Intent(context, DropInTheCarActivity::class.java)
        intent.putExtra(OrderListAdapter.ORDER_LIST_TO_DROP_IN, orderInfo)
        context.startActivity(intent)
    }


    override fun onDestroy() {
        context.unregisterReceiver(receiver)
    }


    fun onMemberPhone() {
        if (context is FragmentActivity) {
            DialogUtil.onSiteDial(context as FragmentActivity, orderInfo?.member?.member_phone)
        }
    }

    fun doMemberAgree() {
        val name = orderInfo?.member?.name ?: ""
        val idNo = orderInfo?.member?.id_card_no ?: ""
        H5Utils.doMemberAgree(context, name, idNo, orderID)
    }


    override fun onOperatingData() {
        val intent = OrderOperatingDataActivity.newInstance(context, orderID)
        context.startActivity(intent)
    }

}