package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.list

import android.content.Intent
import com.jiaoyinbrother.library.base.BaseActivity
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.monkeyking.manager.R

import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.list.fragment.OrdersFragment
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.search.OrderSearchActivity
import com.jiaoyinbrother.monkeyking.manager.widget.OrderManageBar
import kotlinx.android.synthetic.main.activity_manage_orders.*

class OrderManageActivity : BaseActivity() {

    private lateinit var mHandOverFragment: OrdersFragment
    private lateinit var mOnGoingFragment: OrdersFragment
    private lateinit var mSettleFragment: OrdersFragment
    private lateinit var mCompletedFragment: OrdersFragment
    private lateinit var mCancelledFragment: OrdersFragment

    private lateinit var fragments: List<OrdersFragment>

    override fun getLayoutId(): Int {
        return R.layout.activity_manage_orders
    }

    override fun initView() {
        val status = intent.getIntExtra(ConstantsUtil.FRAGMENT_FLAG, 0)
        val carId = intent.getStringExtra(ORDER_CAR_ID)

        mHandOverFragment = OrdersFragment.newInstance(OrdersFragment.ORDER_STATUS_HANDOVER, carId)
        mOnGoingFragment = OrdersFragment.newInstance(OrdersFragment.ORDER_STATUS_ONGOING, carId)
        mSettleFragment = OrdersFragment.newInstance(OrdersFragment.ORDER_STATUS_SETTLE, carId)
        mCompletedFragment = OrdersFragment.newInstance(OrdersFragment.ORDER_STATUS_COMPLETED, carId)
        mCancelledFragment = OrdersFragment.newInstance(OrdersFragment.ORDER_STATUS_CANCELLED, carId)

        fragments = listOf(mHandOverFragment, mOnGoingFragment, mSettleFragment, mCompletedFragment, mCancelledFragment)
        // 所有fragment 都是隐藏状态
        supportFragmentManager.beginTransaction()
                .add(R.id.order_manage_fl, fragments[0])
                .hide(fragments[0])
                .add(R.id.order_manage_fl, fragments[1])
                .hide(fragments[1])
                .add(R.id.order_manage_fl, fragments[2])
                .hide(fragments[2])
                .add(R.id.order_manage_fl, fragments[3])
                .hide(fragments[3])
                .add(R.id.order_manage_fl, fragments[4])
                .hide(fragments[4])
                .commit()
        // 这个时候切换fragment为了去调用fragment onLazyLoad()
        order_manage_bar?.setManageStatusBar(status)
        showHideFragment(status)
    }

    override fun initToolBar() {
        mainTitle?.text = getString(R.string.title_order_manage)
        subImg1.setImageResource(R.mipmap.ic_order_search)
    }

    override fun initData() {
    }

    override fun initListeners() {
        subImg1?.setOnClickListener {
            startActivity(Intent().apply {
                setClass(this@OrderManageActivity, OrderSearchActivity::class.java)
            })
        }

        order_manage_bar?.setListener(object : OrderManageBar.OnOrderManageSortListener {
            override fun onStatus(status: Int) {
                showHideFragment(status)
            }

            override fun onSort(type: Int, isAsc: Boolean) {
                updateSort(type, isAsc)
            }
        })
    }

    private fun updateSort(type: Int, isAsc: Boolean) {
        for (fragment in fragments) {
            fragment.updateSort(type, isAsc)
        }
    }

    private fun showHideFragment(status: Int) {
        val beginTransaction = supportFragmentManager.beginTransaction()
        when (status) {
            OrdersFragment.ORDER_STATUS_HANDOVER -> {
                beginTransaction.show(fragments[0])
                        .hide(fragments[1])
                        .hide(fragments[2])
                        .hide(fragments[3])
                        .hide(fragments[4])
            }
            OrdersFragment.ORDER_STATUS_ONGOING -> {
                beginTransaction.show(fragments[1])
                        .hide(fragments[2])
                        .hide(fragments[3])
                        .hide(fragments[4])
                        .hide(fragments[0])
            }
            OrdersFragment.ORDER_STATUS_SETTLE -> {
                beginTransaction.show(fragments[2])
                        .hide(fragments[3])
                        .hide(fragments[4])
                        .hide(fragments[0])
                        .hide(fragments[1])
            }
            OrdersFragment.ORDER_STATUS_COMPLETED -> {
                beginTransaction.show(fragments[3])
                        .hide(fragments[4])
                        .hide(fragments[0])
                        .hide(fragments[1])
                        .hide(fragments[2])
            }
            OrdersFragment.ORDER_STATUS_CANCELLED -> {
                beginTransaction.show(fragments[4])
                        .hide(fragments[0])
                        .hide(fragments[1])
                        .hide(fragments[2])
                        .hide(fragments[3])
            }
        }
        beginTransaction.commit()
    }


    companion object{
       const val ORDER_CAR_ID = "see_all_car_id"
    }
}