package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.list.fragment

import android.content.Intent
import android.os.Bundle
import android.view.View
import com.jiaoyinbrother.library.bean.OrdersBean
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.Constants
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.adapter.OrderListAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.dropinthecar.DropInTheCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.delivercar.DeliverCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.detail.OrderDetailActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.list.BaseOrdersFragment
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.refundcar.RefundTheCarActivity
import kotlinx.android.synthetic.main.fragment_order_list.*

class OrdersFragment : BaseOrdersFragment<OrdersPresenter>(), IOrdersContract.View {

    private lateinit var mAdapter: OrderListAdapter

    override fun initPresenter(): OrdersPresenter = OrdersPresenter(requireContext(), this)

    override fun getLayoutId(): Int = R.layout.fragment_order_list

    override fun initViews(view: View?, savedInstanceState: Bundle?) {}

    override fun initToolbar(savedInstanceState: Bundle?) {}

    override fun initData() {
        mAdapter = OrderListAdapter(requireContext())
        fg_orders_rv?.adapter = mAdapter

        fg_orders_loading?.setEmptyText("您还没有订单哦~")
        fg_orders_loading?.setErrorText("无法获取数据，可尝试检查网络...")
        fg_orders_loading?.setRetryText("点击重试")

        presenter.getData(arguments)
    }

    override fun initListeners() {
        // 下拉刷新
        fg_orders_refresh?.setOnRefreshListener {
            presenter.onRefresh()
        }
        // 上拉加载
        fg_orders_refresh?.setOnLoadMoreListener {
            presenter.onLoadMore()
        }

        mAdapter.setOrderItemListener(object : OrderListAdapter.OnOrderItemListener {
            override fun onItemClick(type: String, order: OrdersBean) {
                presenter.getInfo(type, order)
            }
        })
    }

    override fun onHiddenChanged(hidden: Boolean) {
        super.onHiddenChanged(hidden)
        presenter.onHiddenChanged(hidden)
    }

    override fun updateSort(type: Int, isAsc: Boolean) {
        presenter.onSort(type, isAsc)
    }

    override fun finishRefreshWithNoMoreData() {
        fg_orders_refresh?.finishRefreshWithNoMoreData()
    }

    override fun finishLoadMoreWithNoMoreData() {
        fg_orders_refresh?.finishLoadMoreWithNoMoreData()
    }

    override fun finishRefresh() {
        fg_orders_refresh?.finishRefresh()
    }

    override fun finishLoadMore() {
        fg_orders_refresh?.finishLoadMore()
    }

    override fun setEnableLoadMore(b: Boolean) {
        fg_orders_refresh?.setEnableLoadMore(b)
    }

    override fun showNoDataView() {
        fg_orders_loading?.showEmpty()
        setEnableLoadMore(false)
    }

    override fun showNetErrView(b: Boolean) {
        fg_orders_loading?.showError()
        setEnableLoadMore(false)
    }

    override fun setOrdersData(data: ArrayList<OrdersBean>) {
        mAdapter.list = data
        mAdapter.notifyDataSetChanged()
        fg_orders_loading?.showContent()
    }

    override fun onOrderInfo(orderId: String) {
        startActivityForResult(Intent().apply {
            setClass(requireContext(), OrderDetailActivity::class.java)
            putExtra(ConstantsUtil.ORDER_ID, orderId)
        }, Constants.ORDER_DETAIL_CODE)
    }

    override fun onDropInTheCar(order: OrdersBean) {
        startActivityForResult(Intent().apply {
            setClass(requireContext(), DropInTheCarActivity::class.java)
            putExtra(OrderListAdapter.ORDER_LIST_TO_DROP_IN, order)
        }, Constants.ORDER_DETAIL_CODE)
    }

    /**
     * 办理取车
     */
    override fun onDeliverTheCar(order: OrdersBean) {
        startActivityForResult(Intent().apply {
            setClass(requireContext(), DeliverCarActivity::class.java)
            putExtra(ConstantsUtil.ORDER_ID, order.orderid)
        }, Constants.ORDER_DETAIL_CODE)
    }

    /**
     * 办理还车
     */
    override fun onConfirmTheCar(order: OrdersBean) {
        startActivityForResult(Intent().apply {
            setClass(requireContext(), RefundTheCarActivity::class.java)
            putExtra(ConstantsUtil.ORDER_ID, order.orderid)
        }, Constants.ORDER_DETAIL_CODE)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        LogUtil.e("Lyy --->>>> onActivityResult")
        presenter.onRefresh()
    }

    companion object {
        // 订单对应状态
        const val ORDER_STATUS_HANDOVER = 0
        const val ORDER_STATUS_ONGOING = 1
        const val ORDER_STATUS_SETTLE = 2
        const val ORDER_STATUS_COMPLETED = 3
        const val ORDER_STATUS_CANCELLED = 4

        // 排序
        const val DEFAULT = 0
        const val TIME_TAKE = 1
        const val TIME_RETURN = 2

        // 排序type
        const val START_TIME = "startTime"
        const val END_TIME = "endTime"

        // 降序
        const val DESC = "DESC"

        // 升序
        const val ASC = "ASC"

        const val ORDER_STATUS = "ORDER_STATUS"
        const val CAR_ID = "CAR_ID"

        fun newInstance(status: Int, carId:String?): OrdersFragment {
            val fragment = OrdersFragment()
            val args = Bundle()
            args.putInt(ORDER_STATUS, status)
            args.putString(CAR_ID, carId)
            fragment.arguments = args
            return fragment
        }
    }
}