package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.list.fragment

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.monkeyking.manager.adapter.OrderListAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.dropinthecar.DropInTheCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.delivercar.DeliverCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.refundcar.RefundTheCarActivity
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class OrdersPresenter(context: Context, view: IOrdersContract.View) :
        BasePresenterImpl<IOrdersContract.View>(context, view), IOrdersContract.Presenter {

    override var mStatus: Int = OrdersFragment.ORDER_STATUS_HANDOVER
    override var isHidden: Boolean = false
    private var isFmCreate = false

    private lateinit var mStatusList: ArrayList<String>

    private val mOrdersData = ArrayList<OrdersBean>()
    private var mPage = 1
    private var mSortType = OrdersFragment.DEFAULT
    private var mSord = ""
    private var mSidx = ""
    private var mCarId:String? = null


    override fun getData(arguments: Bundle?) {
        isFmCreate = true
        mStatus = arguments?.getInt(OrdersFragment.ORDER_STATUS)
                ?: OrdersFragment.ORDER_STATUS_HANDOVER

        mCarId = arguments?.getString(OrdersFragment.CAR_ID)
        mStatusList = getStatusList(mStatus)
    }

    /**
     * 获取订单状态参数
     *
     * @param  status       订单状态
     * @return List<String> 状态数组
     */
    private fun getStatusList(status: Int): ArrayList<String> = when (status) {
        OrdersFragment.ORDER_STATUS_ONGOING -> ArrayList(listOf("ORDER_EXECUTING"))
        OrdersFragment.ORDER_STATUS_SETTLE -> ArrayList(listOf("ORDER_BILL_APPLIED"))
        OrdersFragment.ORDER_STATUS_COMPLETED -> ArrayList(listOf("ORDER_BILL_ACCEPTED", "ORDER_FINISTHED"))
        OrdersFragment.ORDER_STATUS_CANCELLED -> ArrayList(listOf("ORDER_CANCELED"))
        else -> ArrayList(listOf("ORDER_ILLEGAL_PAYED"))
    }

    /**
     * 刷新数据
     */
    override fun onRefresh() {
        if (!isHidden) {
            view?.setEnableLoadMore(true)
            mPage = 1
            onApiOrders()
        }
    }

    /**
     * 加载更多
     */
    override fun onLoadMore() {
        onApiOrders()
    }

//    override fun onResume() {
//        LogUtil.e("Lyy onResume -->> start , hidden = $isHidden")
//        onRefresh()
//    }

    override fun onHiddenChanged(hidden: Boolean) {
        isHidden = hidden
        if (!hidden){
            onLazyLoad()
        }
        LogUtil.e("Lyy ---->>>> status = $mStatus , hidden = $isHidden")
    }

    /**
     * 排序操作
     *
     * @param type
     * @param isAsc
     */
    override fun onSort(type: Int, isAsc: Boolean) {
        mSortType = type
        mSord = if (isAsc) OrdersFragment.ASC else OrdersFragment.DESC
        mSidx = if (mSortType == OrdersFragment.TIME_TAKE) OrdersFragment.START_TIME else OrdersFragment.END_TIME
        onRefresh()
    }

    private fun onApiOrders() {
        val request = OrdersRequest()
        request.manager_id = UserUtil(context).userId.toString()
        if (!TextUtils.isEmpty(mCarId)){
            request.car_id = mCarId
        }
        request.page = mPage
        request.page_size = TimeOrderUtil.PAGE_SIZE
        request.status = 4
        request.status_list = mStatusList
        if (mSortType != OrdersFragment.DEFAULT) {
            request.sidx = mSidx
            request.sord = mSord
        }
        val service = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getOrders(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrdersResult>(context, this) {
            override fun onSuccess(response: OrdersResult) {
                view.dismissLoadingDialog()
                if (mPage == 1) {
                    handleRefreshData(response)
                } else {
                    handleLoadMoreData(response)
                }
            }

            override fun onAPIError(apiException: ApiException?) {
                view.dismissLoadingDialog()
                view.showNetErrView(mOrdersData.size == 0)
            }

            override fun onError(e: Throwable) {
                view.dismissLoadingDialog()
                view.showNetErrView(mOrdersData.size == 0)
            }
        })
    }

    private fun handleRefreshData(response: OrdersResult) {
        val dataList = response.orders
        if (dataList != null && dataList.isNotEmpty()) {
            //刷新成功，有数据
            mOrdersData.clear()
            mOrdersData.addAll(dataList)
            view.setOrdersData(mOrdersData)
            mPage++
            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //刷新成功，数据小于PAGE_SIZE，一页
                view.finishRefreshWithNoMoreData()
            } else {
                //刷新成功，一整页
                view.finishRefresh()
            }
        } else {
            //刷新没有数据，必须显示no data view
            view.finishRefresh()
            view.showNoDataView()
        }
    }

    private fun handleLoadMoreData(response: OrdersResult) {
        val dataList = response.orders
        if (dataList != null && dataList.isNotEmpty()) {
            mOrdersData.addAll(dataList)
            view.setOrdersData(mOrdersData)
            mPage++
            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //加载更多，数据小于PAGE_SIZE，一页
                view.finishLoadMoreWithNoMoreData()
            } else {
                //加载更多，一整页
                view.finishLoadMore()
            }
        } else {
            //加载更多，没有数据
            view.finishLoadMoreWithNoMoreData()
        }
    }


    override fun getInfo(type: String, order: OrdersBean) {
        if (!NetUtil.isNetConnected(context)) {
            view.showToast("呃呃，网络信号太差啦，累死宝宝啦")
            return
        }
        onApiDetail(type, order)
    }


    private fun onApiDetail(type: String, order: OrdersBean) {
        val request = OrderDetailRequest()
        request.uid = UserUtil(context).uid
        request.orderid = order.orderid
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<OrderDetailResult>(context, this) {
                    override fun onSuccess(response: OrderDetailResult) {
                        when (type) {
                            OrderListAdapter.ORDER_LIST_TO_DROP_IN -> {
                                view.onDropInTheCar(order)
                            }
                            OrderListAdapter.ORDER_LIST_TO_DETAIL -> {
                                view.onOrderInfo(order.orderid ?: "")
                            }
                            OrderListAdapter.ORDER_LIST_TO_DELIVER -> {
                                view.onDeliverTheCar(order)
                            }
                            OrderListAdapter.ORDER_LIST_TO_ALSO -> {
                                view.onConfirmTheCar(order)
                            }
                        }
                    }

                    override fun onAPIError(apiException: ApiException?) {
                        view.dismissLoadingDialog()
                        super.onAPIError(apiException)
                    }

                    override fun onError(e: Throwable) {
                        view.dismissLoadingDialog()
                        super.onError(e)
                    }
                })
    }

    private fun onLazyLoad(){
        if (isFmCreate && !isHidden){
            onRefresh()
        }
    }
}