package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.operatingdata

import android.content.Context
import android.content.Intent
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class OperatingDataPresenter(context: Context, view: IOperatingDataContract.View) :
    BasePresenterImpl<IOperatingDataContract.View>(context, view),
    IOperatingDataContract.Presenter {


    private var orderId: String? = null
    private var mPage = 1
    private var mOperatingData = ArrayList<OperatingDataBean>()

    override fun getData(intent: Intent) {
        orderId = intent.getStringExtra(ConstantsUtil.ORDER_ID) ?: return

        onRefresh()
    }

    override fun onRefresh() {
        view.setEnableLoadMore(true)
        mPage = 1
        apiOperatingData()
    }

    override fun onLoadMore() {
        apiOperatingData()
    }

    override fun apiOperatingData() {
        val request = OperatingDataRequest()
        request.orderid = orderId
        request.page = mPage
        request.page_size = ConstantsUtil.PAGE_SIZE
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.operatingData(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : BaseObserver<OperatingDataResult>(context, this) {
                override fun onSuccess(response: OperatingDataResult) {
                    view.dismissLoadingDialog()
                    if (mPage == 1) {
                        handleRefreshData(response)
                    } else {
                        handleLoadMoreData(response)
                    }
                }

                override fun onError(e: Throwable) {
                    view.dismissLoadingDialog()
                    view.showNetErrView(mOperatingData.size == 0)
                }

                override fun onAPIError(apiException: ApiException?) {
                    view.dismissLoadingDialog()
                    view.showNetErrView(mOperatingData.size == 0)
                }
            })
    }

    private fun handleRefreshData(response: OperatingDataResult) {
        val dataList = response.rows
        if (dataList != null && dataList.isNotEmpty()) {
            //刷新成功，有数据
            mOperatingData.clear()
            mOperatingData.add(OperatingDataBean())
            mOperatingData.addAll(dataList)
            view.setOperatingData(mOperatingData)
            mPage++
            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //刷新成功，数据小于PAGE_SIZE，一页
                view.finishRefreshWithNoMoreData()
            } else {
                //刷新成功，一整页
                view.finishRefresh()
            }
        } else {
            //刷新没有数据，必须显示no data view
            view.finishRefresh()
            view.showNoDataView()
        }
    }

    private fun handleLoadMoreData(response: OperatingDataResult) {
        val dataList = response.rows
        if (dataList != null && dataList.isNotEmpty()) {
            mOperatingData.addAll(dataList)
            view.setOperatingData(mOperatingData)
            mPage++
            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //加载更多，数据小于PAGE_SIZE，一页
                view.finishLoadMoreWithNoMoreData()
            } else {
                //加载更多，一整页
                view.finishLoadMore()
            }
        } else {
            //加载更多，没有数据
            view.finishLoadMoreWithNoMoreData()
        }
    }
}