package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.refundcar.fragment.contract

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.activity.GeneralWebViewActivity
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.monkeyking.manager.view.TakeAlsoCarTimeLine
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class RefundContractPresenter(context: Context, view: IRefundContractModel.View)
    : BasePresenterImpl<IRefundContractModel.View>(context, view), IRefundContractModel.Presenter {

    private var mOrderInfo: OrderDetailResult? = null
    private var mDeleteType: Int = ContractPreRequest.DELETE
    private var mUserIDNum: String? = null
    private var mContractState: String? = null
    private var mBillRequest: BillApplyRequest? = null
    private var isSignSuccess = false

    override fun getOrderInfo(result: OrderDetailResult) {
        mOrderInfo = result
        initTimeLine()
    }

    override fun initPreContract() {
        mDeleteType = ContractPreRequest.DELETE
        apiUserDetail()
    }

    override fun getApplyRequest(request: BillApplyRequest) {
        this.mBillRequest = request
    }

    override fun doContractElectronic(mSignType : String?) {
        apiSignContract(mSignType)
    }

    override fun apiUserDetail() {
        if (TextUtils.isEmpty(mOrderInfo?.renterid))
            return
        val request = UserDetailRequestBean()
        request.uid = mOrderInfo?.renterid
        request.public_classification = "N"
        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = mRetrofitService.userGetDetail(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<UserDetailResult>(context, this) {
            override fun onSuccess(response: UserDetailResult) {
                val member = mOrderInfo?.member
                // renter id card
                mUserIDNum = if (TextUtils.isEmpty(member?.id_card_no))
                    response.id_no ?: ""
                else
                    member?.id_card_no ?: ""
                apiPreContract()
            }

            override fun onAPIError(apiException: ApiException?) {
                super.onAPIError(apiException)
                view?.dismissLoadingDialog()
                view.setErrorPager(true)
            }

            override fun onError(e: Throwable) {
                view?.dismissLoadingDialog()
                view.setErrorPager(true)
            }
        })

    }

    override fun apiPreContract() {
        val request = ContractPreRequest()
        request.order_id = mOrderInfo?.orderid
        request.user_id = UserUtil(context).userId.toString()
        request.id_card_no = mUserIDNum
        request.contract_type = ContractPreRequest.RETURN_CONTRACT
        request.delete_contract = mDeleteType
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.preContract(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<ContractPreResult>(context, this) {
            override fun onSuccess(response: ContractPreResult) {
                view?.dismissLoadingDialog()
                view.setErrorPager(false)
                mContractState = response.contract_sign_status
                // 交车信息至合同页面下一步时mDeleteType == 1，从法大大页面返回时mDeleteType == 0，mDeleteType后台是否删除合同：0否1是
                view.setContractState(mContractState ?: "", mDeleteType)
//                when {
//                    // 初始化
//                    mDeleteType == ContractPreRequest.DELETE -> {
//
//                    }
//                    // 合同签署成功
//                    mDeleteType == ContractPreRequest.RETAIN && isSignSuccess -> {
//                        view.setContractState(mContractState ?: "", mDeleteType)
//                    }
//                    // 合同签署未成功
//                    else -> {
//                        IOSAlertDialog(context).builder().setTitle("温馨提示").setMsg("您签署过程中是否遇到问题，如遇到问题可线下签署纸质合同！是否继续签署电子合同？", true)
//                                .setRightButton("是") {}
//                                .setLeftButton("否") {
//                                    view.setContractState(mContractState ?: "", mDeleteType)
//                                }.show()
//                    }
//                }
            }

            override fun onAPIError(apiException: ApiException?) {
                super.onAPIError(apiException)
                view?.dismissLoadingDialog()
                view.setErrorPager(true)
            }

            override fun onError(e: Throwable) {
                view?.dismissLoadingDialog()
                view.setErrorPager(true)
            }
        })
    }

    override fun apiSignContract(mSignType : String?) {
        val request = ContractSignRequest()
        request.orderid = mOrderInfo?.orderid
        request.user_id = UserUtil(context).userId.toString()
        request.contract_type = ContractPreRequest.RETURN_CONTRACT
        request.fuel = mBillRequest?.fuel
        request.miles = mBillRequest?.miles.toString()
        request.fuel_url = mBillRequest?.fuel_url

        if (!TextUtils.isEmpty(mSignType)){

            request.sign_type = mSignType
        }else {

            request.sign_type = "SMS"
        }

        request.real_start_time = mBillRequest?.real_start_time
        request.real_end_time = mBillRequest?.real_end_time
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.signContract(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<ContractSignResult>(context, this) {
            override fun onSuccess(response: ContractSignResult) {
                view?.dismissLoadingDialog()
                if (!TextUtils.isEmpty(response.sign_url)) {
                    val webViewConfigEntity = WebViewConfigEntity()
                    webViewConfigEntity.title = "签署文件"
                    webViewConfigEntity.url = response.sign_url
                    webViewConfigEntity.isNeedProgressBar = true
                    webViewConfigEntity.isNeedShareButton = false
                    webViewConfigEntity.imageUrl = ""
                    webViewConfigEntity.description = ""
                    val intent = Intent(context, GeneralWebViewActivity::class.java)
                    intent.putExtra(CarEntity.FEATURE_ITEM, webViewConfigEntity)
                    context.startActivity(intent)
                }
            }
        })
    }

    override fun apiOrderApply(request: BillApplyRequest) {
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.orderApply(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult) {
                view?.dismissLoadingDialog()
                view?.showToast("提交成功")
                view?.setActivityFinish()
            }
        })
    }

    /**
     * 合同签署成功
     *
     */
    override fun doContractSuccess() {
        isSignSuccess = true
        mDeleteType = ContractPreRequest.RETAIN
        apiPreContract()
    }

    override fun doContractSubmit() {
        when (mContractState) {
            ContractPreResult.SUCCESS -> {
                // 签署成功 直接确认交车
                if (mBillRequest != null) {
                    apiOrderApply(mBillRequest!!)
                }
            }
            ContractPreResult.SIGN_FAILED,
            ContractPreResult.CERTIFICATION_FAILED -> {
                // 签署失败 & 认证失败 跳转最后一个页面
                view.setContractPaper()
            }
            else -> {
                // 未签署 跳转最后一个页面
                view.setContractPaper()
            }
        }
    }

    /**
     * 合同未签署返回
     *
     */
    override fun doContractSignBack() {
        mDeleteType = ContractPreRequest.RETAIN
        apiPreContract()
    }

    /**
     * 初始化引导标题
     */
    private fun initTimeLine() {
        view.setTimeLine(TakeAlsoCarTimeLine.REFUND_CONTRACT, TakeAlsoCarTimeLine.INDEX_THREE)
    }


}