package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.refundcar.fragment.first

import android.content.Context
import android.content.Intent
import android.support.v4.app.Fragment
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.view.View
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.monkeyking.manager.util.MKUtil
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.codepay.QRCodePayActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.codepay.QRCodePayPresenter
import com.jiaoyinbrother.monkeyking.manager.view.TakeAlsoCarTimeLine
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import org.json.JSONException
import org.json.JSONObject
import java.io.Serializable
import java.util.*
import java.util.concurrent.TimeUnit

class RefundFirstPresenter(context: Context, view: IRefundFirstContract.View)
    : BasePresenterImpl<IRefundFirstContract.View>(context, view), IRefundFirstContract.Presenter {
    private var orderID: String? = null
    private var payAmount: Float? = null
    private var payMethodName: String? = null
    private var mScanPayMethods: ArrayList<PayMethodAvaiBean>? = null

    override fun getOrderInfo(result: OrderDetailResult) {
        initTimeLine(result)

        orderID = result.orderid
        // 订单号
        view.setOrderNum("订单号：${result.getOrderShowID()}")
        // 订单渠道
        if (TextUtils.isEmpty(result.channel_name)) {
            view.setOrderChannel("")
        } else {
            view.setOrderChannel("(${result.channel_name})")
        }
        // 车牌号
        view.setCarNum(result.number ?: "")
        val carDeposit = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_VEHICLE)
        // 车辆押金金额
        view.setDepositCostForCar(carDeposit.type_name ?: "",
                MoneyUtil().getMoneyText(carDeposit.amount.toString()))
        // 车辆押金支付状态
        view.setDepositStateForCar(carDeposit.status_name ?: "")
        // 车辆押金描述
        view.setDepositDescForCar(carDeposit.type_description ?: "")
        // 支付宝预授权；冻结车辆押金
        if (carDeposit.freeze_enabled == 1) {
            view.setFreezeDepositVisibilityForCar(View.VISIBLE)
            if (carDeposit.freeze_description != null) {
                val sp = SpannableStringBuilder()
                TextUtil.generatePromptString(sp, carDeposit.freeze_description)
                view.setFreezeDepositDescForCar(sp)
            }
        } else {
            view.setFreezeDepositVisibilityForCar(View.GONE)
        }

        // 违章押金
        val illegalDeposit = Util.getDepositBean(result, ConstantsUtil.PAY_PROJECT_ILLEGAL)
        payAmount = illegalDeposit.amount
        payMethodName = illegalDeposit.status_name
        view.setDepositCostForIllegal(illegalDeposit.type_name
                ?: "", MoneyUtil().getMoneyText(payAmount.toString()))
        // 违章押金支付状态
        when (illegalDeposit.status_code) {
            DepositBean.DEPOSIT_PAYED,
            DepositBean.DEPOSIT_REFUNDED -> {
                // 已支付 | 退款
                paySuccess(illegalDeposit.status_name ?: "")
                view.setCanNext(true)
            }
            DepositBean.DEPOSIT_WAIT_PAYED -> {
                mScanPayMethods = illegalDeposit.pay_method_avai

                if (getOfflineShow(illegalDeposit)) {
                    view.setDepositForIllegalOffline(View.VISIBLE, "线下收取", illegalDeposit.enabled == 1)
                } else {
                    view.setDepositForIllegalOffline(View.GONE, "线下收取", false)
                }

                if (getScanShow(illegalDeposit)){
                    view.setDepositForIllegalScan(View.VISIBLE, "扫码收取", illegalDeposit.enabled == 1)
                } else{
                    view.setDepositForIllegalScan(View.GONE, "扫码收取", false)
                }

                view.setDepositForIllegalState(View.GONE, "")
//                // 未支付
//                if (Util.isOffline(illegalDeposit)) {
//
//                } else {
//                    paySuccess("未支付")
//                }
                view.setCanNext(false)
            }
        }
        // 违章押金描述
        view.setDepositDescForIllegal(illegalDeposit.type_description ?: "")
    }

    override fun requestOrderPay() {
        val request = OrderPayRequest()
        request.orderid = orderID
        request.pay_method = 10// 到店支付
        request.pay_type = OrderPayRequest.PAY_TYPE_ILLEGAL_DEPOSIT
        request.amount = (payAmount!! * 100).toInt()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.payOrder(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrderPayResult>(context,this) {
            override fun onSuccess(response: OrderPayResult) {
                view.dismissLoadingDialog()
                paySuccess("已到店支付")
                view.setCanNext(true)
            }
        })
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (data == null) return
        LogUtil.e("onActivityResult")
        if (requestCode == 999) {
            LogUtil.e("onActivityResult 999")
            val state = data.getStringExtra(QRCodePayPresenter.DEPOSIT_STATUS_NAME)
            paySuccess(state)
            view.setCanNext(true)
        }
    }

    private fun paySuccess(state: String) {
        view.setDepositForIllegalOffline(View.GONE, "", false)
        view.setDepositForIllegalScan(View.GONE, "", false)
        view.setDepositForIllegalState(View.VISIBLE, state)
        payMethodName = state
    }

    /**
     *  线下支付
     * */
    override fun onOfflinePay() {
        IOSAlertDialog(context).builder().setMsg("您确认线下收取押金吗？线下收取后不可更改", true)
                .setRightButton("取消") { }.setLeftButton("确定") {
                    StatisticsUtil.paymentSensorData("订单详情", orderID, "支付车辆押金",
                            true, true, payAmount?.toInt() ?: 0)
                    requestOrderPay()
                }.show()
    }

    /**
     * 扫码支付
     * */
    override fun onScanPay(fragment: Fragment) {
        StatisticsUtil.paymentSensorData("订单详情", orderID, "支付车辆押金",
                true, false, payAmount?.toInt() ?: 0)
        if (MKUtil.isQRCodePay(mScanPayMethods)) {
            val intent = Intent(context, QRCodePayActivity::class.java)
            intent.putExtra(QRCodePayPresenter.ORDER_ID, orderID)
            intent.putExtra(QRCodePayPresenter.AMOUNT, payAmount?.toInt())
            intent.putExtra(QRCodePayPresenter.PAY_TYPE, QRCodePayPresenter.DEPOSIT_ILLEGAL)
            intent.putExtra(QRCodePayPresenter.PAY_AVAI_METHOD, mScanPayMethods as Serializable)
            fragment.startActivityForResult(intent, 999)
        } else {
            view.showToast("暂无收款码可用")
        }
    }

    override fun onPostApplyRequest(checked: Boolean) {
        val request = BillApplyRequest()
        request.orderid = orderID
        request.freeze_deposit = if (checked) 1 else 0
        LiveDataBus.get().with("FirstFragment").postValue(request)
    }

    /**
     * 还车第一步：订单押金核实
     */
    fun handleSensorData() {
        try {
            val jsonObject = JSONObject()
            jsonObject.put("order_id", orderID)
            jsonObject.put("payment_method", payMethodName)
            jsonObject.put("illegal_deposit_amount", payAmount?.toDouble())
            StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_RETURN_CAR_STEP1, jsonObject)
        } catch (e: JSONException) {
            e.printStackTrace()
        }

    }

    private fun initTimeLine(result: OrderDetailResult) {
        if (result.e_contract_status == 1 && result.is_e_contract == 1) {
            view.setTimeLine(TakeAlsoCarTimeLine.REFUND_CONTRACT, TakeAlsoCarTimeLine.INDEX_ZERO)
        } else {
            view.setTimeLine(TakeAlsoCarTimeLine.REFUND_NORMAL, TakeAlsoCarTimeLine.INDEX_ZERO)
        }
    }


    private fun getOfflineShow(deposit: DepositBean): Boolean {
        return deposit.offline_button?.show == 1
    }


    private fun getScanShow(deposit: DepositBean): Boolean {
        return deposit.scan_button?.show == 1
    }
}