package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.refundcar.fragment.third

import android.content.Context
import android.text.TextUtils
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.refundcar.RefundTheCarActivity
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ChannelUtil
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.MoneyUtil
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.monkeyking.manager.view.TakeAlsoCarTimeLine
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class RefundThirdPresenter(context: Context, view: IRefundThirdContract.View)
    : BasePresenterImpl<IRefundThirdContract.View>(context, view), IRefundThirdContract.Presenter {
    private var postApplyRequest: BillApplyRequest? = null
    private var orderInfo: OrderDetailResult? = null
    private var calcInfo: CalcResult? = null
    private var otherAmount: String? = null
    //    private var fuelAmount: String? = null
//    private var penaltyAmount: String? = null
    private var canPageNext = false

    override fun getOrderInfo(result: OrderDetailResult) {
        initTimeLine(result)
        orderInfo = result
        view.setCostInfo(result)

        val bill = result.bill

        if (bill != null) {
            val couponPay = bill.coupon_pay?.toInt() ?: 0
            view.setCouponAmount(couponPay == 0, MoneyUtil().getMoneyText(couponPay.toString()))

            val wkCoinPay = bill.wkcoin_pay?.toInt() ?: 0
            view.setWkCoinAmount(wkCoinPay == 0, MoneyUtil().getMoneyText(wkCoinPay.toString()))

            val oilPay = bill.oil_pay?.toInt() ?: 0
            view.setOildPayAmount(oilPay == 0, MoneyUtil().getMoneyText(oilPay.toString()))

            val deduct = couponPay + wkCoinPay + oilPay
            view.setDeductAmount(deduct == 0, MoneyUtil().getMoneyText(deduct.toString()))

        }
    }


    private var accept_more_pay_amount:String? = null
    override fun getCalcInfo(result: CalcResult) {
        postApplyRequest = RefundTheCarActivity.buildRequest(postApplyRequest, result)
        calcInfo = result
        otherAmount = result.other_amount
        accept_more_pay_amount = result.accept_more_pay_amount?:""
//        fuelAmount = result.fuel_amount
//        penaltyAmount = result.penalty_amount
        view.setCostCalcInfo(result)

        if (canPageNext) {
            canPageNext = false
            if (orderInfo?.e_contract_status == 1 && orderInfo?.is_e_contract == 1) {
                LiveDataBus.get().with(ConstantsUtil.REFUND_CONTRACT_PRE).postValue(true)
            }
            LiveDataBus.get().with("ThirdFragment").postValue(postApplyRequest)
            view.setNextPage(orderInfo?.e_contract_status == 1 && orderInfo?.is_e_contract == 1)
        }
    }

    override fun getApplyRequest(request: BillApplyRequest) {
        postApplyRequest = request
        LogUtil.e("ApplyRequest --->>> $request")
    }

    override fun requestOrderCalc() {
        val request = CalcRequest()
        request.type = "APPLY"
        request.channel = ChannelUtil.getAppChannel(context)
        request.orderid = postApplyRequest?.orderid
        request.real_start_time = postApplyRequest?.real_start_time
        request.real_end_time = postApplyRequest?.real_end_time
        request.fuel = postApplyRequest?.fuel
        request.miles = postApplyRequest?.miles
        request.send_service = orderInfo?.send_service.toString()
        request.return_service = orderInfo?.return_service.toString()
//        request.fuel_amount = fuelAmount ?: "0"
//        request.penalty_amount = penaltyAmount ?: "0"
        request.other_amount = otherAmount ?: "0"

        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.orderCalc(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<CalcResult>(context, this) {
            override fun onSuccess(response: CalcResult) {
                view.dismissLoadingDialog()
                getCalcInfo(response)

            }
        })
    }

    override fun getAmount(other: String) {

        otherAmount = other
        requestOrderCalc()
    }

    override fun requestBillApply() {

        if (postApplyRequest == null)
            return
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(postApplyRequest)
        val observable = service.orderApply(params).debounce(1000, TimeUnit.MILLISECONDS)

        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view?.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult) {
                view?.dismissLoadingDialog()
                view.setApplySuccess()
            }
        })
    }

    override fun onNextPage() {
        if (!TextUtils.isEmpty(accept_more_pay_amount)){
            IOSAlertDialog(context)
                    .builder()
                    .setCancelable(false)
                    .setTitle("结算提醒")
                    .setMsg("此订单结算时还需客户支付${accept_more_pay_amount}元，为避免造成经济损失，请尽快与客户确认结算信息，完成订单结算！", true)
                    .setRightButton("我知道了") {
                        doApply()
                    }
                    .show()
        }  else{
            doApply()
        }
    }

    private fun doApply(){
        canPageNext = true
        requestBillApply()
    }


    private fun initTimeLine(result: OrderDetailResult) {
        if (result.e_contract_status == 1 && result.is_e_contract == 1) {
            view.setTimeLine(TakeAlsoCarTimeLine.REFUND_CONTRACT, TakeAlsoCarTimeLine.INDEX_TWO)
        } else {
            view.setTimeLine(TakeAlsoCarTimeLine.REFUND_NORMAL, TakeAlsoCarTimeLine.INDEX_TWO)
        }
    }
}