package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.search

import android.content.Intent
import android.view.View
import com.jiaoyinbrother.library.bean.OrdersBean
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.widget.GeneralBottomButton
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.adapter.OrderListAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.detail.OrderDetailActivity
import kotlinx.android.synthetic.main.activity_order_search.*

class OrderSearchActivity : MvpBaseActivity<OrderSearchPresenter>(), IOrderSearchContract.View {

    private lateinit var mAdapter: OrderListAdapter

    override fun initPresenter(): OrderSearchPresenter = OrderSearchPresenter(this, this)

    override fun getLayoutId(): Int = R.layout.activity_order_search

    override fun initView() {
        mAdapter = OrderListAdapter(this)
        orders_rv?.adapter = mAdapter
        search_result_loading?.setEmptyText("未查询到相关订单。请重新核对!")
        search_result_loading?.setErrorText("无法获取数据，可尝试检查网络...")
        search_result_loading?.setRetryText("点击重试")
    }

    override fun initToolBar() {
        setPageTitle(false)
    }

    override fun initData() {
    }

    override fun initListeners() {
        // 下拉刷新
        search_result_refresh?.setOnRefreshListener {
            presenter.onRefresh()
        }
        // 上拉加载
        search_result_refresh?.setOnLoadMoreListener {
            presenter.onLoadMore()
        }

        mAdapter.setOrderItemListener(object : OrderListAdapter.OnOrderItemListener {
            override fun onItemClick(type: String, order: OrdersBean) {
                presenter.getInfo(type, order)
            }
        })

        onSearch.setListener(object : GeneralBottomButton.SubmitListener {
            override fun submit() {
                presenter.onSearch()
            }
        })
    }

    override fun finishRefreshWithNoMoreData() {
        search_result_refresh?.finishRefreshWithNoMoreData()
    }

    override fun finishLoadMoreWithNoMoreData() {
        search_result_refresh?.finishLoadMoreWithNoMoreData()
    }

    override fun finishRefresh() {
        search_result_refresh?.finishRefresh()
    }

    override fun finishLoadMore() {
        search_result_refresh?.finishLoadMore()
    }

    override fun setEnableLoadMore(b: Boolean) {
        search_result_refresh?.setEnableLoadMore(b)
    }

    override fun showNoDataView() {
        showList(true)
        search_result_loading?.showEmpty()
        setEnableLoadMore(false)
    }

    override fun showNetErrView(b: Boolean) {
        search_result_loading?.showError()
        setEnableLoadMore(false)
    }

    override fun setOrdersData(data: ArrayList<OrdersBean>) {
        showList(true)
        mAdapter.list = data
        mAdapter.notifyDataSetChanged()
        search_result_loading?.showContent()
    }

    override fun onOrderInfo(orderId: String) {
        startActivity(Intent().apply {
            setClass(this@OrderSearchActivity, OrderDetailActivity::class.java)
            putExtra(ConstantsUtil.ORDER_ID, orderId)
        })
    }

    private fun showList(isList: Boolean) {
        if (isList) {
            orderSearchLayout.visibility = View.GONE
            search_result_refresh.visibility = View.VISIBLE
        } else {
            orderSearchLayout.visibility = View.VISIBLE
            search_result_refresh.visibility = View.GONE
        }
        setPageTitle(isList)
    }

    override fun onBackPressed() {
        if (orderSearchLayout.visibility == View.VISIBLE) {
            super.onBackPressed()
        } else {
            showList(false)
        }
    }

    private fun setPageTitle(b: Boolean) {
        mainTitle?.text = if (b) "订单列表" else "订单查询"
    }


    override fun getSearchInfo(): String {
        return searchInfo?.text?.toString() ?: ""
    }
}