package com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.search

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.TextUtil
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.monkeyking.manager.adapter.OrderListAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.dropinthecar.DropInTheCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.delivercar.DeliverCarActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.refundcar.RefundTheCarActivity
import com.jiaoyinbrother.monkeyking.manager.util.NetUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit
import java.util.regex.Pattern

class OrderSearchPresenter(context: Context, view: IOrderSearchContract.View) :
        BasePresenterImpl<IOrderSearchContract.View>(context, view), IOrderSearchContract.Presenter {


    private val mOrdersData = ArrayList<OrdersBean>()
    private var mPage = 1

    /**
     * 刷新数据
     */
    override fun onRefresh() {
        view?.setEnableLoadMore(true)
        mPage = 1
        onSearchPhone()
    }

    /**
     * 加载更多
     */
    override fun onLoadMore() {
        onSearchPhone()
    }

    override fun onSearch() {

        val info = view.getSearchInfo()
        if (!NetUtil.isNetworkAvailable(context)) {
            view.showToast("无网络")
            return
        }
        if (TextUtils.isEmpty(info)) {
            view.showToast("请输入正确的订单号/手机号/姓名")
            return
        }
        if (isNumeric(info)){
            if (info.length < 8) {
                view.showToast("请输入正确的订单号/手机号/姓名")
                return
            }
            if (info.length in 9..10) {
                view.showToast("请输入正确的订单号/手机号/姓名")
                return
            }
            if (info.length in 12..15) {
                view.showToast("请输入正确的订单号/手机号/姓名")
                return
            }
            val patter = TextUtil.phonePatter
            if (!TextUtil.matchers_input(patter, info) && info.length == 11) {
                view.showToast("请输入正确的订单号/手机号/姓名")
                return
            }
            onRefresh()
        }else{
            onRefresh()
        }

    }

    private fun onSearchPhone() {
        val request = OrdersRequest()
        request.manager_id = UserUtil(context).userId.toString()
        request.key_word = view.getSearchInfo()
        request.page = mPage
        request.page_size = 30
        request.status = 4
        request.status_list = ArrayList(listOf("ORDER_ILLEGAL_PAYED", "ORDER_EXECUTING", "ORDER_BILL_APPLIED", "ORDER_BILL_ACCEPTED", "ORDER_FINISTHED","ORDER_CANCELED"))
        val service = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getOrders(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<OrdersResult>(context, this) {
            override fun onSuccess(response: OrdersResult) {
                view.dismissLoadingDialog()
                if(request.key_word ==null || response.orders ==null)
                    return
                if ((request.key_word?.length == 8 ||request.key_word?.length == 16) && (response.orders?.size!! >= 1)) {
                    view.onOrderInfo(response.orders?.get(0)?.order_id?:"")
                } else {
                    if (mPage == 1) {
                        handleRefreshData(response)
                    } else {
                        handleLoadMoreData(response)
                    }
                }
            }

            override fun onAPIError(apiException: ApiException?) {
                view.dismissLoadingDialog()
                view.showNetErrView(mOrdersData.size == 0)
            }

            override fun onError(e: Throwable) {
                view.dismissLoadingDialog()
                view.showNetErrView(mOrdersData.size == 0)
            }
        })
    }

    private fun handleRefreshData(response: OrdersResult) {
        val dataList = response.orders
        if (dataList != null && dataList.isNotEmpty()) {
            //刷新成功，有数据
            mOrdersData.clear()
            mOrdersData.addAll(dataList)
            view.setOrdersData(mOrdersData)
            mPage++
            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //刷新成功，数据小于PAGE_SIZE，一页
                view.finishRefreshWithNoMoreData()
            } else {
                //刷新成功，一整页
                view.finishRefresh()
            }
        } else {
            //刷新没有数据，必须显示no data view
            view.finishRefresh()
            view.showNoDataView()
        }
    }

    private fun handleLoadMoreData(response: OrdersResult) {
        val dataList = response.orders
        if (dataList != null && dataList.isNotEmpty()) {
            mOrdersData.addAll(dataList)
            view.setOrdersData(mOrdersData)
            mPage++
            if (dataList.size < ConstantsUtil.PAGE_SIZE) {
                //加载更多，数据小于PAGE_SIZE，一页
                view.finishLoadMoreWithNoMoreData()
            } else {
                //加载更多，一整页
                view.finishLoadMore()
            }
        } else {
            //加载更多，没有数据
            view.finishLoadMoreWithNoMoreData()
        }
    }

    override fun getInfo(type: String?, order: OrdersBean?) {
        if (!NetUtil.isNetworkAvailable(context)) {
            view.showToast("无网络")
            return
        }
        onApiDetail(type, order)
    }


    private fun onApiDetail(type: String?, order: OrdersBean?) {
        val request = OrderDetailRequest()
        request.uid = UserUtil(context).uid
        if (order == null){
            request.orderid = view.getSearchInfo()
        } else{
            request.orderid = order.orderid
        }
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<OrderDetailResult>(context, this) {
                    override fun onSuccess(response: OrderDetailResult) {
                        view.dismissLoadingDialog()
                        if (order == null) {
                            view.onOrderInfo(response.orderid?:"")
                        } else {
                            when (type) {
                                OrderListAdapter.ORDER_LIST_TO_DROP_IN -> {
                                    goDropInTheCarActivity(order)
                                }
                                OrderListAdapter.ORDER_LIST_TO_DETAIL -> {
                                    view.onOrderInfo(order.orderid ?: "")
                                }
                                OrderListAdapter.ORDER_LIST_TO_DELIVER -> {
                                    goDeliverCarActivity(order)
                                }
                                OrderListAdapter.ORDER_LIST_TO_ALSO -> {
                                    onConfirmReturnCar(order)
                                }
                            }
                        }
                    }

                    override fun onAPIError(apiException: ApiException?) {
                        view.dismissLoadingDialog()
                        super.onAPIError(apiException)
                    }

                    override fun onError(e: Throwable) {
                        view.dismissLoadingDialog()
                        super.onError(e)
                    }
                })
    }


    private fun goDropInTheCarActivity(order: OrdersBean) {
        val intent = Intent(context, DropInTheCarActivity::class.java)
        intent.putExtra(OrderListAdapter.ORDER_LIST_TO_DROP_IN, order)
        context.startActivity(intent)
    }

    /**
     * 办理取车
     */
    private fun goDeliverCarActivity(order: OrdersBean) {
        val intent = Intent(context, DeliverCarActivity::class.java)
        intent.putExtra(ConstantsUtil.ORDER_ID, order?.orderid)
        context.startActivity(intent)
    }


    /**
     * 办理还车
     */
    fun onConfirmReturnCar(order: OrdersBean) {
        val intent = Intent(context, RefundTheCarActivity::class.java)
        intent.putExtra(ConstantsUtil.ORDER_ID, order?.orderid)
        context.startActivity(intent)
    }

    private fun isNumeric(str: String): Boolean {

        val pattern = Pattern.compile("[0-9]*")

        return pattern.matcher(str).matches()

    }
}