package com.jiaoyinbrother.monkeyking.mvpactivity.ordercontract

import android.content.Context
import android.content.Intent
import android.view.View
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.BaseResult
import com.jiaoyinbrother.library.bean.ContractListResult
import com.jiaoyinbrother.library.bean.OrdersBean
import com.jiaoyinbrother.library.bean.SendContractEmailRequest
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.ordercontract.OrderContractActivity.Companion.ORDER_CONTRACT_ORDER_ID
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.ordercontract.OrderContractActivity.Companion.ORDER_CONTRACT_STATUS
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.ordercontract.OrderContractActivity.Companion.RENT_CONTRACT
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.ordercontract.OrderContractActivity.Companion.RETURN_CONTRACT
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class OrderContractPresenter(context: Context, view: IOrderContract.IOrderContractView) :
    BasePresenterImpl<IOrderContract.IOrderContractView>(context, view), IOrderContract.IOrderContractPresenter {

    /**
     * 获取邮件类型列表
     */
    override fun getEmailList() {

        val request = SendContractEmailRequest()
        request.order_id = mOrderId

        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getContractList(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<ContractListResult>(context, this) {
                    override fun onSuccess(response: ContractListResult) {

                        view.onGetEmailListSuccess(response.contract_array)
                    }
                })
    }

    private var mOrderId : String ?= null

    /**
     * 发送邮件接口
     */
    override fun sendEmail(email: String,type: String) {

        val request = SendContractEmailRequest()
        request.contract_type = type
        request.email = email
        request.order_id = mOrderId

        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.sendContractEmail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
        }.observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : BaseObserver<BaseResult>(context, this) {
                override fun onSuccess(response: BaseResult) {
                    view.showToast("电子合同邮件发送成功")
                    view.onSendEmailSuccess()
                }
            })
    }

    override fun sendRentEmail(email: String?) {

        sendEmailType(email,RENT_CONTRACT)
    }

    override fun sendEvidenceEmail(email: String?) {

        sendEmailType(email,RETURN_CONTRACT)
    }

    override fun initData(intent: Intent) {
        val orderStatus = intent.getIntExtra(ORDER_CONTRACT_STATUS, -1)
        mOrderId = intent.getStringExtra(ORDER_CONTRACT_ORDER_ID)

        //获取邮箱列表
        getEmailList()

    }

    fun sendEmailType(email: String?,type: String){

        if (email !=null && email.indexOf("@") != -1) {
            sendEmail(email,type)
        } else {
            view.showToast("请输入正确的邮箱")
        }
    }
}