package com.jiaoyinbrother.monkeyking.manager.mvpactivity.price

import android.app.Activity
import android.arch.lifecycle.Observer
import android.content.Context
import android.content.Intent
import android.view.WindowManager
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.base.BaseFragment
import com.jiaoyinbrother.library.bean.PriceDetailResult
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.fragment.based.PriceBasedFragment
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.fragment.calendar.PriceCalendarFragment
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog


class PriceSetActivity : MvpBaseActivity<PriceSetPresenter>(), IPriceSetContract.View {


    companion object {
        private const val FIRST = 0
        private const val SECOND = 1

        const val CAR_TYPE_ID = "CAR_TYPE_ID"
        const val PRICE_INFO = "PRICE_INFO"
        const val SWITCH_RADIO = "SWITCH_RADIO"
        const val IS_BASED_PRICE_EDITED = "IS_BASED_PRICE_EDITED"
        const val IS_FLOAT_PRICE_EDITED = "IS_FLOAT_PRICE_EDITED"

        const val REFRESH_PRICE = "REFRESH_PRICE"

        fun newInstance(context: Context, siteCarTypeId: Int?) = Intent().apply {
            setClass(context, PriceSetActivity::class.java)
            putExtra(CAR_TYPE_ID, siteCarTypeId)
        }
    }

    private val mFragments = arrayOfNulls<BaseFragment>(2)
    private var isBasedEdited = false
    private var isFloatEdited = false

    override fun initPresenter(): PriceSetPresenter {
        return PriceSetPresenter(this, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_price_set
    }

    override fun initToolBar() {
        mainTitle.text = "调整价格"
    }

    override fun initView() {
        // 初始化fragment
        if (findFragment(PriceCalendarFragment::class.java) == null) {
            mFragments[FIRST] = PriceCalendarFragment.newInstance()
            mFragments[SECOND] = PriceBasedFragment.newInstance()
            loadMultipleRootFragment(R.id.fl_container, FIRST, mFragments[FIRST], mFragments[SECOND])
        } else {
            mFragments[FIRST] = findFragment(PriceCalendarFragment::class.java)
            mFragments[SECOND] = findFragment(PriceBasedFragment::class.java)
        }
    }

    override fun initData() {
        presenter?.getData(intent)

        LiveDataBus.get().with(SWITCH_RADIO, Boolean::class.java).observe(this, Observer {
            it?.let { self -> setSwitchTab(self) }
        })

        LiveDataBus.get().with(IS_BASED_PRICE_EDITED, Boolean::class.java).observe(this, Observer {
            it?.let { self -> isBasedEdited = self }
        })

        LiveDataBus.get().with(IS_FLOAT_PRICE_EDITED, Boolean::class.java).observe(this, Observer {
            it?.let { self -> isFloatEdited = self }
        })

        // 调用API更新价格日历
        LiveDataBus.get().with(REFRESH_PRICE, Boolean::class.java).observe(this, Observer {
            it?.let { self ->
                if (self) {
                    presenter.requestPrice()
                }
            }
        })
    }

    override fun initListeners() {

    }


    override fun setPriceInfo(result: PriceDetailResult) {
        LiveDataBus.get().with(PRICE_INFO).setValue(result)
    }

    /**
     * 如果返回至工单列表页，工单列表页数据刷新，显示最新的待处理的工单列表
     */
    override fun setResultWorkFlow() {
        setResult(Activity.RESULT_OK)
    }


    //    【1】返回：按钮，点击返回上一级
//    DONE 如果返回至车型价格列表页，页面不需要刷新，显示当前位置
//    DONE 如果返回至工单列表页，工单列表页数据刷新，显示最新的待处理的工单列表
//    校验是否存在未保存的设置（日历价格&基础价格），如果有，提示用户，见左图（点击离开，不执行也不保存本次所有设置，返回至上级页面；点击不离开，停留在调价页面，不执行任何操作）
//    DONE 从调整价格页面退出后，未提交的设置清空即可
    override fun onBackPressed() {
        if (isBasedEdited || isFloatEdited) {
            IOSAlertDialog(this)
                    .builder()
                    .setTitle("温馨提示")
                    .setMsg("存在未提交的修改内容，请确认是否仍要离开", true)
                    .setRightButton("不离开") {
                    }.setLeftButton("离开") {
                        setResultWorkFlow()
                        super.onBackPressed()
                    }.show()
        } else {
            setResultWorkFlow()
            super.onBackPressed()
        }
    }

    override fun setSwitchTab(switch: Boolean) {
        if (switch) {
            showHideFragment(mFragments[FIRST], mFragments[SECOND])
        } else {
            showHideFragment(mFragments[SECOND], mFragments[FIRST])
        }
    }

}