package com.jiaoyinbrother.monkeyking.manager.mvpactivity.price

import android.content.Context
import android.content.Intent
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.PriceDetailRequest
import com.jiaoyinbrother.library.bean.PriceDetailResult
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.text.SimpleDateFormat
import java.util.*
import java.util.concurrent.TimeUnit

class PriceSetPresenter(context: Context, view: IPriceSetContract.View) : BasePresenterImpl<IPriceSetContract.View>(context, view),
        IPriceSetContract.Presenter {

    private var siteCarTypeID: Int = 0

    override fun getData(intent: Intent) {
        siteCarTypeID = intent.getIntExtra(PriceSetActivity.CAR_TYPE_ID, 0)

        requestPrice()
    }

    override fun requestPrice() {
        val request = PriceDetailRequest()
        request.id = siteCarTypeID
        request.car_price_date = getToadyDate()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.apiPriceInfo(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<PriceDetailResult>(context, this) {
            override fun onSuccess(response: PriceDetailResult) {
                view.dismissLoadingDialog()
                view.setPriceInfo(response)
            }

            override fun onAPIError(apiException: ApiException?) {
                view.dismissLoadingDialog()
            }

            override fun onError(e: Throwable) {
                view.dismissLoadingDialog()
            }
        })
    }

    /**
     * 获取当前日期。
     *
     * @return yyyy-MM-dd
     */
    private fun getToadyDate(): String? {
        return try {
            SimpleDateFormat("yyyy-MM-dd").format(Date())
        } catch (e: Exception) {
            ""
        }
    }


}