package com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.batch

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.text.TextUtils
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.library.bean.PriceBatchItemBean
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpdialog.selectdate.SelectDateMVPDialog
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import kotlinx.android.synthetic.main.activity_price_batch.*
import java.lang.StringBuilder
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

/**
 * @author zhangbo
 * @date 2020/7/24
 * Email : b.zhang@wkzuche.com
 * Describe: 批量操作页面
 */
class PriceBatchActivity : MvpBaseActivity<PriceBatchPresenter>(), IPriceBatchContract.View {

    private lateinit var mAdapter: PriceBatchAdapter
    private lateinit var list: ArrayList<PriceBatchItemBean>

    companion object {

        const val AC_PRICE_BATCH_LIST = "AC_PRICE_BATCH_LIST"
        const val AC_PRICE_BATCH_DONE = "AC_PRICE_BATCH_DONE"

        fun newInstance(context: Context) {

            context.startActivity(Intent(context, PriceBatchActivity::class.java))
        }
    }

    override fun initListeners() {

        ac_price_batch_save_tv.setOnClickListener {

            //是否全部数据正确
            var isSuccess = true

            for (index in 0 until list.size - 1) {

                val item = list[index]

                LogUtil.e("item->$item")

                if (item.price < 1) {

                    item.isPriceError = true
                    isSuccess = isSuccess && false
                } else {
                    isSuccess = isSuccess && true
                    item.isPriceError = false
                }

                if (TextUtils.isEmpty(item.startTime) || TextUtils.isEmpty(item.endTime)) {

                    item.isTimeError = true
                    isSuccess = isSuccess && false
                } else {
                    isSuccess = isSuccess && true
                    item.isTimeError = false
                }

            }

            if (!isSuccess) {

                showToast("请填写价格和时间")
                mAdapter.notifyDataSetChanged()
                return@setOnClickListener
            }

            val mToastBuilder = StringBuilder()
            for (index in list.indices - 1) {

                val bean1 = list[index]

                bean1.isPriceError = bean1.price < 1

                for (index2 in index + 1 until list.size - 1) {

                    val bean2 = list[index2]

                    bean2.isPriceError = bean2.price < 1

                    //判断时间重合关系

                    LogUtil.e("time-->" + bean1.getStartTimeStamp() + "---" + bean2.getEndTimeStamp() + "---" + bean1.getEndTimeStamp() + "---" + bean2.getStartTimeStamp())

                    if (bean1.getEndTimeStamp() < bean2.getStartTimeStamp() || bean2.getEndTimeStamp() < bean1.getStartTimeStamp()) {

                        bean1.isTimeError = false
                        bean2.isTimeError = false
                    } else {

                        bean1.isTimeError = true
                        bean2.isTimeError = true
                        mToastBuilder.append("时间段【${bean1.startTime}到${bean1.endTime}】和时间段【${bean2.startTime}到${bean2.endTime}】重叠")
                    }
                }
            }

            if (mToastBuilder.isNotEmpty()) {

                isSuccess = isSuccess && false
                showToast(mToastBuilder.toString())
            }

            if (isSuccess) {

                //需要移除最后一个数据（为添加模块）
                list.removeAt(list.size - 1)
                val intent = Intent()
                intent.putExtra(AC_PRICE_BATCH_LIST, list)
                intent.putExtra(AC_PRICE_BATCH_DONE, true)
                setResult(Activity.RESULT_OK, intent)
                finish()
            } else {

                mAdapter.notifyDataSetChanged()
            }

            LogUtil.e(list.toString())
        }

        mAdapter.setChangeListListener(object : PriceBatchAdapter.ChangeListListener {
            override fun delItem(position: Int) {

                list.removeAt(position)

                mAdapter.list = list
                mAdapter.notifyDataSetChanged()
            }

            override fun addItem() {

                list.add(list.size - 1, PriceBatchItemBean())

                mAdapter.list = list

                LogUtil.e(list.toString())

                mAdapter.notifyItemChanged(list.size - 2)

                ac_price_batch_rv.smoothScrollToPosition(list.size - 1)

            }

            override fun changeItemTime(position: Int, timeTab: Int, startTime: String?, endTime: String?) {

                SelectDateMVPDialog.newMainDialog(
                        startTime,
                        endTime,
                        timeTab,
                        object : SelectDateMVPDialog.OnResultListener {
                            override fun onConfirm(startDate: Date?, endDate: Date?) {

                                val simpleDateFormat = SimpleDateFormat("yyyy-MM-dd", Locale.CHINA)
                                val startDateString = simpleDateFormat.format(startDate)
                                val endDateString = simpleDateFormat.format(endDate)

                                list[position].startTime = startDateString

                                list[position].endTime = endDateString

                                list[position].isTimeError = false

                                mAdapter.list = list
                                mAdapter.notifyItemChanged(position)
                            }

                        },
                        supportFragmentManager)
            }

        })
    }

    override fun initPresenter(): PriceBatchPresenter {
        return PriceBatchPresenter(this, this)
    }

    override fun initToolBar() {

        mainTitle.text = "批量修改"
    }

    override fun initData() {
        mAdapter = PriceBatchAdapter(this)

        ac_price_batch_rv.adapter = mAdapter

        presenter.initIntent(intent)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_price_batch
    }

    override fun initView() {
    }

    override fun onBackPressed() {
//        super.onBackPressed()

        if (!presenter.isSame()){

            IOSAlertDialog(this)
                    .builder()
                    .setTitle("温馨提示")
                    .setMsg("存在未保存的设置，请确认是否仍要离开", true)
                    .setRightButton("不离开") {
                    }
                    .setLeftButton("离开") {

                        list.removeAt(list.size - 1)
                        val intent = Intent()
                        intent.putExtra(AC_PRICE_BATCH_LIST, list)
                        intent.putExtra(AC_PRICE_BATCH_DONE, false)
                        setResult(Activity.RESULT_OK, intent)

                        super.onBackPressed()
                    }
                    .show()
        }else {

            super.onBackPressed()
        }
    }

    override fun setList(list: ArrayList<PriceBatchItemBean>) {

        this.list = list

        mAdapter.list = list
        mAdapter.notifyDataSetChanged()
    }
}