package com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.batch

import android.content.Context
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.view.View
import android.widget.EditText
import android.widget.LinearLayout
import android.widget.TextView
import com.camnter.easyrecyclerview.adapter.EasyRecyclerViewAdapter
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.library.util.TextUtil
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.library.bean.PriceBatchItemBean
import com.jiaoyinbrother.monkeyking.manager.mvpdialog.selectdate.SelectDateMVPDialog
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList
import java.lang.ref.WeakReference as WeakReference


/**
 * @author zhangbo
 * @date 2020/7/24
 * Email : b.zhang@wkzuche.com
 * Describe: 批量修改适配器
 */
class PriceBatchAdapter(val context: Context) : EasyRecyclerViewAdapter() {

    private var mChangeListListener: ChangeListListener? = null

    override fun getRecycleViewItemType(position: Int): Int {

        return if (position == listSize - 1) 1 else 0
    }

    override fun onBindRecycleViewHolder(viewHolder: EasyRecyclerViewHolder?, position: Int) {


        if (getRecycleViewItemType(position) == 1) {

            onViewHolderFooter(viewHolder, position)
        } else {

            onViewHolderItem(viewHolder, position)
        }
    }

    /**
     * 内容条目ViewHolder
     */
    private fun onViewHolderItem(viewHolder: EasyRecyclerViewHolder?, position: Int) {

        lateinit var mTvDel: TextView

        lateinit var mEtPrice: EditText

        lateinit var mEtStartTime: EditText

        lateinit var mEtEndTime: EditText

        lateinit var mLlPrice: LinearLayout

        viewHolder?.apply {

            mTvDel = findViewById(R.id.item_price_batch_del_tv)

            mEtPrice = findViewById(R.id.item_price_batch_price_et)

            mEtStartTime = findViewById(R.id.item_price_batch_time_start_et)

            mEtEndTime = findViewById(R.id.item_price_batch_time_end_et)

            mLlPrice = findViewById(R.id.item_price_batch_price_ll)
        }

        val bean = list[position] as PriceBatchItemBean

        val price = bean.price

        //时间错误
        if (bean.isTimeError) {

            mEtStartTime.background = context.getDrawable(R.drawable.bg_item_price_batch_et_error)
            mEtEndTime.background = context.getDrawable(R.drawable.bg_item_price_batch_et_error)
        } else {

            mEtStartTime.background = context.getDrawable(R.drawable.bg_item_price_batch_et)
            mEtEndTime.background = context.getDrawable(R.drawable.bg_item_price_batch_et)
        }

        //价格错误
        if (bean.isPriceError) {

            mLlPrice.background = context.getDrawable(R.drawable.bg_item_price_batch_et_error)
        } else {

            mLlPrice.background = context.getDrawable(R.drawable.bg_item_price_batch_et)
        }

        if (price > 0) {

            mEtPrice.setText(price.toString())
        } else {

            mEtPrice.setText("")
        }

        if (position == 0) mTvDel.visibility = View.GONE else mTvDel.visibility = View.VISIBLE

        mTvDel.setOnClickListener {

            mChangeListListener?.delItem(position)
        }

        mEtStartTime.setText(bean.startTime)

        mEtEndTime.setText(bean.endTime)

        mEtStartTime.setOnClickListener {

            var strEndTime = mEtEndTime.text.toString()

            mChangeListListener?.changeItemTime(position, SelectDateMVPDialog.SELECT_DATE_TAB_START, mEtStartTime.text.toString(), getEndTimeStr(strEndTime))
        }

        mEtEndTime.setOnClickListener {

            var strEndTime = mEtEndTime.text.toString()

            mChangeListListener?.changeItemTime(position, SelectDateMVPDialog.SELECT_DATE_TAB_END, mEtStartTime.text.toString(), getEndTimeStr(strEndTime))
        }

        mEtPrice.setSelection(mEtPrice.text.length)

        mEtPrice.addTextChangedListener(MyPriceTextChangedListener(context, viewHolder!!, list as ArrayList<PriceBatchItemBean>))
    }

    private fun getEndTimeStr(strEndTime: String): String {

        return if (TextUtils.isEmpty(strEndTime)) {
            val simpleDateFormat = SimpleDateFormat("yyyy-MM-dd", Locale.CHINA)
            simpleDateFormat.format(Date())
        } else {

            strEndTime
        }
    }

    /**
     * 底部添加ViewHolder
     */
    private fun onViewHolderFooter(viewHolder: EasyRecyclerViewHolder?, position: Int) {

        lateinit var mTvAdd: TextView

        viewHolder?.apply {

            mTvAdd = findViewById(R.id.item_price_batch_footer_add_tv)
        }

        mTvAdd.setOnClickListener {

            mChangeListListener?.addItem()
        }
    }

    override fun getItemLayouts(): IntArray {
        return intArrayOf(R.layout.item_price_batch, R.layout.item_price_batch_footer)
    }

    fun setChangeListListener(mChangeListListener: ChangeListListener) {
        this.mChangeListListener = mChangeListListener
    }

    interface ChangeListListener {

        fun delItem(position: Int)

        fun addItem()

        fun changeItemTime(position: Int, timeTab: Int,
                           startTime: String?,
                           endTime: String?)
    }

//    private fun delItem(position: Int){
//
//        list.removeAt(position)
//        notifyDataSetChanged()
//    }
//
//    private fun addItem(bean: PriceBatchItemBean){
//
//        list.add(listSize - 1,bean)
//        notifyDataSetChanged()
//    }

    private class MyPriceTextChangedListener : TextWatcher {

        private var mList: ArrayList<PriceBatchItemBean>

        private var viewHolder: EasyRecyclerViewHolder

        private var context: WeakReference<Context>

        constructor(context: Context, viewHolder: EasyRecyclerViewHolder, mList: ArrayList<PriceBatchItemBean>) {
            this.mList = mList
            this.viewHolder = viewHolder
            this.context = WeakReference(context)
        }


        override fun afterTextChanged(s: Editable?) {

            val mPosition = viewHolder.adapterPosition

            if (mPosition > -1) {
                val content = s.toString()
                if (content.isNotEmpty()) {

                    viewHolder.findViewById<LinearLayout>(R.id.item_price_batch_price_ll).background = context.get()?.getDrawable(R.drawable.bg_item_price_batch_et)
                    mList[mPosition].price = s.toString().toInt()
                } else {

                    mList[mPosition].price = 0
                }
            }
        }

        override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {

        }

        override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
            if (TextUtil.isZeroBegin(s.toString())) {
                viewHolder.findViewById<EditText>(R.id.item_price_batch_price_et).setText("")
            }
        }

    }
}