package com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.fragment.based

import android.arch.lifecycle.Observer
import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.text.Editable
import android.text.method.LinkMovementMethod
import android.view.View
import android.view.WindowManager
import android.view.inputmethod.EditorInfo
import com.bumptech.glide.Glide
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.bean.PriceDetailResult
import com.jiaoyinbrother.library.listeners.SpannableClickListener
import com.jiaoyinbrother.library.util.TextUtil
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseFragment
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.PriceSetActivity
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.monkeyking.manager.widget.EditRow
import kotlinx.android.synthetic.main.fragment_based_price.*

class PriceBasedFragment : MvpBaseFragment<PriceBasedPresenter>(), IPriceBasedContract.View {


    companion object {
        fun newInstance(): PriceBasedFragment {
            return PriceBasedFragment()
        }
    }

    override fun initPresenter(): PriceBasedPresenter {
        return PriceBasedPresenter(requireContext(), this)
    }

    override fun getLayoutId(): Int {
        return R.layout.fragment_based_price
    }

    override fun initToolbar(savedInstanceState: Bundle?) {}

    override fun initViews(view: View?, savedInstanceState: Bundle?) {}

    override fun initData() {
        on_submit_based?.disable()

        LiveDataBus.get().with(PriceSetActivity.PRICE_INFO, PriceDetailResult::class.java).observe(this, Observer {
            presenter.getData(it)
        })

        // 车型押金
        fg_based_car_deposit?.setAction(EditorInfo.IME_ACTION_NEXT)
        fg_based_car_deposit?.setInputType(EditorInfo.TYPE_CLASS_NUMBER)
        fg_based_car_deposit?.setMaxLength(5)
        // 违章押金
        fg_based_illegal_deposit?.setAction(EditorInfo.IME_ACTION_NEXT)
        fg_based_illegal_deposit?.setInputType(EditorInfo.TYPE_CLASS_NUMBER)
        fg_based_illegal_deposit?.setMaxLength(4)
        // 日租金
        fg_based_rent_day?.setAction(EditorInfo.IME_ACTION_DONE)
        fg_based_rent_day?.setInputType(EditorInfo.TYPE_CLASS_NUMBER)
        fg_based_rent_day?.setMaxLength(5)

        val tips = fg_based_tips?.text.toString()
        fg_based_tips?.text = TextUtil.setSpannableClick(
                tips, tips.length - 4, tips.length, ContextCompat.getColor(mActivity, R.color.color_blue_f9),
                object : SpannableClickListener {
                    override fun onClick() {
                        presenter.onTipsDialog()
                    }
                })
        fg_based_tips?.movementMethod = LinkMovementMethod.getInstance()
    }

    override fun onHiddenChanged(hidden: Boolean) {
        super.onHiddenChanged(hidden)
        if (hidden) {
            viewClearFocus()
            // 如果键盘显示 则隐藏
            hideSoftInput()
        }
    }

    override fun initListeners() {
        fg_based_car_deposit?.addTextChangedListener(MyRowTextWatcher(fg_based_car_deposit))
        fg_based_illegal_deposit?.addTextChangedListener(MyRowTextWatcher(fg_based_illegal_deposit))
        fg_based_rent_day?.addTextChangedListener(MyRowTextWatcher(fg_based_rent_day))

        rb_calendar_price?.setOnClickListener {
            LiveDataBus.get().with(PriceSetActivity.SWITCH_RADIO).setValue(true)
        }

        fg_based_insurance_day?.setOnClickListener {
            presenter.onChangeRange(PriceBasedPresenter.RANGE_INSURANCE)
        }

        fg_based_poundage?.setOnClickListener {
            presenter.onChangeRange(PriceBasedPresenter.RANGE_POUNDAGE)
        }

        fg_based_car_service?.setOnClickListener {
            presenter.onChangeRange(PriceBasedPresenter.RANGE_ADDITIONAL)
        }

        on_submit_based?.setOnClickListener {
            presenter.checkBasedPrice()
        }
    }

    override fun setCarImage(image: String?) {
        fg_based_car_pic?.let {
            Glide.with(requireContext())
                    .load(image)
                    .placeholder(R.mipmap.bg_place_image)
                    .error(R.mipmap.bg_place_image)
                    .into(it)
        }
    }

    override fun setCarType(type: String?) {
        fg_based_car_type?.text = type
    }

    override fun setCarInfo(info: String?) {
        fg_based_car_info?.text = info
    }

    override fun setCarDeposit(amount: String) {
        fg_based_car_deposit?.setContent(amount)
        fg_based_car_deposit?.setSelection(amount.length)
    }

    override fun setIllegalDeposit(amount: String) {
        fg_based_illegal_deposit?.setContent(amount)
        fg_based_illegal_deposit?.setSelection(amount.length)
    }

    override fun setDayInsurance(amount: String) {
        fg_based_insurance_day?.setContent(amount)
    }

    override fun setHourInsurance(amount: String) {
        fg_based_insurance_hour?.setContent(amount)
    }

    override fun setPoundage(amount: String) {
        fg_based_poundage?.setContent(amount)
    }

    override fun setCarService(amount: String) {
        fg_based_car_service?.setContent(amount)
    }

    override fun setDayRent(amount: String) {
        fg_based_rent_day?.setContent(amount)
        fg_based_rent_day?.setSelection(amount.length)
    }

    override fun setHourRent(amount: String) {
        fg_based_rent_hour?.setContent(amount)
    }

    override fun setShowAlert(title: String, content: String) {
        IOSAlertDialog(requireContext())
                .builder()
                .setTitle(title)
                .setMsg(content, true)
                .setRightButton("执行调价") {
                    presenter.sureBasedPrice()
                }.setLeftButton("取消调价") {
                }.show()
    }

    override fun setUpdateDone() {
        IOSAlertDialog(requireContext())
                .builder()
                .setCancelable(false)
                .setMsg("设置成功", true)
                .setRightButton("确定") {
                    viewClearFocus()
                    LiveDataBus.get().with(PriceSetActivity.REFRESH_PRICE).setValue(true)
                    setActiveSubmit(false)
                }.show()
    }

    override fun getDepositCar(): String? = fg_based_car_deposit?.getText()

    override fun getDepositIllegal(): String? = fg_based_illegal_deposit?.getText()

    override fun getInsuranceDay(): String? = fg_based_insurance_day?.getText()

    override fun getInsuranceHour(): String? = fg_based_insurance_hour?.getText()

    override fun getPoundage(): String? = fg_based_poundage?.getText()

    override fun getAdditional(): String? = fg_based_car_service?.getText()

    override fun getRentDay(): String? = fg_based_rent_day?.getText()

    override fun getRentHour(): String? = fg_based_rent_hour?.getText()

    override fun setFirstAfter() {
        presenter.setEditState(false)
        on_submit_based?.disable()
    }

    override fun setActiveSubmit(b: Boolean) {
        if (b) on_submit_based?.normal() else on_submit_based?.disable()
    }

    private fun observationEditText(editRow: EditRow) {
        // 联动
        when (editRow) {
            // 联动时租价
            fg_based_rent_day -> {
                presenter.onChangeRentDay(fg_based_rent_day?.getText())
            }
            // 啥也不用做
            else -> {
            }
        }
        presenter.checkModify()
    }

    private fun viewClearFocus(){
        fg_based_car_deposit?.clearFocus()
        fg_based_illegal_deposit?.clearFocus()
        fg_based_rent_day?.clearFocus()
    }

    /**
     * EditText 监听事件
     *
     * @param editRow
     */
    inner class MyRowTextWatcher(private val editRow: EditRow) : EditRow.RowTextWatcher {
        override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
            if (TextUtil.isZeroBegin(s.toString())) {
                editRow.setContent("")
            }
        }

        override fun afterTextChanged(s: Editable?) {
            observationEditText(editRow)
        }

        override fun onFocusChange(v: View?, hasFocus: Boolean) {
//            requireActivity().window.setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE)
        }
    }

}