package com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.fragment.based

import android.content.Context
import android.support.v7.app.AppCompatActivity
import android.text.TextUtils
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.PriceDetailResult
import com.jiaoyinbrother.library.bean.PriceUpdateRequest
import com.jiaoyinbrother.library.bean.PriceUpdateResult
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.MoneyUtil
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.PriceSetActivity
import com.jiaoyinbrother.monkeyking.manager.mvpdialog.selectobject.SelectAnyDialog
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.monkeyking.manager.view.date.SelectDialog
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class PriceBasedPresenter(context: Context, view: IPriceBasedContract.View) : BasePresenterImpl<IPriceBasedContract.View>(context, view),
        IPriceBasedContract.Presenter {


    companion object {
        const val RANGE_INSURANCE = "basic_insurance"
        const val RANGE_POUNDAGE = "poundage_insurance"
        const val RANGE_ADDITIONAL = "additional_insurance"
    }

    override var siteCarTypeId: Int = 0
    override var limitCheck: Int = PriceUpdateRequest.CHECK
    private var mInsuranceData: ArrayList<Int>? = null
    private var mPoundageData: ArrayList<Int>? = null
    private var mAdditionalData: ArrayList<Int>? = null
    private var maximum = 0
    private var minimum = 0
    private var mRuleDescription: String? = null

    override fun getData(data: PriceDetailResult?) {
        if (data == null)
            return

        siteCarTypeId = data.id ?: 0
        mRuleDescription = data.rule_description
        view.setCarImage(data.pic1_url)
        view.setCarType(data.car_type_name)
        view.setCarInfo(data.base_config)


        view.setCarDeposit(if (data.car_deposit ?: 0f > 0) MoneyUtil().getAmount(data.car_deposit.toString()) else "")
        view.setIllegalDeposit(if (data.illegal_deposit ?: 0f > 0) MoneyUtil().getAmount(data.illegal_deposit.toString()) else "")

        data.insurance?.let {
            view.setDayInsurance(if (it.basic_insurance ?: 0f > 0) MoneyUtil().getAmount(it.basic_insurance.toString()) else "")

            view.setHourInsurance(if (it.hour_insurance ?: 0f > 0) MoneyUtil().getAmount(it.hour_insurance.toString()) else "")

            val poundage = if (it.poundage_amount == null) {
                ""
            } else {
                if (it.poundage_amount ?: 0f > 0f) MoneyUtil().getAmount(it.additional_insurance.toString()) else "0"
            }
            view.setPoundage(poundage)

            view.setCarService(if (it.additional_insurance ?: 0f > 0) MoneyUtil().getAmount(it.additional_insurance.toString()) else "")
        }

        data.prices?.let {
            view.setDayRent(if (it.day_price > 0) MoneyUtil().getAmount(it.day_price.toString()) else "")
            onChangeRentDay(it.day_price.toString())
        }

        data.price_range?.let {
            mInsuranceData = it.basic_insurance_range
            mPoundageData = it.poundage_amount_range
            mAdditionalData = it.additional_insurance_range
            maximum = it.illegal_deposit_range?.maximum ?: 0
            minimum = it.illegal_deposit_range?.minimum ?: 0
        }

        // 只是用来区分首次进来 按钮是灰的
        view.setFirstAfter()
    }

    override fun checkBasedPrice() {
        val depositIllegal = view.getDepositIllegal()?.toInt() ?: 0
        if (depositIllegal !in minimum..maximum) {
            view.showToast("违章押金只能设置${minimum}-${maximum}之间的金额")
            return
        }
        limitCheck = PriceUpdateRequest.CHECK
        updateBasedPrice()
    }

    override fun sureBasedPrice() {
        limitCheck = PriceUpdateRequest.UNCHECK
        updateBasedPrice()
    }

    override fun onChangeRentDay(amount: String?) {
        view.setHourRent(if (TextUtils.isEmpty(amount)) "0" else MoneyUtil().ceilToInt(amount!!.toInt()))
    }

    fun setEditState(b: Boolean) {
        LiveDataBus.get().with(PriceSetActivity.IS_BASED_PRICE_EDITED).setValue(b)
    }

    fun checkModify() {
        if (TextUtils.isEmpty(view.getDepositCar())
                || TextUtils.isEmpty(view.getDepositIllegal())
                || TextUtils.isEmpty(view.getInsuranceDay())
                || TextUtils.isEmpty(view.getInsuranceHour())
                || TextUtils.isEmpty(view.getPoundage())
                || TextUtils.isEmpty(view.getAdditional())
                || TextUtils.isEmpty(view.getRentDay())
                || TextUtils.isEmpty(view.getRentHour())) {
            setEditState(false)
            view.setActiveSubmit(false)
            return
        }
        setEditState(true)
        view.setActiveSubmit(true)
    }

    private fun updateBasedPrice() {
        val request = PriceUpdateRequest()
        request.type = PriceUpdateRequest.BASIC
        request.limit_check = limitCheck
        request.id = siteCarTypeId
        request.car_deposit = view.getDepositCar()
        request.illegal_deposit = view.getDepositIllegal()
        request.basic_insurance = view.getInsuranceDay()
        request.hour_insurance = view.getInsuranceHour()
        request.poundage_amount = view.getPoundage()
        request.additional_insurance = view.getAdditional()
        request.day_price = view.getRentDay()
        request.hour_price = view.getRentHour()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.apiPriceUpdate(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<PriceUpdateResult>(context, this) {
                    override fun onSuccess(response: PriceUpdateResult) {
                        view.dismissLoadingDialog()
                        // 当前type  只有一种。
                        if (response.alert != null && response.alert?.type == ConstantsUtil.LIMIT_CHECK) {
                            view.setShowAlert(response.alert?.title ?: "", response.alert?.content
                                    ?: "")
                        } else {
                            view.setUpdateDone()
                        }
                    }
                })
    }


    override fun onChangeRange(type: String) {
        if (getRangeData(type).isEmpty()) {
            return
        }
        SelectAnyDialog.newDialog(
                (context as AppCompatActivity).supportFragmentManager,
                getRangeTitle(type),
                getRangeDefault(type),
                getRangeData(type),
                object : SelectAnyDialog.OnResultListener {
                    override fun onResult(result: String) {
                        setRangeResult(type, result)
                    }
                })
    }

    override fun onTipsDialog() {
        mRuleDescription?.let {
            IOSAlertDialog(context).builder()
                    .setTitle("规则说明")
                    .setMsg(mRuleDescription, false)
                    .setRightButton("知道了") {}
                    .show()
        }
    }

    private fun getRangeDefault(type: String): String? = when (type) {
        RANGE_INSURANCE -> view.getInsuranceDay()
        RANGE_POUNDAGE -> view.getPoundage()
        RANGE_ADDITIONAL -> view.getAdditional()
        else -> ""
    }

    private fun setRangeResult(type: String, it: String?) {
        when (type) {
            RANGE_INSURANCE -> {
                view.setDayInsurance(it ?: "0")
                view.setHourInsurance(if (TextUtils.isEmpty(it)) "0" else MoneyUtil().ceilToInt(it!!.toInt()))
            }
            RANGE_POUNDAGE -> {
                view.setPoundage(it ?: "0")
            }
            RANGE_ADDITIONAL -> {
                view.setCarService(it ?: "0")
            }
            else -> {
            }
        }
        checkModify()
    }


    private fun getRangeData(type: String): ArrayList<Int> {
        return when (type) {
            RANGE_INSURANCE -> mInsuranceData ?: ArrayList()
            RANGE_POUNDAGE -> mPoundageData ?: ArrayList()
            RANGE_ADDITIONAL -> mAdditionalData ?: ArrayList()
            else -> ArrayList()
        }
    }


    private fun getRangeTitle(type: String): String {
        return when (type) {
            RANGE_INSURANCE -> "日租基本保险"
            RANGE_POUNDAGE -> "手续费"
            RANGE_ADDITIONAL -> "车损无忧服务"
            else -> ""
        }
    }
}
