package com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.fragment.calendar

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.support.v4.app.Fragment
import android.text.TextUtils
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.DateTimeUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.bean.PriceBatchItemBean
import com.jiaoyinbrother.library.util.UserUtil
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.PriceSetActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.batch.PriceBatchActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.holiday.PriceHolidayActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.week.PriceWeekActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.worry.PriceWorryActivity
import com.monkey.datetimelibrary.entity.PriceMonthEntity
import com.monkey.datetimelibrary.util.PriceCalendarUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.lang.Exception
import java.text.SimpleDateFormat
import java.util.*
import java.util.concurrent.TimeUnit
import kotlin.collections.ArrayList

class PriceCalendarPresenter(context: Context, view: IPriceCalendarContract.View) : BasePresenterImpl<IPriceCalendarContract.View>(context, view),
        IPriceCalendarContract.Presenter {

    companion object {
        const val MODIFY_BATCH_CODE = 0x99
        const val MODIFY_WEEK_CODE = 0x98
        const val MODIFY_HOLIDAY_CODE = 0x97
        const val MODIFY_ABNORMAL_CODE = 0x96

        const val NUM_MONTH_OF_CALENDAR = 6

        private const val REFRESH_HOLIDAY = 0
        private const val REFRESH_WEEKDAY = 1
        private const val REFRESH_BATCH = 2
        private const val REFRESH_ABNORMAL = 3
    }

    // 起始年
    private var mStartYear = 0

    // 起始月
    private var mStartMonth = 0

    // 日历数据
    private var mRootData = ArrayList<PriceMonthEntity>()
    private var mTempData = ArrayList<PriceMonthEntity>()
    private var siteCarTypeId: Int = 0
    private var limitCheck = PriceUpdateRequest.CHECK

    // 节假日
    private var mHolidayData: ArrayList<HolidaysBean>? = null

    // 节假日修改
    private var mHolidaysBean: HolidaysBean? = null

    // 异常价格修改
    private var mAbnormalData: ArrayList<OtherBean>? = null
    private var mAbnormalBean: ArrayList<OtherBean>? = null

    // 按周修改修改
    private var mWeekBean: PriceWeekBean? = null

    // 批量修改
    private var mBatchBean: ArrayList<PriceBatchItemBean>? = null

    private var isEditWeekDone = false
    private var isEditBatchDone = false
    private var isEditAbnormalDone = false
    private var isEditHolidaysDone = false

    // 用来存顺序的
    private var status = ArrayList<Int>()

    private var abnormalId = 0


    /**
     * 初始化 N 个月数据
     *
     * @return 月历
     */
    override fun init() {
        val calendar = Calendar.getInstance()
        mStartYear = calendar[Calendar.YEAR]
        mStartMonth = calendar[Calendar.MONTH]
        mRootData.clear()
        mRootData.addAll(PriceCalendarUtil().getCalendarData(mStartYear, mStartMonth, NUM_MONTH_OF_CALENDAR))
        view.setCalendarData(mRootData)
    }


    override fun getData(data: PriceDetailResult?) {

        data?.let {
            siteCarTypeId = it.id ?: 0

            view.setCarImage(it.pic1_url ?: "")

            view.setCarType(it.car_type_name ?: "")

            view.setCarInfo(it.base_config ?: "")

            if (it.abnormal_price_workcard != null) {
                abnormalId = it.abnormal_price_workcard?.id ?: 0
                view.setAbnormalTips(it.abnormal_price_workcard?.desc ?: "")
                view.setAbnormalTipVisible(true)
            } else {
                view.setAbnormalTipVisible(false)
            }

            // 先设置 节假日信息
            PriceCalendarUtil().modifyHolidayData(mRootData, it.holidays)

            it.prices?.let { price ->
                PriceCalendarUtil().setPricesData(mRootData, price.others)

                PriceCalendarUtil().setAbnormalData(mRootData, price.abnormal)

                // 异常价格
                mAbnormalData = PriceCalendarUtil().getAbnormalPriceData(price.others, price.abnormal)

                // 节假日List对象
                mHolidayData = it.holidays
            }

            mTempData.clear()
            mTempData.addAll(mRootData)

            view.setCalendarData(mTempData)
        }

        if (mHolidaysBean != null || (mAbnormalBean != null && mAbnormalBean?.size ?: 0 > 0) || mWeekBean != null || mBatchBean != null) {
            updateCalendarData()
        } else {
            view.setFirstAfter()
        }
    }


    /**
     * 跳转异常价格修改
     *
     * @param fragment
     */
    override fun onAbnormalModify(fragment: Fragment) {
        val intent = Intent(context, PriceWorryActivity::class.java)
        if (mAbnormalBean != null && mAbnormalBean?.size ?: 0 > 0) {
            intent.putExtra(PriceWorryActivity.WORRY_PRICE_LIST_KEY, mAbnormalBean)
        } else {
            intent.putExtra(PriceWorryActivity.WORRY_PRICE_LIST_KEY, mAbnormalData)
        }
        fragment.startActivityForResult(intent, MODIFY_ABNORMAL_CODE)
    }


    /**
     * 跳转批量价格修改
     *
     * @param fragment
     */
    override fun onBatchModify(fragment: Fragment) {
        val intent = Intent(context, PriceBatchActivity::class.java)
        intent.putExtra(PriceBatchActivity.AC_PRICE_BATCH_LIST, mBatchBean)
        fragment.startActivityForResult(intent, MODIFY_BATCH_CODE)
    }


    /**
     * 跳转按周价格修改
     *
     * @param fragment
     */
    override fun onWeekModify(fragment: Fragment) {
        val intent = PriceWeekActivity.newInstance(context, mWeekBean)
        fragment.startActivityForResult(intent, MODIFY_WEEK_CODE)
    }


    /**
     * 跳转节假日价格修改
     *
     * @param fragment
     */
    override fun onHolidayModify(fragment: Fragment) {
        if (mHolidayData != null && mHolidayData?.size ?: 0 > 0) {
            fragment.startActivityForResult(PriceHolidayActivity.newInstance(context, mHolidayData!!, mHolidaysBean), MODIFY_HOLIDAY_CODE)
        } else {
            view.showToast("未来6个月无节假日")
        }
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (data == null)
            return
        when {
            // 异常价格修改
            requestCode == MODIFY_ABNORMAL_CODE && resultCode == Activity.RESULT_OK -> {
                mAbnormalBean = data.getSerializableExtra(PriceWorryActivity.WORRY_PRICE_LIST_KEY) as? ArrayList<OtherBean>
                isEditAbnormalDone = data.getBooleanExtra(PriceWorryActivity.NEED_MODIFY, false)
                if (data.getBooleanExtra(PriceWorryActivity.NEED_GONE, false)) {
                    apiSpiderDeal(siteCarTypeId, "REJECT")
                    return
                }
                if (isEditAbnormalDone) {
                    mAbnormalData = mAbnormalBean
                    modifyStatus(REFRESH_ABNORMAL)
                    updateCalendarData()
                }
            }

            // 批量修改
            requestCode == MODIFY_BATCH_CODE && resultCode == Activity.RESULT_OK -> {
                mBatchBean = data.getSerializableExtra(PriceBatchActivity.AC_PRICE_BATCH_LIST) as? ArrayList<PriceBatchItemBean>
                isEditBatchDone = data.getBooleanExtra(PriceBatchActivity.AC_PRICE_BATCH_DONE, false)
                if (isEditBatchDone) {
                    modifyStatus(REFRESH_BATCH)
                    updateCalendarData()
                }
            }

            // 按周修改
            requestCode == MODIFY_WEEK_CODE && resultCode == Activity.RESULT_OK -> {
                mWeekBean = data.getSerializableExtra(PriceWeekActivity.KEY_PRICE_WEEK_BEAN) as? PriceWeekBean
                isEditWeekDone = data.getBooleanExtra(PriceWeekActivity.NEED_MODIFY, false)
                if (isEditWeekDone) {
                    modifyStatus(REFRESH_WEEKDAY)
                    updateCalendarData()
                }
            }

            // 节假日修改
            requestCode == MODIFY_HOLIDAY_CODE && resultCode == Activity.RESULT_OK -> {
                mHolidaysBean = data.getSerializableExtra(PriceHolidayActivity.HOLIDAY_RESULT) as? HolidaysBean
                isEditHolidaysDone = data.getBooleanExtra(PriceHolidayActivity.IS_SUBMIT, false)
                if (isEditHolidaysDone) {
                    modifyStatus(REFRESH_HOLIDAY)
                    updateCalendarData()
                }
            }

            else -> {
            }
        }
    }


    /**
     *  更新日历数据（根据用户操作顺序覆盖内容）
     */
    private fun updateCalendarData() {
        setEditState(true)
        view.setActiveSubmit(true)
        mTempData.clear()
        mTempData.addAll(mRootData)
        for (i in status) {
            when (i) {
                REFRESH_HOLIDAY -> PriceCalendarUtil().modifyHolidayData(mTempData, mHolidaysBean)
                REFRESH_WEEKDAY -> PriceCalendarUtil().modifyWeekdayData(mTempData, mWeekBean)
                REFRESH_BATCH -> PriceCalendarUtil().modifyBatchData(mTempData, mBatchBean)
                REFRESH_ABNORMAL -> PriceCalendarUtil().modifyAbnormalData(mTempData, mAbnormalBean)
            }
        }
        view.setCalendarData(mTempData)
    }


    fun setEditState(boolean: Boolean) {
        LiveDataBus.get().with(PriceSetActivity.IS_FLOAT_PRICE_EDITED).setValue(boolean)
    }

    /**
     * 处理异常价格工单
     *
     * @param siteCarTypeId
     */
    override fun apiSpiderDeal(siteCarTypeId: Int, status: String) {
        val request = SpiderDealRequest()
        request.site_car_type_id = siteCarTypeId
        request.status = status
        request.adjust_price_result = "CONFIRM"
        request.id = abnormalId
        request.handle_user_name = if (TextUtils.isEmpty(UserUtil(context).userName)) {
            UserUtil(context).nikeName ?: ""
        } else {
            UserUtil(context).userName ?: ""
        }
        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = mRetrofitService.spiderDeal(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context, this) {
            override fun onSuccess(response: BaseResult?) {
                view.dismissLoadingDialog()
                //  case 1 存在待处理的异常价格工单的场景：如果进行了调整浮动价格，将工单状态变更为已处理
                view.setAbnormalTipVisible(false)
            }
        })
    }


    override fun checkPriceUpdate() {
        limitCheck = PriceUpdateRequest.CHECK
        updatePrice()
    }


    override fun surePriceUpdate() {
        limitCheck = PriceUpdateRequest.UNCHECK
        updatePrice()
    }


    private fun updatePrice() {
        val request = PriceUpdateRequest()
        request.type = PriceUpdateRequest.PRICE
        request.limit_check = limitCheck
        request.id = siteCarTypeId
        request.prices = getCalendarData(mTempData)
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.apiPriceUpdate(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<PriceUpdateResult>(context, this) {
                    override fun onSuccess(response: PriceUpdateResult) {
                        view.dismissLoadingDialog()
//                        // 当前type  只有一种。
                        if (response.alert != null && response.alert?.type == ConstantsUtil.LIMIT_CHECK) {
                            view.setShowAlert(response.alert?.title ?: "", response.alert?.content
                                    ?: "")
                        } else {
                            mHolidaysBean = null
                            mAbnormalBean = null
                            mWeekBean = null
                            mBatchBean = null
                            view.setUpdateDone()
                        }
                        setEditState(false)
                    }
                })
    }


    /**
     * 根据操作顺序，最近的操作放在后面
     *
     * @param key
     */
    private fun modifyStatus(key: Int) {
        if (status.contains(key)) {
            val index = status.indexOf(key)
            Collections.swap(status, index, status.size - 1)
            LogUtil.e("Lyy status = $status")
        } else {
            status.add(key)
        }
    }


    private fun getCalendarData(data: ArrayList<PriceMonthEntity>): ArrayList<PriceBean> {
        val sdf = SimpleDateFormat("yyyy-MM-dd")
        val list = ArrayList<PriceBean>()
        var price = PriceBean()
        val calendar = Calendar.getInstance()
        var amount = 0
        var priceDate = ""

        for (monthObj in data) {
            val daysList = monthObj.list ?: return list
            for (dayObj in daysList) {
                // 日期必须是被编辑过的
                if (dayObj.isEdited) {

                    // 如果价格变了 or 不是连续的第二天 就重新创建对象 添加到list
                    if (amount != dayObj.float_amount || !TextUtils.equals(priceDate, DateTimeUtil.dateToString(dayObj.year, dayObj.month, dayObj.date))) {
                        amount = dayObj.float_amount
                        price = PriceBean()
                        list.add(price)
                    }

                    if (price.day == 0) {
                        price.day = dayObj.float_amount
                    }

                    if (TextUtils.isEmpty(price.start_time)) {
                        price.start_time = DateTimeUtil.dateToString(dayObj.year, dayObj.month, dayObj.date)
                    }

                    price.end_time = DateTimeUtil.dateToString(dayObj.year, dayObj.month, dayObj.date)
                    try {
                        // 获取到当前日期的第二天转成字符串 下次循环的数据做比较
                        sdf.parse(price.end_time)?.let { calendar.time = it }
                        calendar.add(Calendar.DAY_OF_YEAR, 1)
                        priceDate = sdf.format(calendar.time)
                    } catch (e: Exception) {
                        priceDate = ""
                    }
                }
            }
        }
        return list
    }

}
