package com.jiaoyinbrother.monkeyking.manager.mvpactivity.price.holiday

import android.app.Activity
import android.app.AlertDialog
import android.content.Context
import android.content.Intent
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import com.jiaoyinbrother.library.bean.HolidaysBean
import com.jiaoyinbrother.library.util.TextUtil
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpdialog.selectfestival.SelectFestivalDialog
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import kotlinx.android.synthetic.main.activity_price_holiday.*

class PriceHolidayActivity : MvpBaseActivity<PriceHolidayPresenter>(), PriceHolidayContract.View {

    private var mHolidayList: ArrayList<HolidaysBean>? = null
    private var mSelectFestivalDialog: SelectFestivalDialog? = null
    private var mHolidayResult: HolidaysBean? = null
    private var isEdited = false

    override fun initPresenter(): PriceHolidayPresenter {
        return PriceHolidayPresenter(this, this)
    }

    override fun initToolBar() {
        mainTitle.text = "按节日修改"
    }


    override fun getLayoutId(): Int {
        return R.layout.activity_price_holiday
    }

    override fun initView() {
        mSelectFestivalDialog = SelectFestivalDialog()
    }


    override fun initData() {
        mHolidayList = intent?.getSerializableExtra(HOLIDAY_LIST) as? ArrayList<HolidaysBean>
        val holidaysBean1 = intent?.getSerializableExtra(HOLIDAY_RESULT) as? HolidaysBean
        if (holidaysBean1 != null) {
            mHolidayResult = holidaysBean1?.apply {
                tv_festival_name?.text = holiday_name
                tv_date_content?.text = start_time + " 至 " + end_time
                et_price?.setText(if (day > 0) day.toString() else "")
            }
        } else {

            mHolidayList?.takeIf { it.isNotEmpty() }?.let {
                val holidaysBean = it.get(0)

                mHolidayResult = holidaysBean?.apply {
                    tv_festival_name?.text = holiday_name
                    tv_date_content?.text = start_time + " 至 " + end_time
                }
            }
        }
    }

    override fun initListeners() {
        ll_festival_content?.setOnClickListener {
            mSelectFestivalDialog?.show(supportFragmentManager, mHolidayList, mHolidayResult)
        }
        mSelectFestivalDialog?.setOnListener(object : SelectFestivalDialog.OnListener {
            override fun onSelect(holiday: HolidaysBean?) {
                mHolidayResult = holiday?.apply {
                    tv_festival_name?.text = holiday_name
                    tv_date_content?.text = start_time + " 至 " + end_time
                }
            }

        })

        btn_save?.setOnClickListener {
            et_price?.text?.ifEmpty {
                isEdited = true
                setBackground(true)
                showToast("请填写价格")
                return@setOnClickListener
            }
            mHolidayResult?.day = et_price?.text.toString().toInt()
            val intent = Intent()
            intent.putExtra(HOLIDAY_RESULT, mHolidayResult)
            intent.putExtra(IS_SUBMIT, true)
            setResult(Activity.RESULT_OK, intent)
            finish()
        }

        et_price?.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(s: Editable?) {
                setBackground(TextUtils.isEmpty(s))
            }

            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                if (TextUtil.isZeroBegin(s.toString())) {
                    et_price?.setText("")
                }
            }

        })
    }

    private fun setBackground(b: Boolean) {
        if (!isEdited) {
            isEdited = false
            return
        }
        edit_ll?.setBackgroundResource(if (b) R.drawable.shape_price_week_et_error_bg else R.drawable.shape_price_week_et_bg)
    }


    override fun onBackPressed() {
        if (TextUtils.isEmpty(et_price?.text)) {
            finish()
        } else {
            IOSAlertDialog(this)
                    .builder()
                    .setTitle("温馨提示")
                    .setMsg("存在未保存的设置，请确认是否仍要离开", true)
                    .setRightButton("不离开") {
                    }
                    .setLeftButton("离开") {
                        mHolidayResult?.day = et_price?.text.toString().toInt()
                        val intent = Intent()
                        intent.putExtra(HOLIDAY_RESULT, mHolidayResult)
                        intent.putExtra(IS_SUBMIT, false)
                        setResult(Activity.RESULT_OK, intent)
                        finish()
                    }
                    .show()
        }

    }


    companion object {
        const val HOLIDAY_LIST = "HOLIDAY_LIST"
        const val HOLIDAY_RESULT = "HOLIDAY_RESULT"
        const val IS_SUBMIT = "IS_SUBMIT"

        fun newInstance(context: Context, holidays: ArrayList<HolidaysBean>, holidayBean: HolidaysBean?) = Intent().apply {
            setClass(context, PriceHolidayActivity::class.java)
            putExtra(HOLIDAY_LIST, holidays)
            putExtra(HOLIDAY_RESULT, holidayBean)
        }

    }
}