package com.jiaoyinbrother.monkeyking.manager.mvpactivity.relet

import com.alibaba.android.arouter.facade.annotation.Route
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.bean.CalcDifferenceBean
import com.jiaoyinbrother.library.bean.OrderDetailResult
import com.jiaoyinbrother.library.util.ARouterConstantUtil
import com.jiaoyinbrother.library.util.DateTimeUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.adapter.CostAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.mvpdialog.selectdatetime.SelectDateTimeMVPDialog
import com.monkey.datetimelibrary.util.TimeGenerateUtil
import kotlinx.android.synthetic.main.activity_relet.*
import java.util.*

@Route(path = ARouterConstantUtil.ACTIVITY_ORDER_RELET_URL)
class ReletActivity : MvpBaseActivity<ReletPresenter>(), ReletContract.View {

    private var mCostAdapter: CostAdapter? = null
    private var mOrderId: String? = null
    private var mStartTime: String? = null
    private var mEndTime: String? = null
    private var mNewEndTime: String? = null
    private var mDeadline: String? = null

    override fun initPresenter(): ReletPresenter {
        return ReletPresenter(this, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_relet
    }

    override fun initToolBar() {
        mainTitle.text = "续租"
    }

    override fun initView() {
        mCostAdapter = CostAdapter()
        cost_rv?.adapter = mCostAdapter
    }

    override fun initData() {
        observeOrderInfo()
    }

    private var resultListener = object : SelectDateTimeMVPDialog.OnResultListener {
        override fun onConfirm(startDate: Date?, endDate: Date?) {

            LogUtil.e("onconfirm ----- relet")

            if (endDate != null) {
                LogUtil.e("updateStartEndDates ----- endDate=" + DateTimeUtil.dateToString(endDate))
                mNewEndTime = DateTimeUtil.dateToString(endDate)
                setUpdateTimeView()
                presenter?.requestOrderCalc(mOrderId, mNewEndTime)
            }
        }

    }

    override fun initListeners() {
        select_time_ll?.setOnClickListener { v ->
            val endTime = DateTimeUtil.stringToDate(mNewEndTime)
            val minEndTime = DateTimeUtil.stringToDate(mEndTime)
            val maxEndTime = DateTimeUtil.stringToDate(mDeadline)
            SelectDateTimeMVPDialog.newOrderReletDialog(
                    endTime, minEndTime, maxEndTime,
                    resultListener, supportFragmentManager
            )
        }
        submit_btn?.setOnClickListener { v ->
            presenter?.requestOrderModify(mOrderId, mNewEndTime)
        }
    }

    private fun observeOrderInfo() {
        try {
            //order detail
            val orderDetail =
                    intent.getSerializableExtra(ARouterConstantUtil.ACTIVITY_PARAM_ORDER_DETAIL) as OrderDetailResult
            presenter.getData(orderDetail)
            mOrderId = orderDetail?.orderid
            val start_time = orderDetail?.start_time
            val end_time = orderDetail?.end_time
            mStartTime = start_time
            mEndTime = end_time
            mNewEndTime = DateTimeUtil.getDefaultReletDate(end_time)
            setDefaultTimeView(start_time, end_time)


            //Deadline
            mDeadline =
                    intent.getSerializableExtra(ARouterConstantUtil.ACTIVITY_PARAM_RELET_DEADLINE) as String
            LogUtil.e("mDeadline =" + mDeadline)
            if (DateTimeUtil.compareBigToDuration(mDeadline, mNewEndTime)) {
//            if (DateTimeUtil.compareBigToDuration(mNewEndTime,mDeadline)) {
                LogUtil.e("in compareBigToDuration")
                mNewEndTime = mDeadline
            }
            //只有在和deadline,比较后，才能更新界面
            setUpdateTimeView()
            presenter?.requestOrderCalc(mOrderId, mNewEndTime)
        } catch (e: Exception) {
            LogUtil.e("observeOrderInfo, e= $e")
        }

    }

    private fun setDefaultTimeView(start_time: String?, end_time: String?) {
        start_time_tv?.text = DateTimeUtil.getDate(start_time) + " " + DateTimeUtil.getTime(start_time)
        end_time_tv?.text = DateTimeUtil.getDate(end_time) + " " + DateTimeUtil.getTime(end_time)
        duration_tv?.text = TimeGenerateUtil.getTimePeriodString(start_time, end_time, TimeGenerateUtil.TYPE_DAY_HOUR)
        new_start_time_tv?.text = start_time_tv?.text
    }

    private fun setUpdateTimeView() {
        LogUtil.e("setUpdateTimeView, mNewEndTime$mNewEndTime")
        select_end_time_tv?.text = mNewEndTime
        new_end_time_tv?.text = DateTimeUtil.getDate(mNewEndTime) + " " + DateTimeUtil.getTime(mNewEndTime)
        new_duration_tv?.text =
                TimeGenerateUtil.getTimePeriodString(mStartTime, mNewEndTime, TimeGenerateUtil.TYPE_DAY_HOUR)
        relet_duration_tv?.text =
                TimeGenerateUtil.getTimePeriodString(mEndTime, mNewEndTime, TimeGenerateUtil.TYPE_DAY_MAIN_HOUR)
    }

    override fun onDifference(diff: CalcDifferenceBean?) {
        total_title_tv?.text = diff?.title
        total_amount_tv?.text = diff?.content
        total_desc_tv?.text = if (diff?.amount ?: 0f > 0) "(应支付)" else if (diff?.amount ?: 0f < 0) "(应退还)" else ""
        mCostAdapter?.list = diff?.bill_show
        mCostAdapter?.notifyDataSetChanged()
    }

    override fun onReletSuccess() {
//        setResult(RESULT_OK)
        onFinish()
    }

    private fun onFinish() {
        //用于刷新
        LiveDataBus.get().with(CarEntity.BROADCAST_JPUSH_ORDER_STATUS_CHANGED)
            .setValue(null)
        finish()
    }

}
