package com.jiaoyinbrother.monkeyking.manager.mvpactivity.relet

import android.content.Context
import android.support.v4.app.FragmentActivity
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.*
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class ReletPresenter(context: Context, view: ReletContract.View) : BasePresenterImpl<ReletContract.View>(context, view),
        ReletContract.Presenter {


    private var phones = ArrayList<String>()

    override fun getData(order: OrderDetailResult?) {
        if (!Util.isEmptyList(order?.return_site?.site_phones)) {
            phones.addAll(order?.return_site?.site_phones ?: ArrayList())
        }
    }

    override fun requestOrderCalc(orderId: String?, endTime: String?) {
        val request = CalcRequest()
        request.end_time = endTime
        if (!TextUtils.isEmpty(orderId)) {
            request.orderid = orderId
        }
        request.type = CalcRequest.RELET
        val channel = ChannelUtil.getAppChannel(context)
        request.channel = channel
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.orderCalc(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<CalcResult>(context, this) {
                    override fun onSuccess(response: CalcResult) {
                        view.dismissLoadingDialog()
                        view?.onDifference(response?.difference)
                    }
                })
    }

    override fun requestOrderModify(orderId: String?, endTime: String?) {
        val request = OrderModifyRequest()
        if (!TextUtils.isEmpty(orderId)) {
            request.orderid = orderId
        }
        request.type = CalcRequest.RELET
        val time = OrderModifyRequest.Time()
        time?.end = endTime
        request.time = time
        request.city = UserLocationUtil(context).userSelectCityName
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.orderModify(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<OrderModifyResult>(context, this) {
                    override fun onSuccess(response: OrderModifyResult) {
                        view.dismissLoadingDialog()
                        view.onReletSuccess()
                    }
                })
    }

//    override fun requestOrderDetail(orderID: String?) {
//        LogUtil.e("Reletpresenter, order/get_detail")
//        val request = OrderDetailRequest()
//        request.uid = UserUtil(context).uid
//        request.orderid = orderID
//        val service = RetrofitHelper.getInstance(context).service
//        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
//        val observable = service.getOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
//        observable.subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).subscribe(
//                object : BaseObserver<OrderDetailResult>(context, this) {
//                    override fun onSuccess(response: OrderDetailResult) {
//                        view.dismissLoadingDialog()
//                        view.onStartPay()
//                    }
//
//                })
//    }

}