package com.jiaoyinbrother.monkeyking.manager.mvpactivity.setting

import android.app.ProgressDialog
import android.content.Intent
import android.view.View
import cn.jpush.android.api.JPushInterface
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.util.CheckVersionUtil
import com.jiaoyinbrother.library.bean.AndroidBean
import com.jiaoyinbrother.library.util.AppCacheUtil
import com.jiaoyinbrother.library.util.ConstantsUtil
import com.jiaoyinbrother.library.util.StatisticsUtil
import com.jiaoyinbrother.monkeyking.manager.activity.SafeCenterActivity
import kotlinx.android.synthetic.main.act_more.*
import java.lang.NullPointerException
import java.util.ArrayList

class SettingActivity : MvpBaseActivity<SettingPresenter>(), ISettingContract.View,
    View.OnClickListener {

    //版本升级
    private var pdDialog: ProgressDialog? = null// 版本升级进度条


    override fun initPresenter(): SettingPresenter {
        return SettingPresenter(this, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.act_more
    }

    override fun initView() {
    }

    override fun initToolBar() {
        mainTitle.text = "设置"
    }

    override fun initData() {
        /* 版本升级进度 */
        pdDialog = ProgressDialog(this@SettingActivity)// 创建进度对话框
        pdDialog?.setTitle("正在下载")// 设置标题
        pdDialog?.setCancelable(false)
        pdDialog?.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL)// 设置进度条样

        presenter.registerCheckVersionReceiver()
        presenter.getData()
    }

    override fun initListeners() {
        btn_push.isChecked = AppCacheUtil(this).jPushStatus
        btn_push.setOnCheckedChangeListener { _, isChecked ->
            StatisticsUtil.onEvent(this, ConstantsUtil.MINE_SETTINGS_CLOSE_PUSH, "")
            if (isChecked) {
                JPushInterface.resumePush(this@SettingActivity)
                showToast("已开启推送")
            } else {
                JPushInterface.stopPush(this@SettingActivity)
                showToast("已关闭推送")
            }
            AppCacheUtil(this@SettingActivity).jPushStatus = isChecked
        }
        personal_info_relative.setOnClickListener(this)
        contact_relative.setOnClickListener(this)
        contact_us_relative.setOnClickListener(this)
        rate_relative.setOnClickListener(this)
        check_version_relative.setOnClickListener(this)
        clear_cache_relative.setOnClickListener(this)
        logout_relative.setOnClickListener(this)
        safe_center_layout.setOnClickListener(this)
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.personal_info_relative -> presenter.onPersonalInfo()

            R.id.contact_relative -> presenter.onContact()

            R.id.contact_us_relative -> presenter.onContactUs()

            R.id.rate_relative -> presenter.onMarket()

            R.id.check_version_relative -> presenter.onCheckVersion()

            R.id.clear_cache_relative -> presenter.onClearCache()

            R.id.logout_relative -> presenter.onLogout()

            R.id.safe_center_layout -> SafeCenterActivity.newInstance(this)
        }
    }

    override fun setCacheSize(size: String) {
        cache_status.text = size
    }

    override fun setVersionName(name: String) {
        version_name.text = name
    }

    override fun setVersionUpdateDialog(android: AndroidBean) {
        CheckVersionUtil(this).showUploadVersionDialog(android, this)
    }

    override fun setVersionNoDataDialog() {
        CheckVersionUtil(this).showNoUploadDialog()
    }

    override fun setLogoutSuccess() {
        val intent = Intent()
        setResult(LOGOUT_OK, intent)
        finish()
    }

    override fun setDownloadCompleted() {
        pdDialog?.dismiss()
    }

    override fun setUpdateProgress(progress: Int) {
        if (progress < 0 || progress > 100) {
            pdDialog?.dismiss()
        } else {
            pdDialog?.show()
            pdDialog?.progress = progress
        }
    }

    override fun onDestroy() {
        presenter.onDestroy()
        super.onDestroy()
    }

    companion object {
        const val LOGOUT_OK = 100
    }
}