package com.jiaoyinbrother.monkeyking.manager.mvpactivity.setting

import android.annotation.SuppressLint
import android.app.NotificationManager
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.net.Uri
import android.os.AsyncTask
import android.os.Handler
import android.os.Message
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.activity.ContactsActivity
import com.jiaoyinbrother.monkeyking.manager.activity.GeneralWebViewActivity
import com.jiaoyinbrother.monkeyking.manager.activity.PersonalInfoActivity
import com.jiaoyinbrother.monkeyking.manager.jpush.MKPushUtil
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.login.LoginActivity
import com.jiaoyinbrother.monkeyking.manager.util.CheckVersionUtil
import com.jiaoyinbrother.monkeyking.manager.util.EMUtil
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.BaseResult
import com.jiaoyinbrother.library.bean.CheckVersionResult
import com.jiaoyinbrother.library.bean.UserDetailRequestBean
import com.jiaoyinbrother.library.bean.WebViewConfigEntity
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.monkeyking.manager.push.PushUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class SettingPresenter(context: Context, view: ISettingContract.View) :
        BasePresenterImpl<ISettingContract.View>(context, view), ISettingContract.Presenter {


    private var mProgressReceiver: ProgressReceiver? = null

    fun registerCheckVersionReceiver() {
        mProgressReceiver = ProgressReceiver()
        val filter = IntentFilter()
        filter.addAction(ConstantsUtil.ACTION_DOWNLOAD_PROGRESS)
        filter.addAction(ConstantsUtil.ACTION_DOWNLOAD_COMPOLETED)
        context.registerReceiver(mProgressReceiver, filter)
    }

    private inner class ProgressReceiver : BroadcastReceiver() {

        override fun onReceive(context: Context, intent: Intent) {
            val action = intent.action
            when (action) {
                ConstantsUtil.ACTION_DOWNLOAD_PROGRESS -> {
                    val total = intent.getLongExtra(ConstantsUtil.EXTRA_TOTAL, 0)
                    val current = intent.getLongExtra(ConstantsUtil.EXTRA_CURRENT, 0)
                    mHandler.sendMessage(mHandler.obtainMessage(MSG_UPDATE_PROGRESS, total.toInt(), current.toInt()))
                }
                ConstantsUtil.ACTION_DOWNLOAD_COMPOLETED -> {
                    val result = intent.getIntExtra(ConstantsUtil.EXTRA_RESULT, -1)
                    CheckVersionUtil(context).handleDownloadComplete(intent)
                    mHandler.sendMessage(mHandler.obtainMessage(MSG_DOWNLOAD_COMPLETED, result))
                }
            }
        }
    }


    private val mHandler = @SuppressLint("HandlerLeak")
    object : Handler() {

        override fun handleMessage(msg: Message) {
            when (msg.what) {
                ConstantsUtil.DESTROY -> removeMessages(MSG_UPDATE_PROGRESS)
                MSG_UPDATE_PROGRESS -> {
                    //升级进度条开启，关闭loading
                    view.dismissLoadingDialog()
                    val total = msg.arg1 / 1000
                    val current = msg.arg2 / 1000
                    var progress = 0
                    if (total > 0) {
                        if (total > current)
                            progress = (current * 100 / total)
                    }
                    view.setUpdateProgress(progress)
                }
                MSG_DOWNLOAD_COMPLETED -> {
                    view.dismissLoadingDialog()
                    view.showToast("下载成功")
                    view.setDownloadCompleted()
                }
                else -> {
                }
            }
        }
    }

    fun onDestroy() {
        context.unregisterReceiver(mProgressReceiver)
    }


    override fun getData() {

        // 获取许需要清除的缓存大小
        val cacheSize = FileUtils.getTotalCacheSize(context)
        view.setCacheSize(cacheSize)

        view.setVersionName("版本号V" + AppAndSysInfoUtil.getAppVersion(context))
    }

    /**
     * 个人信息
     *
     **/
    fun onPersonalInfo() {
        val intent = Intent(context, PersonalInfoActivity::class.java)
        context.startActivity(intent)
    }

    /**
     * 紧急联系人
     *
     **/
    fun onContact() {
        val intent = Intent()
        if (UserUtil(context).isLogin()) {
            StatisticsUtil.onEvent(context, ConstantsUtil.MINE_SETTINGS_CONTACTS, "")
            intent.setClass(context, ContactsActivity::class.java)
        } else {
            intent.setClass(context, LoginActivity::class.java)
        }
        context.startActivity(intent)
    }

    fun onContactUs() {
        StatisticsUtil.onEvent(context, ConstantsUtil.MINE_SETTINGS_CONTACT_US, "")
        val webViewConfigEntity = WebViewConfigEntity()
        webViewConfigEntity.title = "联系我们"
        webViewConfigEntity.isNeedProgressBar = true
        webViewConfigEntity.url = UrlUtil.getText_url() + "/#/profile/more/contact_us"
        webViewConfigEntity.isNeedShareButton = false
        val intent = Intent(context, GeneralWebViewActivity::class.java)
        intent.putExtra(CarEntity.FEATURE_ITEM, webViewConfigEntity)
        context.startActivity(intent)
    }


    fun onMarket() {
        StatisticsUtil.onEvent(context, ConstantsUtil.MINE_SETTINGS_APP_SCORE, "")
        val uri = Uri.parse("market://details?id=${context.packageName}")
        val intent = Intent(Intent.ACTION_VIEW, uri)
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        try {
            context.startActivity(intent)
        } catch (e: Exception) {
            view.showToast("没找到相关市场")
        }
    }


    fun onCheckVersion() {
        requestVersion()
    }

    override fun requestVersion() {
        val request = CheckVersionUtil(context).getCheckVersionEntity()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.checkVersion(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<CheckVersionResult>(context, this) {
                    override fun onSuccess(response: CheckVersionResult) {
                        view.dismissLoadingDialog()
                        if (CheckVersionUtil(context).checkVersion(response)) {
                            view.setVersionUpdateDialog(response.android)
                        } else {
                            view.setVersionNoDataDialog()
                        }
                    }
                })
    }


    fun onClearCache() {
        StatisticsUtil.onEvent(context, ConstantsUtil.MINE_SETTINGS_CLEAN_CACHE, "")
        DeleteCacheAsyncTask().execute()
    }

    @SuppressLint("StaticFieldLeak")
    private inner class DeleteCacheAsyncTask : AsyncTask<Void, Void, Int>() {
        override fun doInBackground(vararg Integer: Void): Int? {
            FileUtils.clearWebViewCache(context)
            GeneralWebViewActivity.clearCookies(context)
            return 1
        }

        override fun onPostExecute(temp: Int?) {
            super.onPostExecute(temp)
            val cacheSize = FileUtils.getTotalCacheSize(context)
            view.setCacheSize(cacheSize)
            view.showToast("清除成功")
        }
    }


    fun onLogout() {
        val userUtil = UserUtil(context)
        if (userUtil.isLogin()) {
            val notificationManager = context.getSystemService(NotificationManager::class.java)
            notificationManager.cancelAll()
            handleLogout()
            //先logout,再更新Jpush

            PushUtil.apiPush(context,this,true)

            userUtil.logout()

            MKPushUtil.updateJPush(context)
            GeneralWebViewActivity.clearCookies(context)
            LiveDataBus.get().with(ConstantsUtil.LIVEDATA_LOGOUT).postValue(true)
            EMUtil.instance.logoutEM()
            view.setLogoutSuccess()
        }
    }

    private fun handleLogout() {
        val request = UserDetailRequestBean()
        request.uid = UserUtil(context).uid
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.logout(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {}
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<BaseResult>(context, this) {
                    override fun onSuccess(response: BaseResult) {
                        LogUtil.e("用户登出成功")
                    }
                })
    }


    companion object {
        const val MSG_UPDATE_PROGRESS = 0x30
        const val MSG_DOWNLOAD_COMPLETED = 0x31
    }
}