package com.jiaoyinbrother.monkeyking.manager.mvpactivity.store.dummy

import android.content.Context
import android.content.Intent
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.UserUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.lang.ref.ReferenceQueue
import java.util.concurrent.TimeUnit

class VirtualStoresPresenter(context: Context, view: IVirtualStoresContract.View) :
        BasePresenterImpl<IVirtualStoresContract.View>(context, view), IVirtualStoresContract.Presenter {
    override val list: MutableList<SiteBean> = mutableListOf()

        private var mSiteId:String? = null

    fun getData(intent:Intent){
        mSiteId = intent.getStringExtra(VirtualStoresActivity.SITE_ID)

        apiStoreList()
    }

    override fun apiStoreList() {
        val request = SitesRequest()
        request.manager_id = UserUtil(context).userId.toString()
        request.sort_by_car_total = 0
        request.site_id = mSiteId
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.siteList(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<SitesResult>(context, this) {
                    override fun onSuccess(response: SitesResult) {
                        view.dismissLoadingDialog()
                        val responseList = response.sites ?: emptyList<SiteBean>()
                        if (responseList.isEmpty()) {
                            view?.setNoDataView()
                        } else {
                            list.clear()
                            list.addAll(responseList)
                            view?.updateStoreList()
                        }
                    }

                    override fun onError(e: Throwable) {
                        view?.dismissLoadingDialog()
                        view?.setNoDataView()
                        super.onError(e)
                    }

                    override fun onAPIError(apiException: ApiException?) {
                        view?.dismissLoadingDialog()
                        view?.setNoDataView()
                        super.onAPIError(apiException)
                    }

                })
    }

}