package com.jiaoyinbrother.monkeyking.manager.mvpactivity.store.list

import android.app.Activity
import android.content.Intent
import android.text.TextUtils
import com.jiaoyinbrother.library.bean.SiteBean
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.store.modify.StoreModifyInfoActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.store.dummy.VirtualStoresActivity
import kotlinx.android.synthetic.main.activity_store_list.*

class StoreListActivity : MvpBaseActivity<StoreListPresenter>(), IStoreListContract.View {

    private val REQUEST_CODE_STORE_MODIFY = 0x0010

    private val adapter by lazy {
        StoreListAdapter(presenter.list, StoreListAdapter.STORE_ENTITY)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_store_list
    }

    override fun initView() {
        refresh_layout?.setEnableLoadMore(false)
        refresh_layout?.setOnRefreshListener {
            presenter.apiStoreList()
        }
        store_list_rv?.adapter = adapter
    }

    override fun initToolBar() {
        mainTitle?.text = "门店"
    }

    override fun initData() {
        presenter.apiStoreList()
    }

    override fun initListeners() {
        adapter.setListener(object : StoreListAdapter.StoreListListener {
            override fun onVirtualStore(position: Int, data: SiteBean) {
                startActivity(VirtualStoresActivity.newIntent(this@StoreListActivity, data.id
                        ?: ""))
            }

            override fun onBaseRule(position: Int, data: SiteBean) {
                startActivityForResult(StoreModifyInfoActivity.newIntent(this@StoreListActivity, data.id
                        ?: "", false, position), REQUEST_CODE_STORE_MODIFY)
            }
        })
    }

    override fun initPresenter(): StoreListPresenter {
        return StoreListPresenter(this, this)
    }

    override fun updateStoreList() {
        refresh_layout?.finishRefresh()
        adapter.notifyDataSetChanged()
        loading_layout?.showContent()
    }

    override fun setNoDataView() {
        refresh_layout?.finishRefresh()
        loading_layout?.setEmptyText("登录账号未绑定有效门店")
        loading_layout?.showEmpty()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == REQUEST_CODE_STORE_MODIFY && resultCode == Activity.RESULT_OK && data != null) {
            val position = data.getIntExtra(StoreModifyInfoActivity.POSITION, -1)
            if (position >= 0) {
                try {
                    val siteBean = presenter.list[position]
                    val contact = data.getStringExtra(StoreModifyInfoActivity.CONTACT)
                    val phone = data.getStringExtra(StoreModifyInfoActivity.PHONE)
                    val time = data.getStringExtra(StoreModifyInfoActivity.TIME)
                    if (!TextUtils.isEmpty(contact)) {
                        siteBean.contact = contact
                    }
                    if (!TextUtils.isEmpty(phone)) {
                        siteBean.site_phone = phone
                    }
                    val split = time?.split(StoreModifyInfoActivity.TIME_JOINER)
                    if (split?.size == 2 && !TextUtils.isEmpty(split[0]) && !TextUtils.isEmpty(split[1])) {
                        siteBean.from_time = split[0]
                        siteBean.to_time = split[1]
                    }
                    adapter.notifyItemChanged(position)
                } catch (e: Exception) {
                }
            }
        }
    }
}