package com.jiaoyinbrother.monkeyking.manager.mvpactivity.store.modify

import android.os.Bundle
import android.support.v4.app.DialogFragment
import android.support.v4.app.FragmentManager
import android.text.TextUtils
import android.view.Gravity
import android.view.View
import android.view.WindowManager
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpdialog.BaseDialogFragment
import kotlinx.android.synthetic.main.dialog_start_and_end_time.*
import java.lang.Exception
import java.text.SimpleDateFormat
import java.util.*

class StartAndEndTimePicker : BaseDialogFragment() {

    private var mListener: OnResultListener? = null
    private var mInStart = true
    private val mHourList = IntRange(0, 23).map { if (it < 10) "0${it}" else it.toString() }
    private val mMinuteList = IntRange(0, 59).map { if (it < 10) "0${it}" else it.toString() }

    override fun getLayoutId(): Int {
        return R.layout.dialog_start_and_end_time
    }

    override fun initViews(view: View?, savedInstanceState: Bundle?) {
        wv_hour?.data = mHourList
        wv_minute?.data = mMinuteList
    }

    override fun initToolbar(savedInstanceState: Bundle?) {}

    override fun initListeners() {
        home_calendar_pickup?.setOnClickListener {
            mInStart = true
            updateHead()
            updateWheel()
        }
        home_calendar_return?.setOnClickListener {
            mInStart = false
            updateHead()
            updateWheel()
        }
        wv_hour?.setOnItemSelectedListener { _, _, _ ->
            updateTime()
            updateMinuteWheel()
        }
        wv_minute?.setOnItemSelectedListener { _, _, _ ->
            updateTime()
        }
        tv_submit?.setOnClickListener {
            mListener?.onConfirm(start_date_tv?.text?.toString()
                    ?: "00:00", end_date_tv?.text?.toString() ?: "00:00")
            dismiss()
        }
        tv_cancel?.setOnClickListener {
            dismiss()
        }
    }

    private fun updateHead() {
        if (mInStart) {
            home_calendar_pickup?.setBackgroundColor(resources.getColor(R.color.color_FF3434))
            start_name_tv?.setTextColor(resources.getColor(R.color.color_0))
            start_date_tv?.setTextColor(resources.getColor(R.color.color_0))
            home_calendar_return?.setBackgroundColor(resources.getColor(R.color.color_e5e5e5))
            end_name_tv?.setTextColor(resources.getColor(R.color.color_444444))
            end_date_tv?.setTextColor(resources.getColor(R.color.color_444444))
        } else {
            home_calendar_pickup?.setBackgroundColor(resources.getColor(R.color.color_e5e5e5))
            start_name_tv?.setTextColor(resources.getColor(R.color.color_444444))
            start_date_tv?.setTextColor(resources.getColor(R.color.color_444444))
            home_calendar_return?.setBackgroundColor(resources.getColor(R.color.color_FF3434))
            end_name_tv?.setTextColor(resources.getColor(R.color.color_0))
            end_date_tv?.setTextColor(resources.getColor(R.color.color_0))
        }
    }

    private fun updateMinuteWheel() {
        val sourceTime = if (mInStart) end_date_tv?.text.toString() else start_date_tv?.text.toString()
        val (hour, minute) = getHourAndMinute(sourceTime)
        val time = if (mInStart) start_date_tv?.text.toString() else end_date_tv?.text.toString()
        val sourceMinuteList = mutableListOf<String>()
        if (time.startsWith(hour)) {
            if (mInStart) {
                for (s in mMinuteList) {
                    sourceMinuteList.add(s)
                    if (TextUtils.equals(s, minute)) {
                        break
                    }
                }

            } else {
                for (i in mMinuteList.size - 1 downTo 0) {
                    val s = mMinuteList[i]
                    sourceMinuteList.add(0, s)
                    if (TextUtils.equals(s, minute)) {
                        break
                    }
                }
            }
        } else {
            sourceMinuteList.addAll(mMinuteList)
        }
        if (mInStart) {
            val oldData = wv_minute?.data?.get(wv_minute?.selectedItemPosition ?: 0)
            wv_minute?.data = sourceMinuteList
            var indexOf = sourceMinuteList.indexOf(oldData)
            if (indexOf < 0) {
                indexOf = sourceMinuteList.size - 1
                val endTime = start_date_tv?.text?.toString()
                start_date_tv?.text = "${endTime?.subSequence(0, 3)}${sourceMinuteList[indexOf]}"
            }
            wv_minute?.selectedItemPosition = indexOf
        } else {
            val oldData = wv_minute?.data?.get(wv_minute?.selectedItemPosition ?: 0)
            wv_minute?.data = sourceMinuteList
            var indexOf = sourceMinuteList.indexOf(oldData)
            if (indexOf < 0) {
                indexOf = 0
                val endTime = end_date_tv?.text?.toString()
                end_date_tv?.text = "${endTime?.subSequence(0, 3)}${sourceMinuteList[indexOf]}"
            }
            wv_minute?.selectedItemPosition = indexOf
        }
    }

    private fun getHourAndMinute(time: String): Pair<String, String> {
        val split = time.split(":")
        return if (split.size == 2) {
            Pair(split[0], split[1])
        } else {
            Pair("00", "00")
        }
    }

    private fun updateTime() {
        val hour = wv_hour?.data?.get(wv_hour?.selectedItemPosition ?: 0) ?: "00"
        val minute = wv_minute?.data?.get(wv_minute?.selectedItemPosition ?: 0) ?: "00"
        (if (mInStart) start_date_tv else end_date_tv)?.text = "${hour}:${minute}"
    }

    override fun initData() {
        checkEndTimeAndInit()
        mInStart = true
        updateWheel()
    }

    private fun checkEndTimeAndInit() {
        val simpleDateFormat = SimpleDateFormat("HH:mm")
        val startTime = try {
            simpleDateFormat.parse(arguments?.getString(START_TIME) ?: "00:00")
        } catch (e: Exception) {
            val instance = Calendar.getInstance()
            instance.set(Calendar.HOUR_OF_DAY, 0)
            instance.set(Calendar.MINUTE, 0)
            instance.time
        }
        var endTime = try {
            simpleDateFormat.parse(arguments?.getString(END_TIME) ?: "23:59")
        } catch (e: Exception) {
            val instance = Calendar.getInstance()
            instance.set(Calendar.HOUR_OF_DAY, 23)
            instance.set(Calendar.MINUTE, 59)
            instance.time
        }
        if (startTime > endTime) {
            val instance = Calendar.getInstance()
            instance.set(Calendar.HOUR_OF_DAY, 23)
            instance.set(Calendar.MINUTE, 59)
            endTime = instance.time
        }
        start_date_tv?.text = simpleDateFormat.format(startTime)
        end_date_tv?.text = simpleDateFormat.format(endTime)
    }

    private fun updateWheel() {
        updateWheelSource()
        val time = if (mInStart) start_date_tv?.text.toString() else end_date_tv?.text.toString()
        val (hour, minute) = getHourAndMinute(time)
        val hourIndexOf = wv_hour?.data?.indexOf(hour) ?: 0
        wv_hour?.selectedItemPosition = if (hourIndexOf < 0) 0 else hourIndexOf
        val minuteIndexOf = wv_minute?.data?.indexOf(minute) ?: 0
        wv_minute?.selectedItemPosition = if (minuteIndexOf < 0) 0 else minuteIndexOf
    }

    private fun updateWheelSource() {
        val sourceTime = if (mInStart) end_date_tv?.text.toString() else start_date_tv?.text.toString()
        val (sourceHour, sourceMinute) = getHourAndMinute(sourceTime)
        val sourceHourList = mutableListOf<String>()
        if (mInStart) {
            for (s in mHourList) {
                sourceHourList.add(s)
                if (TextUtils.equals(s, sourceHour)) {
                    break
                }
            }

        } else {
            for (i in mHourList.size - 1 downTo 0) {
                val s = mHourList[i]
                sourceHourList.add(0, s)
                if (TextUtils.equals(s, sourceHour)) {
                    break
                }
            }
        }
        wv_hour?.data = sourceHourList
        updateMinuteWheel()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setStyle(DialogFragment.STYLE_NO_FRAME, R.style.Dialog_Transparent)
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)

        //初始化样式
        dialog?.window?.setLayout(WindowManager.LayoutParams.MATCH_PARENT, WindowManager.LayoutParams.WRAP_CONTENT)
        dialog?.window?.setGravity(Gravity.BOTTOM)
        dialog?.window?.setWindowAnimations(R.style.dialog_anim_down2up)
    }

    companion object {

        private const val START_TIME = "start_time"
        private const val END_TIME = "end_time"

        fun showTimePickerDialog(manager: FragmentManager, startTime: String?, entTime: String?, listener: OnResultListener) {
            val selectDateTimeMVPDialog = StartAndEndTimePicker()
            selectDateTimeMVPDialog.mListener = listener
            val arguments = Bundle()
            arguments.putString(START_TIME, startTime)
            arguments.putString(END_TIME, entTime)
            selectDateTimeMVPDialog?.arguments = arguments
            selectDateTimeMVPDialog.show(manager, "StartAndEndTimePicker")
        }

    }

    interface OnResultListener {
        fun onConfirm(startDate: String, endDate: String)
    }
}