package com.jiaoyinbrother.monkeyking.manager.mvpactivity.store.modify

import android.content.Context
import android.content.Intent
import android.support.v4.app.FragmentManager
import android.support.v7.app.AlertDialog
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.Gravity
import android.view.View
import android.widget.FrameLayout
import android.widget.TextView
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.monkeyking.manager.R
import com.leochuan.CenterSnapHelper
import com.leochuan.ScaleLayoutManager
import com.leochuan.ViewPagerLayoutManager
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.lang.Exception
import java.util.concurrent.TimeUnit

class StoreModifyInfoPresenter(context: Context, view: IStoreModifyInfoContract.View) :
        BasePresenterImpl<IStoreModifyInfoContract.View>(context, view), IStoreModifyInfoContract.Presenter {
    private var mSiteID: String? = null
    private var mSiteResult: SiteResult? = null
    override var isVirtualSite: Boolean = false

    private val nightServiceAmountList = arrayOf("0", "10", "20", "30", "40", "50")

    override fun apiSiteDetail() {
        val request = SiteDetailRequest()
        request.id = mSiteID
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getSiteDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<SiteResult>(context, this) {
                    override fun onSuccess(response: SiteResult) {
                        view.dismissLoadingDialog()
                        mSiteResult = response
                        view.closeView()
                    }
                })
    }

    override fun restoreData() {
        view.restoreData(mSiteResult)
    }

    override fun checkChange(contact: String,
                             phone: String,
                             fromTime: String,
                             toTime: String,
                             intervalTime: String,
                             prepareTime: String,
                             minimum: String,
                             nightServiceAmount: String): Boolean {
        return !(TextUtils.equals(mSiteResult?.contact, contact) &&
                TextUtils.equals(mSiteResult?.site_phone, phone) &&
                TextUtils.equals(mSiteResult?.from_time, fromTime) &&
                TextUtils.equals(mSiteResult?.to_time, toTime) &&
                TextUtils.equals(mSiteResult?.order_interval_time?.toString(), intervalTime) &&
                TextUtils.equals(mSiteResult?.prepare_time?.toString(), prepareTime) &&
                TextUtils.equals(mSiteResult?.minimum?.toString(), minimum) &&
                TextUtils.equals(mSiteResult?.night_service_amount?.toString(), nightServiceAmount))
    }

    override fun apiModifySite(contact: String,
                               phone: String,
                               fromTime: String,
                               toTime: String,
                               intervalTime: Int,
                               prepareTime: Int,
                               minimum: Int,
                               nightServiceAmount: String) {
        val request = ModifySiteRequest()
        request.partyid = mSiteID
        request.name = mSiteResult?.name
        request.contact = contact
        request.partytypeid = mSiteResult?.type
        request.site_phone = phone
        request.from_time = fromTime
        request.to_time = toTime
        request.order_interval_time = intervalTime
        request.prepare_time = prepareTime
        request.minimum = minimum
        val nightServiceAmountInt = try {
            nightServiceAmount.toInt()
        } catch (e: Exception) {
            0
        }
        request.night_service_amount = nightServiceAmountInt
        request.office_site = mSiteResult?.address
        request.phone = mSiteResult?.phone
        if (isVirtualSite && view.getSyncSiteStatus()){
            request.sync_virtual_sites = 1
        }
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.modifySite(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<BaseResult>(context, this) {
                    override fun onSuccess(response: BaseResult) {
                        view.dismissLoadingDialog()
                        mSiteResult?.contact = contact
                        mSiteResult?.site_phone = phone
                        mSiteResult?.from_time = fromTime
                        mSiteResult?.to_time = toTime
                        mSiteResult?.order_interval_time = intervalTime
                        mSiteResult?.prepare_time = prepareTime
                        mSiteResult?.minimum = minimum
                        mSiteResult?.night_service_amount = nightServiceAmountInt
                        view.showToast("修改成功")
                        view.closeView()
                    }
                })
    }

    override fun showTimeDialog(supportFragmentManager: FragmentManager, time: String) {
        val split = time.split(StoreModifyInfoActivity.TIME_JOINER)
        val (fromTime, toTime) = if (split.size != 2) {
            Pair("00:00", "00:00")
        } else {
            Pair(
                    if (TextUtils.isEmpty(split[0].trim())) "00:00" else split[0].trim(),
                    if (TextUtils.isEmpty(split[1].trim())) "0:00" else split[1].trim()
            )
        }
        StartAndEndTimePicker.showTimePickerDialog(supportFragmentManager,
                fromTime,
                toTime,
                object : StartAndEndTimePicker.OnResultListener {
                    override fun onConfirm(startDate: String, endDate: String) {
                        view.showBusinessTime(startDate,endDate)
                    }

                })
    }

    override fun showServiceAmountDialog(text: String) {
        val dialogView = View.inflate(context, R.layout.dialog_select_site, null)
        val dialog = AlertDialog.Builder(context, R.style.wk_pub_dialog_style).setView(dialogView).create()
        dialog.show()
        // 调整dialog背景大小
        val window = dialog.window
        window?.setGravity(Gravity.BOTTOM)
        val display = window?.windowManager?.defaultDisplay
        val layoutParams = dialogView.layoutParams
        layoutParams.height = ((display?.height?.times(0.4)) as? Int)
                ?: FrameLayout.LayoutParams.WRAP_CONTENT
        layoutParams.width = display?.width
                ?: FrameLayout.LayoutParams.MATCH_PARENT
        dialogView.layoutParams = layoutParams
        val rvSites = dialogView.findViewById<RecyclerView>(R.id.rv_site)
        val cancelTv = dialogView.findViewById<TextView>(R.id.tv_cancel)
        val titleTv = dialogView.findViewById<TextView>(R.id.tv_title)
        val submitTv = dialogView.findViewById<TextView>(R.id.tv_submit)
        val builder = ScaleLayoutManager.Builder(context, 0)
        titleTv.text = "夜间服务费"
        builder.setMaxAlpha(1f)
        builder.setMinAlpha(0.7f)
        builder.setOrientation(ViewPagerLayoutManager.VERTICAL)
        val viewPagerLayoutManager = ScaleLayoutManager(builder)
        rvSites.layoutManager = viewPagerLayoutManager
        val adapter = SiteNightServiceAmountAdapter(nightServiceAmountList)
        rvSites.adapter = adapter
        val centerSnapHelper = CenterSnapHelper()
        centerSnapHelper.attachToRecyclerView(rvSites)
        var indexOfFirst = nightServiceAmountList.indexOfFirst { TextUtils.equals(text, it) }
        if (indexOfFirst < 0) {
            indexOfFirst = 0
        }
        rvSites.scrollToPosition(indexOfFirst)
        cancelTv.setOnClickListener {
            dialog.dismiss()
        }
        submitTv.setOnClickListener {
            dialog.dismiss()
            view.updateNightServiceAmount(nightServiceAmountList[viewPagerLayoutManager.currentPosition])
        }
    }

    override fun initIntent(intent: Intent?) {
        mSiteID = intent?.getStringExtra(StoreModifyInfoActivity.SITE_ID)
    }

}