package com.jiaoyinbrother.monkeyking.manager.mvpactivity.storeinfo

import android.arch.lifecycle.Observer
import android.content.Context
import android.content.Intent
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.bean.CommentListRequest
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.commentlist.CommentListActivity
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.SiteResult
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.commentlist.CommentListPresenter.Companion.COMMENT_LIST_DATA_GROUP_ID
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.commentlist.CommentListPresenter.Companion.COMMENT_LIST_DATA_SITE_ID
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.lang.StringBuilder
import java.util.concurrent.TimeUnit

class StoreInfoPresenter(context: Context, view: IStoreInfoContract.IStoreInfoView) :
        BasePresenterImpl<IStoreInfoContract.IStoreInfoView>(context, view), IStoreInfoContract.IStoreInfoPresenter {

    private var mSiteID: String? = null
    private var mGroupID: String? = null
    private var mGroupName: String? = null

    companion object {
        const val STORE_INFO_SITE_ID = "STORE_INFO_SITE_ID"
    }

    override fun initLiveDataBus(activity: StoreInfoActivity) {
        LiveDataBus.get().with(STORE_INFO_SITE_ID, String::class.java).observeSticky(activity, Observer {
            mSiteID = it
            getStoreInfo()
        })
    }

    override fun goCommentListActivity(context: Context) {
        val intent = Intent(context, CommentListActivity::class.java)
        intent.putExtra(COMMENT_LIST_DATA_SITE_ID, mSiteID)
        intent.putExtra(COMMENT_LIST_DATA_GROUP_ID, mGroupID)
        context.startActivity(intent)

    }

    override fun getStoreInfo() {

        val request = CommentListRequest()
        request.site_id = mSiteID
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.getStoreInfo(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe { }.observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<SiteResult>(context, this) {
                    override fun onSuccess(response: SiteResult?) {
                        view.getDataSuccess()
                        view.setStoreIcon(response?.logo_url)
                        if (response?.group_type == "BRAND_JM") {
                            view.setShowBrandJM(response?.site_name)
                        } else {
                            view.setStoreName(response?.site_name)
                        }
                        view.setStoreAddress(response?.address)
                        view.setStoreLocation(response?.location)


                        mGroupID = response?.group_id
                        mGroupName = response?.group_name

                        val mStoreTime = StringBuilder()
                        mStoreTime.append(response?.from_time)
                        mStoreTime.append("-")
                        mStoreTime.append(response?.to_time)
                        view.setStoreTime(mStoreTime)

                        view.setStoreCall(response?.site_phone)

                        val mark = response?.mark


                        val commentDatas = mark?.datas

                        if (commentDatas != null && commentDatas.isNotEmpty()) {
                            val bean = commentDatas[0]
                            view.setStoreCommentShowAvatar(bean.headimg_url)
                            view.setStoreCommentShowName(bean.name)
                            view.setStoreCommentShowTime(bean.time)
                            val i = bean.rate?.toDouble()
                            val stringBuilder = StringBuilder()
                            stringBuilder.append(i.toString())
                            stringBuilder.append("分 ")
                            stringBuilder.append(setScoreText(i ?: 0.0))
                            view.setStoreCommentShowScore(stringBuilder.toString())
                            view.setStoreCommentShowContent(bean.comment)
                            view.setStoreCommentShowImages(bean.images?: emptyList())
                            view.setStoreCommentShowReplay(bean.reply_list ?: emptyList())

                        } else {
                            view.setCommentIsNull()
                        }
                        val rateScore = mark?.rate

                        view.setStoreCommentScore(rateScore ?: 0.0)
                        var text: String? = null
                        if (rateScore != null) {
                            text = setScoreText(rateScore)
                            view.setStoreCommentState(text)
                        }
                        view.setStoreCommentHardware(mark?.rate_car_hardware ?: 0.0)//服务质量
                        view.setStoreCommentService(mark?.rate_service_level ?: 0.0)//车辆状况
                        view.setStoreCommentClean(mark?.rate_car_clean ?: 0.0)//车辆状况
                        view.setStoreCommentConvenient(mark?.rate_take_return ?: 0.0)//取还便捷

                        //门店基本概述

                        val mRentNum = response?.rent_num

                        val mStoreInfoContent = StringBuilder()
                        if (rateScore != 0.0) {
                            mStoreInfoContent.append(rateScore)
                            mStoreInfoContent.append("分 ")
                            mStoreInfoContent.append(text)
                        } else {
                            mStoreInfoContent.append("暂无评价")
                        }

                        if (mRentNum != null && mRentNum > 0) {
                            mStoreInfoContent.append(" 半年租")
                            mStoreInfoContent.append(response?.rent_num)
                            mStoreInfoContent.append("次")
                        } else {
                            if (rateScore == 0.0) {
                                mStoreInfoContent.append("与订单")
                            } else {
                                mStoreInfoContent.append("暂无订单")
                            }
                        }
                        view.setStoreContent(mStoreInfoContent)

                    }

                    override fun onError(e: Throwable) {
                        super.onError(e)
                        view.getDataError()
                    }
                })
    }

    private fun setScoreText(rateScore: Double): String {
        var text = ""
        if (rateScore > 2.9 && rateScore < 4) {
            text = "还不错"
        } else if (rateScore >= 3.9 && rateScore < 4.5) {
            text = "挺好哒"
        } else if (rateScore >= 4.5) {
            text = "棒极啦"
        }
        return text
    }


}