package com.jiaoyinbrother.monkeyking.manager.mvpactivity.systemnotification

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.activity.GeneralWebViewActivity
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.UserUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

/**
 * Created by Sgq
 * Create Date 2019/4/19 and 11:53
 * desc:系统公告
 */
class SysNotificationPresenter(context: Context, view: SysNotiffcationContract.view) : BasePresenterImpl<SysNotiffcationContract.view>(context, view),
        SysNotiffcationContract.presenter {


    fun onMessageDetail(bulletinsBean: BulletinsBean) {
        if (!TextUtils.isEmpty(bulletinsBean.url)) {
            val webViewConfigEntity = WebViewConfigEntity()
            webViewConfigEntity.title = bulletinsBean.bulletin_name
            webViewConfigEntity.url = bulletinsBean.url
            webViewConfigEntity.isNeedProgressBar = true
            webViewConfigEntity.isNeedShareButton = false
            webViewConfigEntity.imageUrl = ""
            webViewConfigEntity.description = ""

            val intent = Intent(context, GeneralWebViewActivity::class.java)
            intent.putExtra(CarEntity.FEATURE_ITEM, webViewConfigEntity)
            context.startActivity(intent)
        }
    }


    //修改消息状态
    fun requestMessageStatus(bulletinsBean: BulletinsBean) {
        val request = MessageStatusRequest()
        request.user_id = UserUtil(context).userId.toString()
        request.bulletin_id = bulletinsBean.bulletin_id.toString()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.messageStatusRead(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<BaseResult>(context, this) {
                    override fun onSuccess(response: BaseResult) {
                        view.dismissLoadingDialog()
                        view.onSuccessMessageStatus(response)
                    }

                    override fun onAPIError(apiException: ApiException?) {
                        super.onAPIError(apiException)
                        onMessageDetail(bulletinsBean)
                    }
                })
    }


    //系统公告列表
    override fun getSysTipList(page: Int) {
        val request = BulletinsRequest()
        request.user_id = UserUtil(context).userId.toString()
        request.page = page
        request.page_size = 20
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.bulletinList(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<BulletinsResult>(context, this) {
                    override fun onSuccess(response: BulletinsResult) {
                        view.dismissLoadingDialog()
                        view?.onSuccessMessageList(response)
                    }

                    override fun onAPIError(apiException: ApiException?) {
                        super.onAPIError(apiException)
                        view?.onFailedMessageList()
                    }
                })
    }



    fun requestMessageAllRead() {
        val request = BaseRequestBean()
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.systemMsgAllRead(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(
                object : BaseObserver<BaseResult>(context, this) {
                    override fun onSuccess(response: BaseResult) {
                        view.dismissLoadingDialog()
                        view?.showToast("已读设置完成")
                        view?.onAllReadRefresh()
                    }

                })
    }
}
