package com.jiaoyinbrother.monkeyking.manager.mvpactivity.timebooksuccess


import android.app.Activity
import android.content.Context
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.view.View
import android.widget.ImageView
import com.baidu.mapapi.map.BaiduMap
import com.baidu.mapapi.map.BitmapDescriptorFactory
import com.baidu.mapapi.map.MarkerOptions
import com.baidu.mapapi.model.LatLng
import com.bumptech.glide.Glide
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.timecancelorder.TimeCancelOrderActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.timeunlockingcar.TimeUnlockingCarActivity
import com.jiaoyinbrother.monkeyking.manager.util.DialogUtil
import com.jiaoyinbrother.monkeyking.manager.util.MKUtil
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.util.*
import com.sensorsdata.analytics.android.sdk.SensorsDataAPI
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import kotlinx.android.synthetic.main.activity_time_book_success.*
import org.json.JSONException
import org.json.JSONObject
import java.util.concurrent.TimeUnit


class TimeBookSuccessActivity : MvpBaseActivity<TimeBookSuccessPresenter>(), TimeBookSuccessView {

    private val mContext: Context = this
    private var mBaiduMap: BaiduMap? = null
    private var mUserUtil: UserUtil? = null
    private var mUserLocationUtil: UserLocationUtil? = null
//    private var mMapUtil: MapUtil? = null
    private var mSensorUtil: SensorUtil? = null
    private var mAuthUtil: AuthUtil? = null
    private var mImgNavigation: ImageView? = null


    override fun initPresenter(): TimeBookSuccessPresenter {
        return TimeBookSuccessPresenter(this, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_time_book_success
    }

    override fun initView() {
        map_view.showZoomControls(false)
        mBaiduMap = map_view.map
        mImgNavigation = findViewById(R.id.navigation_iv)
    }

    override fun initToolBar() {
        mainTitle?.text = "预订成功"
        subTitle?.text = "取消订单"
    }

    private var mDisposable: Disposable? = null

    override fun initData() {
        mUserUtil = UserUtil(mContext)
        mUserLocationUtil = UserLocationUtil(mContext)
//        mMapUtil = MapUtil(mBaiduMap)
        mSensorUtil = SensorUtil(mContext)
        mAuthUtil = AuthUtil(mContext)

        mOrderId = intent?.getStringExtra(ConstantsUtil.ORDER_ID) ?: ""

    }

    override fun onResume() {
        super.onResume()
        mSensorUtil?.registerOrientationSensor(mSensorListener)
        //轮询
        mDisposable = Observable.interval(0, 1, TimeUnit.MINUTES)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ min ->
                    presenter?.requestOrderDetail(getOrderDetailRequest())
                })
    }

    override fun onPause() {
        super.onPause()
        mSensorUtil?.unregisterOrientationSensor()
        mDisposable?.dispose()
    }

    private fun getOrderDetailRequest(): TimeOrderDetailRequest {
        val request = TimeOrderDetailRequest()
        request.user_id = mUserUtil?.userId?.toString()
        request.order_id = mOrderId
        return request
    }

    private var mSite: SiteBean? = null
    private var mCar: TimeCarBean? = null
    private var mTotalAmount = 0f
    private var mOrderId = ""
    private var mOrderStatus = ""

    override fun onOrderDetail(result: TimeOrderDetailResult?) {
        mCar = result?.car
        mSite = result?.site
        mTotalAmount = result?.bill?.total_amount ?: 0f
        mOrderStatus = result?.status ?: ""

        setPromptText(result?.prompt)
        setData()
        val center = LatLng(mSite?.location?.lat ?: 0.0, mSite?.location?.lng ?: 0.0)
//        mMapUtil?.setMapCenter(center)
        setMapMarker(mSite)

        if (TextUtils.equals(mOrderStatus, TimeOrderUtil.TIME_ORDER_CANCELED)) {
            mDisposable?.dispose()
        }
    }

    private fun setData() {

        Glide.with(this@TimeBookSuccessActivity).load(mCar?.pic_url).into(car_img_iv)

        car_num_tv?.text = mCar?.number
        car_info_tv?.text = mCar?.car_type_name + "|" + mCar?.description + "|"
        car_oil_tv?.text = mCar?.oil_percent?.toString() + "%"
        price_mile_tv?.text = mTotalAmount.toString()
        site_name_tv?.text = mSite?.name
        site_address_tv?.text = mSite?.address
    }

    private fun setPromptText(prompt: ArrayList<NameColorBean>?) {
        if (prompt == null || prompt.size == 0) {
            prompt_ll?.visibility = View.GONE
            return
        }
        prompt_ll?.visibility = View.VISIBLE
        val spannableString = SpannableStringBuilder("")
        TextUtil.generatePromptString(spannableString, prompt)
        prompt_tv.setText(spannableString)
    }


    override fun initListeners() {
        service_iv?.setOnClickListener(listener)
        subTitle?.setOnClickListener(listener)
        navigation_iv?.setOnClickListener(listener)
        light_whistle_tv?.setOnClickListener(listener)
        unlock_tv?.setOnClickListener(listener)
    }

    private fun setMapMarker(site: SiteBean?) {
//        if (site == null) return
//        val icon = BitmapDescriptorFactory.fromBitmap(mMapUtil?.getMarkerBitmap(mContext, "P"))
//        val point = LatLng(site.location?.lat ?: 0.0, site.location?.lng ?: 0.0)
//        val option = MarkerOptions()
//                .position(point)
//                .icon(icon)
//
//        mBaiduMap?.addOverlay(option)
    }

    private val listener = View.OnClickListener { v ->
        when (v) {
            service_iv -> {
                DialogUtil.setMainServiceDialog(this@TimeBookSuccessActivity, null)
            }
            subTitle -> {
                presenter?.getOrderCancelCheck(getCancelCheckRequest())
                SensorsDataAPI.sharedInstance().setViewID(subTitle, "order_sc_submit_cancel")

            }
            navigation_iv -> {
                MapUtil().startChooseMap(mContext, UserLocationUtil(mContext).userGPSLocation, mSite?.location!!, mSite?.name
                        ?: "")
                SensorsDataAPI.sharedInstance().setViewID(mImgNavigation, "order_sc_navigation")
            }
            light_whistle_tv -> {
                //showToast("闪灯鸣笛")
                presenter.requestCarControl(getCarControlRequest())
            }
            unlock_tv -> {
                presenter?.postTimeUserRequest(mUserUtil?.getUserRequest()!!)
            }
        }
    }

    private fun getCancelCheckRequest(): TimeCheckCancelRequest {
        val request = TimeCheckCancelRequest()
        request.order_id = mOrderId
        request.user_id = mUserUtil?.userId.toString()
        return request
    }

    private fun getRequestBean(): TimeUnlockingRequest {
        val request = TimeUnlockingRequest()
        request.order_id = mOrderId
        request.user_id = mUserUtil?.userId.toString()
        request.is_unlock = TimeUnlockingCarActivity.NO_NEED_UNLOCK
        return request
    }

    override fun onStartUnlockSuccess() {
        TimeUnlockingCarActivity.newInstance(mContext, mOrderId)
        //finish()
    }

    override fun setCheckCancel(result: TimeCheckCancelResult?) {
        if (result == null) return
        val title = result?.title
        val msg = result?.description
        DialogUtil.showDialogTimeCancelChance(mContext, title, msg, object : DialogUtil.OnDialogClickListener {
            override fun onLeft() {
                TimeCancelOrderActivity.startOrderCancel(mContext as Activity, TimeCancelOrderActivity.TYPE_BEFORE_UNLOCK, mOrderId, mCar?.car_id ?: "", result)
            }

            override fun onRight() {
            }

        })
    }

    override fun onUserDetail(result: UserDetailResult?) {
        when (mOrderStatus) {
            TimeOrderUtil.TIME_ORDER_ACCEPTED -> {
                presenter?.requestStartUnlock(getRequestBean())
            }
            TimeOrderUtil.TIME_ORDER_UNLOCKING -> {
                sensorsData(mOrderId, mCar?.car_id)
                TimeUnlockingCarActivity.newInstance(mContext, mOrderId)
            }
            else -> {
                finish()
            }
        }
    }


    private val mSensorListener = object : SensorUtil.MySensorListener {
        override fun onSensorChanged(de: Float) {
            doOnSensorChanged(de)
        }

    }

    private fun doOnSensorChanged(de: Float) {
//        var locationPoint = LatLng(mUserLocationUtil?.userGPSLocation?.lat
//                ?: 0.0, mUserLocationUtil?.userGPSLocation?.lng ?: 0.0)
//        var radius = mUserLocationUtil?.userGPSRadius ?: 20f
//        mMapUtil?.setMapLocation(de, locationPoint, radius)
    }

    private fun getCarControlRequest(): TimeCarControlRequest {
        val request = TimeCarControlRequest()
        request.user_id = mUserUtil?.userId.toString()
        request.role = mUserUtil?.userRole.toString()
        request.order_id = mOrderId
        request.car_id = mCar?.car_id
        request.control_type = CarControlUtil().carWhisleFlash
        return request
    }

    override fun onCarControlSuccess() {
        sensorsData1(mOrderId, mCar?.car_id ?: "", true, "")
    }


    private fun sensorsData(orderId: String, carId: String?) {
        val properties = JSONObject()
        try {
            properties.put("order_id", orderId)
            properties.put("car_id", carId)
            StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_SC_UNLOCKCAR_STEP1, properties)
        } catch (e: JSONException) {
            e.printStackTrace()
        }
    }

    private fun sensorsData1(orderId: String, carId: String, isSuccess: Boolean, reason: String) {
        val properties = JSONObject()
        try {
            properties.put("order_id", orderId)
            properties.put("car_id", carId)
            properties.put("is_success", isSuccess)
            properties.put("fail_reason", reason)
            StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_SC_WARNING, properties)
        } catch (e: JSONException) {
            e.printStackTrace()
        }
    }

    override fun onBackPressed() {

        MKUtil.goBackMainPage(this, MKUtil.GO_MAIN_TIME)
    }

}
