package com.jiaoyinbrother.monkeyking.manager.mvpactivity.timecancelorder

import android.app.Activity
import android.content.Intent
import android.graphics.Rect
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.View
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.adapter.TimeCancelOrderAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.util.MKUtil
import com.jiaoyinbrother.library.bean.TimeCancelOrderBean
import com.jiaoyinbrother.library.bean.TimeCheckCancelResult
import com.jiaoyinbrother.library.bean.TimeOrderCancelRequest
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.library.widget.GeneralBottomButton
import kotlinx.android.synthetic.main.activity_time_cancel_order.*
import org.json.JSONObject

class TimeCancelOrderActivity : MvpBaseActivity<TimeCancelOrderPresenter>(), TimeCancelOrderContract.View, EasyRecyclerViewHolder.OnItemClickListener {


    private var mOrderId = ""
    private var mCarId = ""
    private var mTimeCheckCancelResult: TimeCheckCancelResult? = null
//    private var mActivityType = ""

    private var cancelOrderAdapter = TimeCancelOrderAdapter(this@TimeCancelOrderActivity)
    private var orderBeanlist: ArrayList<TimeCancelOrderBean> = ArrayList<TimeCancelOrderBean>()
    private var mActivityType = ""

    override fun onOrderCancelSuccess() {
        onSensorCancelOrder(true)

        MKUtil.goBackMainPage(this@TimeCancelOrderActivity)
    }

    override fun onOrderCancelFail() {
        onSensorCancelOrder(false)
    }

    override fun initPresenter(): TimeCancelOrderPresenter {
        return TimeCancelOrderPresenter(this@TimeCancelOrderActivity, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_time_cancel_order
    }

    override fun initView() {

    }

    override fun initToolBar() {
        mainTitle?.text = "取消订单"
    }


    override fun initData() {

        try {
            mOrderId = intent?.getStringExtra(ConstantsUtil.ORDER_ID) ?: ""
            mCarId = intent?.getStringExtra(ConstantsUtil.CAR_ID) ?: ""
            mActivityType = intent?.getStringExtra(ConstantsUtil.ACTIVITY_TYPE) ?: ""
            mTimeCheckCancelResult = intent?.getSerializableExtra(ConstantsUtil.EXTRA_SERIALIZED) as TimeCheckCancelResult

        } catch (e: Exception) {

        }
        if (TextUtils.isEmpty(mOrderId) || TextUtils.isEmpty(mActivityType) || mTimeCheckCancelResult == null) {
            finish()
            return
        }

        val list = mTimeCheckCancelResult?.reasons
        list?.forEach({
            var orderBean = TimeCancelOrderBean()
            orderBean.content = it
            orderBean.isSelected = false
            orderBeanlist.add(orderBean)
        })

        cancelOrderAdapter.list = orderBeanlist;
        recycler_cancel_reason.adapter = cancelOrderAdapter
        recycler_cancel_reason.addItemDecoration(SpaceItemDecoration(DensityUtil.dip2px(this@TimeCancelOrderActivity, 10f)))

        goSubmit.disable()
        LogUtil.e("mActivityType =" + mActivityType)
        if (mActivityType == TYPE_BEFORE_UNLOCK) {
            LogUtil.e("TYPE_BEFORE_UNLOCK")
            goSubmit.setText("立即取消")
            textview_title.text = "您为什么取消订单？"
            textview_subcontent_1.visibility = View.GONE
            textview_subcontent_2.visibility = View.GONE
        } else {
            LogUtil.e("TYPE_BEFORE_UNLOCK else")
            goSubmit.setText("取消并锁车")
            textview_title.text = "取消订单后将立即锁车"
            textview_subcontent_1.visibility = View.VISIBLE
            textview_subcontent_2.visibility = View.VISIBLE
        }
    }

    override fun initListeners() {
        cancelOrderAdapter.setOnItemClickListener(this)

        goSubmit.setListener(object : GeneralBottomButton.SubmitListener {
            override fun submit() {
                presenter.orderCancel(getOrderCancelRequest())
            }
        })


    }

    private fun getOrderCancelRequest(): TimeOrderCancelRequest {
        var timeOrderCancelRequest = TimeOrderCancelRequest()
        timeOrderCancelRequest.order_id = mOrderId

        var selectReasons = ArrayList<String>()
        orderBeanlist.forEach({
            if (it.isSelected)
                selectReasons.add(it.content)
        })
        timeOrderCancelRequest.reasons = selectReasons

        timeOrderCancelRequest.user_id = UserUtil(this@TimeCancelOrderActivity).userId.toString()

        return timeOrderCancelRequest
    }

    override fun onItemClick(convertView: View?, position: Int) {

//             实现单选，每次点击时，先将所有的selected设为false，
        if (mActivityType == TYPE_BEFORE_UNLOCK) {
            orderBeanlist.forEach({
                it.isSelected = false
            })
            //        并且将当前点击的item 设为true， 刷新整个视图
            orderBeanlist[position].isSelected = true
        } else {
            //多选情况，改变选择状态
            orderBeanlist[position].isSelected = !orderBeanlist[position].isSelected
        }

        //有选择条目，才可以提交
        var enable = false
        orderBeanlist.forEach({
            if (it.isSelected)
                enable = true
        })

        if (enable)
            goSubmit.normal()
        else
            goSubmit.disable()

        cancelOrderAdapter.notifyDataSetChanged();

    }

    companion object {
        const val TYPE_AFTER_UNLOCK = "TYPE_AFTER_UNLOCK"
        const val TYPE_BEFORE_UNLOCK = "TYPE_BEFORE_UNLOCK"

        fun startOrderCancel(activity: Activity, type: String, orderId: String, carId: String, result: TimeCheckCancelResult) {
            val intent = Intent(activity, TimeCancelOrderActivity::class.java)
            intent.putExtra(ConstantsUtil.ORDER_ID, orderId)
            intent.putExtra(ConstantsUtil.ACTIVITY_TYPE, type)
            intent.putExtra(ConstantsUtil.CAR_ID, carId)
            intent.putExtra(ConstantsUtil.EXTRA_SERIALIZED, result)
            activity.startActivityForResult(intent, ConstantsUtil.ORDER_CANCEL_CODE)
        }
    }

    private fun onSensorCancelOrder(status: Boolean) {

        var jsonObject = JSONObject()
        try {
            var entrace = ""

            var lockStatus = 1
            if (mActivityType == TYPE_BEFORE_UNLOCK){
                entrace = "预定成功"
                lockStatus = 1
            }else{
                entrace = "验车"
                lockStatus = 2
            }
            jsonObject.put("entrance", entrace)
            jsonObject.put("order_id", mOrderId)
            jsonObject.put("car_id", mCarId)
            jsonObject.put("lock_status", lockStatus)

            var selectReasons = ArrayList<String>()
            orderBeanlist.forEach({
                if (it.isSelected)
                    selectReasons.add(it.content)
            })
            jsonObject.put("cancel_reasons", StatisticsUtil.listToJsonArray(selectReasons))
            jsonObject.put("is_success", status)
            StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_SC_CancelOrder, jsonObject)

        } catch (e: Exception) {

        }
    }

    private class SpaceItemDecoration(private val space: Int) : RecyclerView.ItemDecoration() {

        override fun getItemOffsets(outRect: Rect, view: View,
                                    parent: RecyclerView, state: RecyclerView.State?) {
            outRect.bottom = space
        }

    }

}
