package com.jiaoyinbrother.monkeyking.manager.mvpactivity.timecashierdesk

import android.app.Activity
import android.content.Intent
import android.view.View
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.activity.PayMethodActivity
import com.jiaoyinbrother.monkeyking.manager.adapter.PayMethodAdapter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.timepayresult.TimePayResultActivity
import com.jiaoyinbrother.monkeyking.manager.util.SharedPreferencesUtil
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.monkeyking.manager.wxutil.WXUitls
import com.jiaoyinbrother.library.base.BaseBean
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.library.widget.GeneralBottomButton
import kotlinx.android.synthetic.main.activity_cashier_desk.*
import org.json.JSONObject

class TimeCashierDeskActivity : MvpBaseActivity<TimeCashierDeskPresenter>(), TimeCashierDeskContract.View, EasyRecyclerViewHolder.OnItemClickListener {

    private var cashierDeskParamBean: CashierDeskParamBean? = null
    private var mAdapter = PayMethodAdapter(this@TimeCashierDeskActivity)

    //当前选中的支付方式
    private var mPayMethod: PayMethodAvaiBean? = null
    private var orderPayResult = OrderPayResult()


    companion object {
        //typeAPI类型
        const val TYPE_ORDER_PAY = "TYPE_PAY"
        const val TYPE_DEPOSIT_PAY = "TYPE_DEPOSIT"

        //orderPayType类型
//        支付类别(1：车辆押金，2：违章押金，3：租金，4：维修押金，5：清洁费，6：停运损失费，7：超期还车费，8：违章违约金，9：加速折旧费，10：随车物品损失费，11：其他)
        //只有ORDER_PAY_RENTAL_FEE = 3 写死，其他取后台接口
        const val ORDER_PAY_RENTAL_FEE = 3

        fun startTimeCashierDesk(activity: Activity, param: CashierDeskParamBean) {
            val intent = Intent(activity, TimeCashierDeskActivity::class.java)
            intent.putExtra(ConstantsUtil.EXTRA_SERIALIZED, param)
            activity.startActivity(intent)
        }
    }

    override fun onItemClick(convertView: View?, position: Int) {
        mPayMethod = mAdapter?.getItem<PayMethodAvaiBean>(position)

        if (mPayMethod == null)
            return

        mAdapter?.selectPayMethod(mPayMethod?.id ?: -1)

        LogUtil.e("当前点击支付方式:${mPayMethod.toString()}")
    }


    override fun updateTimeDepositResult(result: OrderPayResult) {

        startPayMethodActivity(result)
    }


    override fun updateTimeOrderResult(result: OrderPayResult) {
        startPayMethodActivity(result)
    }


    override fun initPresenter(): TimeCashierDeskPresenter {
        return TimeCashierDeskPresenter(this@TimeCashierDeskActivity, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_cashier_desk
    }

    override fun initView() {

    }

    override fun initToolBar() {
        mainTitle.text = "支付收银台"
    }

    override fun initData() {
        try {
            cashierDeskParamBean = intent?.getSerializableExtra(ConstantsUtil.EXTRA_SERIALIZED) as CashierDeskParamBean
        } catch (e: Exception) {

        }

        if (cashierDeskParamBean == null || Util.isEmptyList(cashierDeskParamBean?.payMethodAvailable)) {
            finish()
            return
        }
        if (Util.isEmptyList(cashierDeskParamBean?.payMethodAvailable)) {
            cashierDeskParamBean?.payMethodAvailable = defaultPayMethodList()
        }

        //初始化选择项
        mPayMethod = getPayMethod()
        updateView()

        //进入页面，埋点E_PAYMENT
        StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_PAYMENT, getGeneralSensorPayment())
    }

    /**
     * 初始化默认可用支付列表
     * */
    private fun defaultPayMethodList(): ArrayList<PayMethodAvaiBean> {
        val payMethodList = ArrayList<PayMethodAvaiBean>()
        payMethodList.add(PayMethodAvaiBean(PayMethodAvaiBean.WECHATPAY, "微信支付", "", 1))
        payMethodList.add(PayMethodAvaiBean(PayMethodAvaiBean.ALIPAY, "支付宝支付", "", 0))
        payMethodList.add(PayMethodAvaiBean(PayMethodAvaiBean.YIBAOPAY, "银行卡支付", "", 0))
        return payMethodList
    }

    override fun initListeners() {
        mAdapter?.setOnItemClickListener(this)
        onPay.setListener(object : GeneralBottomButton.SubmitListener {
            override fun submit() {
                handlePay()
            }
        })
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 101 && resultCode == Activity.RESULT_OK) {
            startPayResultActivity(data)
        }

        dismissLoadingDialog()
    }

    private fun startPayResultActivity(data: Intent?) {
        val code = data?.extras?.getInt(PayMethodActivity.RESULT_CODE)

        var properties = getGeneralSensorPayment()
        properties.put("payment_platform", mPayMethod?.name ?: "")
        var result = code == PayMethodActivity.SUCC
        properties.put("is_success", result)
        StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_PAYMENT_RESULT, properties)


        val intent = Intent(this@TimeCashierDeskActivity, TimePayResultActivity::class.java)
        intent.putExtra("entrance", cashierDeskParamBean?.entrance)

        //生成activityType
        var activityType = ""
        when (cashierDeskParamBean?.typeAPI) {
            TYPE_ORDER_PAY -> {
                activityType = if (code == PayMethodActivity.SUCC)
                    TimePayResultActivity.ORDER_PAY_SUCCESS
                else
                    TimePayResultActivity.ORDER_PAY_FAIL
            }
            TYPE_DEPOSIT_PAY -> {
                activityType = if (code == PayMethodActivity.SUCC)
                    TimePayResultActivity.DEPOSIT_PAY_SUCCESS
                else
                    TimePayResultActivity.DEPOSIT_PAY_FAIL
            }
        }
        intent.putExtra(ConstantsUtil.ACTIVITY_TYPE, activityType)

        startActivityForResult(intent, ConstantsUtil.PAYMENT_RESULT)

        //成功，则关闭收银台
        if (code == PayMethodActivity.SUCC)
            finish()
    }

    private fun handlePay() {
//        埋点
        if (!WXUitls.initWxApi(this).isWXAppInstalled && mPayMethod?.id == PayMethodAvaiBean.WECHATPAY) {
            showToast("请先安装微信")
            return
        }

        var properties = getGeneralSensorPayment()
        properties.put("payment_platform", mPayMethod?.name ?: "")
        StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_CHOOSE_PAYMENT_PLATFORM, properties)

        if (cashierDeskParamBean?.typeAPI == TYPE_DEPOSIT_PAY)

            presenter.postTimeDepositPay(getTimeDepositRequest())
        else {
            presenter.postTimeOrderPay(getTimeOrderPayRequest())
        }

    }

    /**
     * order/pay request
     * */
    private fun getTimeDepositRequest(): RechargeRequest {
        var rechargeRequest = RechargeRequest()
        rechargeRequest.user_id = UserUtil(this@TimeCashierDeskActivity).userId.toString()
        rechargeRequest.pay_method = mPayMethod?.id!!
        rechargeRequest.amount = (cashierDeskParamBean?.amount!! * 100).toInt()
        return rechargeRequest
    }


    private fun getTimeOrderPayRequest(): OrderPayRequest {
        var orderPayRequest = OrderPayRequest()
        if (cashierDeskParamBean != null) {
            orderPayRequest.order_id = cashierDeskParamBean?.orderId
            orderPayRequest.pay_method = mPayMethod?.id!!
            orderPayRequest.pay_type = cashierDeskParamBean?.orderPayType!!
            orderPayRequest.amount = (cashierDeskParamBean?.amount!! * 100).toInt()
        }
        return orderPayRequest
    }

    override fun onBackPressed() {
        IOSAlertDialog(this@TimeCashierDeskActivity).builder().setMsg("您还未支付哦，确定要离开吗？", true)
                .setCancelable(false).setRightButton("再想想") { }.setLeftButton("去意已决") {
                    finish()
                    super.onBackPressed()
                }.show()
    }


    /**
     * 初始化支付收银台页面
     * */
    private fun updateView() {
        // 预付租金提示
        payPrompt.visibility = View.GONE
        // 支付金额
        var amount = MoneyUtil().getMoneyText(cashierDeskParamBean?.amount.toString())
        payAmount.text = TextUtil.setForegroundSize("${amount}", 0.5f, 0, 1)
        // 支付项目
        payProject.text = cashierDeskParamBean?.description
        // 押金提示
        if (cashierDeskParamBean?.typeAPI == TYPE_DEPOSIT_PAY) {
            promptDepositLayout.visibility = View.VISIBLE
            promptDeposit.text = cashierDeskParamBean?.warmPrompt
        } else {
            promptDepositLayout.visibility = View.GONE
        }


        // 支付方式
        mAdapter?.setAmout(0, cashierDeskParamBean?.amount!!.toInt())
        mAdapter?.list = PayUtil(this@TimeCashierDeskActivity).getAvaiPayMethod(cashierDeskParamBean?.payMethodAvailable!!)
        mAdapter?.selectPayMethod(mPayMethod?.id ?: -1)

        payMethodRv.adapter = mAdapter

    }

    /**
     * 初始化默认支付渠道(UNIONPAY:0，WECHATPAY:6，ALIPAY:7，ACCOUNTPAY:8,YIBAOPAY:12)
     * */
    private fun getPayMethod(): PayMethodAvaiBean {
        val payMethodList = PayUtil(this@TimeCashierDeskActivity).getAvaiPayMethod(cashierDeskParamBean?.payMethodAvailable!!)
        // 存在用户习惯， 并且支付方式包含用户习惯
        for (payMethod: PayMethodAvaiBean in payMethodList) {
            if (payMethod.id == SharedPreferencesUtil.getInstance().payments) {// 选中
                LogUtil.e("getPayMethod ------> user finish")
                return payMethod
            }
        }
        // 后台提供勾选项
        for (payMethod: PayMethodAvaiBean in payMethodList) {
            if (payMethod.check == 1) {// 选中
                LogUtil.e("getPayMethod ------> system finish")
                return payMethod
            }
        }
        // 默认勾选第一项
        LogUtil.e("getPayMethod ------> defualt finish")
        return payMethodList[0]
    }


    private fun startPayMethodActivity(result: OrderPayResult) {
        showLoadingDialog()

        orderPayResult = result
        SharedPreferencesUtil.getInstance().savePayments(mPayMethod?.id ?: -1)

        val payMethodBean = PayInfoBean(PayInfoBean.PAYMENT_TYPE_ZUCHE, orderPayResult.url, orderPayResult.tn, cashierDeskParamBean?.amount, orderPayResult.pay_method, orderPayResult.order_str)
        val intent = Intent(this@TimeCashierDeskActivity, PayMethodActivity::class.java)
        intent.putExtra(ConstantsUtil.START_PAYMETHOD, payMethodBean)
        intent.putExtra("disable_pay_channels", orderPayResult.disable_pay_channels)
        intent.putExtra("enable_pay_channels", orderPayResult.enable_pay_channels)
        startActivityForResult(intent, 101)
    }

    private fun getGeneralSensorPayment(): JSONObject {

        var jsonObject = JSONObject()
        try {
            var isDeposit = false
            var orderID = ""
            var paymentType = "分时_" + cashierDeskParamBean?.description
            if (cashierDeskParamBean?.typeAPI == TYPE_DEPOSIT_PAY) {
                isDeposit = true
                orderID = ""
                paymentType = "支付分时押金"
            } else {
                isDeposit = true
                orderID = cashierDeskParamBean?.orderId!!
                if (cashierDeskParamBean?.orderPayType == TimeCashierDeskActivity.ORDER_PAY_RENTAL_FEE)
                    isDeposit = false

                paymentType = "支付分时用车费用"
            }

            jsonObject.put("entrance", cashierDeskParamBean?.entrance)
            jsonObject.put("order_id", orderID)
            jsonObject.put("payment_type", paymentType)
            jsonObject.put("is_deposit", isDeposit)
            jsonObject.put("is_offline", false)
            jsonObject.put("order_amount", cashierDeskParamBean?.amount)

        } catch (e: Exception) {

        }

        return jsonObject
    }

    class CashierDeskParamBean : BaseBean() {
        //        埋点数据
        var entrance = ""
        var typeAPI: String = ""
        var orderPayType = -1
        var orderId: String? = ""
        var description = ""
        var amount = 0.0f
        var payMethodAvailable: List<PayMethodAvaiBean> = ArrayList<PayMethodAvaiBean>()
        var warmPrompt = ""

        override fun toString(): String {
            return "CashierDeskParamBean(type='$typeAPI', orderId=$orderId, description='$description', amount=$amount, payMethodAvailable=$payMethodAvailable, warmPrompt='$warmPrompt')"
        }


    }

}
