package com.jiaoyinbrother.monkeyking.manager.mvpactivity.timecheckcar

import android.content.Context
import android.content.Intent
import android.support.v4.content.ContextCompat
import android.text.*
import android.view.View
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.timecancelorder.TimeCancelOrderActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.timefindissue.TimeFindIssueActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.timeknownissue.TimeKnownIssueActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.timeusingcar.TimeUsingCarActivity
import com.jiaoyinbrother.monkeyking.manager.util.DialogUtil
import com.jiaoyinbrother.monkeyking.manager.util.MKUtil
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.library.widget.GeneralBottomButton
import com.sensorsdata.analytics.android.sdk.SensorsDataAPI
import io.reactivex.Flowable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import kotlinx.android.synthetic.main.activity_time_check_car.*
import org.json.JSONException
import org.json.JSONObject
import java.util.concurrent.TimeUnit

class TimeCheckCarActivity : MvpBaseActivity<TimeCheckCarPresenter>(), ITimeCheckCarContract.view {


    private var carID = ""
    private var orderID = ""
    private var isFromList = false
    private var isFirst = true
    private var userUtil: UserUtil? = null
    private var mCountDown: Disposable? = null
    private var carControlType = ""
    private var isUsingCar = false

    override fun initPresenter(): TimeCheckCarPresenter {
        return TimeCheckCarPresenter(this, this)
    }

    override fun getLayoutId(): Int {
        orderID = intent.getStringExtra(ConstantsUtil.ORDER_ID)
        isFromList = intent.getBooleanExtra(ConstantsUtil.IS_FROM_LIST, false)
        return R.layout.activity_time_check_car
    }

    override fun initView() {
    }

    override fun initToolBar() {
        mainTitle.text = "验车"
        subTitle.text = "取消订单"
    }

    override fun initData() {
        userUtil = UserUtil(this)
        mAddFindIssue.text = TextUtil.setForegroundColor(mAddFindIssue.text.toString(), ContextCompat.getColor(this, R.color.color_6), 0, 1)
        presenter.requestOrderInfo(getOrderInfoRequest())
        checkUsingCar()
    }

    override fun initListeners() {
        // 取消订单
        subTitle.setOnClickListener {
            presenter.requestCheckCancel(getCheckCancel())

            SensorsDataAPI.sharedInstance().setViewID(subTitle, "order_sc_checkcar_cancel")
        }
        // 查看详情
        mDetailKnownIssue.setOnClickListener {
            TimeKnownIssueActivity.newInstance(this, orderID, carID)
            SensorsDataAPI.sharedInstance().setViewID(mDetailKnownIssue, "order_sc_checkcar_detail")

        }
        // 添加问题
        mAddFindIssue.setOnClickListener {
            TimeFindIssueActivity.newInstance(this, orderID, carID, false)
            SensorsDataAPI.sharedInstance().setViewID(mAddFindIssue, "order_sc_checkcar_new_problem")
        }
        // 开锁
        openCar.setOnClickListener {
            presenter.requestCarControl(getCarControlRequest(CarControlUtil().carUnlock), 1)
        }
        // 锁车
        lockCar.setOnClickListener {
            presenter.requestCarControl(getCarControlRequest(CarControlUtil().carLock), 1)
        }
        // 验车完成，开始用车
        onSubmit.setListener(object : GeneralBottomButton.SubmitListener {
            override fun submit() {
                presenter.requestUsingCar(getUsingCarRequest())
            }
        })
    }


    /**
     * 返回首页
     * */
    override fun onBackPressed() {
        if (!isFromList) MKUtil.goBackMainPage(this)
        super.onBackPressed()
    }

    override fun onDestroy() {
        mCountDown?.dispose()
        super.onDestroy()
    }

    /**
     * 获取订单详情
     *
     * @return TimeOrderDetailRequest
     * */
    private fun getOrderInfoRequest(): TimeOrderDetailRequest {
        LogUtil.e("Looping-----------------------------------------------------")
        val request = TimeOrderDetailRequest()
        request.order_id = orderID
        request.user_id = userUtil?.userId.toString()
        return request
    }

    /**
     * 获取验车详情
     *
     * @return TimeCheckCarRequest
     * */
    private fun getQuestionsRequest(): TimeCheckCarRequest {
        val request = TimeCheckCarRequest()
        request.user_id = userUtil?.userId.toString()
        request.car_id = carID
        request.order_id = orderID
        return request
    }

    /**
     * 分时订单取消检查
     *
     * @return TimeCheckCancelRequest
     * */
    private fun getCheckCancel(): TimeCheckCancelRequest {
        val request = TimeCheckCancelRequest()
        request.order_id = orderID
        request.user_id = userUtil?.userId.toString()
        return request
    }

    /**
     * 开始用车
     *
     * @return TimeUsingCarRequest
     * */
    private fun getUsingCarRequest(): TimeUsingCarRequest {
        val request = TimeUsingCarRequest()
        request.order_id = orderID
        return request
    }

    /**
     * 分时车辆控制
     *
     * @return TimeCarControlRequest
     * */
    private fun getCarControlRequest(type: String): TimeCarControlRequest {
        carControlType = type
        val request = TimeCarControlRequest()
        request.user_id = userUtil?.userId.toString()
        request.role = userUtil?.userRole.toString()
        request.order_id = orderID
        request.car_id = carID
        request.control_type = type
        return request
    }

    /**
     * 轮训订单详情，查看是否已经验车完成
     * */
    private fun checkUsingCar() {
        mCountDown = Flowable.intervalRange(0, 10, 0, 1, TimeUnit.MINUTES)
                .observeOn(AndroidSchedulers.mainThread())
                .doOnNext { _ ->
                    presenter.requestOrderInfo(getOrderInfoRequest())
                }
                .doOnComplete {
                    if (isUsingCar)
                        return@doOnComplete
                    isUsingCar = true
                    presenter.requestUsingCar(getUsingCarRequest())
                }
                .subscribe()
    }

    /**
     * 通知已有问题&添加问题 关闭页面
     * */
    private fun sendBroadcast() {
        val intent = Intent()
        intent.action = CHECK_FINISH
        intent.putExtra(CHECK_FINISH, CHECK_FINISH)
        this.sendBroadcast(intent)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (resultCode) {
            // 发现问题
            TimeFindIssueActivity.RESULT_SUCC -> presenter?.requestCheckQuestions(getQuestionsRequest())
            // 取消订单
            ConstantsUtil.ORDER_CANCEL_CODE -> MKUtil.goBackMainPage(this)
        }
    }

    /**
     * 订单详情
     * */
    override fun setOrderInfo(result: TimeOrderDetailResult) {
        carID = result.car?.car_id.toString()
        // prompt
        val spannableString = SpannableStringBuilder("")
        TextUtil.generatePromptString(spannableString, result.prompt)
        prompt.text = spannableString

        if (isFirst) {
            isFirst = false
            presenter.requestCheckQuestions(getQuestionsRequest())
        }

        if (TimeOrderUtil(this).orderUsingCar(result.status) && !isUsingCar) {
            isUsingCar = true
            startUsingCar()
        }
    }

    /**
     * 验车完成，开始用车
     * */
    override fun setUsingCar() {
        startUsingCar()
    }

    private fun startUsingCar() {
        if (isUsingCar)
            showToast("订单已开始，如有问题请联系客服")
        sendBroadcast()
        TimeUsingCarActivity.newInstance(this@TimeCheckCarActivity, orderID)
        finish()
    }

    /**
     * 发现问题 result
     * */
    override fun setNewQuestions(questions: String) {
        mTvFindIssue.visibility = if (TextUtils.isEmpty(questions)) View.GONE else View.VISIBLE
        mTvFindIssue.text = if (TextUtils.isEmpty(questions)) "" else questions
    }

    /**
     * 已有问题 result
     * */
    override fun setOldQuestions(questions: String) {
        if (TextUtils.isEmpty(questions)) {
            mTvKnownIssue.text = "暂无问题"
            mDetailKnownIssue.isEnabled = false
            mDetailKnownIssue.setTextColor(ContextCompat.getColor(this, R.color.color_3))
        } else {
            mTvKnownIssue.text = questions
            mDetailKnownIssue.isEnabled = true
            mDetailKnownIssue.setTextColor(ContextCompat.getColor(this, R.color.color_5))
        }
    }

    /**
     * 取消次数检查
     * */
    override fun setCheckCancel(result: TimeCheckCancelResult) {
        DialogUtil.showDialogTimeCancelChance(this, result.title, result.description, object : DialogUtil.OnDialogClickListener {
            override fun onLeft() {
                TimeCancelOrderActivity.startOrderCancel(this@TimeCheckCarActivity, TimeCancelOrderActivity.TYPE_AFTER_UNLOCK, orderID, carID, result)
            }

            override fun onRight() {}
        })
    }

    override fun setCarControlSuccess() {
        if (carControlType == CarControlUtil().carLock) {
            showToast("锁车成功")
            sensorsData(ConstantsUtil.E_SC_LOCKCAR, true, "")
        } else {
            showToast("开锁成功")
            sensorsData(ConstantsUtil.E_SC_UNLOCKCAR_STEP2, true, "")
        }
    }

    override fun setCarControlFailed(msg: String) {
        showToast(msg)
        if (carControlType == CarControlUtil().carLock) {
            sensorsData(ConstantsUtil.E_SC_LOCKCAR, false, msg)
        } else {
            sensorsData(ConstantsUtil.E_SC_UNLOCKCAR_STEP2, false, msg)
        }
    }

    companion object {
        const val CHECK_FINISH = "CHECK_FINISH"

        fun newInstance(context: Context, orderId: String, isFromList: Boolean) {
            val intent = Intent(context, TimeCheckCarActivity::class.java)
            intent.putExtra(ConstantsUtil.ORDER_ID, orderId)
            intent.putExtra(ConstantsUtil.IS_FROM_LIST, isFromList)
            context.startActivity(intent)
        }
    }

    /**
     * 神策埋点
     * */
    private fun sensorsData(key: String, success: Boolean, reason: String) {
        val properties = JSONObject()
        try {
            properties.put("order_id", orderID)
            properties.put("car_id", carID)
            properties.put("is_success", success)
            properties.put("fail_reason", reason)
            StatisticsUtil.onSensorsDataTrack(key, properties)
        } catch (e: JSONException) {
            e.printStackTrace()
        }
    }

}