package com.jiaoyinbrother.monkeyking.manager.mvpactivity.timecheckcar

import android.content.Context
import com.alibaba.android.arouter.launcher.ARouter
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.retrofit.RetrofitHelper2
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class TimeCheckCarPresenter(context: Context, view: ITimeCheckCarContract.view) :
        BasePresenterImpl<ITimeCheckCarContract.view>(context, view), ITimeCheckCarContract.presenter {

    private var errType: Int = 0

    /**
     * 分时订单详情
     *
     * @param request: TimeOrderDetailRequest
     */
    override fun requestOrderInfo(request: TimeOrderDetailRequest) {
        errType = 0
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<TimeOrderDetailResult>(context,this) {
                    override fun onSuccess(response: TimeOrderDetailResult) {
                        view.setOrderInfo(response)
                    }
                })
    }

    /**
     * 分时验车问题查询
     *
     * @param request: TimeCheckCarRequest
     */
    override fun requestCheckQuestions(request: TimeCheckCarRequest) {
        errType = 0
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeCheckQuestions(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<TimeCheckCarResult>(context,this) {
                    override fun onSuccess(response: TimeCheckCarResult) {
                        view.dismissLoadingDialog()
                        view.setOldQuestions(response.old_questions ?: "")
                        view.setNewQuestions(response.new_questions ?: "")
                    }
                })
    }

    /**
     * 分时订单开始用车
     *
     * @param request: TimeUsingCarRequest
     */
    override fun requestUsingCar(request: TimeUsingCarRequest?) {
        errType = 0
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeUsingCar(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<BaseResult>(context,this) {
                    override fun onSuccess(response: BaseResult) {
                        view.dismissLoadingDialog()
                        view.setUsingCar()
                    }
                })
    }

    /**
     * 分时订单取消检查
     *
     * @param request: TimeCheckCancelRequest
     */
    override fun requestCheckCancel(request: TimeCheckCancelRequest) {
        errType = 0
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeCheckCancel(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<TimeCheckCancelResult>(context,this) {
                    override fun onSuccess(response: TimeCheckCancelResult) {
                        view.dismissLoadingDialog()
                        view.setCheckCancel(response)
                    }
                })
    }

    /**
     * 分时订单取消检查
     *
     * @param request: TimeCheckCancelRequest
     */
    override fun requestCarControl(request: TimeCarControlRequest, errType: Int) {
        this.errType = errType
        val service = RetrofitHelper2.getInstance(context).service
        val params = RetrofitHelper2.getInstance(context).convertRequestToMap(request)
        val observable = service.timeCarControl(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<BaseResult>(context,this) {
                    override fun onSuccess(response: BaseResult) {
                        view.dismissLoadingDialog()
                        view.setCarControlSuccess()
                    }
                })
    }


    override fun handleException(apiExcpetion: ApiException?) {
        view.dismissLoadingDialog()
        if (errType == 0) {
            super.handleException(apiExcpetion)
        } else {
            when (apiExcpetion?.code) {
                ApiException.SERVER_EXCEPTION, ApiException.OTHER_EXCEPTION, ApiException.API_ERROR_EXCEPTION -> view.setCarControlFailed(apiExcpetion.message)
                ApiException.TOKEN_EXPIRED_EXCEPTION -> {
                    view.showToast(apiExcpetion.message)
                    ARouter.getInstance().build("/user/loginactivity").navigation()
                }
                else -> view.setCarControlFailed(apiExcpetion?.message)
            }
        }
    }

}