package com.jiaoyinbrother.monkeyking.manager.mvpactivity.timefeesdetail

import android.content.Context
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

/**
 * Created by Administrator on 2018/8/3.
 */

class TimeFeesDetailPresenter(context: Context, view: ITimeFeesDetailContract.view) :
        BasePresenterImpl<ITimeFeesDetailContract.view>(context, view), ITimeFeesDetailContract.presenter {

    /**
     * 账单计算
     * */
    override fun requestBillCalc(request: TimeBillCalcRequest?) {
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeBillCalc(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<TimeBillCalcResult>(context,this) {
                    override fun onSuccess(response: TimeBillCalcResult?) {
                        view.dismissLoadingDialog()
                        view.setBillCalcSuccess(response)
                    }
                })
    }

    /**
     * 订单详情
     * */
    override fun requestOrderInfo(request: TimeOrderDetailRequest) {
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<TimeOrderDetailResult>(context,this) {
                    override fun onSuccess(response: TimeOrderDetailResult?) {
                        view.dismissLoadingDialog()
                        view.setOrderSuccess(response)
                    }
                })
    }

    /**
     * 确认账单
     * */
    override fun requestOrderAccept(request: TimeOrderAcceptRequest) {
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeOrderAccept(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<BaseResult>(context,this) {
                    override fun onSuccess(response: BaseResult?) {
                        view.dismissLoadingDialog()
                        view.setAcceptSuccess()
                    }
                })
    }
}
