package com.jiaoyinbrother.monkeyking.manager.mvpactivity.timefindissue

import android.content.Context
import com.alibaba.android.arouter.launcher.ARouter
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class TimeFindIssuePresenter(context: Context, view: ITimeFindIssueContract.view) :
        BasePresenterImpl<ITimeFindIssueContract.view>(context, view), ITimeFindIssueContract.presenter {

    private var errType: Int = 0

    // 获取问题部位
    override fun requestPosition() {
        errType = 0

        val service = RetrofitHelper.getInstance(context).service

        val params = RetrofitHelper.getInstance(context).convertRequestToMap(BaseRequestBean())

        val observable = service.timeGetPositions(params).debounce(1000, TimeUnit.MILLISECONDS)

        observable.subscribeOn(Schedulers.io()).doOnSubscribe {

            view.showLoadingDialog()

        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<TimeGetPositionResult>(context,this) {
            override fun onSuccess(response: TimeGetPositionResult?) {

                val top = ArrayList<PositionsBean>()
                val ahead = ArrayList<PositionsBean>()
                val behind = ArrayList<PositionsBean>()
                val left = ArrayList<PositionsBean>()
                val right = ArrayList<PositionsBean>()
                val inside = ArrayList<PositionsBean>()

                val positions = response?.positions
                if (positions != null && positions.size > 0) {
                    for (p in positions) {
                        when (p.p_position) {
                        // 车辆顶部
                            "TOP" -> top.add(p)
                        // 车辆前方
                            "AHEAD" -> ahead.add(p)
                        // 车辆后方
                            "BEHIND" -> behind.add(p)
                        // 车辆左侧
                            "LEFT" -> left.add(p)
                        // 车辆右侧
                            "RIGHT" -> right.add(p)
                        // 车辆内饰
                            "INSIDE" -> inside.add(p)
                        }
                    }
                }

                view.onTop(top)
                view.onAhead(ahead)
                view.onBehind(behind)
                view.onLeft(left)
                view.onRight(right)
                view.onInside(inside)

                view.dismissLoadingDialog()
            }
        })

    }

    // 提交
    override fun requestData(request: TimeFindIssueRequest) {
        errType = 1

        val service = RetrofitHelper.getInstance(context).service

        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        val observable = service.timeFindIssueAdd(params).debounce(1000, TimeUnit.MILLISECONDS)

        observable.subscribeOn(Schedulers.io()).doOnSubscribe {

            view.showLoadingDialog()

        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<BaseResult>(context,this) {
            override fun onSuccess(response: BaseResult?) {

                view.dismissLoadingDialog()

                view.onSuccess()

            }
        })
    }

    override fun handleException(apiExcpetion: ApiException) {
        if (errType == 0) {
            super.handleException(apiExcpetion)
        } else {
            view.dismissLoadingDialog()
            when (apiExcpetion.code) {
                ApiException.SERVER_EXCEPTION, ApiException.OTHER_EXCEPTION, ApiException.API_ERROR_EXCEPTION -> view.showToast("提交失败，请稍后重试")
                ApiException.TOKEN_EXPIRED_EXCEPTION -> {
                    view.showToast(apiExcpetion.message)
                    ARouter.getInstance().build("/user/loginactivity").navigation()
                }
                else -> view.showToast("提交失败，请稍后重试")
            }
        }

    }
}