package com.jiaoyinbrother.monkeyking.manager.mvpactivity.timeunlockingcar

import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import android.text.SpannableStringBuilder
import android.text.TextUtils
import android.view.View
import com.bumptech.glide.Glide
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.timecheckcar.TimeCheckCarActivity
import com.jiaoyinbrother.monkeyking.manager.util.MKUtil
import com.jiaoyinbrother.monkeyking.manager.util.NetUtil
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.library.bean.TimeOrderDetailRequest
import com.jiaoyinbrother.library.bean.TimeOrderDetailResult
import com.jiaoyinbrother.library.bean.TimeUnlockingRequest
import com.jiaoyinbrother.library.bean.TimeUnlockingResult
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.library.widget.GeneralBottomButton
import com.jiaoyinbrother.library.widget.TimeUnlockDialog
import io.reactivex.Flowable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import kotlinx.android.synthetic.main.activity_lock_picking_car.*
import org.json.JSONException
import org.json.JSONObject
import java.util.concurrent.TimeUnit


class TimeUnlockingCarActivity : MvpBaseActivity<TimeUnlockingPresenter>(), ITimeUnlockingContract.view {

    private var mCountDown: Disposable? = null
    private var isCountDownSuccess = false
    private var mOrderCancel: Disposable? = null
    private var orderID = ""
    private var carID = ""
    private var userUtil: UserUtil? = null

    override fun initPresenter(): TimeUnlockingPresenter {
        return TimeUnlockingPresenter(this, this)
    }

    override fun getLayoutId(): Int {
        orderID = intent.getStringExtra(ConstantsUtil.ORDER_ID)
        return R.layout.activity_lock_picking_car
    }

    override fun initView() {}

    override fun initToolBar() {
        mainTitle.text = "开锁"
    }

    @SuppressLint("SetTextI18n")
    override fun initData() {
        userUtil = UserUtil(this)

        val spTools = SharePreferenceTools(this)
        isCountDownSuccess = spTools.getBoolean("${orderID}_isNotFirstRun", false)
        spTools.putBoolean("${orderID}_isNotFirstRun", true)
        LogUtil.e("是否是第一次进入 --->>> $isCountDownSuccess")

        if (isCountDownSuccess) unlocked.normal() else unlocked.disable()

        presenter.requestOrderInfo(getOrderInfoRequest())
    }

    override fun initListeners() {
        // 确认开锁
        unlocked.setListener(object : GeneralBottomButton.SubmitListener {
            override fun submit() {
                if (NetUtil.isNetworkAvailable(this@TimeUnlockingCarActivity)) {
                    presenter.requestUnlockData(getUnlockRequest())
                } else {
                    IOSAlertDialog(this@TimeUnlockingCarActivity)
                            .builder()
                            .setMsg("请确定您连接了网络，并重试。如有问题请联系客服", true)
                            .setRightButton("知道了", { LogUtil.e("dismiss dialog") })
                            .show()
                }
            }
        })
    }

    override fun onStart() {
        super.onStart()
        checkOrderCancel()
    }

    /**
     * 1 minutes check order cancel
     * */
    private fun checkOrderCancel() {
        // 每分钟查询一次订单是否被自动取消
        mOrderCancel = Flowable.interval(ORDER_CANCEL, TimeUnit.MINUTES)
                .onBackpressureDrop()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ _ ->
                    presenter.requestOrderInfo(getOrderInfoRequest())
                })
    }

    /**
     * 获取订单详情
     *
     * @return TimeOrderDetailRequest
     * */
    private fun getOrderInfoRequest(): TimeOrderDetailRequest {
        val request = TimeOrderDetailRequest()
        request.order_id = orderID
        request.user_id = userUtil?.userId.toString()
        return request
    }

    /**
     * 5 seconds count down active bottom button
     *
     * 从0开始发射5个数字为：0-6依次输出，延时0s执行，每1s发射一次。
    每1s发射一次**/

    private fun startCountDown() {
        mCountDown = Flowable.intervalRange(0, 6, 0, 1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .doOnNext { count ->
                    unlocked.setText("确认开锁(${COUNT_DOWN - count}" + "s)")
                }
                .doOnComplete {
                    mCountDown?.dispose()
                    isCountDownSuccess = true
                    unlocked.setText("确认开锁")
                    unlocked.normal()
                }
                .subscribe()
    }

    /**
     * 确认开锁request
     *
     * @return TimeUnlockingRequest
     **/
    private fun getUnlockRequest(): TimeUnlockingRequest {
        val request = TimeUnlockingRequest()
        request.order_id = orderID
        request.user_id = userUtil?.userId.toString()
        request.is_unlock = NEED_UNLOCK
        return request
    }

    override fun onDestroy() {
        mCountDown?.dispose()
        mOrderCancel?.dispose()
        super.onDestroy()
    }

    /**
     * 订单详情info
     *
     * @param result:TimeOrderDetailResult
     * */
    @SuppressLint("SetTextI18n")
    override fun onOrderSuccess(result: TimeOrderDetailResult) {
        if (!isCountDownSuccess) startCountDown()

        if (TimeOrderUtil(this).orderCancel(result.status)) {
            mOrderCancel?.dispose()
            IOSAlertDialog(this@TimeUnlockingCarActivity)
                    .builder()
                    .setCancelable(false)
                    .setTitle("温馨提示")
                    .setMsg("您的预约保留时间已结束，订单已为您自动取消，如有问题请联系客服。", true)
                    .setRightButton("知道了", {
                        MKUtil.goBackMainPage(this)
                        finish()
                    })
                    .show()
        } else {
            val car = result.car
            if (car != null) {
                carID = car.car_id

                Glide.with(this@TimeUnlockingCarActivity).load(car.pic_url)
                        .placeholder(R.mipmap.empty).into(carImg)
                carNumber.text = car.number
                carInfo.text = car?.car_type_name+"|"+car?.description+"|"
                car_oil_tv?.text = car?.oil_percent?.toString()+"%"
            }

            if (!TextUtils.isEmpty(result.tips)) {
                prompt.text = result.tips
            }

            if (result.prompt != null) {
                val spannableString = SpannableStringBuilder("")
                TextUtil.generatePromptString(spannableString, result.prompt)
                countDownPrompt.text = spannableString
            }

        }
    }

    /**
     * 开锁成功
     *
     * @param result:TimeUnlockingResult
     * */
    override fun onUnlockSuccess(result: TimeUnlockingResult) {
        sensorsData(true, "")
        mOrderCancel?.dispose()
        TimeUnlockDialog(this@TimeUnlockingCarActivity)
                .builder()
                .setMsg(result.image.toString())
                .setPositiveButton("去验车", View.OnClickListener {
                    TimeCheckCarActivity.newInstance(this, orderID, false)
                    finish()
                })
                .show()
    }

    override fun onFailed(reason: String) {
        sensorsData(false, reason)
        showToast(reason)
    }

    companion object {
        const val COUNT_DOWN = 5L// 倒计时5秒
        const val ORDER_CANCEL = 1L
        const val NEED_UNLOCK = "1"
        const val NO_NEED_UNLOCK = "0"

        fun newInstance(context: Context, orderId: String) {
            val intent = Intent(context, TimeUnlockingCarActivity::class.java)
            intent.putExtra(ConstantsUtil.ORDER_ID, orderId)
            context.startActivity(intent)
        }
    }

    /**
     * 埋点
     * */
    private fun sensorsData(success: Boolean, reason: String) {
        val properties = JSONObject()
        try {
            properties.put("order_id", orderID)
            properties.put("car_id", carID)
            properties.put("is_success", success)
            properties.put("fail_reason", reason)
            StatisticsUtil.onSensorsDataTrack(ConstantsUtil.E_SC_UNLOCKCAR_STEP2, properties)
        } catch (e: JSONException) {
            e.printStackTrace()
        }
    }
}