package com.jiaoyinbrother.monkeyking.manager.mvpactivity.timeunlockingcar

import android.content.Context
import com.alibaba.android.arouter.launcher.ARouter
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.retrofit.RetrofitHelper2
import com.jiaoyinbrother.library.util.LogUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class TimeUnlockingPresenter(context: Context, view: ITimeUnlockingContract.view) :
        BasePresenterImpl<ITimeUnlockingContract.view>(context, view), ITimeUnlockingContract.presenter {


    override fun handleException(apiExcpetion: ApiException) {
        view.dismissLoadingDialog()
        when (apiExcpetion.code) {
            ApiException.SERVER_EXCEPTION, ApiException.OTHER_EXCEPTION, ApiException.API_ERROR_EXCEPTION -> view.onFailed(apiExcpetion.message)
            ApiException.TOKEN_EXPIRED_EXCEPTION -> {
                view.showToast(apiExcpetion.message)
                ARouter.getInstance().build("/user/loginactivity").navigation()
            }
            else -> view.onFailed(apiExcpetion.message)
        }
    }

    override fun requestUnlockData(request: TimeUnlockingRequest) {

        val service = RetrofitHelper2.getInstance(context).service

        val params = RetrofitHelper2.getInstance(context).convertRequestToMap(request)

        val observable = service.timeUnlocking(params).debounce(1000, TimeUnit.MILLISECONDS)

        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<TimeUnlockingResult>(context,this) {
                    override fun onSuccess(response: TimeUnlockingResult?) {
                        view.dismissLoadingDialog()
                        view.onUnlockSuccess(response)
                    }
                })
    }


    override fun requestOrderInfo(request: TimeOrderDetailRequest?) {
        LogUtil.e("TimeUnlockingPresenter ----------------------------------------------------")
        val service = RetrofitHelper.getInstance(context).service

        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        val observable = service.timeOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)

        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<TimeOrderDetailResult>(context,this) {
                    override fun onSuccess(response: TimeOrderDetailResult?) {
                        view.onOrderSuccess(response)
                    }
                })
    }
}