package com.jiaoyinbrother.monkeyking.manager.mvpactivity.timeusingcar

import android.content.Context
import com.alibaba.android.arouter.launcher.ARouter
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.ApiException
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.retrofit.RetrofitHelper2
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class TimeUsingCarPresenter(context: Context, view: ITimeUsingCarContract.view) :
        BasePresenterImpl<ITimeUsingCarContract.view>(context, view), ITimeUsingCarContract.presenter {


    private var errType: Int = 0
    /**
     * 订单详情
     * */
    override fun requestOrderInfo(request: TimeOrderDetailRequest?) {
        errType = 0
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeOrderDetail(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {}
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<TimeOrderDetailResult>(context,this) {
                    override fun onSuccess(response: TimeOrderDetailResult?) {
                        view.setOrderSuccess(response)
                    }
                })
    }

    /**
     * 轮询计算订单
     * */
    override fun requestCalcInfo(request: TimeOrderCalcRequest?) {
        errType = 0
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeOrderCalc(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {}
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<TimeOrderCalcResult>(context,this) {
                    override fun onSuccess(response: TimeOrderCalcResult?) {
                        view.setCalcSuccess(response)
                    }
                })
    }

    /**
     * 分时订单取消检查
     */
    override fun requestCarControl(request: TimeCarControlRequest, errType: Int) {
        this.errType = errType
        val service = RetrofitHelper2.getInstance(context).service
        val params = RetrofitHelper2.getInstance(context).convertRequestToMap(request)
        val observable = service.timeCarControl(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<BaseResult>(context,this) {
                    override fun onSuccess(response: BaseResult) {
                        view.dismissLoadingDialog()
                        view.setCarControlSuccess()
                    }
                })
    }


    /**
     * 车辆所在网点变更
     */
    override fun requestSiteModify(request: TimeSiteModifyRequest) {
        this.errType = 0
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeSiteModify(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {
                    view.showLoadingDialog()
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<BaseResult>(context,this) {
                    override fun onSuccess(response: BaseResult) {
                        view.dismissLoadingDialog()
                        view.setSiteModify()
                    }
                })
    }

    /**
     * 去还车
     * */
    override fun requestBackCar(request: TimeOrderCalcRequest) {
        errType = 0
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.timeOrderCalc(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable
                .subscribeOn(Schedulers.io())
                .doOnSubscribe {}
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(object : BaseObserver<TimeOrderCalcResult>(context,this) {
                    override fun onSuccess(response: TimeOrderCalcResult?) {
                        view.setBackCar(response)
                    }
                })
    }

    override fun handleException(apiExcpetion: ApiException?) {
        view.dismissLoadingDialog()
        if (errType == 0) {
            super.handleException(apiExcpetion)
        } else {
            when (apiExcpetion?.code) {
                ApiException.SERVER_EXCEPTION, ApiException.OTHER_EXCEPTION, ApiException.API_ERROR_EXCEPTION -> view.setCarControlFailed(apiExcpetion.message)
                ApiException.TOKEN_EXPIRED_EXCEPTION -> {
                    view.showToast(apiExcpetion.message)
                    ARouter.getInstance().build("/user/loginactivity").navigation()
                }
                else -> view.setCarControlFailed(apiExcpetion?.message)
            }
        }
    }
}