package com.jiaoyinbrother.monkeyking.manager.mvpactivity.uploadcheckimage

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.text.TextUtils
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.CheckImgBean
import com.jiaoyinbrother.library.impl.UploadListener
import com.jiaoyinbrother.library.util.AliYunPictureUtil
import com.jiaoyinbrother.library.util.BitmapUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.util.PhotoUtils
import com.jiaoyinbrother.library.widget.BottomDialog
import com.jiaoyinbrother.monkeyking.manager.CarEntity
import com.jiaoyinbrother.monkeyking.manager.Constants
import com.jiaoyinbrother.monkeyking.manager.activity.ShowBigImageActivity
import com.jiaoyinbrother.monkeyking.manager.util.ImageSelectorUtils
import java.io.File
import kotlin.collections.ArrayList

class UploadCheckImagePresenter(context: Context, view: IUploadCheckImageContract.View)
    : BasePresenterImpl<IUploadCheckImageContract.View>(context, view), IUploadCheckImageContract.Presenter {


    companion object {
        private val PIC_OIL = BitmapUtil.SYSTEM_BITMAP_PATH + "check_car_oil.jpg"
        private val PIC_FRONT = BitmapUtil.SYSTEM_BITMAP_PATH + "check_car_front.jpg"
        private val PIC_FRONT_LEFT = BitmapUtil.SYSTEM_BITMAP_PATH + "check_car_front_left.jpg"
        private val PIC_LEFT = BitmapUtil.SYSTEM_BITMAP_PATH + "check_car_left.jpg"
        private val PIC_BACK_LEFT = BitmapUtil.SYSTEM_BITMAP_PATH + "check_car_back_left.jpg"
        private val PIC_FRONT_RIGHT = BitmapUtil.SYSTEM_BITMAP_PATH + "check_car_front_right.jpg"
        private val PIC_RIGHT = BitmapUtil.SYSTEM_BITMAP_PATH + "check_car_right.jpg"
        private val PIC_BACK_RIGHT = BitmapUtil.SYSTEM_BITMAP_PATH + "check_car_back_right.jpg"

        const val INDEX_OIL = 0
        const val INDEX_FRONT = 1
        const val INDEX_LEFT_FRONT = 2
        const val INDEX_LEFT = 3
        const val INDEX_LEFT_BACK = 4
        const val INDEX_RIGHT_BACK = 5
        const val INDEX_RIGHT = 6
        const val INDEX_RIGHT_FRONT = 7

    }

    private var mSelectIndex = 0

    private var mUploadUtil: AliYunPictureUtil? = null
    private var postImages = ArrayList<String>()
    private var status = 0

    private var imagesList = ArrayList<CheckImgBean>()


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            CarEntity.REQUEST_CAMERA -> {
                if (resultCode == Activity.RESULT_OK) {
                    handlePhoto(File(getFilePath()))
                }
            }

            Constants.REQUEST_PIC -> {
                if (data != null) {
                    val imagePath = data.getStringArrayListExtra(ImageSelectorUtils.SELECT_RESULT)
                    if (imagePath != null && imagePath.size > 0) {
                        if (imagePath[0] == null) return

                        handlePhoto(File(imagePath[0]))
                    }
                }
            }
        }
    }

    private fun handlePhoto(actualImage: File?) {
        if (actualImage == null)
            return

        try {
            val path = PhotoUtils.compressFile(context, actualImage)
            setPath(path)
        } catch (e: Exception) {
        }
    }

    private fun getFilePath(): String {
        return when (mSelectIndex) {
            INDEX_OIL -> PIC_OIL
            INDEX_FRONT -> PIC_FRONT
            INDEX_LEFT_FRONT -> PIC_FRONT_LEFT
            INDEX_LEFT -> PIC_LEFT
            INDEX_LEFT_BACK -> PIC_BACK_LEFT
            INDEX_RIGHT_FRONT -> PIC_FRONT_RIGHT
            INDEX_RIGHT -> PIC_RIGHT
            INDEX_RIGHT_BACK -> PIC_BACK_RIGHT
            else -> PIC_OIL
        }
    }

    private fun setPath(path: File) {
        LogUtil.e("mSelectIndex =  $mSelectIndex")
        imagesList[mSelectIndex].path = path.absolutePath
        imagesList[mSelectIndex].url = ""

        updateUI()
    }

    override fun onGetPic(index: Int) {
        mSelectIndex = index
        showMenu(getFilePath())
    }

    private fun showMenu(path: String) {
        BottomDialog(context)
                .builder()
                .addItems(arrayOf("拍照", "相册"))
                .setCancel("取消") {}
                .setItemClick { _, i ->
                    when (i) {
                        0 -> view.setCamera(path)
                        1 -> view.setAlbum()
                    }
                }.show()
    }


    override fun showBigImage(url: String) {
        ShowBigImageActivity.newInstanceUri(context, url, false)
    }

    override fun onDelPath(index: Int) {

        imagesList[index].url = ""
        imagesList[index].path = ""

        updateUI()
    }


    private fun activeSure() {
        if ((!TextUtils.isEmpty(imagesList[0].url) || !TextUtils.isEmpty(imagesList[0].path))
                && (!TextUtils.isEmpty(imagesList[1].url) || !TextUtils.isEmpty(imagesList[1].path))) {
            view.setActiveSure(true)
        } else {
            view.setActiveSure(false)
        }
    }


    override fun onLoadPhotos() {
        view.showLoadingDialog()
        //清除list,否则会有多张图片


        postImages.clear()

        var index = 1
        for (entity in imagesList) {
            if (!TextUtils.isEmpty(entity.path)) {
                entity.alkey = index
                index++
                postImages.add(entity.path ?: "")
            }
        }

        mUploadUtil = AliYunPictureUtil.getInstance(context)
        mUploadUtil?.setDatas(AliYunPictureUtil.BUCKET_TYPE_PUBLIC, postImages, AliYunPictureUtil.VALIDATE_CAR, listener)

    }

    private val listener = object : UploadListener {

        override fun onUploadComplete(success: java.util.HashMap<Int, String>, failure: java.util.HashMap<Int, String>) {
            view?.dismissLoadingDialog()
            if (success.size > 0) {
                postImages.clear()

                for (entity in imagesList) {
                    if (entity.alkey != -1) {
                        entity.url = success[entity.alkey]
                        entity.alkey = -1
                    }
                }

                if (status == 4) {
                    view.setResultStatus4(imagesList)
                } else {
                    view.setResultStatus5(imagesList)
                }
            } else {
                view.showToast("上传失败")
            }
        }

        override fun onUploadComplete(net: String?, local: String?) {}
    }


    override fun getData(intent: Intent) {
        status = intent.getIntExtra("ORDER_STATUS", 0)
        when (status) {
            4 -> view.setHideReturn()
            5 -> view.setHideTake()
        }

        imagesList = intent.getSerializableExtra("CHECK_IMAGES") as ArrayList<CheckImgBean>

        if (imagesList.size == 0) {
            for (i in 0..INDEX_RIGHT_FRONT) {
                val entity = CheckImgBean()
                entity.index = i
                imagesList.add(entity)
            }
        }
        updateUI()


    }

    private fun updateUI() {
        for (entity in imagesList) {
            when (entity.index) {
                INDEX_OIL -> {
                    if (status == 4) {
                        view.setOilPhoto(if (TextUtils.isEmpty(entity.url)) entity.path else entity.url)
                    } else {
                        view.setReturnOilPhoto(if (TextUtils.isEmpty(entity.url)) entity.path else entity.url)
                    }
                }

                INDEX_FRONT -> view.setFrontPhoto(if (TextUtils.isEmpty(entity.url)) entity.path else entity.url)

                INDEX_LEFT_FRONT -> view.setLeftFrontPhoto(if (TextUtils.isEmpty(entity.url)) entity.path else entity.url)

                INDEX_LEFT -> view.setLeftPhoto(if (TextUtils.isEmpty(entity.url)) entity.path else entity.url)

                INDEX_LEFT_BACK -> view.setLeftBackPhoto(if (TextUtils.isEmpty(entity.url)) entity.path else entity.url)

                INDEX_RIGHT_BACK -> view.setRightBackPhoto(if (TextUtils.isEmpty(entity.url)) entity.path else entity.url)

                INDEX_RIGHT -> view.setRightPhoto(if (TextUtils.isEmpty(entity.url)) entity.path else entity.url)

                INDEX_RIGHT_FRONT -> view.setRightFrontPhoto(if (TextUtils.isEmpty(entity.url)) entity.path else entity.url)
            }
        }
        activeSure()
    }
}
