package com.jiaoyinbrother.monkeyking.manager.mvpactivity.workflowlist

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v4.view.ViewPager
import android.view.View
import com.jiaoyinbrother.library.bean.OrderReletApplyListRowsBean
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.library.widget.OrderListSortView
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.adapter.MyFragmentAdapter
import com.jiaoyinbrother.monkeyking.manager.fragment.workflowlist.WorkflowListFragment
import com.jiaoyinbrother.monkeyking.manager.fragment.workflowlist.WorkflowListPresenter
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.MvpBaseActivity

import kotlinx.android.synthetic.main.activity_work_flow_list.*

class WorkFlowListActivity : MvpBaseActivity<WorkflowPresenter>(), IWorkflowContract.view {
    private var mAdapter: MyFragmentAdapter? = null

    private var pendingListFragment: WorkflowListFragment? = null
    private var approvedListFragment: WorkflowListFragment? = null
    private var deniedListFragment: WorkflowListFragment? = null
    private var cancelListFragment: WorkflowListFragment? = null

    companion object {
        const val START_PAGE_NUM_KEY = "START_PAGE_NUM_KEY"

        const val PAGE_TYPE_KEY = "PAGE_TYPE_KEY"
        const val PAGE_TYPE_RELET = 0
        const val PAGE_TYPE_MODIFY = 1
        const val PAGE_TYPE_ADVANCE = 2
        const val PAGE_TYPE_AUDITING = 3

        fun newInstance(context: Context, pageNo: Int, type: Int) {
            LogUtil.e("WorkFlowListActivity, newInstance, pageNo=$pageNo, type=$type")
            val intent = Intent(context, WorkFlowListActivity::class.java)
            intent.putExtra(START_PAGE_NUM_KEY, pageNo)
            intent.putExtra(PAGE_TYPE_KEY, type)
            context.startActivity(intent)
        }
    }

    override fun initListeners() {
        managerTitleBack?.setOnClickListener {
            finish()
        }

        auditing_radio_button.setOnClickListener {
            LogUtil.e("auditing_radio_button clicked")
            managerOrderViewPage.currentItem = 0
            pendingListFragment?.updataFragment()
        }

        approved_radio_button?.setOnClickListener {
            LogUtil.e("approved_radio_button clicked")
            managerOrderViewPage.currentItem = 1
            approvedListFragment?.updataFragment()
        }

        rejected_radio_button?.setOnClickListener {
            LogUtil.e("rejected_radio_button clicked")
            managerOrderViewPage.currentItem = 2
            deniedListFragment?.updataFragment()
        }

        canceled_radio_button?.setOnClickListener {
            LogUtil.e("canceled_radio_button clicked")
            managerOrderViewPage.currentItem = 3
            cancelListFragment?.updataFragment()
        }


        // 排序
        managerOrderSortView.setListener(object : OrderListSortView.OrderSortListener {
            override fun onDefault(sort: Int) {
                onSortOrder(sort, false)
            }

            override fun onTakeCar(sort: Int, isASC: Boolean) {
                onSortOrder(sort, isASC)
            }

            override fun onReturnCar(sort: Int, isASC: Boolean) {
                onSortOrder(sort, isASC)
            }
        })
    }

    override fun initPresenter(): WorkflowPresenter {
        return WorkflowPresenter(this, this)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_work_flow_list
    }

    override fun initView() {

    }

    override fun initToolBar() {

    }

    override fun initData() {
        val initPageNo = intent.getIntExtra(START_PAGE_NUM_KEY, 0)
        val type = intent.getIntExtra(PAGE_TYPE_KEY, 0)

        var pageTitle = ""
        when (type) {
            PAGE_TYPE_MODIFY -> pageTitle = "修改时间工单"
            PAGE_TYPE_RELET -> pageTitle = "续租工单"
            PAGE_TYPE_ADVANCE -> pageTitle = "提前还车工单"
            PAGE_TYPE_AUDITING -> {

                pageTitle = "异常价格工单"
                approved_radio_button.text = "已处理"
            }
        }
        page_title_textview?.text = pageTitle

        managerOrderSortView.visibility = if (type == PAGE_TYPE_AUDITING) View.GONE else View.VISIBLE
        managerOrderViewPage.offscreenPageLimit = 4

        //初始化Adapter---------------------
        mAdapter = MyFragmentAdapter(this, supportFragmentManager)

        //续租工单
        pendingListFragment = WorkflowListFragment()
        val bundle1 = Bundle()
        bundle1.putString(WorkflowListPresenter.WORK_LIST_APPLY_STATUS_KEY,
                OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_AUDITING)
        bundle1.putInt(PAGE_TYPE_KEY, type)
        pendingListFragment?.arguments = bundle1
        mAdapter?.addFragment(pendingListFragment)

        //同意
        approvedListFragment = WorkflowListFragment()
        val bundle2 = Bundle()
        bundle2.putString(WorkflowListPresenter.WORK_LIST_APPLY_STATUS_KEY,
                OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_AUDITED)
        bundle2.putInt(PAGE_TYPE_KEY, type)
        approvedListFragment?.arguments = bundle2
        mAdapter?.addFragment(approvedListFragment)

        //拒绝
        deniedListFragment = WorkflowListFragment()
        val bundle3 = Bundle()
        bundle3.putString(WorkflowListPresenter.WORK_LIST_APPLY_STATUS_KEY,
                OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_REJECT)
        bundle3.putInt(PAGE_TYPE_KEY, type)
        deniedListFragment?.arguments = bundle3
        mAdapter?.addFragment(deniedListFragment)

        //取消
        cancelListFragment = WorkflowListFragment()
        val bundle4 = Bundle()
        bundle4.putString(WorkflowListPresenter.WORK_LIST_APPLY_STATUS_KEY,
                OrderReletApplyListRowsBean.WORK_LIST_APPLY_STATUS_CANCEL)
        bundle4.putInt(PAGE_TYPE_KEY, type)
        cancelListFragment?.arguments = bundle4
        mAdapter?.addFragment(cancelListFragment)

        //设置adapter----------------
        managerOrderViewPage.adapter = mAdapter

        setPageNo(initPageNo)
    }

    private fun setPageNo(pageNo: Int) {
        when (pageNo) {
            0 -> {
                managerOrderViewPage.currentItem = 0
                auditing_radio_button.isChecked = true
            }
            3 -> {
                //定位fragment
                managerOrderViewPage.currentItem = 3
                canceled_radio_button.isChecked = true
            }
        }
    }

    private fun onSortOrder(sortType: Int, isASC: Boolean) {
        pendingListFragment?.onSort(sortType, isASC)
        approvedListFragment?.onSort(sortType, isASC)
        deniedListFragment?.onSort(sortType, isASC)
        cancelListFragment?.onSort(sortType, isASC)
    }

}
