package com.jiaoyinbrother.monkeyking.manager.mvpdialog.cartypesearch

import android.arch.lifecycle.Observer
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.support.v4.app.DialogFragment
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import android.view.WindowManager
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.library.bean.CarTypeSearchRowsBean
import com.jiaoyinbrother.library.util.HawkUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpdialog.MVPDialogFragment
import com.jiaoyinbrother.monkeyking.manager.util.SharedPreferencesUtil
import kotlinx.android.synthetic.main.activity_hand_write.view.*
import kotlinx.android.synthetic.main.dialog_car_type_search.*

class CarTypeSearchDialog : MVPDialogFragment<CarTypeSearchPresenter>(), ICarTypeSearchContract.View{

    private var mHotListAdapter:OneLineItemAdapter? = null

    private var mHistoryListAdapter:CarTypeSearchAdapter? = null

    private var mSearchListAdapter:CarTypeSearchAdapter? = null

    companion object{
        const val CAR_TYPE_SEARCH_CHECK = "CAR_TYPE_SEARCH_CHECK"
        const val CAR_TYPE_HOT_CHECK = "CAR_TYPE_HOT_CHECK"
    }

    override fun getLayoutId(): Int {
        return R.layout.dialog_car_type_search
    }

    override fun initViews(view: View?, savedInstanceState: Bundle?) {
        setStyle(DialogFragment.STYLE_NO_FRAME, R.style.Dialog_Transparent)
    }

    override fun initToolbar(savedInstanceState: Bundle?) {

    }

    override fun initListeners() {
        search_textview?.setOnClickListener {
            val str = search_et.text.toString()
            if (str.isNotEmpty()){

                presenter?.onSearchClicked(str)
            }else{

                showToast("请输入搜索内容！")
            }
        }
        back_imageview?.setOnClickListener {
            dismiss()
        }

        search_et.addTextChangedListener(object : TextWatcher{
            override fun afterTextChanged(p0: Editable?) {

                val content = p0.toString()

                if (content.isEmpty()){

                    search_list_recycler_view.visibility = View.GONE
                    no_data_ll.visibility = View.GONE
                }

            }

            override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
            }

            override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
            }

        })

    }

    override fun initData() {

        presenter?.initData()
        mHotListAdapter = OneLineItemAdapter(context!!)
        hot_list_recycler_view?.adapter = mHotListAdapter

        mHistoryListAdapter = CarTypeSearchAdapter()
        history_list_recycler_view?.adapter = mHistoryListAdapter

        mSearchListAdapter = CarTypeSearchAdapter()
        search_list_recycler_view?.adapter = mSearchListAdapter

        presenter?.setHistoryCarTypeList()

        LiveDataBus.get().with(CAR_TYPE_SEARCH_CHECK,CarTypeSearchRowsBean::class.java).observe(this, Observer {

            if (it != null){
                HawkUtil().setCarTypeSearchHistory(it)
            }

            dismiss()
        })

        LiveDataBus.get().with(CAR_TYPE_HOT_CHECK,String::class.java).observe(this, Observer {

            if (it?.isNotEmpty() == true){

                presenter?.onSearchClicked(it)
            }

        })
    }

    override fun initPresenter(): CarTypeSearchPresenter {
        return CarTypeSearchPresenter(context!!,this)
    }

    override fun updateHotCarTypeList(list: List<String>) {
        LogUtil.e("updateHotCarTypeList,list =$list")

        if (list.isNotEmpty()){
            car_type_search_hot_title?.visibility = View.VISIBLE
            hot_list_recycler_view?.visibility = View.VISIBLE
        }else{
            car_type_search_hot_title?.visibility = View.GONE
            hot_list_recycler_view?.visibility = View.GONE
        }
        mHotListAdapter?.list = list
        mHotListAdapter?.notifyDataSetChanged()
    }

    override fun updateCarTypeList(list: List<CarTypeSearchRowsBean>?) {
        LogUtil.e("updateCarTypeList,list =$list")

        if (list == null || list.isEmpty()){

            no_data_ll?.visibility = View.VISIBLE
            search_list_recycler_view?.visibility = View.GONE
        }else{
            no_data_ll?.visibility = View.GONE
            search_list_recycler_view?.visibility = View.VISIBLE
        }

        mSearchListAdapter?.list = list
        mSearchListAdapter?.notifyDataSetChanged()
        search_list_recycler_view?.scrollToPosition(0)

    }

    override fun updateHistoryCarTypeList(list: List<CarTypeSearchRowsBean>) {

        if (list.isNotEmpty()){
            car_type_search_history_title?.visibility = View.VISIBLE
            history_list_recycler_view?.visibility = View.VISIBLE
        }else{
            car_type_search_history_title?.visibility = View.GONE
            history_list_recycler_view?.visibility = View.GONE
        }

        mHistoryListAdapter?.list = list
        mHistoryListAdapter?.notifyDataSetChanged()

    }


    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        dialog?.window?.setBackgroundDrawable(ColorDrawable(Color.WHITE))
        dialog?.window?.setLayout(WindowManager.LayoutParams.MATCH_PARENT, WindowManager.LayoutParams.MATCH_PARENT)
        dialog?.window?.setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN)
    }

}
