package com.jiaoyinbrother.monkeyking.manager.mvpdialog.cartypesearch

import android.content.Context
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.bean.*
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.util.HawkUtil
import com.jiaoyinbrother.library.util.LogUtil
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit


class CarTypeSearchPresenter(context: Context, view: ICarTypeSearchContract.View) :
        BasePresenterImpl<ICarTypeSearchContract.View>(context, view), ICarTypeSearchContract.Presenter {

    override fun setHistoryCarTypeList() {
        val list = HawkUtil().getCarTypeSearchHistory()

        view.updateHistoryCarTypeList(list)
    }

    override fun initData() {
        LogUtil.e("CarTypeSearchPresenter initData")
        apiHotCarTypeSearch()
    }

    private fun apiHotCarTypeSearch() {
        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val request = HotCarTypeSearchRequest()
        request.key = "cartype.add.hotname"

        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        val observable = mRetrofitService.hotCarTypeSearch(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<HotCarTypeSearchResult>(context, this) {
            override fun onSuccess(response: HotCarTypeSearchResult?) {

                if (response?.value != null)
                    view.updateHotCarTypeList(response.value!!)

                view.dismissLoadingDialog()
            }
        })

    }

    private fun apiCarTypeSearch(keyword: String) {
        val mRetrofitService = RetrofitHelper.getInstance(context).service
        val request = CarTypeSearchRequest()
        request.name = keyword
        request.page = 1
        request.page_size = 100

        val hashMap = RetrofitHelper.getInstance(context).convertRequestToMap(request)

        val observable = mRetrofitService.carTypeSearch(hashMap).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread()).subscribe(object : BaseObserver<CarTypeSearchResult>(context, this) {
            override fun onSuccess(response: CarTypeSearchResult?) {

                view.updateCarTypeList(response?.rows)

                view.dismissLoadingDialog()
            }
        })
    }

    override fun onSearchClicked(keyword: String) {
        LogUtil.e("CarTypeSearchPresenter onSearchClicked")
        apiCarTypeSearch(keyword)
    }

}
