package com.jiaoyinbrother.monkeyking.manager.mvpdialog.selectdate

import android.content.Context
import android.os.Bundle
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.util.DateTimeUtil
import java.lang.Exception
import java.text.SimpleDateFormat
import java.util.*

open class SelectDatePresenter(context: Context, view: ISelectDateContract.View) :
        BasePresenterImpl<ISelectDateContract.View>(context, view), ISelectDateContract.Presenter {
    private val TIME_SPAN = 5

    //    用户选择的tab
    private var mSelectDateTab = SelectDateMVPDialog.SELECT_DATE_TAB_START

    private val simpleDateFormat = SimpleDateFormat("yyyy-MM-dd", Locale.CHINA)

    //用户选择的开始时间，也是初始化传入的起始时间;
    private var mSelectedStartTime: Date? = null

    //用户选择的结束时间，也是初始化传入的还车时间
    private var mSelectedEndTime: Date? = null

    //时间控件日期的开始时间
    private var mMaxDate: Date? = null

    //时间控件日期的结束时间
    private var mMinDate: Date? = null

    //日历控件的数据list
    private val mMonthListDate = mutableListOf<String>()
    private val mDayListDate = mutableListOf<String>()
    private val dayOfMonthMap = mutableMapOf<String, MutableList<String>>()

    //回调
    private var mOnResultListener: SelectDateMVPDialog.OnResultListener? = null

    override fun initData(arguments: Bundle, onResultListener: SelectDateMVPDialog.OnResultListener) {
        mSelectDateTab = arguments.getInt(SelectDateMVPDialog.SELECTED_DATE_TAB_TYPE, SelectDateMVPDialog.SELECT_DATE_TAB_END)
        try {
            arguments.getString(SelectDateMVPDialog.SELECT_DATE_START_DATE)?.let {
                mSelectedStartTime = simpleDateFormat.parse(it)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        try {
            arguments.getString(SelectDateMVPDialog.SELECT_DATE_END_DATE)?.let {
                mSelectedEndTime = simpleDateFormat.parse(it)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        mOnResultListener = onResultListener
        mMinDate = Date()
        mMaxDate = DateTimeUtil.getTheLastDayAfterMonths(TIME_SPAN)
        setDateList()
        refreshUi()
    }

    private fun refreshUi() {
        mSelectedStartTime = mSelectedStartTime ?: mMinDate
        mSelectedEndTime = mSelectedEndTime ?: mMaxDate
        view.setDateTabView(mSelectDateTab == SelectDateMVPDialog.SELECT_DATE_TAB_START)
        val date = if (mSelectDateTab == SelectDateMVPDialog.SELECT_DATE_TAB_START) mSelectedStartTime else mSelectedEndTime
        val calendar = Calendar.getInstance(Locale.CHINA)
        calendar.time = date
        mDayListDate.clear()
        val year = calendar[Calendar.YEAR]
        val month = calendar[Calendar.MONTH]
        val yearString = "${year}-${month + 1}"
        mDayListDate.addAll(dayOfMonthMap[yearString] ?: emptyList())
        view.setDayView(mDayListDate.map {
            val split = it.split("-")
            if (split.size == 2) {
                "${split[0]}日${getWeek(split[1].toInt())}"
            } else {
                ""
            }
        })
        view.setYearIndex(mMonthListDate.indexOf(yearString))
        val dayString = "${calendar[Calendar.DAY_OF_MONTH]}-${calendar[Calendar.DAY_OF_WEEK]}"
        view.setDayIndex(mDayListDate.indexOf(dayString))

        val start = Calendar.getInstance()
        start.time = mSelectedStartTime ?: mMinDate
        val end = Calendar.getInstance()
        end.time = mSelectedEndTime ?: mMaxDate
        val endDay = end[Calendar.DAY_OF_YEAR]
        val startDay = start[Calendar.DAY_OF_YEAR]
        if (start[Calendar.YEAR] == end[Calendar.YEAR]) {
            view.setTitle("${endDay - startDay + 1}天")
        } else {
            view.setTitle("${start.getActualMaximum(Calendar.DAY_OF_YEAR) - startDay + endDay + 1}天")
        }
        view.setStartAndEndDate(simpleDateFormat.format(mSelectedStartTime), simpleDateFormat.format(mSelectedEndTime))
    }

    private fun setDateList() {
        val start = Calendar.getInstance()
        start.time = mMinDate
        val end = Calendar.getInstance()
        end.time = mMaxDate
        var prevMonth = start[Calendar.MONTH]
        var prevYear = start[Calendar.YEAR]
        var days = mutableListOf<String>()
        while (start <= end) {
            val currentMonth = start[Calendar.MONTH]
            if (prevMonth != currentMonth) {
                val key = "${prevYear}-${prevMonth + 1}"
                mMonthListDate.add(key)
                dayOfMonthMap[key] = days
                days = mutableListOf<String>()
                prevYear = start[Calendar.YEAR]
                prevMonth = currentMonth
            }
            days.add("${start[Calendar.DAY_OF_MONTH]}-${start[Calendar.DAY_OF_WEEK]}")
            start.add(Calendar.DAY_OF_YEAR, 1)
        }
        val key = "${end[Calendar.YEAR]}-${end[Calendar.MONTH] + 1}"
        mMonthListDate.add(key)
        dayOfMonthMap[key] = days
        view.setYearView(mMonthListDate.map {
            val split = it.split("-")
            if (split.size == 2) {
                "${split[0]}年${split[1]}月"
            } else {
                ""
            }
        })
    }

    private fun getWeek(day: Int) = when (day) {
        Calendar.SUNDAY -> "周日"
        Calendar.MONDAY -> "周一"
        Calendar.TUESDAY -> "周二"
        Calendar.WEDNESDAY -> "周三"
        Calendar.THURSDAY -> "周四"
        Calendar.FRIDAY -> "周五"
        Calendar.SATURDAY -> "周六"
        else -> "周"
    }

    override fun onTabClicked(tab: Int) {
        if (tab != mSelectDateTab) {
            mSelectDateTab = tab
            refreshUi()
        }
    }


    override fun onLoopEnd() {
        val yearSplit = mMonthListDate[view.getYearIndex()].split("-")
        val daySplit = mDayListDate[view.getDayIndex()].split("-")
        if (daySplit.size == 2 && yearSplit.size == 2) {
            var select = Calendar.getInstance(Locale.CHINA)
            select.time = simpleDateFormat.parse("${yearSplit[0]}-${yearSplit[1]}-${daySplit[0]}")
            //获取loop end时候，时间
            if (mSelectDateTab == SelectDateMVPDialog.SELECT_DATE_TAB_START) {
                val end = Calendar.getInstance(Locale.CHINA)
                end.time = mSelectedEndTime
                if (select > end) {
                    mSelectedEndTime = select.time
                }
                mSelectedStartTime = select.time
            } else {
                val start = Calendar.getInstance(Locale.CHINA)
                start.time = mSelectedStartTime
                if (select < start) {
                    select = start
                }
                mSelectedEndTime = select.time
            }
            refreshUi()
        }
    }

    override fun onConfirm() {
        mOnResultListener?.onConfirm(mSelectedStartTime, mSelectedEndTime)
        view.viewDismiss()
    }


}
