package com.jiaoyinbrother.monkeyking.mvpdialog.selectdatetime

import android.content.Context
import android.os.Bundle
import android.view.View
import com.jiaoyinbrother.library.util.DateTimeUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.util.SelectDateTimeUtil
import com.monkey.datetimelibrary.util.TimeGenerateUtil
import java.util.*

class SelectDateTimeMainPresenter(context: Context, view: ISelectDateTimeContract.View) :
    SelectDateTimePresenter(context,view),
    ISelectDateTimeContract.Presenter {

    //    用户选择的tab
    private var mSelectDateTab = SelectDateTimeMVPDialog.SELECT_DATE_TAB_START

    //用户选择的开始时间，也是初始化传入的起始时间;
    private var mSelectedStartTime: Date? = null
    //用户选择的结束时间，也是初始化传入的还车时间
    private var mSelectedEndTime: Date? = null

    //时间控件日期的开始时间
    private var mMaxTime: Date? = null
    //时间控件日期的结束时间
    private var mMinTime: Date? = null

    //日历控件的数据list
    private var mDayList: List<String>? = null
    private var mDayListDate: List<Date>? = null
    private var mHourList = ArrayList<String>()
    private var mMinuteList = ArrayList<String>()

    //回调
    private var mOnResultListener: SelectDateTimeMVPDialog.OnResultListener? = null

    override fun initData(arguments: Bundle, onResultListener: SelectDateTimeMVPDialog.OnResultListener) {

        LogUtil.e("initData arguments = $arguments")
        mSelectDateTab = arguments.getInt(SelectDateTimeMVPDialog.SELECTED_TIME_TAB_TYPE)
        mOnResultListener = onResultListener

        //只初始化数据
        initMainData(arguments)

        //刷新hours,minute----没有变化，初始化处理;必须先处理inithours,在refreshAllUI
        initHours()

        refreshAllUI()


    }

    override fun onStartTimeTabClicked() {
        LogUtil.e("onStartTimeTabClicked")
        mSelectDateTab = SelectDateTimeMVPDialog.SELECT_DATE_TAB_START
        //refresh
        refreshAllUI()
    }


    override fun onEndTimeTabClicked() {
        LogUtil.e("onEndTimeTabClicked")
        mSelectDateTab = SelectDateTimeMVPDialog.SELECT_DATE_TAB_END

        refreshAllUI()
    }


    override fun onLoopEnd() {
        LogUtil.e("onLoopEnd")
        //获取loop end时候，时间
        val calendar = SelectDateTimeUtil().getSelectedCalendar(mDayListDate!!,
            view.getDateIndex(),view.getHourIndex(),view.getMinuteIndex())
        val selectTime = calendar.time

        //检查用户选中的时间，是否是否非法时间，
        checkIsTimeValid(selectTime)

        //更新界面
        refreshAllUI()
    }

    override fun onConfirm() {
        LogUtil.e("onConfirm***********************************************")

        if (mOnResultListener != null) {
            if (mSelectedStartTime != null)
                LogUtil.e("onConfirm, mSelectedStartTime = " + DateTimeUtil.dateToString(mSelectedStartTime))

            if (mSelectedEndTime != null)
                LogUtil.e("onConfirm, mSelectedEndTime = " + DateTimeUtil.dateToString(mSelectedEndTime))

            mOnResultListener?.onConfirm(mSelectedStartTime, mSelectedEndTime)
        }

        view.viewDismiss()
    }

    private fun refreshAllUI() {

        //刷新UI之前，检查是否取车时间非法，如果非法，重置。
        if(mSelectedStartTime!=null)
            checkIsStartTimeValid(mSelectedStartTime!!)

//        1. 重新生成日期列表
        refreshDateList()

//        2. 刷新日期
        refreshShowDate()

//        3. 刷新时间tab
        refreshDateTab()
    }

    private fun refreshShowDate() {
        refreshShowDateMainType()
    }


    private fun refreshShowDateMainType() {

        LogUtil.e("refreshShowDateMainType")
        var selectTime = DateTimeUtil.getStartTimeDateFormat().time
        when (mSelectDateTab) {
            SelectDateTimeMVPDialog.SELECT_DATE_TAB_START -> {
                if (mSelectedStartTime != null)
                    selectTime = mSelectedStartTime
            }
            SelectDateTimeMVPDialog.SELECT_DATE_TAB_END -> {
                if (mSelectedEndTime != null)
                    selectTime = mSelectedEndTime
            }
        }
        doRefreshShowDate(selectTime)
    }


    private fun refreshDateTab() {
        //刷新days
        if (mSelectedStartTime != null)
            view.setStartDateTabText(DateTimeUtil.dateToStringMMDDHHMM(mSelectedStartTime))

        if (mSelectedEndTime != null)
            view.setEndDateTabText(DateTimeUtil.dateToStringMMDDHHMM(mSelectedEndTime))

        refreshMainDateTab()

    }

    private fun refreshMainDateTab() {
        view.setDateTabVisibility(View.VISIBLE)
        setTimeDuration()
        when (mSelectDateTab) {
            SelectDateTimeMVPDialog.SELECT_DATE_TAB_START -> view.setDateTabView(true)
            SelectDateTimeMVPDialog.SELECT_DATE_TAB_END -> view.setDateTabView(false)
        }
    }


    private fun refreshDateList() {
        //刷新days
        initMainDays()
        if (mDayList != null)
            view.setDayView(mDayList!!)

    }



    private fun checkIsTimeValid(selectTime: Date) {

        LogUtil.e("checkIsTimeValid, selectTime = " + DateTimeUtil.dateToString(selectTime))
        when (mSelectDateTab) {
            SelectDateTimeMVPDialog.SELECT_DATE_TAB_START ->{
                checkIsStartTimeValid(selectTime)
            }
            SelectDateTimeMVPDialog.SELECT_DATE_TAB_END ->{
                reCalculateMinMaxEndDateForMainType()

                if (mMaxTime != null && mMinTime != null)
                    mSelectedEndTime = doCheckIsTimeValid(selectTime, mMinTime!!, mMaxTime!!)
            }
        }
    }

    private fun checkIsStartTimeValid(selectTime: Date) {
        LogUtil.e("checkIsStartTimeValid")
        reCalculateMinMaxStartDateForMainType()

        if (mMaxTime != null && mMinTime != null)
            mSelectedStartTime = doCheckIsTimeValid(selectTime, mMinTime!!, mMaxTime!!)

        //需要检查还车时间，参考setSureDate
        val calendarMax = SelectDateTimeUtil().getLastDayOfEndDate()
        if (mSelectedEndTime != null)
            mSelectedEndTime =
                SelectDateTimeUtil().adjustEndDateByStartDate(selectTime, mSelectedEndTime!!, calendarMax.time)
    }


    private fun doCheckIsTimeValid(
        selectTime: Date,
        minTime: Date,
        maxTime: Date
    ): Date? {
        LogUtil.e("selectTime = " + DateTimeUtil.dateToString(selectTime))
        LogUtil.e("minTime = " + DateTimeUtil.dateToString(minTime))
        LogUtil.e("maxTime = " + DateTimeUtil.dateToString(maxTime))
        val ret = when {
            selectTime.before(minTime) -> // 上限
                // 还车时间要大于当前时间！
                SelectDateTimeUtil().getDateWithValidMinute(minTime).time
            selectTime.after(maxTime) -> // 下限
                // 还车时间只能选择订单原还车时间之前！
                mMaxTime
            else -> selectTime
        }
        LogUtil.e("finalTime = " + DateTimeUtil.dateToString(ret))
        return ret
    }

    private fun setTimeDuration() {
        if (mSelectedStartTime != null && mSelectedEndTime != null)
            view.setTitle(
                DateTimeUtil.getTimePeriodString(
                    mSelectedStartTime,
                    mSelectedEndTime,
                    DateTimeUtil.TYPE_DAY_HOUR
                )
            )
    }


    private fun initMainData(arguments: Bundle) {
        try {
            mSelectedStartTime = arguments.getSerializable(SelectDateTimeMVPDialog.START_TIME) as Date?
            mSelectedEndTime = arguments.getSerializable(SelectDateTimeMVPDialog.END_TIME) as Date?
        } catch (e: Exception) {

        }

    }


    private fun initMainDays() {
        when (mSelectDateTab) {
            SelectDateTimeMVPDialog.SELECT_DATE_TAB_START -> reCalculateMinMaxStartDateForMainType()
            SelectDateTimeMVPDialog.SELECT_DATE_TAB_END -> reCalculateMinMaxEndDateForMainType()
        }

        if (mMinTime == null || mMaxTime == null)
            return

        mDayListDate = TimeGenerateUtil.getDayList(mMinTime, mMaxTime)
        mDayList = TimeGenerateUtil.convertStringListToDateList(mDayListDate)
    }

    private fun reCalculateMinMaxStartDateForMainType() {
        LogUtil.e("reCalculateMinMaxStartDateForMainType")
        // 设置选择时间的上限
        mMinTime = Date()

        //计算下辖
        mMaxTime = SelectDateTimeUtil().getLastDayOfStartDate().time
    }

    private fun reCalculateMinMaxEndDateForMainType() {
        LogUtil.e("reCalculateMinMaxEndDateForMainType")
        // 还车时间请设置为一小时之后！
        val calendarMin = Calendar.getInstance()
        calendarMin.time = mSelectedStartTime
        calendarMin.add(Calendar.HOUR_OF_DAY, 1)

        mMinTime = calendarMin.time

        //计算下辖
        mMaxTime = SelectDateTimeUtil().getLastDayOfEndDate().time
    }

    private fun initHours() {

        mHourList = SelectDateTimeUtil().getHourList()
        mMinuteList = SelectDateTimeUtil().getMinuteList()

        view.setHourView(mHourList)
        view.setMinuteView(mMinuteList)
    }



    private fun doRefreshShowDate(selectTime: Date) {
        LogUtil.e("doRefreshShowDate, selectTime = " + DateTimeUtil.dateToString(selectTime))
        val validSelectTime = SelectDateTimeUtil().getDateWithValidMinute(selectTime).time

        LogUtil.e("doRefreshShowDate, validSelectTime = " + DateTimeUtil.dateToString(validSelectTime))
        if (mDayListDate == null) {
            LogUtil.e("doRefreshShowDate, mDayListDate error")
            return
        }

        val calendar = Calendar.getInstance()
        calendar.time = validSelectTime
        val minute = calendar.get(Calendar.MINUTE)
        //处理传入不是30，0 问题
        if (minute != 0 && minute != 30) {
            LogUtil.e("doRefreshShowDate, minute error")
            return
        }

        when (minute) {
            0 -> view.setMinuteIndex(0)
            30 -> view.setMinuteIndex(1)
        }


        mDayListDate!!.forEachIndexed { index, date ->
            if (DateTimeUtil.isSameDay(validSelectTime, date))
                view?.setDateIndex(index)
        }

        view.setHourIndex(calendar.get(Calendar.HOUR_OF_DAY))

    }

}

