package com.jiaoyinbrother.monkeyking.mvpdialog.selectdatetime

import android.content.Context
import android.os.Bundle
import android.view.View
import com.jiaoyinbrother.library.base.BasePresenterImpl
import com.jiaoyinbrother.library.util.DateTimeUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.util.SelectDateTimeUtil
import com.jiaoyinbrother.monkeyking.mvpdialog.selectdatetime.SelectDateTimeMVPDialog.Builder.SELECT_DATE_TAB_END
import com.jiaoyinbrother.monkeyking.mvpdialog.selectdatetime.SelectDateTimeMVPDialog.Builder.SELECT_DATE_TAB_START
import com.jiaoyinbrother.monkeyking.mvpdialog.selectdatetime.SelectDateTimeMVPDialog.Builder.TYPE_ADVANCE_BACK
import com.jiaoyinbrother.monkeyking.mvpdialog.selectdatetime.SelectDateTimeMVPDialog.Builder.TYPE_MAIN_START_END
import com.jiaoyinbrother.monkeyking.mvpdialog.selectdatetime.SelectDateTimeMVPDialog.Builder.TYPE_ORDER_MODIFY
import com.jiaoyinbrother.monkeyking.mvpdialog.selectdatetime.SelectDateTimeMVPDialog.Builder.TYPE_RELET
import com.monkey.datetimelibrary.util.TimeGenerateUtil
import java.text.DecimalFormat
import java.util.*

open class SelectDateTimePresenter(context: Context, view: ISelectDateTimeContract.View) :
    BasePresenterImpl<ISelectDateTimeContract.View>(context, view), ISelectDateTimeContract.Presenter {

    //功能已经移植到4个presenter,以后删除掉
    private var mDialogType = TYPE_MAIN_START_END
    private var mModifyType = SelectDateTimeMVPDialog.MODITY_0

    //    用户选择的tab
    private var mSelectDateTab = SELECT_DATE_TAB_START

    //用户选择的开始时间，也是初始化传入的起始时间;
    private var mSelectedStartTime: Date? = null
    //用户选择的结束时间，也是初始化传入的还车时间
    private var mSelectedEndTime: Date? = null

    //时间控件日期的开始时间
    private var mMaxTime: Date? = null
    //时间控件日期的结束时间
    private var mMinTime: Date? = null

    //日历控件的数据list
    private var mDayList: List<String>? = null
    private var mDayListDate: List<Date>? = null
    private var mHourList = ArrayList<String>()
    private var mMinuteList = ArrayList<String>()

    //回调
    private var mOnResultListener: SelectDateTimeMVPDialog.OnResultListener? = null

    override fun initData(arguments: Bundle, onResultListener: SelectDateTimeMVPDialog.OnResultListener) {

        LogUtil.e("initData arguments = $arguments")
        mDialogType = arguments.getInt(SelectDateTimeMVPDialog.DIALOG_TYPE)
        mSelectDateTab = arguments.getInt(SelectDateTimeMVPDialog.SELECTED_TIME_TAB_TYPE)
        mOnResultListener = onResultListener

        //只初始化数据
        when (mDialogType) {
            TYPE_MAIN_START_END -> {
                initMainData(arguments)
            }
            TYPE_ORDER_MODIFY -> {
                initModifyData(arguments)
            }
            TYPE_RELET -> {
                initReletData(arguments)
            }
            TYPE_ADVANCE_BACK -> {
                initAdvanceBackData(arguments)
            }
        }

        //刷新hours,minute----没有变化，初始化处理;必须先处理inithours,在refreshAllUI
        initHours()

        refreshAllUI()


    }

    override fun onStartTimeTabClicked() {
        LogUtil.e("onStartTimeTabClicked")
        mSelectDateTab = SELECT_DATE_TAB_START
        //refresh
        refreshAllUI()
    }


    override fun onEndTimeTabClicked() {
        LogUtil.e("onEndTimeTabClicked")
        mSelectDateTab = SELECT_DATE_TAB_END

        refreshAllUI()
    }


    override fun onLoopEnd() {
        LogUtil.e("onLoopEnd")
        //获取loop end时候，时间
        val calendar = getSelectedCalendar()
        val selectTime = calendar.time

        //检查用户选中的时间，是否是否非法时间，
        checkIsTimeValid(selectTime)

        //更新界面
        refreshAllUI()
    }

    override fun onConfirm() {
        LogUtil.e("onConfirm***********************************************")

        if (mOnResultListener != null) {
            if (mSelectedStartTime != null)
                LogUtil.e("onConfirm, mSelectedStartTime = " + DateTimeUtil.dateToString(mSelectedStartTime))

            if (mSelectedEndTime != null)
                LogUtil.e("onConfirm, mSelectedEndTime = " + DateTimeUtil.dateToString(mSelectedEndTime))

            mOnResultListener?.onConfirm(mSelectedStartTime, mSelectedEndTime)
        }

        view.viewDismiss()
    }

    private fun refreshAllUI() {
//        1. 重新生成日期列表
        refreshDateList()

//        2. 刷新日期
        refreshShowDate()

//        3. 刷新时间tab
        refreshDateTab()
    }

    private fun refreshShowDate() {
        when (mDialogType) {
            TYPE_MAIN_START_END -> {
                refreshShowDateMainType()
            }
            TYPE_ORDER_MODIFY -> {
                refreshShowDateModify()
            }
            TYPE_RELET, TYPE_ADVANCE_BACK -> {
                refreshShowDateAdvanceBack()
            }
        }
    }

    private fun refreshShowDateModify() {
        LogUtil.e("refreshShowDateModify")
        refreshShowDateMainType()
    }

    private fun refreshShowDateMainType() {

        LogUtil.e("refreshShowDateMainType")
        var selectTime = DateTimeUtil.getStartTimeDateFormat().time
        when (mSelectDateTab) {
            SELECT_DATE_TAB_START -> {
                if (mSelectedStartTime != null)
                    selectTime = mSelectedStartTime
            }
            SELECT_DATE_TAB_END -> {
                if (mSelectedEndTime != null)
                    selectTime = mSelectedEndTime
            }
        }
        doRefreshShowDate(selectTime)
    }

    private fun refreshShowDateAdvanceBack() {

        if (mSelectedEndTime != null) {
            doRefreshShowDate(mSelectedEndTime!!)
        }

    }

    private fun refreshDateTab() {
        //刷新days
        if (mSelectedStartTime != null)
            view.setStartDateTabText(DateTimeUtil.dateToStringMMDDHHMM(mSelectedStartTime))

        if (mSelectedEndTime != null)
            view.setEndDateTabText(DateTimeUtil.dateToStringMMDDHHMM(mSelectedEndTime))

        when (mDialogType) {
            TYPE_MAIN_START_END -> {
                refreshMainDateTab()
            }
            TYPE_ORDER_MODIFY -> {
                refreshModifyDateTab()
            }
            TYPE_RELET, TYPE_ADVANCE_BACK -> {
                refreshAdvanceBackDateTab()
            }
        }

    }

    private fun refreshAdvanceBackDateTab() {
        view.setTitle("还车时间")
        view.setDateTabVisibility(View.GONE)
    }

    private fun refreshModifyDateTab() {
        when (mModifyType) {
            SelectDateTimeMVPDialog.MODITY_0 -> {
                refreshMainDateTab()
            }
            SelectDateTimeMVPDialog.MODITY_1 -> {
                refreshAdvanceBackDateTab()
            }
            SelectDateTimeMVPDialog.MODITY_2 -> {
                refreshMainDateTab()
            }
            SelectDateTimeMVPDialog.MODITY_3 -> {
                refreshAdvanceBackDateTab()
            }
        }
    }

    private fun refreshMainDateTab() {
        view.setDateTabVisibility(View.VISIBLE)
        setTimeDuration()
        when (mSelectDateTab) {
            SELECT_DATE_TAB_START -> view.setDateTabView(true)
            SELECT_DATE_TAB_END -> view.setDateTabView(false)
        }
    }


    private fun refreshDateList() {
        //刷新days
        when (mDialogType) {
            TYPE_MAIN_START_END -> {
                initMainDays()
            }
            TYPE_ORDER_MODIFY -> {
                initOrderModifyDays()
            }
            TYPE_RELET -> {
                initReletDays()
            }
            TYPE_ADVANCE_BACK -> {
                initAdvanceBackDays()
            }
        }
        if (mDayList != null)
            view.setDayView(mDayList!!)

    }

    private fun initAdvanceBackDays() {
        if (mSelectedEndTime == null || mMaxTime == null) {
            LogUtil.e("initAdvanceBackDays invalid date")
            return
        }

        mMinTime = Calendar.getInstance().time
        mDayListDate = TimeGenerateUtil.getDayList(mMinTime, mMaxTime)
        mDayList = TimeGenerateUtil.convertStringListToDateList(mDayListDate)
    }

    private fun initReletDays() {
        if (mSelectedEndTime == null || mMaxTime == null || mMinTime == null) {
            LogUtil.e("initReletDays invalid date")
            return
        }

        mDayListDate = TimeGenerateUtil.getDayList(mMinTime, mMaxTime)
        mDayList = TimeGenerateUtil.convertStringListToDateList(mDayListDate)
    }


    private fun getSelectedCalendar(): Calendar {
        val dateIndex = view.getDateIndex()
        val hourIndex = view.getHourIndex()
        val minuteIndex = view.getMinuteIndex()

        //date, 从生成的时间取回
        val calendar = Calendar.getInstance()
        if (mDayListDate != null && dateIndex < mDayListDate!!.size)
            calendar.time = mDayListDate!![dateIndex]

        //1-24 对应 1-24小时
        calendar.set(Calendar.HOUR_OF_DAY, hourIndex)

        //0,1 对应 0,30分钟
        if (minuteIndex == 0)
            calendar.set(Calendar.MINUTE, 0)
        else
            calendar.set(Calendar.MINUTE, 30)

        LogUtil.e("calendar = " + DateTimeUtil.dateToString(calendar.time))
        return calendar
    }


    private fun checkIsTimeValid(selectTime: Date) {

        LogUtil.e("checkIsTimeValid, selectTime = " + DateTimeUtil.dateToString(selectTime))
        when (mDialogType) {
            TYPE_MAIN_START_END -> {
                when (mSelectDateTab) {
                    SELECT_DATE_TAB_START ->{
                        reCalculateMinMaxStartDateForMainType()

                        if (mMaxTime != null && mMinTime != null)
                            mSelectedStartTime = doCheckIsTimeValid(selectTime, mMinTime!!, mMaxTime!!)

                        //需要检查还车时间，参考setSureDate
                        val calendarMax = SelectDateTimeUtil().getLastDayOfEndDate()
                        if(mSelectedEndTime!=null)
                            mSelectedEndTime = SelectDateTimeUtil().adjustEndDateByStartDate(selectTime, mSelectedEndTime!!, calendarMax.time)

                    }
                    SELECT_DATE_TAB_END ->{
                        reCalculateMinMaxEndDateForMainType()

                        if (mMaxTime != null && mMinTime != null)
                            mSelectedEndTime = doCheckIsTimeValid(selectTime, mMinTime!!, mMaxTime!!)
                    }
                }


            }
            TYPE_ADVANCE_BACK -> {
                setAdvanceBackTime(selectTime)
            }
            TYPE_RELET -> {
                setOrderReletTime(selectTime)
            }
            TYPE_ORDER_MODIFY -> {
                when (mSelectDateTab) {
                    SELECT_DATE_TAB_START -> {
                        reCalculateMinMaxStartDateModifyType2()
                        if (mMaxTime != null && mMinTime != null)
                            mSelectedStartTime = doCheckIsTimeValid(selectTime, mMinTime!!, mMaxTime!!)
                    }
                    SELECT_DATE_TAB_END -> {
                        reCalculateMinMaxEndDateModifyType2()
                        if (mMaxTime != null && mMinTime != null)
                            mSelectedEndTime = doCheckIsTimeValid(selectTime, mMinTime!!, mMaxTime!!)

                        //需要考虑其他因素
                        val calendarMax = SelectDateTimeUtil().getLastDayOfEndDate()
                        if(mSelectedEndTime!=null)
                            mSelectedEndTime = SelectDateTimeUtil().adjustEndDateByStartDate(selectTime, mSelectedEndTime!!, calendarMax.time)
                    }
                }


            }

        }

    }

    private fun setOrderReletTime(selectTime: Date) {
        LogUtil.e("setOrderReletTime")
        //不需要检查取车时间

        //检查还车时间
        if (mMaxTime != null && mMinTime != null)
            mSelectedEndTime = doCheckIsTimeValid(selectTime, mMinTime!!, mMaxTime!!)
    }

    private fun setAdvanceBackTime(selectTime: Date) {
        LogUtil.e("setAdvanceBackTime")

        val currentTime = Date()
        //不需要检查取车时间

        //检查还车时间
        //还车时间最小是当前时间，最大是maxTime
        if (mMaxTime != null)
            mSelectedEndTime = doCheckIsTimeValid(selectTime, currentTime, mMaxTime!!)

    }

    private fun doCheckIsTimeValid(
        selectTime: Date,
        minTime: Date,
        maxTime: Date
    ): Date? {
        LogUtil.e("selectTime = " + DateTimeUtil.dateToString(selectTime))
        LogUtil.e("minTime = " + DateTimeUtil.dateToString(minTime))
        LogUtil.e("maxTime = " + DateTimeUtil.dateToString(maxTime))
        val ret = when {
            selectTime.before(minTime) -> // 上限
                // 还车时间要大于当前时间！
                getDateWithValidMinute(minTime).time
            selectTime.after(maxTime) -> // 下限
                // 还车时间只能选择订单原还车时间之前！
                mMaxTime
            else -> selectTime
        }
        LogUtil.e("finalTime = " + DateTimeUtil.dateToString(ret))
        return ret
    }

    private fun setTimeDuration() {
        if (mSelectedStartTime != null && mSelectedEndTime != null)
            view.setTitle(
                DateTimeUtil.getTimePeriodString(
                    mSelectedStartTime,
                    mSelectedEndTime,
                    DateTimeUtil.TYPE_DAY_HOUR
                )
            )
    }


    private fun initReletData(arguments: Bundle) {

        try {
            mSelectedEndTime = arguments.getSerializable(SelectDateTimeMVPDialog.END_TIME) as Date?
            mMinTime = arguments.getSerializable(SelectDateTimeMVPDialog.MIN_TIME) as Date?
            mMaxTime = arguments.getSerializable(SelectDateTimeMVPDialog.MAX_TIME) as Date?
        } catch (e: Exception) {

        }

    }

    private fun initAdvanceBackData(arguments: Bundle) {
        try {
            mSelectedEndTime = arguments.getSerializable(SelectDateTimeMVPDialog.END_TIME) as Date?
            mMaxTime = arguments.getSerializable(SelectDateTimeMVPDialog.MAX_TIME) as Date?
        } catch (e: Exception) {

        }

    }

    private fun initMainData(arguments: Bundle) {
        try {
            mSelectedStartTime = arguments.getSerializable(SelectDateTimeMVPDialog.START_TIME) as Date?
            mSelectedEndTime = arguments.getSerializable(SelectDateTimeMVPDialog.END_TIME) as Date?
        } catch (e: Exception) {

        }

    }

    private fun initModifyData(arguments: Bundle) {
        mModifyType = arguments.getInt(SelectDateTimeMVPDialog.MODIFY_TYPE)
        try {
            mSelectedStartTime = arguments.getSerializable(SelectDateTimeMVPDialog.START_TIME) as Date?
            mSelectedEndTime = arguments.getSerializable(SelectDateTimeMVPDialog.END_TIME) as Date?
            mMinTime = arguments.getSerializable(SelectDateTimeMVPDialog.MIN_TIME) as Date?
            mMaxTime = arguments.getSerializable(SelectDateTimeMVPDialog.MAX_TIME) as Date?
        } catch (e: Exception) {

        }

    }

    private fun initOrderModifyDays() {
        when (mModifyType) {
            SelectDateTimeMVPDialog.MODITY_0 -> {
                initMainDays()
            }
            SelectDateTimeMVPDialog.MODITY_1 -> {

                mSelectDateTab = SELECT_DATE_TAB_END
                initMainDays()
            }
            SelectDateTimeMVPDialog.MODITY_2 -> {
                initOrderModifyType2()
            }
            SelectDateTimeMVPDialog.MODITY_3 -> {

                mSelectDateTab = SELECT_DATE_TAB_END
                initOrderModifyType2()
            }
        }
    }

    private fun initOrderModifyType2() {

        when (mSelectDateTab) {
            SELECT_DATE_TAB_START -> reCalculateMinMaxStartDateModifyType2()
            SELECT_DATE_TAB_END -> reCalculateMinMaxEndDateModifyType2()
        }

        if (mMinTime == null || mMaxTime == null)
            return

        mDayListDate = TimeGenerateUtil.getDayList(mMinTime, mMaxTime)
        mDayList = TimeGenerateUtil.convertStringListToDateList(mDayListDate)

    }

    private fun reCalculateMinMaxStartDateModifyType2() {
        LogUtil.e("reCalculateMinMaxStartDateForMainType")
        // 设置选择时间的上限
        mMinTime = Date()

        //计算下辖
//        传入
    }

    private fun reCalculateMinMaxEndDateModifyType2() {
        // 设置选择时间的上限
        //传入

        //计算下辖
        mMaxTime = SelectDateTimeUtil().getLastDayOfEndDate().time

    }

    private fun initMainDays() {
        when (mSelectDateTab) {
            SELECT_DATE_TAB_START -> reCalculateMinMaxStartDateForMainType()
            SELECT_DATE_TAB_END -> reCalculateMinMaxEndDateForMainType()
        }

        if (mMinTime == null || mMaxTime == null)
            return

        mDayListDate = TimeGenerateUtil.getDayList(mMinTime, mMaxTime)
        mDayList = TimeGenerateUtil.convertStringListToDateList(mDayListDate)
    }

    private fun reCalculateMinMaxStartDateForMainType() {
        LogUtil.e("reCalculateMinMaxStartDateForMainType")
        // 设置选择时间的上限
        mMinTime = Date()

        //计算下辖
        mMaxTime = SelectDateTimeUtil().getLastDayOfStartDate().time
    }

    private fun reCalculateMinMaxEndDateForMainType() {
        LogUtil.e("reCalculateMinMaxEndDateForMainType")
        // 还车时间请设置为一小时之后！
        val calendarMin = Calendar.getInstance()
        calendarMin.time = mSelectedStartTime
        calendarMin.add(Calendar.HOUR_OF_DAY, 1)

        mMinTime = calendarMin.time

        //计算下辖
        mMaxTime = SelectDateTimeUtil().getLastDayOfEndDate().time
    }

    private fun initHours() {
        mHourList.clear()

        // 如果是个数,则显示为"02"的样式
        val patten = "00"
        val decimal = DecimalFormat(patten)

        for (i in 0..23) {
            mHourList.add(decimal.format(i.toLong()))
        }

        mMinuteList.clear()
        mMinuteList.add("00")
        mMinuteList.add("30")

        view.setHourView(mHourList)
        view.setMinuteView(mMinuteList)
    }

    private fun getDateWithValidMinute(date: Date): Calendar {
        val calendar = Calendar.getInstance()
        LogUtil.e("getDateWithValidMinute, in date = " + DateTimeUtil.dateToString(date))
        calendar.time = date
        val minute = calendar.get(Calendar.MINUTE)
        //处理传入不是30，0 问题
        if (minute == 0) {

        } else if (minute > 0 && minute <= 30) {
            calendar.set(Calendar.MINUTE, 30)
        } else {
            //+ 30分钟，然后分钟设置为0，可以解决跨月，跨年等问题
            calendar.add(Calendar.MINUTE, 30)
            calendar.set(Calendar.MINUTE, 0)
        }

        LogUtil.e("getDateWithValidMinute, out date = " + DateTimeUtil.dateToString(calendar.time))
        return calendar
    }

    private fun doRefreshShowDate(selectTime: Date) {
        LogUtil.e("doRefreshShowDate, selectTime = " + DateTimeUtil.dateToString(selectTime))
        val validSelectTime = getDateWithValidMinute(selectTime).time

        LogUtil.e("doRefreshShowDate, validSelectTime = " + DateTimeUtil.dateToString(validSelectTime))
        if (mDayListDate == null) {
            LogUtil.e("doRefreshShowDate, mDayListDate error")
            return
        }

        val calendar = Calendar.getInstance()
        calendar.time = validSelectTime
        val minute = calendar.get(Calendar.MINUTE)
        //处理传入不是30，0 问题
        if (minute != 0 && minute != 30) {
            LogUtil.e("doRefreshShowDate, minute error")
            return
        }

        when (minute) {
            0 -> view.setMinuteIndex(0)
            30 -> view.setMinuteIndex(1)
        }


        mDayListDate!!.forEachIndexed { index, date ->
            if (DateTimeUtil.isSameDay(validSelectTime, date))
                view?.setDateIndex(index)
        }

        view.setHourIndex(calendar.get(Calendar.HOUR_OF_DAY))

    }

}
