package com.jiaoyinbrother.monkeyking.mvpdialog.selectdatetime

import android.content.Context
import android.os.Bundle
import android.view.View
import com.jiaoyinbrother.library.util.DateTimeUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.util.SelectDateTimeUtil
import com.monkey.datetimelibrary.util.TimeGenerateUtil
import java.text.DecimalFormat
import java.util.*

class SelectDateTimeReletPresenter(context: Context, view: ISelectDateTimeContract.View) :
    SelectDateTimePresenter(context,view),
    ISelectDateTimeContract.Presenter{

    //用户选择的结束时间，也是初始化传入的还车时间
    private var mSelectedEndTime: Date? = null

    //时间控件日期的开始时间
    private var mMaxTime: Date? = null
    //时间控件日期的结束时间
    private var mMinTime: Date? = null

    //日历控件的数据list
    private var mDayList: List<String>? = null
    private var mDayListDate: List<Date>? = null
    private var mHourList = ArrayList<String>()
    private var mMinuteList = ArrayList<String>()

    //回调
    private var mOnResultListener: SelectDateTimeMVPDialog.OnResultListener? = null

    override fun initData(arguments: Bundle, onResultListener: SelectDateTimeMVPDialog.OnResultListener) {
        LogUtil.e("SelectDateTimeReletPresenter initData")
        LogUtil.e("initData arguments = $arguments")
        mOnResultListener = onResultListener

        //只初始化数据
        initReletData(arguments)

        //刷新hours,minute----没有变化，初始化处理;必须先处理inithours,在refreshAllUI
        initHours()

        refreshAllUI()
    }

    override fun onLoopEnd() {
        LogUtil.e("onLoopEnd")
        //获取loop end时候，时间
        if(mDayListDate==null)
            return

        val calendar = SelectDateTimeUtil().getSelectedCalendar(mDayListDate!!,
            view.getDateIndex(),view.getHourIndex(),view.getMinuteIndex())
        val selectTime = calendar.time

        //检查用户选中的时间，是否是否非法时间，
        checkIsTimeValid(selectTime)

        //更新界面
        refreshAllUI()
    }

    override fun onConfirm() {
        LogUtil.e("onConfirm***********************************************")

        if (mOnResultListener != null) {
            if (mSelectedEndTime != null)
                LogUtil.e("onConfirm, mSelectedEndTime = " + DateTimeUtil.dateToString(mSelectedEndTime))

            mOnResultListener?.onConfirm(null, mSelectedEndTime)
        }

        view.viewDismiss()
    }

    override fun onStartTimeTabClicked() {
        LogUtil.e("onStartTimeTabClicked")
    }

    override fun onEndTimeTabClicked() {
        LogUtil.e("onEndTimeTabClicked")
    }


    private fun initReletData(arguments: Bundle) {
        try {
            mSelectedEndTime = arguments.getSerializable(SelectDateTimeMVPDialog.END_TIME) as Date?
            mMinTime = arguments.getSerializable(SelectDateTimeMVPDialog.MIN_TIME) as Date?
            mMaxTime = arguments.getSerializable(SelectDateTimeMVPDialog.MAX_TIME) as Date?
        } catch (e: Exception) {

        }
    }


    private fun initHours() {

        mHourList = SelectDateTimeUtil().getHourList()
        mMinuteList = SelectDateTimeUtil().getMinuteList()

        view.setHourView(mHourList)
        view.setMinuteView(mMinuteList)
    }


    private fun refreshAllUI() {
//        1. 重新生成日期列表
        refreshDateList()

//        2. 刷新日期
        refreshShowDate()

//        3. 刷新时间tab
        refreshDateTab()
    }

    private fun refreshShowDate() {

        if (mSelectedEndTime != null) {
            doRefreshShowDate(mSelectedEndTime!!)
        }

    }

    private fun refreshDateTab() {
        //刷新days
        if (mSelectedEndTime != null)
            view.setEndDateTabText(DateTimeUtil.dateToStringMMDDHHMM(mSelectedEndTime))

        view.setTitle("还车时间")
        view.setDateTabVisibility(View.GONE)
    }


    private fun refreshDateList() {
        //刷新days
        initReletDays()

        if (mDayList != null)
            view.setDayView(mDayList!!)

    }
    private fun initReletDays() {
        if (mSelectedEndTime == null || mMaxTime == null || mMinTime == null) {
            LogUtil.e("initReletDays invalid date")
            return
        }

        mDayListDate = TimeGenerateUtil.getDayList(mMinTime, mMaxTime)
        mDayList = TimeGenerateUtil.convertStringListToDateList(mDayListDate)
    }

    private fun checkIsTimeValid(selectTime: Date) {
        LogUtil.e("setOrderReletTime")
        //不需要检查取车时间

        //检查还车时间
        if (mMaxTime != null && mMinTime != null)
            mSelectedEndTime = doCheckIsTimeValid(selectTime, mMinTime!!, mMaxTime!!)
    }


    private fun doCheckIsTimeValid(
        selectTime: Date,
        minTime: Date,
        maxTime: Date
    ): Date? {
        LogUtil.e("selectTime = " + DateTimeUtil.dateToString(selectTime))
        LogUtil.e("minTime = " + DateTimeUtil.dateToString(minTime))
        LogUtil.e("maxTime = " + DateTimeUtil.dateToString(maxTime))
        val ret = when {
            selectTime.before(minTime) -> // 上限
                // 还车时间要大于当前时间！
                SelectDateTimeUtil().getDateWithValidMinute(minTime).time
            selectTime.after(maxTime) -> // 下限
                // 还车时间只能选择订单原还车时间之前！
                mMaxTime
            else -> selectTime
        }
        LogUtil.e("finalTime = " + DateTimeUtil.dateToString(ret))
        return ret
    }


    private fun doRefreshShowDate(selectTime: Date) {
        LogUtil.e("doRefreshShowDate, selectTime = " + DateTimeUtil.dateToString(selectTime))
        val validSelectTime = SelectDateTimeUtil().getDateWithValidMinute(selectTime).time

        LogUtil.e("doRefreshShowDate, validSelectTime = " + DateTimeUtil.dateToString(validSelectTime))
        if (mDayListDate == null) {
            LogUtil.e("doRefreshShowDate, mDayListDate error")
            return
        }

        val calendar = Calendar.getInstance()
        calendar.time = validSelectTime
        val minute = calendar.get(Calendar.MINUTE)
        //处理传入不是30，0 问题
        if (minute != 0 && minute != 30) {
            LogUtil.e("doRefreshShowDate, minute error")
            return
        }

        when (minute) {
            0 -> view.setMinuteIndex(0)
            30 -> view.setMinuteIndex(1)
        }


        mDayListDate!!.forEachIndexed { index, date ->
            if (DateTimeUtil.isSameDay(validSelectTime, date))
                view?.setDateIndex(index)
        }

        view.setHourIndex(calendar.get(Calendar.HOUR_OF_DAY))

    }
}
