package com.jiaoyinbrother.monkeyking.manager.push.hw

import android.text.TextUtils
import com.huawei.hms.push.HmsMessageService
import com.huawei.hms.push.RemoteMessage
import com.hyphenate.chat.EMClient
import com.jiaoyinbrother.library.util.AppCacheUtil
import com.jiaoyinbrother.library.util.LogUtil

class HwPushReceiver : HmsMessageService() {

    /**
     * 1、申请Token如果当次调用失败，PUSH会自动重试申请，成功后则以onNewToken接口返回。
     *
     *
     * 2、如果服务端识别token过期，服务端刷新token也会以onNewToken方式返回。
     *
     *
     * 3、华为设备上EMUI版本低于10.0申请token时，以onNewToken方式返回。
     */
    override fun onNewToken(token: String?) {
        if (!TextUtils.isEmpty(token)) {
            AppCacheUtil(applicationContext).pushRegId = token!!
            EMClient.getInstance().sendHMSPushTokenToServer(token)
        }
    }

    /**
     * 1、APP调用此接口可用于接收透传消息。
     *
     *
     * 2、开发者如果订阅了主题消息或者接收下行设备组消息抑或是APP服务器推送透传消息到客户端APP，
     *
     *
     * 3、客户端APP要接收消息需要创建服务子类继承HmsMessageService，并重写onMessageReceived回调方法。
     *
     *
     * 4、开发者需在10s内完成onMessageReceived方法回调处理，超过10s开发者自己起线程处理。
     */
    override fun onMessageReceived(remoteMessage: RemoteMessage?) {
        LogUtil.e("HW push msg = " + remoteMessage?.toString())
    }

    /**
     * APP调用此接口可获取上行消息或设备组消息发送结果。
     */
    override fun onMessageSent(msgId: String?) {
        super.onMessageSent(msgId)
        LogUtil.e("HWPUSH onMessageSent = " + msgId)
    }

    /**
     * APP调用此接口可获取上行消息或设备组消息发送结果。
     */
    override fun onSendError(msgId: String?, exception: Exception?) {
        LogUtil.e("HWPUSH onSendError = $msgId : exception :$exception")
        super.onSendError(msgId, exception)
    }

}