package com.jiaoyinbrother.monkeyking.manager.push.mi

import android.app.Application
import android.content.Context
import android.text.TextUtils
import com.alibaba.android.arouter.launcher.ARouter
import com.hyphenate.push.platform.mi.EMMiMsgReceiver
import com.jiaoyinbrother.library.util.ARouterConstantUtil
import com.jiaoyinbrother.library.util.AppCacheUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.push.PushUtil
import com.xiaomi.mipush.sdk.ErrorCode
import com.xiaomi.mipush.sdk.MiPushClient
import com.xiaomi.mipush.sdk.MiPushCommandMessage
import com.xiaomi.mipush.sdk.MiPushMessage
import org.json.JSONException
import org.json.JSONObject

class MiPushReceiver : EMMiMsgReceiver() {

    /**
     * 方法用来接收服务器向客户端发送的透传消息
     */
    override fun onReceivePassThroughMessage(context: Context?, msg: MiPushMessage?) {
        LogUtil.e("MI Push onNotificationMessageArrived : " + msg?.toString())
    }

    /**
     * 方法用来接收服务器向客户端发送的通知消息，这个回调方法会在用户手动点击通知后触发
     */
    override fun onNotificationMessageClicked(context: Context?, msg: MiPushMessage?) {
        LogUtil.e("MI Push onNotificationMessageClicked : " + msg?.toString())
        if (TextUtils.isEmpty(msg?.content)) {
            ARouter.init(context as Application?)
            ARouter.getInstance().build(ARouterConstantUtil.ACTIVITY_MAIN).navigation()
            return
        }
        try {
            val jsonObject = JSONObject(msg?.content)
            val fromId = jsonObject.optString("f")
            PushUtil.notifyChatPage(context, fromId)
        } catch (e: JSONException) {
            LogUtil.e("fromId JSONException : " + e)
        }

    }


    /**
     * 方法用来接收服务器向客户端发送的通知消息，这个回调方法是在通知消息到达客户端时触发。
     * 另外应用在前台时不弹出通知的通知消息到达客户端也会触发这个回调函数。
     */
    override fun onNotificationMessageArrived(context: Context?, msg: MiPushMessage?) {
        LogUtil.e("MI Push onNotificationMessageArrived : " + msg?.toString())
    }

    /**
     * 方法用来接收客户端向服务器发送命令后的响应结果。
     */
    override fun onCommandResult(context: Context?, msg: MiPushCommandMessage?) {
        val command = msg?.command
        val arguments = msg?.commandArguments
        val cmdArg1 = if (arguments != null && arguments?.size > 0) arguments?.get(0) else null
        if (MiPushClient.COMMAND_REGISTER == command && msg?.resultCode == ErrorCode.SUCCESS.toLong()) {
            if (!TextUtils.isEmpty(cmdArg1) && context != null) {
                AppCacheUtil(context).pushRegId = cmdArg1 ?: ""
            }
        }
    }

    /**
     * 方法用来接收客户端向服务器发送注册命令后的响应结果。
     */
    override fun onReceiveRegisterResult(context: Context?, msg: MiPushCommandMessage?) {
        super.onReceiveRegisterResult(context, msg)
        LogUtil.e("MI Push onReceiveRegisterResult : " + msg?.toString())
    }

}