package com.jiaoyinbrother.monkeyking.manager.util

import android.content.Context
import android.content.Intent
import android.os.Build
import android.provider.Settings
import com.jiaoyinbrother.monkeyking.manager.network.DownloadFileManager
import com.jiaoyinbrother.monkeyking.manager.view.IOSdialog.IOSAlertDialog
import com.jiaoyinbrother.library.bean.AndroidBean
import com.jiaoyinbrother.library.bean.CheckVersionBean
import com.jiaoyinbrother.library.bean.CheckVersionResult
import com.jiaoyinbrother.library.util.*
import com.jiaoyinbrother.library.view.IBaseView
import com.jiaoyinbrother.library.widget.CheckVersionDialog
import java.io.File
import java.util.*


class CheckVersionUtil(private val context: Context) {

    fun getCheckVersionEntity(): CheckVersionBean {
        val cve = CheckVersionBean()
        cve.appVersion = AppAndSysInfoUtil.getAppVersion(context)
        cve.channel = ChannelUtil.getAppChannel(context)
        return cve
    }


    /**
     * @param result        result   checkVersion结果
     */

    fun checkVersion(result: CheckVersionResult?): Boolean {
        LogUtil.e("checkVersion")
        var isNewVersion = false
        if (result?.android != null) {
            try {
                val android = result.android
                AppCacheUtil(context).updateMD5 = android?.md5 ?: ""
                val local_version = AppAndSysInfoUtil.getAppVersion(context)
                val oldVersion = Integer.valueOf(local_version.replace(".", ""))
                val newVersion = android?.latestVersion?.replace(".", "")?.let { Integer.valueOf(it) } ?:0
                LogUtil.e("oldVersion = $oldVersion ; newVersion=$newVersion")
                if (newVersion > oldVersion) {
                    isNewVersion = true
                }
            } catch (e: Exception) {
                LogUtil.e(e.message)

            }

        }
        return isNewVersion
    }

    fun showNoUploadDialog() {
        IOSAlertDialog(context).builder().setTitle("版本更新").setMsg("已为最新版本", true)
                .setRightButton("确定") { }.show()
    }

    fun showUploadVersionDialog(android: AndroidBean, view: IBaseView?) {
        val newVersion = android.latestVersion?.replace(".", "")?.let { Integer.valueOf(it) }

        val dialog = CheckVersionDialog(context)
        val forced = android.forced
        dialog.show(forced)
        val list = ArrayList<String>()
        if (android.feature?.size!! > 0) {
            for (s in android.feature!!) {
                list.add(s)
            }
        }
        dialog.setContent(list)
        /* 强制更新*/

        dialog.setOnUpdateClickListener {
            dialog.dismiss()
            //下载文件开始，显示loading,避免在下载开始----到收到progress之间，用户可以进行其他操作。
            view?.showLoadingDialog()
            DownloadFileManager.getInstance(context).insertTask(android.url, "V$newVersion.apk")
        }

        dialog.setOnIgnoreClickListener {
            dialog.dismiss()
        }
    }


    fun handleDownloadComplete(intent: Intent) {
        val action = intent.action
        val result = intent.getIntExtra(ConstantsUtil.EXTRA_RESULT, 1)
        if (action == ConstantsUtil.ACTION_DOWNLOAD_COMPOLETED) {
            val path = intent.getStringExtra(ConstantsUtil.EXTRA_DEST_PATH)
            if (result == ConstantsUtil.RESULT_SUCCESS) {
                // MD5 校验文件完整性{}
                // 获取JSON解析到的MD5值
                val MD5Value = AppCacheUtil(context).updateMD5

                // MD5加密后的字符串
                val MD5str = MD5Util.md5sum(path)
                LogUtil.e("Local MD5str = $MD5str")
                LogUtil.e("MD5Value = $MD5Value")

                // MD5校验
                if (MD5str != null && MD5str.equals(MD5Value, ignoreCase = true)) {
                    // 文件校验成功拉起安装
                    installApk(path)
                } else {
                    // 下载文件不完成,文件删除
                    OldToastUtil.show(context, "客官下载出问题啦，请重新下载")
                }
            } else {
            }
        }
    }

    // 第一次未安装退出后，广播无法在第二次拉起安装界面，广播只有一次
    private fun installApk(apkPath: String) {
        //Android8.0 版本更新无法自动安装问题解决
        if (Build.VERSION.SDK_INT >= 26) {
            //先获取是否有安装未知来源应用的权限
            val installAllowed = context.packageManager.canRequestPackageInstalls()
            //有权限
            if (installAllowed) {
                //安装apk
                installToApk(apkPath)
            } else {
                //将用户引导至安装未知应用界面。
                val intent = Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES)
                context.startActivity(intent)
            }
        } else {
            installToApk(apkPath)
        }
    }

    private fun installToApk(apkPath: String) {
        val installIntent = Intent()
        installIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        installIntent.action = Intent.ACTION_VIEW
        installIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
        installIntent.setDataAndType(
                PhotoUtils.getUriFromFile(context, File(apkPath)),
                "application/vnd.android.package-archive"
        )
        context.startActivity(installIntent)
    }
}
