package com.jiaoyinbrother.monkeyking.manager.util

import android.content.Context
import android.support.v4.app.FragmentActivity
import android.text.TextUtils
import com.hyphenate.EMCallBack
import com.hyphenate.EMMessageListener
import com.hyphenate.chat.EMClient
import com.hyphenate.chat.EMConversation
import com.hyphenate.chat.EMMessage
import com.hyphenate.chat.EMOptions
import com.hyphenate.easeui.EaseUI
import com.hyphenate.push.EMPushConfig
import com.hyphenate.push.EMPushHelper
import com.hyphenate.push.EMPushType
import com.hyphenate.push.PushListener
import com.jeremyliao.livedatabus.LiveDataBus
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.chat.ChatFragment
import com.jiaoyinbrother.library.bean.EMChatBaseBean
import com.jiaoyinbrother.library.bean.ImInfoBean
import com.jiaoyinbrother.library.util.ARouterUtil
import com.jiaoyinbrother.library.util.DateTimeUtil
import com.jiaoyinbrother.library.util.LogUtil
import com.jiaoyinbrother.monkeyking.manager.AppAccount
import com.tbruyelle.rxpermissions2.RxPermissions
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.*


class EMUtil {

    private constructor()

    /**
     * 静态内部类单利
     */
    companion object {
        val instance: EMUtil
            get() = InstanceHolder.mEMUtil
    }

    private class InstanceHolder {
        companion object {
            val mEMUtil = EMUtil()
        }

    }

    fun initEM(applicationContext: Context) {
        val options = EMOptions()
        // 默认添加好友时，是不需要验证的，改成需要验证
        options.acceptInvitationAlways = true
        // 是否自动将消息附件上传到环信服务器，默认为True是使用环信服务器上传下载，如果设为 false，需要开发者自己处理附件消息的上传和下载
        options.autoTransferMessageAttachments = true
        // 是否自动下载附件类消息的缩略图等，默认为 true 这里和上边这个参数相关联
        options.setAutoDownloadThumbnail(true)
        //自动登录
        options.autoLogin = true
        // 配置推送的接口
        val builder = EMPushConfig.Builder(applicationContext)
        builder.enableMiPush(AppAccount.MI_PUSH_ID, AppAccount.MI_PUSH_KEY)
                .enableOppoPush(AppAccount.OPPO_PUSH_KEY, AppAccount.OPPO_PUSH_SECERT)
                .enableVivoPush()
                .enableHWPush()
        options.pushConfig = builder.build()

        try{
            //初始化
            EMClient.getInstance().init(applicationContext, options);
            //在做打包混淆时，关闭debug模式，避免消耗不必要的资源
            EMClient.getInstance().setDebugMode(true);

            EaseUI.getInstance().init(applicationContext, options);
        }catch (e:Exception){

        }
    }


    fun loginEM(im_username: String, im_password: String) {
        if (TextUtils.isEmpty(im_username) || TextUtils.isEmpty(im_password) || EMClient.getInstance() == null) return
        Thread(object : Runnable {

            override fun run() {
                EMClient.getInstance().login(im_username, im_password, object : EMCallBack {
                    //回调
                    override fun onSuccess() {
                        loadAllConversations()
                        LogUtil.e("登录聊天服务器成功！")
                    }

                    override fun onProgress(progress: Int, status: String) {

                    }

                    override fun onError(code: Int, message: String) {
                        LogUtil.e("登录聊天服务器失败！" + ",code:" + code + ",message:" + message)
                    }
                })
            }

        }).start()
    }

    fun logoutEM() {
        if (EMClient.getInstance() == null) return
        EMClient.getInstance().logout(true)
    }

    fun loadAllConversations() {
        try{
            if (EMClient.getInstance() == null) return
            if (EMClient.getInstance().chatManager() == null) return
            //EMClient.getInstance().groupManager().loadAllGroups()
            EMClient.getInstance().chatManager().loadAllConversations()
        } catch (e:Exception){}

    }


    private fun startChat(activity: FragmentActivity) {
//        if (EMClient.getInstance() == null || EMClient.getInstance().chatManager() == null) return

        if (EMClient.getInstance() == null) return
        if (EMClient.getInstance().chatManager() == null) return

        loadAllConversations()
        val mRxPermissions = RxPermissions(activity)
        val disposable = mRxPermissions.request(
                android.Manifest.permission.ACCESS_COARSE_LOCATION,
                android.Manifest.permission.CAMERA,
                android.Manifest.permission.RECORD_AUDIO,
                android.Manifest.permission.WRITE_EXTERNAL_STORAGE,
                android.Manifest.permission.READ_EXTERNAL_STORAGE
        )
                .subscribe { aBoolean ->
                    if (aBoolean ?: false) {
                        ARouterUtil.instance.navigateEMChat()
                    } else {

                    }
                }
    }


    fun startEMChat(activity: FragmentActivity, infoBean: ImInfoBean?) {
        if (TextUtils.isEmpty(infoBean?.im_username)) return
        val emChatBaseBean = EMChatBaseBean()
        emChatBaseBean.to_username = infoBean?.im_username
        emChatBaseBean.to_headimg_url = infoBean?.im_headimg_url
        emChatBaseBean.to_nickname = infoBean?.im_nickname
        LiveDataBus.get().with(ChatFragment.EM_CHAT_DATA).postValue(emChatBaseBean)
        startChat(activity)
    }


    fun addMessageListener(onMessageListener: OnMessageListener?) {
        if (EMClient.getInstance() == null) return
        if (EMClient.getInstance().chatManager() == null) return
        if (onMessageListener == null) return
        if (!mOnMessageListenerList.contains(onMessageListener)) {
            mOnMessageListenerList.add(onMessageListener)
        }
        EMClient.getInstance().chatManager().addMessageListener(msgListener)
    }

    private var mOnMessageListenerList = ArrayList<OnMessageListener>()

    fun removeMessageListener(onMessageListener: OnMessageListener) {
        if (mOnMessageListenerList.contains(onMessageListener)) {
            mOnMessageListenerList.remove(onMessageListener)
        }
        if (mOnMessageListenerList.size == 0) {
            clearMessageListener()
        }
    }

    private fun clearMessageListener() {
        if (EMClient.getInstance() == null) return
        if (EMClient.getInstance().chatManager() == null) return
        EMClient.getInstance().chatManager().removeMessageListener(msgListener)
    }

    interface OnMessageListener {
        fun onMessageReceived(messages: List<EMMessage>)
    }

    private var msgListener: EMMessageListener = object : EMMessageListener {

        override fun onMessageReceived(messages: List<EMMessage>) {
            LogUtil.e("onMessageReceived - " + messages)
            //收到消息
            for (msgListener in mOnMessageListenerList) {
                msgListener.onMessageReceived(messages)
            }
        }

        override fun onCmdMessageReceived(messages: List<EMMessage>) {
            //收到透传消息
        }

        override fun onMessageRead(messages: List<EMMessage>) {
            //收到已读回执
        }

        override fun onMessageDelivered(message: List<EMMessage>) {
            //收到已送达回执
        }

        override fun onMessageRecalled(messages: List<EMMessage>) {
            //消息被撤回
        }

        override fun onMessageChanged(message: EMMessage, change: Any) {
            //消息状态变动
        }
    }


    //消息列表排序
    fun sortImInfo(ims: List<ImInfoBean>?, mConversationsMap: Map<String, EMConversation>?): List<ImInfoBean> {
        var sortedList = ArrayList<ImInfoBean>()
        if (ims != null) {
            for (item in ims) {
                item.lastTime = getInfoBeanLastTime(item, mConversationsMap)
                sortedList.add(item)
            }
            LogUtil.e("sortImInfo, sortedList = $sortedList")
            Collections.sort(sortedList, infoBeanComparator)
        }

        return sortedList
    }

    private val infoBeanComparator = Comparator<ImInfoBean> { lhs, rhs ->
        LogUtil.e("CarTypeListUtil, siteDefaultComparator, lhs = $lhs")
        lhs.lastTime.compareTo(rhs.lastTime) * (-1)

    }

    private fun getInfoBeanLastTime(item: ImInfoBean, mConversationsMap: Map<String, EMConversation>?): Long {

        val emConversation = mConversationsMap?.get(item.im_username)
        var lastTime = emConversation?.lastMessage?.msgTime ?: 0
        LogUtil.e("getInfoBeanLastTime, lasttime = $lastTime")
        return lastTime
    }

    //IM日期格式处理
    fun getIMDateString(msgDate: Date): String {
        val dayDiff = DateTimeUtil.getTimeDayDifference(msgDate, Date())
        var dateString = ""
        when (dayDiff) {
            0 -> {
                val sdf = SimpleDateFormat("H:mm")
                try {
                    dateString = sdf.format(msgDate)
                } catch (e: ParseException) {
                    LogUtil.e("dateToString 格式错误")
                }
            }
            1 -> {
                dateString = "昨天"
            }
            else -> {
                val sdf = SimpleDateFormat("yyyy-MM-dd")
                try {
                    dateString = sdf.format(msgDate)
                } catch (e: ParseException) {
                    LogUtil.e("dateToString 格式错误")
                }
            }
        }

        LogUtil.e("getIMDateString, dateString = " + dateString)
        return dateString
    }


    fun addPushListener() {
        EMPushHelper.getInstance().setPushListener(object : PushListener() {
            override fun onError(pushType: EMPushType, errorCode: Long) {
                // TODO: 开发者会在这个回调中收到使用推送的相关错误信息，各推送类型的error code开发者可以自己去各推送平台官网查询错误原因。
                LogUtil.e("EMPush addPushListener onError pushType : $pushType - $errorCode")
            }

            override fun isSupportPush(pushType: EMPushType, pushConfig: EMPushConfig?): Boolean {
                // TODO：开发者可以复写该方法控制设备是否支持某推送的判断。
                LogUtil.e("EMPush addPushListener isSupportPush pushType : $pushType - $pushConfig")
                LogUtil.e("EMPush addPushListener isSupportPush ${super.isSupportPush(pushType, pushConfig)}")
                return super.isSupportPush(pushType, pushConfig)

            }
        })

    }
}