package com.jiaoyinbrother.monkeyking.manager.util

import android.content.Context
import android.support.v4.app.FragmentActivity
import com.jiaoyinbrother.library.base.IBasePresenter
import com.jiaoyinbrother.library.bean.PhoneConvertRequest
import com.jiaoyinbrother.library.bean.PhoneConvertResult
import com.jiaoyinbrother.library.retrofit.BaseObserver
import com.jiaoyinbrother.library.retrofit.RetrofitHelper
import com.jiaoyinbrother.library.view.IBaseView
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class PhoneConvertUtil private constructor() {


    fun apiPhoneConvert(context: Context, phone: String, siteID: String?, view: IBaseView, presenter: IBasePresenter) {
        val request = PhoneConvertRequest()
        request.phone_number = phone
        request.site_id = if (siteID?.isNotEmpty()?:false) {
            siteID
        } else {
            PhoneConvertRequest.SITE_ID_DETAULT
        }
        val service = RetrofitHelper.getInstance(context).service
        val params = RetrofitHelper.getInstance(context).convertRequestToMap(request)
        val observable = service.phoneConvert(params).debounce(1000, TimeUnit.MILLISECONDS)
        observable.subscribeOn(Schedulers.io()).doOnSubscribe {
            view.showLoadingDialog()
        }.observeOn(AndroidSchedulers.mainThread())
            .subscribe(object : BaseObserver<PhoneConvertResult>(context, presenter) {
                override fun onSuccess(response: PhoneConvertResult) {
                    view.dismissLoadingDialog()
                    DialogUtil.getCallPermission(context as? FragmentActivity, response.dial_number)
                }

                override fun onError(e: Throwable) {
                    view.dismissLoadingDialog()
                    DialogUtil.getCallPermission(context as? FragmentActivity, phone)
                }
            })
    }


    companion object {
        private var instance: PhoneConvertUtil? = null

        fun getInstance(): PhoneConvertUtil? {
            if (instance == null) {
                instance = PhoneConvertUtil()
            }
            return instance
        }
    }
}