package com.jiaoyinbrother.monkeyking.manager.view

import android.content.Context
import android.support.constraint.ConstraintLayout
import android.text.TextUtils
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.request.RequestOptions
import com.jiaoyinbrother.library.util.DensityUtil
import com.jiaoyinbrother.monkeyking.manager.R
import kotlinx.android.synthetic.main.view_check_image.view.*

class CheckImageView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : ConstraintLayout(context, attrs, defStyleAttr) {


    private var mBg: Int = 0
    private var viewWidth: Int = 0
    private var mPhotoUrl: String = ""

    private fun getAttr(attrs: AttributeSet? = null) {
        val typedArray = context.obtainStyledAttributes(attrs, R.styleable.CheckImage)
        // 背景
        mBg = typedArray.getResourceId(R.styleable.CheckImage_background, 0)
        typedArray.recycle()
    }

    init {
        if (attrs != null) {
            getAttr(attrs)
        }
        val inflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
        inflater.inflate(R.layout.view_check_image, this)

        val wm = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        viewWidth = (wm.defaultDisplay.width - DensityUtil.dip2px(context, 54f)) / 3
        initData()
        initListener()
    }

    private fun initListener() {
        image.setOnClickListener {
            if (TextUtils.isEmpty(mPhotoUrl)) {
                listener?.onGetPhoto()
            } else {
                listener?.onBigPhoto(mPhotoUrl)
            }
        }
        clear.setOnClickListener {
            mPhotoUrl = ""
            Glide.with(context).load(mBg).placeholder(mBg).into(image)
            listener?.onDelPhoto()
        }
    }

    private fun initData() {
        val params = ConstraintLayout.LayoutParams(width, width)
        this.layoutParams = params
        Glide.with(context).load(mBg).into(image)
    }

    fun setData(url: String?) {
        mPhotoUrl = url ?: ""
        clear.visibility = if (TextUtils.isEmpty(url)) View.GONE else View.VISIBLE

        val options = RequestOptions()
                .fitCenter()
                .diskCacheStrategy(DiskCacheStrategy.NONE)
                .skipMemoryCache(true)
                .placeholder(mBg)
                .error(mBg)



        Glide.with(context).load(mPhotoUrl).apply(options).into(image)
    }

    private var listener: CheckCarClickListener? = null

    fun setListener(listener: CheckCarClickListener) {
        this.listener = listener
    }

    interface CheckCarClickListener {
        fun onBigPhoto(url: String)
        fun onDelPhoto()
        fun onGetPhoto()
    }


}