package com.jiaoyinbrother.monkeyking.manager.view

import android.app.Dialog
import android.content.Context
import android.text.Editable
import android.text.TextWatcher
import android.view.Display
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import android.widget.*
import com.bumptech.glide.Glide
import com.jiaoyinbrother.monkeyking.manager.R

class GraphicCodeDialog(val context: Context) : View.OnClickListener, TextWatcher {

    private var dialog: Dialog? = null
    private var display: Display? = null
    private var listener: GraphicCodeListener? = null

    private var onSure: Button? = null
    private var errMsgTv: TextView? = null
    private var codeEt: ClearEditText? = null
    private var codeImg: ImageView? = null
    private var cancelImg: ImageView? = null
    private var mainView: LinearLayout? = null

    fun builder(): GraphicCodeDialog {
        val view = LayoutInflater.from(context).inflate(R.layout.view_graphic_code, null)
        mainView = view.findViewById(R.id.mainView)
        onSure = view.findViewById(R.id.sure)
        errMsgTv = view.findViewById(R.id.errMsg)
        codeImg = view.findViewById(R.id.imgCode)
        codeEt = view.findViewById(R.id.etCode)
        cancelImg = view.findViewById(R.id.cancel)

        dialog = Dialog(context, R.style.AlertDialogStyle)
        dialog?.setContentView(view)
        dialog?.setCancelable(false)
        val width = (display?.width?.times(0.8))?.toInt() ?: 0
        mainView?.layoutParams = FrameLayout.LayoutParams(width, LinearLayout.LayoutParams.WRAP_CONTENT)

//        setListener
        onSure?.setOnClickListener(this)
        codeImg?.setOnClickListener(this)
        cancelImg?.setOnClickListener(this)
        codeEt?.addTextChangedListener(this)

        onSure?.isEnabled = false
        return this
    }

    fun setErrMsg(errMsg: String): GraphicCodeDialog {
        errMsgTv?.text = errMsg
        return this
    }

    fun setImgCode(res: String): GraphicCodeDialog {

        if(context!=null && codeImg!=null)
            Glide.with(context!!).load(res).placeholder(R.mipmap.bg_empty).into(codeImg!!)

        return this
    }

    fun show() {
        try {
            dialog?.show()
        } catch (e: Exception) {
        }

    }

    fun dismiss() {
        try {
            dialog?.dismiss()
        } catch (e: Exception) {
        }

    }

    fun isShowing(): Boolean? {
        return dialog?.isShowing
    }

    init {
        val windowManager = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        display = windowManager.defaultDisplay
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.cancel -> {
               listener?.onCancel()
            }

            R.id.imgCode -> {
                listener?.onImgCode()
            }

            R.id.sure -> {
                listener?.onSure(codeEt?.text.toString())
            }
        }
    }

    override fun afterTextChanged(s: Editable?) {
        val length = s?.length
        onSure?.isEnabled = length == 4
    }

    override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}

    override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}

    fun setListener(l: GraphicCodeListener) {
        this.listener = l
    }

    interface GraphicCodeListener {

        fun onCancel()

        fun onImgCode()

        fun onSure(code: String)
    }
}