package com.jiaoyinbrother.monkeyking.manager.widget

import android.content.Context
import android.support.constraint.ConstraintLayout
import android.support.v4.content.ContextCompat
import android.text.Editable
import android.text.InputFilter
import android.text.TextUtils
import android.text.TextWatcher
import android.util.AttributeSet
import android.util.TypedValue
import android.view.View
import android.view.inputmethod.EditorInfo
import com.jiaoyinbrother.library.util.px2Dp
import com.jiaoyinbrother.monkeyking.manager.R
import kotlinx.android.synthetic.main.view_edit_row.view.*

class EditRow @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) :
        ConstraintLayout(context, attrs, defStyleAttr), TextWatcher ,View.OnFocusChangeListener{


    companion object {
        const val TEXT_SIZE = 13F
    }

    // 是否启动编辑框
    private var isEdit = false
    private var isEnable = true

    // 标题文本
    private var mTitleText: String? = ""
    private var mTitleSize: Float = TEXT_SIZE
    private var mTitleColor: Int = 0

    // 尾部文本
    private var mEndText: String? = ""
    private var mEndSize: Float = TEXT_SIZE
    private var mEndColor: Int = 0

    // 输入文本
    private var mHintText: String? = ""
    private var mHintColor: Int = 0

    // 内容
    private var mTextSize: Float = TEXT_SIZE
    private var mTextColor: Int = 0

//    private var mRowEditEnable: Boolean = true
//    private var mRowEditIsSelectable: Boolean = true


    init {
        View.inflate(context, R.layout.view_edit_row, this)
        getAttr(attrs)
        initData()
        initListener()
    }

    private fun initListener() {
        view_row_content_et?.addTextChangedListener(this)
        view_row_content_et?.setOnFocusChangeListener(this)
    }

    private fun getAttr(attrs: AttributeSet?) {
        if (attrs == null)
            return
        val typedArray = context.obtainStyledAttributes(attrs, R.styleable.EditRow)
        isEdit = typedArray.getBoolean(R.styleable.EditRow_isEdit, false)
        isEnable = typedArray.getBoolean(R.styleable.EditRow_isEnable, true)
        // 标题
        mTitleText = typedArray.getString(R.styleable.EditRow_title_text)
        mTitleSize = typedArray.getDimension(R.styleable.EditRow_title_size, 0f)
        mTitleColor = typedArray.getResourceId(R.styleable.EditRow_title_color, 0)
        // 输入框
        mHintText = typedArray.getString(R.styleable.EditRow_hint_text)
        mHintColor = typedArray.getResourceId(R.styleable.EditRow_hint_color, 0)
        mTextSize = typedArray.getDimension(R.styleable.EditRow_text_size, 0f)
        mTextColor = typedArray.getResourceId(R.styleable.EditRow_text_color, 0)
        // 尾部
        mEndText = typedArray.getString(R.styleable.EditRow_end_text)
        mEndSize = typedArray.getDimension(R.styleable.EditRow_end_size, 0f)
        mEndColor = typedArray.getResourceId(R.styleable.EditRow_end_color, 0)
//        mRowEditEnable = typedArray.getBoolean(R.styleable.EditRow_edit_enable, true)
//        mRowEditIsSelectable = typedArray.getBoolean(R.styleable.EditRow_edit_isSelectable, true)
        typedArray.recycle()
    }

    private fun initData() {
        // 初始化 title
        setTitle(mTitleText)
        view_row_title?.visibility = if (TextUtils.isEmpty(mTitleText)) View.GONE else View.VISIBLE
        view_row_title?.setTextSize(TypedValue.COMPLEX_UNIT_DIP, getSize(mTitleSize))
        view_row_title?.setTextColor(getColor(mTitleColor, R.color.color_282828))
        // 初始化 edit
        initEdit()
        // 初始化 end
        view_row_end_text?.text = mEndText
        view_row_end_text?.visibility = if (TextUtils.isEmpty(mEndText)) View.GONE else View.VISIBLE
        view_row_end_text?.setTextSize(TypedValue.COMPLEX_UNIT_DIP, getSize(mEndSize))
        view_row_end_text?.setTextColor(getColor(mEndColor, R.color.color_999999))
        // 初始化是否可编辑
        isRowEdit(isEdit)
        // 初始化是否可点击
        isRowEnable(isEnable)
    }

    private fun initEdit() {
        // 编辑框
        view_row_content_et?.hint = mHintText
        view_row_content_et?.setHintTextColor(getColor(mHintColor, R.color.color_999999))
        view_row_content_et?.setTextSize(TypedValue.COMPLEX_UNIT_DIP, getSize(mTextSize))
        view_row_content_et?.setTextColor(getColor(mTextColor, R.color.color_333333))
        // 文本框
        view_row_content_tv?.hint = mHintText
        view_row_content_tv?.setHintTextColor(getColor(mHintColor, R.color.color_999999))
        view_row_content_tv?.setTextSize(TypedValue.COMPLEX_UNIT_DIP, getSize(mTextSize))
        view_row_content_et?.setTextColor(getColor(mTextColor, R.color.color_333333))
    }

    /**
     * 设置控件是否可编辑
     * @param b
     */
    private fun isRowEdit(b: Boolean) {
        view_row_content_et.visibility = if (b) View.VISIBLE else View.GONE
        view_row_content_tv.visibility = if (b) View.GONE else View.VISIBLE
    }

    /**
     * 设置控件enable状态
     * @param b
     */
    fun isRowEnable(b: Boolean) {
        isEnable = b
        view_row_content_tv.isEnabled = b
        view_row_content_et.isEnabled = b
        view_row_content_et.isFocusable = b
        view_row_content_et.isFocusableInTouchMode = b
        if (b) {
            view_row_content_ll?.setBackgroundResource(R.drawable.bg_white_stroke_gray)
        } else {
            view_row_content_ll?.setBackgroundResource(R.drawable.bg_white_stroke_gray_disable)
        }
    }

    /**
     * 设置标题文本
     * @param text
     */
    fun setTitle(text: String?) {
        view_row_title?.text = text
    }

    /**
     * 设置内容文本
     * @param text
     */
    fun setContent(text: String?) {
        if (isEdit) {
            view_row_content_et?.setText(text)
        } else {
            view_row_content_tv?.text = text
        }
    }

    /**
     * 设置标识文本
     * @param text
     */
    fun setEndText(text: String?) {
        view_row_end_text?.text = text
    }


    override fun afterTextChanged(s: Editable?) {
        listener?.afterTextChanged(s)
    }

    override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
    }

    override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
        listener?.onTextChanged(s, start, before, count)
    }

    override fun onFocusChange(v: View?, hasFocus: Boolean) {
        listener?.onFocusChange(v,hasFocus)
    }

    /**
     * 设置输入框最大长度
     * @param length
     */
    fun setMaxLength(length: Int) {
        view_row_content_et?.filters = arrayOf<InputFilter>(object : InputFilter.LengthFilter(length) {})
    }

    /**
     * 设置输入框类型
     * @param type
     */
    fun setInputType(type: Int) {
        view_row_content_et?.inputType = type
    }

    /**
     * 设置输入框action
     * EditorInfo.IME_ACTION_DONE
     */
    fun setAction(action: Int? = -1) {
        view_row_content_et?.imeOptions = if (action == -1) EditorInfo.IME_ACTION_DONE else action
                ?: 0
    }

    fun setSelection(index: Int) {
        view_row_content_et?.setSelection(index)
    }

    override fun clearFocus() {
        view_row_content_et?.clearFocus()
    }

    fun getText(): String? = if (isEdit) view_row_content_et?.text?.toString() else view_row_content_tv?.text?.toString()

    private fun getSize(size: Float) = (if (size > 0) size else TEXT_SIZE).px2Dp(context).toFloat()

    private fun getColor(resId: Int, defId: Int) = ContextCompat.getColor(context, if (resId > 0) resId else defId)


    private var listener: RowTextWatcher? = null

    fun addTextChangedListener(listener: RowTextWatcher) {
        this.listener = listener
    }

    interface RowTextWatcher {
        fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int)
        fun afterTextChanged(s: Editable?)
        fun onFocusChange(v: View?, hasFocus: Boolean)
    }

}