package com.jiaoyinbrother.monkeyking.manager.widget

import android.content.Context
import android.support.v4.content.ContextCompat
import android.util.AttributeSet
import android.view.View
import android.widget.FrameLayout
import android.widget.ImageView
import com.bumptech.glide.Glide
import com.jiaoyinbrother.monkeyking.manager.R
import com.jiaoyinbrother.monkeyking.manager.mvpactivity.order.list.fragment.OrdersFragment
import kotlinx.android.synthetic.main.view_order_manage_bar.view.*

class OrderManageBar @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) :
        FrameLayout(context, attrs, defStyleAttr) {

    private var listener: OnOrderManageSortListener? = null
    private var mSortType = OrdersFragment.DEFAULT
    private var isTimeTakeAsc = true
    private var isTimeReturnAsc = true

    init {
        View.inflate(context, R.layout.view_order_manage_bar, this)
        initData()
        initListeners()
    }

    private fun initData() {
        view_order_manage_status_bar.check(R.id.view_order_manage_status_handover)
        setTextSelectedRes(mSortType)
        setIconDefaultRes(view_order_manage_sort_time_take_icon, isTimeTakeAsc)
        setIconDefaultRes(view_order_manage_sort_time_return_icon, isTimeReturnAsc)
    }

    private fun initListeners() {
        view_order_manage_status_bar?.setOnCheckedChangeListener { _, checkedId ->
            when (checkedId) {
                R.id.view_order_manage_status_ongoing -> {
                    listener?.onStatus(OrdersFragment.ORDER_STATUS_ONGOING)
                }
                R.id.view_order_manage_status_in_settlement -> {
                    listener?.onStatus(OrdersFragment.ORDER_STATUS_SETTLE)
                }
                R.id.view_order_manage_status_completed -> {
                    listener?.onStatus(OrdersFragment.ORDER_STATUS_COMPLETED)
                }
                R.id.view_order_manage_status_cancelled -> {
                    listener?.onStatus(OrdersFragment.ORDER_STATUS_CANCELLED)
                }
                else -> {
                    listener?.onStatus(OrdersFragment.ORDER_STATUS_HANDOVER)
                }
            }
        }
        view_order_manage_sort_default?.setOnClickListener {
            onSortDefault()
        }
        view_order_manage_sort_time_take?.setOnClickListener {
            onSortTimeTake()
        }
        view_order_manage_sort_time_return?.setOnClickListener {
            onSortTimeReturn()
        }
    }

    private fun onSortDefault() {
        mSortType = OrdersFragment.DEFAULT
        setTextSelectedRes(mSortType)
        setIconDefaultRes(view_order_manage_sort_time_take_icon, true)
        setIconDefaultRes(view_order_manage_sort_time_return_icon, true)
        listener?.onSort(mSortType, true)
    }

    private fun onSortTimeTake() {
        mSortType = OrdersFragment.TIME_TAKE
        setTextSelectedRes(mSortType)
        isTimeTakeAsc = !isTimeTakeAsc
        setIconSelectedRes(view_order_manage_sort_time_take_icon, isTimeTakeAsc)
        isTimeReturnAsc = true
        setIconDefaultRes(view_order_manage_sort_time_return_icon, isTimeReturnAsc)
        listener?.onSort(mSortType, isTimeTakeAsc)
    }

    private fun onSortTimeReturn() {
        mSortType = OrdersFragment.TIME_RETURN
        setTextSelectedRes(mSortType)
        isTimeReturnAsc = !isTimeReturnAsc
        setIconSelectedRes(view_order_manage_sort_time_return_icon, isTimeReturnAsc)
        isTimeTakeAsc = true
        setIconDefaultRes(view_order_manage_sort_time_take_icon, isTimeTakeAsc)
        listener?.onSort(mSortType, isTimeReturnAsc)
    }

    private fun setIconSelectedRes(view: ImageView?, isAsc: Boolean) {
        view?.let {
            Glide.with(context).load(
                    if (isAsc) {
                        R.mipmap.ic_order_item_sort_des
                    } else {
                        R.mipmap.ic_order_item_sort_asc
                    }
            ).into(view)
        }
    }

    private fun setIconDefaultRes(view: ImageView?, isAsc: Boolean) {
        view?.let {
            Glide.with(context).load(
                    if (isAsc) {
                        R.mipmap.ic_order_item_sort_default_asc
                    } else {
                        R.mipmap.ic_order_item_sort_default_des
                    }
            ).into(view)
        }
    }

    private fun setTextSelectedRes(type: Int) {
        view_order_manage_sort_default?.setTextColor(ContextCompat.getColor(context, R.color.color_4))
        view_order_manage_sort_time_take_name?.setTextColor(ContextCompat.getColor(context, R.color.color_4))
        view_order_manage_sort_time_return_name?.setTextColor(ContextCompat.getColor(context, R.color.color_4))
        when (type) {
            OrdersFragment.DEFAULT ->
                view_order_manage_sort_default?.setTextColor(ContextCompat.getColor(context, R.color.color_6))
            OrdersFragment.TIME_TAKE ->
                view_order_manage_sort_time_take_name?.setTextColor(ContextCompat.getColor(context, R.color.color_6))
            OrdersFragment.TIME_RETURN ->
                view_order_manage_sort_time_return_name?.setTextColor(ContextCompat.getColor(context, R.color.color_6))
        }
    }

    private fun getViewId(status: Int) = when (status) {
        OrdersFragment.ORDER_STATUS_HANDOVER -> R.id.view_order_manage_status_handover
        OrdersFragment.ORDER_STATUS_ONGOING -> R.id.view_order_manage_status_ongoing
        OrdersFragment.ORDER_STATUS_SETTLE -> R.id.view_order_manage_status_in_settlement
        OrdersFragment.ORDER_STATUS_COMPLETED -> R.id.view_order_manage_status_completed
        else -> R.id.view_order_manage_status_cancelled
    }

    fun setManageStatusBar(status: Int) {
        view_order_manage_status_bar?.check(getViewId(status))
    }

    fun setListener(listener: OnOrderManageSortListener) {
        this.listener = listener
    }

    interface OnOrderManageSortListener {
        fun onStatus(status: Int)
        fun onSort(type: Int, isAsc: Boolean)
    }
}